package com.megabras.bluelogg.DT70KV;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.os.Environment;
import android.util.Log;

import com.megabras.bluelogg.R;
import com.megabras.modbus.DT70KV.Measurement;
import com.megabras.modbus.DT70KV.TestData;

import java.io.File;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.Observer;

public final class Database extends SQLiteOpenHelper
{

    private String
            TAG          = "Database";

    private static final String
            TEXT_TYPE    = " TEXT",
            INT_TYPE     = " INTEGER",
            FLOAT_TYPE   = " FLOAT",
            DOUBLE_TYPE  = " DOUBLE",
            DATE_TYPE    = " DATETIME",
            COMMA_SEP    = ",";

    private String
            root         = Environment.getExternalStorageDirectory().toString(),
            model,
            serial;

    private File
            dir,
            tempDir,
            rootDir;

    private com.megabras.modbus.DT70KV.FormatData
            formatData;

    public Database(Context context, String model, String serial, String dbName)
    {
        super(context, dbName, null, Entry.DB_VERSION);
        createFolder(context, model);
        this.model  = model;
        this.serial = serial;
        createFileList();
        formatData = new com.megabras.modbus.DT70KV.FormatData();
    }

    private void createFileList()
    {
        String SQL = "CREATE TABLE IF NOT EXISTS " + Entry.FILE_LIST + " (\n"
                + "\t" + Entry.FILE_ID               + " INTEGER PRIMARY KEY, \n"
                + "\t" + Entry.GLOVE_SERIAL          + " TEXT   , \n"
                + "\t" + Entry.TEST_NUMBER           + " REAL   , \n"
                + "\t" + Entry.DATE                  + " INTEGER  \n"
                + ");";

        SQLiteDatabase db = this.getWritableDatabase();
        db.execSQL(SQL);
    }

    public void save(TestData testData)
    {
        long TABLE_NAME;
        if ( testData.getID() > -1 )
            TABLE_NAME = testData.getID();
        else
            TABLE_NAME = new Date().getTime();

        String sTABLE_NAME = "ID" + TABLE_NAME;

        SQLiteDatabase db = this.getWritableDatabase();

        Cursor c  = db.rawQuery("SELECT * FROM sqlite_master WHERE type='table' AND name='" + sTABLE_NAME + "';", null);
        int count = c.getCount();
        c.close();
        // TABLE ALREADY EXITS
        if( count > 0 )
            return;

        String FIELD_ENTRIES =
                Entry.DESCRIPTION   + TEXT_TYPE + COMMA_SEP +
                Entry.COL_A         + TEXT_TYPE + COMMA_SEP +
                Entry.COL_B         + TEXT_TYPE + COMMA_SEP +
                Entry.COL_C         + TEXT_TYPE + COMMA_SEP +
                Entry.COL_D         + TEXT_TYPE + COMMA_SEP +
                Entry.COL_E         + TEXT_TYPE;

        String SQL_CREATE_TABLE =
                "CREATE TABLE " + sTABLE_NAME + " (" + FIELD_ENTRIES + " )";


        db.execSQL(SQL_CREATE_TABLE);

        db.beginTransaction();

        ContentValues values = new ContentValues();
        values.put( Entry.DESCRIPTION, Entry.MODEL );
        values.put( Entry.COL_A      , testData.getModel() );
        db.insertWithOnConflict(sTABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);

        values = new ContentValues();
        values.put( Entry.DESCRIPTION, Entry.SERIAL );
        values.put( Entry.COL_A      , testData.getSerial() );
        db.insertWithOnConflict(sTABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);

        values = new ContentValues();
        values.put( Entry.DESCRIPTION, Entry.MEMORY_MAP );
        values.put( Entry.COL_A      , testData.getMemoryMap() );
        db.insertWithOnConflict(sTABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);

        values = new ContentValues();
        values.put( Entry.DESCRIPTION, Entry.HARDWARE );
        values.put( Entry.COL_A      , testData.getHardware() );
        db.insertWithOnConflict(sTABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);

        values = new ContentValues();
        values.put( Entry.DESCRIPTION, Entry.FIRMWARE );
        values.put( Entry.COL_A      , testData.getFirmware() );
        db.insertWithOnConflict(sTABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);

        values = new ContentValues();
        values.put( Entry.DESCRIPTION, Entry.TEST_NUMBER );
        values.put( Entry.COL_A      , testData.getTestNumber() );
        db.insertWithOnConflict(sTABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);

        values = new ContentValues();
        values.put( Entry.DESCRIPTION, Entry.DATE );
        values.put( Entry.COL_A      , testData.getDate().getTime()+"" );
        db.insertWithOnConflict(sTABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);

        values = new ContentValues();
        values.put( Entry.DESCRIPTION, Entry.OP_MODE );
        values.put( Entry.COL_A      , testData.getTestMode() );
        db.insertWithOnConflict(sTABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);

        values = new ContentValues();
        values.put( Entry.DESCRIPTION, Entry.GLOVE_SERIAL );
        values.put( Entry.COL_A      , testData.getGloveSerial() );
        db.insertWithOnConflict(sTABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);

        values = new ContentValues();
        values.put( Entry.DESCRIPTION, Entry.GLOVE_CLASS );
        values.put( Entry.COL_A      , testData.getGloveClass() );
        db.insertWithOnConflict(sTABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);

        values = new ContentValues();
        values.put( Entry.DESCRIPTION, Entry.GPS_ACCURACY );
        values.put( Entry.COL_A      , testData.getGpsAcuraccy()+"" );
        db.insertWithOnConflict(sTABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);

        values = new ContentValues();
        values.put( Entry.DESCRIPTION, Entry.GPS_ALTITUDE );
        values.put( Entry.COL_A      , testData.getGpsAltitude()+"" );
        db.insertWithOnConflict(sTABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);

        values = new ContentValues();
        values.put( Entry.DESCRIPTION, Entry.GPS_LATITUDE );
        values.put( Entry.COL_A      , testData.getGpsLatitude()+"" );
        db.insertWithOnConflict(sTABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);

        values = new ContentValues();
        values.put( Entry.DESCRIPTION, Entry.GPS_LONGITUDE );
        values.put( Entry.COL_A      , testData.getGpsLongitude()+"" );
        db.insertWithOnConflict(sTABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);

        values = new ContentValues();
        values.put( Entry.DESCRIPTION, Entry.RESULT );
        values.put( Entry.COL_A      , testData.getTestResult() );
        db.insertWithOnConflict(sTABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);

        for (Measurement m : testData.getMeasurement())
        {
            values = new ContentValues();
            values.put( Entry.DESCRIPTION, Entry.MEASURE );
            values.put( Entry.COL_A      , m.getElapsedTotal() );
            values.put( Entry.COL_B      , m.getVolt()         );
            values.put( Entry.COL_C      , m.getAmpRT()        );
            values.put( Entry.COL_D      , m.getStats()        );
            db.insertWithOnConflict(sTABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);
        }

        db.setTransactionSuccessful();
        db.endTransaction();

        testData.setID(TABLE_NAME);
        testData.isIncomplete(false);
        addNewTable(testData);
    }

    public void addNewTable(TestData testData)
    {
        long TABLE_NAME;
        if ( testData.getID() > -1 )
            TABLE_NAME = testData.getID();
        else
            TABLE_NAME = new Date().getTime();

        testData.setID(TABLE_NAME);

        SQLiteDatabase db = this.getWritableDatabase();
        db.beginTransaction();

        ContentValues values = new ContentValues();
        values.put( Entry.FILE_ID     , TABLE_NAME                   );
        values.put( Entry.GLOVE_SERIAL, testData.getGloveSerial()    );
        values.put( Entry.TEST_NUMBER , testData.getTestNumber()     );
        if ( testData.getDate() != null )
            values.put( Entry.DATE        , testData.getDate().getTime() );
        else
            values.put( Entry.DATE        , 0                            );

        db.insertWithOnConflict(Entry.FILE_LIST, null, values, SQLiteDatabase.CONFLICT_REPLACE);
        db.setTransactionSuccessful();
        db.endTransaction();
    }

    void updateGpsData(TestData testData)
    {
        try
        {
            String TABLE_NAME = "ID" + testData.getID();
            SQLiteDatabase db = this.getWritableDatabase();
            db.beginTransaction();

            ContentValues values;

            values = new ContentValues();
            values.put(Entry.DESCRIPTION, Entry.GPS_LATITUDE );
            values.put(Entry.COL_A, testData.getGpsLatitude());
            db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);

            values = new ContentValues();
            values.put(Entry.DESCRIPTION, Entry.GPS_LONGITUDE );
            values.put(Entry.COL_A, testData.getGpsLongitude());
            db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);

            values = new ContentValues();
            values.put(Entry.DESCRIPTION, Entry.GPS_ALTITUDE );
            values.put(Entry.COL_A, testData.getGpsAltitude());
            db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);

            values = new ContentValues();
            values.put(Entry.DESCRIPTION, Entry.GPS_ACCURACY );
            values.put(Entry.COL_A, testData.getGpsAcuraccy());
            db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);

            db.setTransactionSuccessful();
            db.endTransaction();
        }
        catch (Exception e)
        {
            Log.e(TAG, e.getMessage());
        }
    }

    List<FileDetails> getSavedFilesList(int dateFormat, int timeFormat)
    {
        try
        {
            List<FileDetails>
                    filesList = new ArrayList<>();
            SQLiteDatabase db = this.getReadableDatabase();
            Cursor c          = db.rawQuery("SELECT * FROM " + Entry.FILE_LIST + " ORDER BY " + Entry.DATE + " ;", null);

            if (c.moveToFirst())
            {
                while ( !c.isAfterLast() )
                {
                    FileDetails file  = new FileDetails();
                    file.FILE_NAME    = c.getLong  (0);
                    file.GLOVE_SERIAL = c.getString(1);
                    file.TEST_NUMBER  = c.getInt   (2);
                    String sDate      = c.getString(3);
                    long   lDate      = Long.parseLong(sDate);
                    file.DATE         = formatData.dateTime(new Date(lDate), dateFormat, timeFormat);

                    filesList.add(file);
                    c.moveToNext();
                }
            }
            c.close();
            return filesList;
        }
        catch (Exception e)
        {
            return null;
        }
    }

    TestData getResult(Observer observer, long TABLE_NAME, String gloveSerial)
    {
        TestData testData = new TestData( observer, model, serial );
        Cursor c = null;
        try
        {
            testData.setID(TABLE_NAME);
            testData.setGloveSerial(gloveSerial);
            SQLiteDatabase db = this.getReadableDatabase();
            c                 = db.rawQuery("SELECT * FROM " + "ID"+TABLE_NAME, null);

            int count = c.getCount();

            if( count > 0 )
            {
                if (c.moveToFirst())
                {
                    while ( !c.isAfterLast() )
                    {
                        String colA = c.getString(0);
                        String colB = c.getString(1);
                        String colC = c.getString(2);
                        String colD = c.getString(3);
                        String colE = c.getString(4);

                        switch ( colA )
                        {
                            case Entry.MODEL:
                                testData.setModel(colB);
                                break;
                            case Entry.SERIAL:
                                testData.setSerial(colB);
                                break;
                            case Entry.MEMORY_MAP:
                                if ( colB != null )
                                    testData.setMemoryMap(Integer.parseInt(colB));
                                break;
                            case Entry.HARDWARE:
                                if ( colB != null )
                                    testData.setHardware(Integer.parseInt(colB));
                            case Entry.FIRMWARE:
                                if ( colB != null )
                                    testData.setFirmware(Integer.parseInt(colB));
                                break;
                            case Entry.TEST_NUMBER:
                                testData.setTestNumber( Integer.parseInt(colB) );
                                break;
                            case Entry.DATE:
                                Date datef = new Date();
                                datef.setTime(Long.parseLong(colB));
                                testData.setDate(datef);
                                break;
                            case Entry.OP_MODE:
                                testData.setTestMode( Integer.parseInt(colB) );
                                break;
                            case Entry.GLOVE_SERIAL:
                                testData.setGloveSerial( colB );
                                break;
                            case Entry.GLOVE_CLASS:
                                testData.setGloveClass( Integer.parseInt(colB) );
                                break;
                            case Entry.RESULT:
                                testData.setTestResult(Integer.parseInt(colB));
                                break;
                            case Entry.GPS_LATITUDE:
                                testData.setGpsLatitude(Double.parseDouble(colB));
                                break;
                            case Entry.GPS_LONGITUDE:
                                testData.setGpsLongitude(Double.parseDouble(colB));
                                break;
                            case Entry.GPS_ALTITUDE:
                                testData.setGpsAltitude(Double.parseDouble(colB));
                                break;
                            case Entry.GPS_ACCURACY:
                                testData.setGpsAcuraccy(Double.parseDouble(colB));
                                break;
                            case Entry.MEASURE:
                                Measurement m = new Measurement();
                                m.setElapsedTotal( Integer.parseInt   (colB) );
                                m.setVolt        ( Double.parseDouble (colC) );
                                m.setAmpRT       ( Double.parseDouble (colD) );
                                m.setStats       ( Integer.parseInt   (colE) );
                                testData.addMeasument(m);
                                break;
                        }
                        c.moveToNext();
                    }
                }
            }

            c.close();
            if ( testData.getTestNumber() < 0)
                testData.isIncomplete(true);
            else
                testData.isIncomplete(false);
            return testData;
        }
        catch (Exception e)
        {
            if ( c != null ) c.close();
            Log.e(TAG, "ERROR: " + e.getMessage() );
            testData.isIncomplete(true);
            return testData;
        }
    }

    void deleteTable(String serial, long name)
    {
        try
        {
            SQLiteDatabase db = this.getWritableDatabase();
            String SQL_DELETE_TABLE = "DROP TABLE IF EXISTS " + "ID" + name;
            db.execSQL(SQL_DELETE_TABLE);

            db.delete(Entry.FILE_LIST, Entry.FILE_ID + "=" + name, null);
            deleteFiles( serial, name+"");
        }
        catch (Exception ignored)
        {
        }
    }

    //##########################################################################
    //# Folders
    //##########################################################################

    private boolean isExternalStorageAvaiable()
    {
        String state = Environment.getExternalStorageState();
        return Environment.MEDIA_MOUNTED.equals(state);
    }

    public boolean createFolder(Context context, String model)
    {
        if ( !this.isExternalStorageAvaiable() )
            return false;

        rootDir     = new File(root + "/" + context.getResources().getString(R.string.company));
        if ( !rootDir.exists() )
            rootDir.mkdir();

        dir     = new File(root + "/" + context.getResources().getString(R.string.company)+ "/" + model);
        tempDir = new File(dir + "/temp");

        if ( !dir.exists() )
            dir.mkdir();

        if ( !tempDir.exists() )
            tempDir.mkdir();

        return dir.exists();
    }

    public File getFolder()
    {
        if ( !rootDir.exists() )
            rootDir.mkdir();

        if ( !dir.exists() )
            dir.mkdir();

        return dir;
    }

    File getTemporaryFolder()
    {
        if ( !rootDir.exists() )
            rootDir.mkdir();

        if ( !dir.exists() )
            dir.mkdir();

        if ( !tempDir.exists() )
            tempDir.mkdir();

        return tempDir;
    }

    public File getPictureFile(String serial, String timeStamp)
    {
        String img = serial + "_" + timeStamp + ".jpg";
        return new File(dir, img);
    }

    public File getPictureFile(String serial, String testNumber, String timeStamp)
    {
        String img = serial + "_" + testNumber + "_" + timeStamp + ".jpg";
        return new File(dir, img);
    }

    File getAudioFile(String serial, String timeStamp)
    {
        String audioSuffix = "_annotation.3gp";
        String audio = serial + "_" + timeStamp + audioSuffix;
        return new File(dir, audio);
    }

    File getMapSnapshotFile(String serial, String timeStamp)
    {
        String mapSuffix = "_map_snapshot.jpg";
        String map = serial + "_" + timeStamp + mapSuffix;
        return new File(dir, map);
    }

    @SuppressWarnings("ResultOfMethodCallIgnored")
    private void deleteFiles(String serial, String timeStamp)
    {
        if ( dir.listFiles() == null )
            return;
        for ( File child : Objects.requireNonNull( dir.listFiles() ) )
        {
            if( child.getName().contains( serial + "_" + timeStamp ) )
            {
                child.delete();
            }
        }
    }

    @Override public void onCreate(SQLiteDatabase db) {}
    @Override public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) { }

    public static abstract class Entry
    {
        static final int
                DB_VERSION = 2;

        static final String
                FILE_LIST       = "FILE_LIST",
                FILE_ID         = "FILE_ID",
                DESCRIPTION     = "DESCRIPTION",
                COL_A           = "COL_A",
                COL_B           = "COL_B",
                COL_C           = "COL_C",
                COL_D           = "COL_D",
                COL_E           = "COL_E";

        static final String
                MODEL           = "MODEL",
                SERIAL          = "SERIAL",
                MEMORY_MAP      = "MEMORY_MAP",
                FIRMWARE        = "FIRMWARE",
                HARDWARE        = "HARDWARE",
                OP_MODE         = "OP_MODE",
                GLOVE_SERIAL    = "GLOVE_SERIAL",
                GLOVE_CLASS     = "GLOVE_CLASS",
                DATE            = "DATE",
                TEST_NUMBER     = "TEST_NUMBER",
                MEASURE         = "MEASURE",
                RESULT          = "RESULT",
                GPS_LATITUDE    = "GPS_LATITUDE",
                GPS_LONGITUDE   = "GPS_LONGITUDE",
                GPS_ALTITUDE    = "GPS_ALTITUDE",
                GPS_ACCURACY    = "GPS_ACCURACY";
    }

}

