package com.megabras.bluelogg.DT70KV;

import android.content.Context;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import com.megabras.bluelogg.R;
import com.megabras.modbus.DT70KV.Measurement;

import androidx.annotation.NonNull;
import androidx.fragment.app.Fragment;

public class MainScreen extends Fragment
{

    private final String
            TAG             = "MainScreen",
            NONE            = " - - - ";

    private int
            layoutID        =  R.layout.dt70kv_fgr_main;

    private Context
            context;

    private TextView
            ui_imn_pass,
            ui_imn_fail,
            ui_imn_lock,
            ui_imn_spark,
            ui_imn_aborted,
            ui_imn_overcurrent,
            ui_imn_emergency,
            ui_imn_protection,
            tvTestNumber,
            ui_txw_standard,
            ui_txw_glove_class,
            ui_txw_timer,
            ui_txw_max_voltage,
            ui_txw_max_current,
            def_ramp_up,
            def_ramp_down,
            ui_txw_rt_elapsed,
            ui_txw_rt_ramp_elapsed,
            ui_txw_rt_volts,
            ui_txw_rt_amps;

    private ImageView
            ic_ramp,
            ic_ramp_fix,
            ic_ramp_up,
            ic_ramp_down;

    private String[]
            modes;

    private com.megabras.modbus.DT70KV.FormatData
            formatData;

    public MainScreen()
    {
        formatData = new com.megabras.modbus.DT70KV.FormatData();
    }

    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState)
    {
        context   = inflater.getContext();
        View view = inflater.inflate(layoutID, container, false);
        view.setBackgroundColor(context.getResources().getColor(R.color.transparent));
        return view;
    }

    @Override
    public void onStart()
    {
        super.onStart();
        if ( this.getView() == null )
            return;
        modes                   = context.getResources().getStringArray(R.array.glove_class);
        ui_imn_pass             = this.getView().findViewById(R.id.result_pass);
        ui_imn_fail             = this.getView().findViewById(R.id.result_fail);
        ui_imn_aborted          = this.getView().findViewById(R.id.result_aborted);
        ui_imn_lock             = this.getView().findViewById(R.id.result_lock);
        ui_imn_protection       = this.getView().findViewById(R.id.result_protection);
        ui_imn_overcurrent      = this.getView().findViewById(R.id.result_overcurrent);
        ui_imn_emergency        = this.getView().findViewById(R.id.result_emergency);
        ui_imn_spark            = this.getView().findViewById(R.id.result_spark);
        tvTestNumber            = this.getView().findViewById(R.id.lb_test_number);
        ui_txw_standard         = this.getView().findViewById(R.id.lb_standard);
        ui_txw_glove_class      = this.getView().findViewById(R.id.lb_glove_class);
        ui_txw_timer            = this.getView().findViewById(R.id.lb_timer);
        ui_txw_max_voltage      = this.getView().findViewById(R.id.lb_max_voltage);
        ui_txw_max_current      = this.getView().findViewById(R.id.lb_max_current);
        def_ramp_up             = this.getView().findViewById(R.id.lb_ramp_up);
        def_ramp_down           = this.getView().findViewById(R.id.lb_ramp_down);
        ui_txw_rt_elapsed       = this.getView().findViewById(R.id.lb_time_elapsed);
        ui_txw_rt_ramp_elapsed  = this.getView().findViewById(R.id.lb_time_ramp);
        ui_txw_rt_volts         = this.getView().findViewById(R.id.lb_voltage);
        ui_txw_rt_amps          = this.getView().findViewById(R.id.lb_current);
        ic_ramp                 = this.getView().findViewById(R.id.ic_ramp);
        ic_ramp_fix             = this.getView().findViewById(R.id.ic_ramp_fix);
        ic_ramp_up              = this.getView().findViewById(R.id.ic_ramp_up);
        ic_ramp_down            = this.getView().findViewById(R.id.ic_ramp_down);
    }

    public void setTestNumber(int number)
    {
        if ( number <= 0)
            updateTextView( tvTestNumber, NONE   );
        else
            updateTextView( tvTestNumber, number );
    }

    void setStandard(String standard)
    {
        updateTextView( ui_txw_standard, standard );
    }

    void setTestDuration(String time)
    {
        updateTextView( ui_txw_timer, time );
    }

    void setGloveClass(int gloveClass)
    {
        if ( gloveClass < modes.length )
            updateTextView( ui_txw_glove_class, modes[gloveClass] );
        else
            updateTextView( ui_txw_glove_class, NONE );
    }

    void setMaxVoltage(String voltage)
    {
        updateTextView( ui_txw_max_voltage, voltage );
    }

    void setMaxAmp(String amp)
    {
        updateTextView( ui_txw_max_current, amp );
    }

    void setMeasurement(Measurement mea, int testDuration)
    {
        int testTime = testDuration - mea.getElapsed();
        if ( testTime >= 0 )
            updateTextView( ui_txw_rt_ramp_elapsed, formatData.elapsed( testTime              ) );
        updateTextView( ui_txw_rt_elapsed     , formatData.elapsed( mea.getElapsedTotal() ) );
        updateTextView( ui_txw_rt_volts       , formatData.volts  ( mea.getVolt()         ) );
        updateTextView( ui_txw_rt_amps        , formatData.amps   ( mea.getAmpRT()        ) );
    }

    private void updateTextView(TextView tv, int value)
    {
        updateTextView( tv, value + "");
    }

    private void updateTextView(TextView tv, String text)
    {
        if ( tv == null )
            return;
        if ( !tv.getText().equals(text) )
            tv.setText(text);
    }

    private void updateVisibility(View v, int visibility)
    {
        if ( v == null )
            return;
        if ( v.getVisibility() != visibility )
            v.setVisibility( visibility );
    }

    void waitonDisplay()
    {
        updateTextView( ui_txw_rt_volts, context.getString(R.string.wait) );
        updateTextView( ui_txw_rt_amps , context.getString(R.string.wait) );
    }

    void rampUp()
    {
        updateVisibility(ic_ramp     , View.GONE);
        updateVisibility(ic_ramp_up  , View.VISIBLE);
        updateVisibility(ic_ramp_fix , View.GONE);
        updateVisibility(ic_ramp_down, View.GONE);
    }

    void rampDown()
    {
        updateVisibility(ic_ramp     , View.GONE);
        updateVisibility(ic_ramp_up  , View.GONE);
        updateVisibility(ic_ramp_fix , View.GONE);
        updateVisibility(ic_ramp_down, View.VISIBLE);
    }

    void rampFix()
    {
        updateVisibility(ic_ramp     , View.GONE);
        updateVisibility(ic_ramp_up  , View.GONE);
        updateVisibility(ic_ramp_fix , View.VISIBLE);
        updateVisibility(ic_ramp_down, View.GONE);
    }

    void rampHide()
    {
        updateVisibility(ic_ramp     , View.GONE);
        updateVisibility(ic_ramp_up  , View.GONE);
        updateVisibility(ic_ramp_fix , View.GONE);
        updateVisibility(ic_ramp_down, View.GONE);
    }

    void isEmergencyStop(boolean visible)
    {
        updateVisibility(ui_imn_emergency, (visible ? View.VISIBLE : View.GONE));
    }

    void isGenerationLock(boolean visible)
    {
        updateVisibility(ui_imn_lock, (visible ? View.VISIBLE : View.GONE));
    }

    void isSpark(boolean visible)
    {
        updateVisibility(ui_imn_spark, (visible ? View.VISIBLE : View.GONE));
    }

    void isOvercurrent(boolean visible)
    {
        updateVisibility(ui_imn_overcurrent, (visible ? View.VISIBLE : View.GONE));
    }

    void isInternalProtection(boolean visible)
    {
        updateVisibility(ui_imn_protection, (visible ? View.VISIBLE : View.GONE));
    }

    void isResultPass(boolean visible)
    {
        updateVisibility(ui_imn_pass, (visible ? View.VISIBLE : View.GONE));
    }

    void isResultFail(boolean visible)
    {
        updateVisibility(ui_imn_fail, (visible ? View.VISIBLE : View.GONE));
    }

    void isResultAborted(boolean visible)
    {
        updateVisibility(ui_imn_aborted, (visible ? View.VISIBLE : View.GONE));
    }

    public void resetRT()
    {
        updateTextView  (tvTestNumber          , NONE);
        updateTextView  (ui_txw_rt_elapsed     , NONE);
        updateTextView  (ui_txw_rt_ramp_elapsed, NONE);
        updateTextView  (ui_txw_rt_volts       , NONE);
        updateTextView  (ui_txw_rt_amps        , NONE);
        updateVisibility(ui_imn_pass           , View.GONE);
        updateVisibility(ui_imn_fail           , View.GONE);
        updateVisibility(ui_imn_aborted        , View.GONE);
        updateVisibility(ui_imn_lock           , View.GONE);
        updateVisibility(ui_imn_overcurrent    , View.GONE);
        updateVisibility(ui_imn_emergency      , View.GONE);
        updateVisibility(ui_imn_spark          , View.GONE);
    }

    public void resetAll()
    {
        updateTextView  (tvTestNumber          , NONE);
        updateTextView  (ui_txw_glove_class    , NONE);
        updateTextView  (ui_txw_max_voltage    , NONE);
        updateTextView  (ui_txw_max_current    , NONE);
        updateTextView  (ui_txw_standard       , NONE);
        updateTextView  (ui_txw_timer          , NONE);
        updateTextView  (ui_txw_rt_elapsed     , NONE);
        updateTextView  (ui_txw_rt_ramp_elapsed, NONE);
        updateTextView  (ui_txw_rt_volts       , NONE);
        updateTextView  (ui_txw_rt_amps        , NONE);
        updateTextView  (def_ramp_up           , NONE);
        updateTextView  (def_ramp_down         , NONE);
        updateVisibility(ui_imn_pass           , View.GONE);
        updateVisibility(ui_imn_fail           , View.GONE);
        updateVisibility(ui_imn_aborted        , View.GONE);
        updateVisibility(ui_imn_lock           , View.GONE);
        updateVisibility(ui_imn_overcurrent    , View.GONE);
        updateVisibility(ui_imn_emergency      , View.GONE);
        updateVisibility(ui_imn_spark          , View.GONE);
    }
}

