package com.megabras.bluelogg.DT70KV;

import android.content.Context;
import android.content.res.Configuration;
import android.graphics.drawable.Drawable;
import android.text.Spannable;
import android.text.SpannableString;
import android.text.style.ImageSpan;
import android.util.Log;
import android.view.View;

import com.megabras.bluelogg.R;
import com.megabras.bluelogg.DT70KV.Charts;
import com.megabras.bluelogg.DT70KV.MainScreen;
import com.megabras.modbus.DT70KV.Device;
import com.megabras.modbus.DT70KV.FormatData;
import com.megabras.modbus.DT70KV.Measurement;
import com.megabras.modbus.DT70KV.TestData;

import java.util.ArrayList;
import java.util.List;

import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentManager;
import androidx.fragment.app.FragmentStatePagerAdapter;

public class PagerAdapter extends FragmentStatePagerAdapter
{

    private final String
            TAG = "PagerAdapter";

    private Context
            context;

    private MainScreen
            mainScreen;

    private Charts
            voltageChart,
            currentChart,
            table;

    private com.megabras.modbus.DT70KV.FormatData
            formatData;

    private final List<Fragment> mFragmentList = new ArrayList<>();

    private final List<CharSequence> mFragmentTitleList = new ArrayList<>();

    public PagerAdapter(FragmentManager fm, Context context)
    {
        super(fm);
        this.context = context;
        mainScreen   = new MainScreen();
        currentChart = new Charts();
        currentChart.setChartType( Charts.CURRENT_CHART, context.getString(R.string.md_current) );
        voltageChart = new Charts();
        voltageChart.setChartType( Charts.VOLTAGE_CHART, context.getString(R.string.md_voltage) );
        table        = new Charts();
        table.setChartType( Charts.TABLE, "" );

        mFragmentList.add(mainScreen);
        mFragmentList.add(voltageChart);
        mFragmentList.add(currentChart);
        mFragmentList.add(table);

        mFragmentTitleList.add(convertTitle(0));
        mFragmentTitleList.add(convertTitle(1));
        mFragmentTitleList.add(convertTitle(2));
        mFragmentTitleList.add(convertTitle(3));

        formatData = new com.megabras.modbus.DT70KV.FormatData();
    }

    @Override
    public Fragment getItem(int i)
    {
        return mFragmentList.get(i);
    }

    @Override
    public int getCount()
    {
        return mFragmentList.size();
    }

    @Override
    public CharSequence getPageTitle(int position)
    {
        return mFragmentTitleList.get(position);
    }

    private CharSequence convertTitle(int position)
    {
        int[] images;
        if(screenSizeIsLarge(context))
            images = imageResId;
        else
            images = imageResIdSmall;

        Drawable image      = context.getResources().getDrawable(images[position]);
        image.setBounds(0, 0, image.getIntrinsicWidth(), image.getIntrinsicHeight());
        SpannableString sb  = new SpannableString(" ");
        ImageSpan imageSpan = new ImageSpan(image, ImageSpan.ALIGN_BOTTOM);
        sb.setSpan(imageSpan, 0, 1, Spannable.SPAN_EXCLUSIVE_EXCLUSIVE);
        return sb;
    }

    private int[] imageResId =
    {
        R.drawable.ic_analog,
        R.drawable.ic_chart_voltage,
        R.drawable.ic_chart_current,
        R.drawable.ic_table
    };

    private int[] imageResIdSmall =
    {
        R.drawable.ic_analog_20,
        R.drawable.ic_chart_voltage_20,
        R.drawable.ic_chart_current_20,
        R.drawable.ic_table_20
    };

    void setTestNumber(int number)
    {
        for (Fragment f : mFragmentList)
        {
            if (f instanceof Charts)
            {
                Charts c = (Charts)f;
                if (c.getChartType()==Charts.TABLE)
                {
                    c.setTestNumber(number);
                }
            }
            else if (f instanceof MainScreen)
            {
                MainScreen m = (MainScreen)f;
                m.setTestNumber(number);
            }
        }
    }

    void setGloveSerial(String serial)
    {
        table.setGloveSerial( serial );
    }

    void setTestDate(String dateTime)
    {
        for (Fragment f : mFragmentList)
        {
            if (f instanceof Charts)
            {
                Charts c = (Charts)f;
                if (c.getChartType()==Charts.TABLE)
                {
                    c.setDate(dateTime);
                }
            }
        }
    }

    void setTestMode(String mode)
    {
        for (Fragment f : mFragmentList)
        {
            if (f instanceof Charts)
            {
                Charts c = (Charts)f;
                if (c.getChartType()==Charts.TABLE)
                {
                    c.setGloveClass(mode);
                }
            }
        }
    }

    void waitonDisplay()
    {
        mainScreen.waitonDisplay();
    }

    void fillChartsAndTable(TestData testData, Device device)
    {
        String[] gloveClasses = context.getResources().getStringArray(R.array.glove_class);
        for (Fragment f : mFragmentList)
        {
            if (f instanceof Charts)
            {
                Charts c = (Charts)f;
                c.reset();

                if ( c.getChartType() == Charts.TABLE )
                {
                    c.setAllValues  ( testData.getMeasurement() );
                    c.setTestNumber ( testData.getTestNumber()  );
                    c.setGloveSerial( testData.getGloveSerial() );
                    if ( testData.getGloveClass() > -1 )
                        c.setGloveClass( gloveClasses[testData.getGloveClass()] );
                    if ( testData.getTestResult() > -1 )
                        c.setResult( testData.getTestResult() );
                    if ( testData.getDate() != null && testData.getTestNumber() > -1)
                        c.setDate( formatData.dateTime( testData.getDate(), device.getDateFormat(), device.getTimeFormat()) );
                }
                else
                {
                    c.setAllValues( testData.getMeasurement() );
                }
            }
        }
    }

    void resetMainScreen()
    {
        for (Fragment f : mFragmentList)
        {
            if (f instanceof MainScreen)
            {
                MainScreen m = (MainScreen)f;
                m.resetAll();
            }
        }
    }

    void resetCharts()
    {
        for (Fragment f : mFragmentList)
        {
            if (f instanceof Charts)
            {
                Charts c = (Charts)f;
                c.reset();
            }
        }
    }

    void updateChartsAndTable(Measurement measurement)
    {
        for (Fragment f : mFragmentList)
        {
            if (f instanceof Charts)
            {
                Charts c = (Charts)f;
                c.update(measurement, true);
            }
        }
    }

    MainScreen mainScreen()
    {
        return mainScreen;
    }

    Charts table()
    {
        return table;
    }

    Charts chartCurrent()
    {
        return currentChart;
    }

    Charts chartVoltage()
    {
        return voltageChart;
    }

    private boolean screenSizeIsLarge(Context context)
    {
        return (context.getResources().getConfiguration().screenLayout &
                Configuration.SCREENLAYOUT_SIZE_MASK) == Configuration.SCREENLAYOUT_SIZE_LARGE;
    }
}



