package com.megabras.bluelogg.DT70KV;

import android.content.Context;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.TextView;

import com.megabras.bluelogg.R;

import java.util.ArrayList;
import java.util.List;

public class TableBaseAdapter extends BaseAdapter
{
    private LayoutInflater
            inflater;

    private List<TableValues>
            tableValues;

    private boolean
            showResistance = true;

    private String
            testNumber   = " - - -",
            reason       = " - - -",
            gloveSerial  = " - - -",
            gloveClass   = " - - -",
            result       = " - - -",
            date         = " - - -";

    public TableBaseAdapter(Context context)
    {
        tableValues  = new ArrayList<>();
        inflater     = LayoutInflater.from(context);
    }

    static class ViewHolder
    {
        TextView
                tv_test_number,
                tv_test_date,
                ui_txw_glove_serial,
                tv_glove_class,
                tv_reason,
                tv_result,
                tv_elapsed,
                tv_voltage,
                tv_current,
                ui_txw_ohms;
    }

    public int getCount()
    {
        return tableValues.size();
    }

    public TableValues getItem(int position)
    {
        return tableValues.get(position);
    }

    @Override
    public long getItemId(int position)
    {
        return 0;
    }

    public void add(String time, String voltage, String current)
    {
        tableValues.add( new TableValues(time, voltage, current) );
    }

    public void remove(int position)
    {
        tableValues.remove(position);
    }

    public void clear()
    {
        if ( testNumber!=null )
            testNumber = " - - -";
        if ( reason!=null )
            reason = " - - -";
        if ( gloveClass!=null )
            gloveClass = " - - -";
        if ( result!=null )
            result = " - - -";
        if ( date!=null )
            date = " - - -";
        if ( gloveSerial!=null )
            gloveSerial = " - - -";

        tableValues.clear();
    }

    public void repaint()
    {
        notifyDataSetChanged();
    }

    private TextView getTV(int id, View v)
    {
        return ( (TextView)v.findViewById(id) );
    }

    public void setTestNumber(int testNumber)
    {
        if ( testNumber > -1 )
            this.testNumber = testNumber+"";
        else
            this.testNumber = " - - - ";
    }

    public void setGloveClass(String gloveClass)
    {
        this.gloveClass = gloveClass;
    }

    public void setGloveSerial(String gloveSerial)
    {
        this.gloveSerial = gloveSerial;
    }

    public void setDate(String date)
    {
        this.date = date;
    }

    public void setResult(int result)
    {
        String hex          = com.megabras.modbus.MUtils.intToHex(result, 4);
        boolean[] statusOne = com.megabras.modbus.MUtils.hexToBitArray(hex);

        if ( statusOne[0] )
            this.reason = inflater.getContext().getString(R.string.spark_detection).toUpperCase();
        else if (statusOne[1])
            this.reason = inflater.getContext().getString(R.string.overcurrent).toUpperCase();

        if ( statusOne[2] && statusOne[3] )
            this.result = inflater.getContext().getString(R.string.pass);
        else if ( !statusOne[2] && statusOne[3] )
            this.result = inflater.getContext().getString(R.string.fail);
        else if (statusOne[2])
            this.result = inflater.getContext().getString(R.string.aborted);
        else
            this.result = " - - - ";
        notifyDataSetChanged();
    }

    @SuppressWarnings("RedundantCast")
    public View getView(int position, View vi, ViewGroup parent)
    {
        int headerLayout = R.layout.dt70kv_table_header;
        int lineLayout   = (int)R.layout.table_item;
        if ( position==0 ) // First result
        {
            if ( vi == null || vi.getId()!=lineLayout)
            {
                ViewHolder h        = new ViewHolder();
                vi                  = inflater.inflate(headerLayout, parent, false);
                h.tv_test_number    = getTV(R.id.tv_number          , vi);
                h.tv_test_date      = getTV(R.id.tv_date            , vi);
                h.ui_txw_glove_serial = getTV(R.id.ui_txw_glove_serial, vi);
                h.tv_glove_class    = getTV(R.id.tv_glove_class     , vi);
                h.tv_reason         = getTV(R.id.tv_reason          , vi);
                h.tv_result         = getTV(R.id.tv_test_result     , vi);
                h.tv_elapsed        = getTV(R.id.col_a              , vi);
                h.tv_voltage        = getTV(R.id.col_b              , vi);
                h.tv_current        = getTV(R.id.col_c              , vi);
                vi.setTag(h);
            }
            ViewHolder h = (ViewHolder) vi.getTag();
            h.tv_test_number    .setText( testNumber );
            h.tv_test_date      .setText( date );
            h.ui_txw_glove_serial .setText( gloveSerial );
            h.tv_glove_class    .setText( gloveClass );
            h.tv_reason         .setText( reason );
            h.tv_result         .setText( result );
            h.tv_elapsed        .setText( tableValues.get(position).getTime()       );
            h.tv_voltage        .setText( tableValues.get(position).getVoltage()    );
            h.tv_current        .setText( tableValues.get(position).getCurrent()    );
        }
        else // Second or higher result
        {
            if ( vi == null || (int)vi.getId()!= headerLayout)
            {
                ViewHolder h    = new ViewHolder();
                vi              = inflater.inflate(lineLayout, parent, false);
                h.tv_elapsed    = getTV(R.id.col_a, vi);
                h.tv_voltage    = getTV(R.id.col_b, vi);
                h.tv_current    = getTV(R.id.col_c, vi);
                h.ui_txw_ohms   = getTV(R.id.col_d, vi);
                vi.setTag(h);
            }
            ViewHolder h = (ViewHolder) vi.getTag();
            h.tv_elapsed.setText       ( tableValues.get(position).getTime()    );
            h.tv_voltage.setText       ( tableValues.get(position).getVoltage() );
            h.tv_current.setText       ( tableValues.get(position).getCurrent() );
            h.ui_txw_ohms.setVisibility( View.GONE                              );
        }
        return vi;
    }
}

