package com.megabras.bluelogg.MDXXKVR;

import android.content.Context;
import android.content.res.Configuration;
import android.graphics.drawable.Drawable;
import android.text.Spannable;
import android.text.SpannableString;
import android.text.style.ImageSpan;

import com.megabras.bluelogg.R;
import com.megabras.bluelogg.extended.SlidingTabLayout;

import java.util.ArrayList;
import java.util.List;

import androidx.annotation.NonNull;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentManager;
import androidx.fragment.app.FragmentStatePagerAdapter;
import androidx.viewpager.widget.ViewPager;


@SuppressWarnings("WeakerAccess")
public class PagerAdapter extends FragmentStatePagerAdapter
{
    @SuppressWarnings("unused")
    private final String
            TAG = "PagerAdapter";

    private Context
            context;

    private ViewPager
            mViewPager;

    private SlidingTabLayout
            mSlidingTabLayout;

    protected MainFragment
            mainScreen;

    protected ChartFragment
            resistanceChart,
            voltageChart,
            currentChart,
            iddChart,
            table;

    private final List<Fragment>
            mFragmentList = new ArrayList<>();

    private final List<CharSequence>
            mFragmentTitleList = new ArrayList<>();

    public PagerAdapter( FragmentManager fm, Context context )
    {
        super(fm);
        this.context = context;
        mainScreen = new MainFragment();
        resistanceChart = new ChartFragment();
        resistanceChart.setChartType( ChartFragment.RESISTANCE_CHART, context.getString(R.string.md_resistance) );
        currentChart = new ChartFragment( );
        currentChart.setChartType( ChartFragment.CURRENT_CHART, context.getString(R.string.md_current) );
        voltageChart = new ChartFragment( );
        voltageChart.setChartType( ChartFragment.VOLTAGE_CHART, context.getString(R.string.md_voltage) );
        iddChart = new ChartFragment( );
        iddChart.setChartType( ChartFragment.IDD_CHART, context.getString(R.string.dielectric_discharge).toUpperCase() );
        table = new ChartFragment();
        table.setChartType( ChartFragment.TABLE, "" );

        mFragmentList.add( mainScreen );
        mFragmentList.add( resistanceChart );
        mFragmentList.add( voltageChart );
        mFragmentList.add( currentChart );
        mFragmentList.add( iddChart );
        mFragmentList.add( table );

        mFragmentTitleList.add( convertTitle( 0 ) );
        mFragmentTitleList.add( convertTitle( 1 ) );
        mFragmentTitleList.add( convertTitle( 2 ) );
        mFragmentTitleList.add( convertTitle( 3 ) );
        mFragmentTitleList.add( convertTitle( 4 ) );
        mFragmentTitleList.add( convertTitle( 5 ) );
    }

    public void setViewPagerRef( ViewPager viewPager )
    {
        mViewPager = viewPager;
    }

    public void setSlidingTabLayoutRef( SlidingTabLayout slidingTabLayout )
    {
        mSlidingTabLayout = slidingTabLayout;
    }

    @Override
    public Fragment getItem( int i )
    {
        return mFragmentList.get( i );
    }

    @Override
    public int getCount()
    {
        return mFragmentList.size();
    }

    @Override
    public int getItemPosition( @NonNull Object object )
    {
        return PagerAdapter.POSITION_NONE;
    }

    public MainFragment getMainFragment()
    {
        return mainScreen;
    }

    public ChartFragment chartResistance()
    {
        return resistanceChart;
    }

    public ChartFragment chartCurrent()
    {
        return currentChart;
    }

    public ChartFragment chartVoltage()
    {
        return voltageChart;
    }

    public ChartFragment chartIDD()
    {
        return iddChart;
    }

    public ChartFragment table()
    {
        return table;
    }

    private CharSequence convertTitle( int position )
    {
        int[] images;
        if ( screenSizeIsLarge( context ) )
            images = imageResId;
        else
            images = imageResIdSmall;

        Drawable image = context.getResources().getDrawable( images[position] );
        image.setBounds( 0, 0, image.getIntrinsicWidth(), image.getIntrinsicHeight() );
        SpannableString sb = new SpannableString( " " );
        ImageSpan imageSpan = new ImageSpan( image, ImageSpan.ALIGN_BOTTOM );
        sb.setSpan( imageSpan, 0, 1, Spannable.SPAN_EXCLUSIVE_EXCLUSIVE );
        return sb;
    }

    private int[] imageResId = {
            R.drawable.ic_analog,
            R.drawable.ic_chart_resistance,
            R.drawable.ic_chart_voltage,
            R.drawable.ic_chart_current,
            R.drawable.ic_chart_dd,
            R.drawable.ic_table
    };

    private int[] imageResIdSmall = {
            R.drawable.ic_analog_20,
            R.drawable.ic_chart_resistance_20,
            R.drawable.ic_chart_voltage_20,
            R.drawable.ic_chart_current_20,
            R.drawable.ic_chart_dd_20,
            R.drawable.ic_table_20
    };

    @Override
    public CharSequence getPageTitle( int position )
    {
        return mFragmentTitleList.get( position );
    }

    public void setIDDEnabled( boolean show )
    {
        int iddPos = 4;
        boolean contains = false;
        for ( Fragment f : mFragmentList )
        {
            if ( f instanceof ChartFragment )
            {
                ChartFragment c = (ChartFragment) f;
                if ( c.getChartType() == ChartFragment.IDD_CHART )
                    contains = true;
            }
        }
        if ( show )
        {
            if ( !contains )
            {
                mFragmentList.add( iddPos, iddChart );
                mFragmentTitleList.add( iddPos, convertTitle( iddPos ) );
                notifyDataSetChanged();
                if ( mViewPager != null && mSlidingTabLayout != null )
                {
                    mSlidingTabLayout.setViewPager( mViewPager );
                }
            }
        }
        else
        {
            if ( contains )
            {
                mFragmentList.remove( iddPos );
                mFragmentTitleList.remove( iddPos );
                notifyDataSetChanged();
                if ( mViewPager != null && mSlidingTabLayout != null )
                {
                    mSlidingTabLayout.setViewPager( mViewPager );
                }
            }
        }
    }

    private boolean screenSizeIsLarge( Context context )
    {
        return (context.getResources().getConfiguration().screenLayout &
                Configuration.SCREENLAYOUT_SIZE_MASK) == Configuration.SCREENLAYOUT_SIZE_LARGE;
    }
}



