package com.megabras.bluelogg.MPKXX5R;

import android.content.Context;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.TextView;

import com.megabras.bluelogg.R;
import com.megabras.modbus.MPKXX5R.FormatData;
import com.megabras.modbus.MPKXX5R.Measurement;
import com.megabras.modbus.MPKXX5R.RecordData;
import com.megabras.modbus.MPKXX5R.TestData;

import java.util.ArrayList;
import java.util.List;

import androidx.fragment.app.Fragment;

//import android.support.v4.app.Fragment;

public class ChartFragment extends Fragment
{
    @SuppressWarnings("unused")
    private final String
            TAG                 = "ChartFragment";

    private LinearLayout
            layout;

//    private View
//            objectView;

    private TextView
            ui_table_txw_record_name,
            ui_txw_longitude,
            ui_txw_latitude,
            ui_txw_altitude,
            ui_txw_accuracy;

//    private ListView
//            list;

    private TableBaseAdapter
            adapter;

    private List<TableValues>
            tableValues = new ArrayList<>();

    private String[]
            modes,
            enableDisable;

    private Context
            context;

    @SuppressWarnings("NullableProblems")
    @Override
    public View onCreateView( LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState )
    {
        context = inflater.getContext();
        return inflater.inflate( R.layout.md10kvx_fgr_online, container, false );
    }

    @Override
    public void onStart()
    {
        super.onStart();
        table();
    }

    public void table()
    {
        modes = getResources().getStringArray( R.array.mpkxx5r_mode_auto_manual );
        enableDisable = getResources().getStringArray( R.array.enable_disable );
        if ( this.getView() != null )
        {
            layout = this.getView().findViewById( R.id.chart );
        }
        else
        {
            return;
        }

        if ( layout.getChildCount() > 1 )
        {
            return;
        }
        View objectView = View.inflate( context, R.layout.mpkxx5r_table_object, null );

        ui_txw_longitude  = objectView.findViewById( R.id.tv_longitude );
        ui_txw_latitude   = objectView.findViewById( R.id.tv_latitude );
        ui_txw_altitude   = objectView.findViewById( R.id.tv_altitude );
        ui_txw_accuracy   = objectView.findViewById( R.id.tv_accuracy );
        ui_table_txw_record_name = objectView.findViewById( R.id.tv_object_name );

        ui_table_txw_record_name.setText( "" );
        ui_txw_longitude.setText( "" );
        ui_txw_latitude.setText( "" );
        ui_txw_altitude.setText( "" );
        ui_txw_accuracy.setText( "" );

        layout.addView( objectView );

        ListView list = new ListView( layout.getContext() );
        list.setDividerHeight( 0 );
        list.setTranscriptMode( ListView.TRANSCRIPT_MODE_ALWAYS_SCROLL );
        list.setStackFromBottom( true );
        layout.addView( list );

        adapter = new TableBaseAdapter( layout.getContext() );
        list.setAdapter( adapter );
    }

    void loadSavedData( RecordData recordData )
    {
        if ( recordData != null )
        {
            reset();
            updateRecordData( recordData.getRecordName() );
            updateGps( recordData );

            for ( TestData td : recordData.getTestDatas() )
            {
                updateTestNumber( td.getTestNumber() );
                updateHeaderInfo( td );
                int counter = 0;
                for ( Measurement m : td.getMeasurements() )
                {
                    boolean repaint = counter == td.getMeasurements().size() - 1;
                    updateMeasurement( m, td.isBsgEnabled(), repaint );
                    counter++;
                }
            }
        }
        else
        {
            updateRecordData( "" );
            updateGps( null );
            reset();
        }
    }

    private void updateGps( RecordData recordData )
    {
        if ( recordData != null )
        {
            ui_txw_longitude.setText( recordData.getGpsLongitudeAsString() );
            ui_txw_latitude.setText( recordData.getGpsLatitudeAsString() );
            //ui_txw_altitude.setText( String.format( Locale.getDefault(),"%.1f m", recordData.getGpsAltitude() ) );
            //ui_txw_accuracy.setText( String.format( Locale.getDefault(),"\u00B1 %.1f m", recordData.getGpsAccuracy() ) );
            ui_txw_altitude.setText( recordData.getGpsAltitudeAsString() );
            ui_txw_accuracy.setText( recordData.getGpsAccuracyAsString() );
        }
        else
        {
            ui_txw_longitude.setText( "" );
            ui_txw_latitude.setText( "" );
            ui_txw_altitude.setText( "" );
            ui_txw_accuracy.setText( "" );
        }
    }

    void updateTestNumber( int testNumber )
    {
        adapter.setTestNumber( testNumber );
    }

    void updateHeaderInfo( TestData testData )
    {
        String mode  = "";
        String timer = "- - -";
        String bsg   = "- - -";

        if ( testData.getMode() < modes.length )
        {
            mode = modes[testData.getMode()];
        }
        if ( testData.isTimerEnabled() )
        {
            timer = FormatData.elapsed( testData.getTimer() );
        }
        if ( enableDisable.length >= 2 )
        {
            bsg = testData.isBsgEnabled() ? enableDisable[0] : enableDisable[1];
        }
        adapter.setHeaderInfo( testData.getDateAsString(), mode, timer, bsg);
    }

//    void showTable( boolean show )
//    {
//        if ( list != null )
//        {
//            if ( show )
//                list.setVisibility( View.VISIBLE );
//            else
//                list.setVisibility( View.GONE );
//        }
//        if ( objectView != null )
//        {
//            if ( show )
//                objectView.setVisibility( View.VISIBLE );
//            else
//                objectView.setVisibility( View.GONE );
//        }
//    }

    void updateRecordData( String recordDataName )
    {
        ui_table_txw_record_name.setText( recordDataName );
    }

    @SuppressWarnings("SameParameterValue")
    void updateMeasurement( Measurement mea, boolean isBsgEnabled, boolean repaint )
    {
        adapter.add( mea.getElapsedString(), mea.getOhmString(), mea.getAmpString(), mea.getClampAmpString( isBsgEnabled ) );
        if ( repaint )
        {
            adapter.repaint();
        }
    }

    public void repaint()
    {
        adapter.repaint();
    }

    public void setAllValues()
    {
        adapter.addAll( tableValues );
    }

    public void reset()
    {
        try
        {
            if ( layout == null )
            {
                table();
            }
            tableValues.clear();
            adapter.clear();
        }
        catch (Exception e)
        {
            android.util.Log.e("ERROR", e.getMessage()+"");
        }
    }
}
