package com.megabras.bluelogg.MPKXX5R;

import android.annotation.SuppressLint;
import android.app.DatePickerDialog;
import android.app.Dialog;
import android.app.TimePickerDialog;
import android.content.Context;
import android.content.SharedPreferences;
import android.graphics.Color;
import android.os.Handler;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.Window;
import android.view.WindowManager;
import android.view.inputmethod.InputMethodManager;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.DatePicker;
import android.widget.EditText;
import android.widget.FrameLayout;
import android.widget.ImageButton;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.Spinner;
import android.widget.Switch;
import android.widget.TextView;
import android.widget.TimePicker;

import com.megabras.bluelogg.extended.CommonUI;
import com.megabras.bluelogg.MUtils;
import com.megabras.bluelogg.Main;
import com.megabras.bluelogg.R;
import com.megabras.bluelogg.bluetooth.BluetoothClassic;
import com.megabras.bluelogg.extended.Audio;
import com.megabras.bluelogg.extended.GPS;
import com.megabras.bluelogg.extended.GoogleMaps;
import com.megabras.bluelogg.extended.LogCat;
import com.megabras.bluelogg.extended.MapData;
import com.megabras.bluelogg.extended.Picture;
import com.megabras.bluelogg.extended.SlidingTabLayout;
import com.megabras.modbus.MPKXX5R.DeviceOld;
import com.megabras.modbus.MPKXX5R.FormatData;
import com.megabras.modbus.MPKXX5R.Protocol;
import com.megabras.modbus.MPKXX5R.RecordData;
import com.megabras.modbus.Message;
import com.megabras.modbus.Modbus;

import java.io.File;
import java.lang.ref.WeakReference;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Locale;
import java.util.Observable;
import java.util.Observer;

import androidx.annotation.NonNull;
import androidx.fragment.app.FragmentActivity;
import androidx.viewpager.widget.ViewPager;


public class Controller extends com.megabras.bluelogg.Controller implements GPS.GpsInterface, Picture.PictureInterface, com.megabras.CommInterface, Observer
{

    /***********************************************************************************************
     * REGION Fields
     ***********************************************************************************************/

    @SuppressWarnings({"FieldCanBeLocal", "unused"})
    private final String
            TAG = "MPKXX5R.Controller";

    public static String
            model,
            serial;

    private FragmentActivity
            fragmentActivity;

    private MenuItem
            ui_menu_item_options;

    private MainFragment
            ui_main_screen;

    private ChartFragment
            ui_table;

    private ListView
            ui_liw_files;

    private Button
            ui_btn_start,
            ui_btn_stop,
            ui_btn_file_delete,
            ui_btn_file_close,
            ui_btn_file_open,
            ui_btn_file_export;

    private ImageButton
            ui_ibtn_save,
            ui_ibtn_setup,
            ui_ibtn_printer,
            ui_ibtn_pwm_small_up,
            ui_ibtn_pwm_small_down,
            ui_ibtn_pwm_medium_up,
            ui_ibtn_pwm_medium_down,
            ui_ibtn_pwm_large_up,
            ui_ibtn_pwm_large_down,
            ui_ibtn_file_manager,
            ui_ibtn_gps,
            ui_ibtn_picture,
            ui_ibtn_voice,
            ui_ibtn_map;

    private TextView
            ui_setup_txw_auto_amp,
            ui_setup_txw_timer,
            ui_setup_txw_record_name,
            ui_general_txw_free_memory,
            ui_general_txw_date,
            ui_general_txw_time;

    private Spinner
            ui_setup_spn_mode,
            ui_setup_spn_fan,
            ui_general_spn_auto_connection,
            ui_general_spn_date_format,
            ui_general_spn_time_format,
            ui_general_spn_language;

    private Switch
            ui_setup_swt_auto_save,
            ui_setup_swt_save_here,
            ui_setup_swt_bsg,
            ui_setup_swt_timer;

    private LinearLayout
            ui_setup_lyt_auto_amp,
            ui_setup_lyt_timer,
            ui_setup_lyt_fan;

    private Dialog
            ui_dlg_progress,
            ui_dlg_change_record_data,
            ui_dlg_start,
            ui_dlg_finished,
            ui_dlg_cancelled,
            ui_dlg_auto_pilot;

    private ViewPager
            VIEW_PAGER;

    private View
            VIEW_SETUP,
            VIEW_GENERAL,
            VIEW_FILES;

    private FrameLayout
            OVERLAY;

    private Protocol
            modbus = new Protocol();

    private DeviceOld
            device;

    private String
            BOX_MESSAGE         = "",
            PREF_SAVE_IN_DEVICE = "PREF_SAVE_IN_DEVICE";

    private int
            BOX_COLOR = 0;

    private boolean
            BOX_BUSY = false,
            SAVE_IN_DEVICE;

    private CommonUI
            mCommonUI;

    private Picture
            picture;

    private GPS
            mGPS;

    private GoogleMaps
            map;

    private SharedPreferences
            sharedPrefs;

    private Audio
            audio;

    private Date
            USER_DATE;

    private final Database
            database;

    private RecordData
            recordData;

    private ReportPDF
            report;

    private int
            REC_ID_EXPORT = -1;

    /***********************************************************************************************
     * REGION Constructor
     ***********************************************************************************************/

    public Controller( FragmentActivity fragmentActivity, CommonUI ui, String model, String serial )
    {
        this.fragmentActivity = fragmentActivity;
        OVERLAY      = MUtils.getOverlayInstance( fragmentActivity, OVERLAY_ID );
        VIEW_GENERAL = View.inflate( fragmentActivity, R.layout.main_fgr_general_setup, null );
        VIEW_SETUP   = View.inflate( fragmentActivity, R.layout.mpkxx5r_fgr_test_setup, null );
        VIEW_FILES   = View.inflate( fragmentActivity, R.layout.mpkxx5r_files_manager , null );
        mCommonUI    = ui;
        picture      = Picture.getInstance();
        picture.initialize( fragmentActivity, this );
        mGPS         = new GPS( this, fragmentActivity );
        audio        = new Audio( fragmentActivity );
        map          = new GoogleMaps( fragmentActivity );

        Controller.model  = model;
        Controller.serial = serial;

        database     = new Database( fragmentActivity, model, serial );
        device       = new DeviceOld( this );
        device.addObserver( this );
        report       = new ReportPDF( fragmentActivity, database.getFolder(), database.getTemporaryFolder(), new mHandler( this ) );

        sharedPrefs    = fragmentActivity.getSharedPreferences( "BLUELOGG" + model, Context.MODE_PRIVATE );
        SAVE_IN_DEVICE = sharedPrefs.getBoolean( PREF_SAVE_IN_DEVICE, true );

        createPageFilesManager();
        //createPageGeneralSetup();
        createPageTestSetup();

        LogCat.LEVEL = LogCat.LEVEL_NONE;
    }

    /***********************************************************************************************
     * REGION On Destroy
     ***********************************************************************************************/

    public void onClose()
    {
        if ( mGPS != null )
        {
            mGPS.cancel();
        }
        if ( picture != null )
        {
            picture.close();
        }
    }

    public void setToolBarId( int toolBarId )
    {
        LinearLayout ui_lyt_toolbar = fragmentActivity.findViewById( toolBarId );
        ui_lyt_toolbar.findViewById( R.id.bt_details ).setVisibility( View.GONE );
        ui_lyt_toolbar.findViewById( R.id.bt_save ).setVisibility( View.GONE );
        ui_lyt_toolbar.findViewById( R.id.bt_export ).setVisibility( View.GONE );

        ui_ibtn_map = ui_lyt_toolbar.findViewById( R.id.btn_map );
        ui_ibtn_map.setOnClickListener( v ->
        {
            if ( Main.permissionToMemoryAccepted )
            {
                map();
            }
            else
            {
                ((Main) fragmentActivity).requestPermission( Main.REQUEST_MEMORY_MAP_PERMISSION );
            }

        } );

        ui_ibtn_file_manager = ui_lyt_toolbar.findViewById( R.id.bt_manager );
        ui_ibtn_file_manager.setOnClickListener( v ->
        {
            loadFilesManagerPage();
            showOverlay( VIEW_FILES );
        } );

        ui_ibtn_picture = ui_lyt_toolbar.findViewById( R.id.bt_picture );
        ui_ibtn_picture.findViewById( R.id.bt_picture ).setOnClickListener( v ->
        {
            if ( Main.permissionToPictureAccepted )
            {
                takePicture();
            }
            else
            {
                ((Main) fragmentActivity).requestPermission( Main.REQUEST_PICTURE_PERMISSION );
            }
        } );

        ui_ibtn_gps = ui_lyt_toolbar.findViewById( R.id.bt_gps );
        ui_ibtn_gps.setOnClickListener( v ->
        {
            if ( Main.permissionToGPSAccepted )
            {
                getGPS();
            }
            else
            {
                ((Main) fragmentActivity).requestPermission( Main.REQUEST_GPS_PERMISSION );
            }
        } );

        ui_ibtn_voice = ui_lyt_toolbar.findViewById( R.id.bt_record );
        ui_ibtn_voice.setOnClickListener( v ->
        {
            if ( Main.permissionToRecordAccepted )
            {
                recordAudio();
            }
            else
            {
                ((Main) fragmentActivity).requestPermission( Main.REQUEST_RECORD_AUDIO_PERMISSION );
            }
        } );
    }

    public void setAutoConn( boolean enable )
    {
        device.setAutoConnection( enable );
    }

    public void setDateTimeFormat( int dateFormat, int timeFormat )
    {
        device.setDateFormat( dateFormat );
        device.setTimeFormat( timeFormat );
    }

    public boolean getAutoConn()
    {
        return device.getAutoConnection();
    }

    private void createPageFilesManager()
    {
        ui_liw_files         = getLV( R.id.list_savedfiles, VIEW_FILES );
        ui_btn_file_close    = getBTN( R.id.bt_close      , VIEW_FILES );
        ui_btn_file_delete   = getBTN( R.id.bt_delete     , VIEW_FILES );
        ui_btn_file_open     = getBTN( R.id.bt_open       , VIEW_FILES );
        ui_btn_file_export   = getBTN( R.id.bt_export     , VIEW_FILES );
        Button ui_btn_cancel = getBTN( R.id.bt_cancel     , VIEW_FILES );

        ui_btn_cancel.setOnClickListener( v -> closeOverlay() );

        ui_liw_files.setAdapter( new TableFilesAdapter( fragmentActivity ) );
        ui_liw_files.setChoiceMode( ListView.CHOICE_MODE_SINGLE );

        ui_liw_files.setOnItemClickListener(  ( adapterView, view, i, l ) ->
        {
            ui_btn_file_delete.setEnabled( true );
            ui_btn_file_open.setEnabled( true );
            ui_btn_file_export.setEnabled( true );
        } );

        ui_btn_file_open.setOnClickListener( v ->
        {
            int selection = ui_liw_files.getCheckedItemPosition();
            if ( selection > -1 )
            {
                TableFilesAdapter adapter = (TableFilesAdapter) ui_liw_files.getAdapter();
                recordData = database.getRecordData( adapter.getItem( selection ).id );
                recordData.setDateTimeFormat( device.getDateFormat(), device.getTimeFormat() );
                if ( recordData != null )
                {
                    ui_table.loadSavedData( recordData );
                    VIEW_PAGER.setCurrentItem( 1 );
                    closeOverlay();
                }
            }
        } );

        ui_btn_file_close.setOnClickListener( v ->
        {
            recordData = null;
            if ( !device.getRecordName().equals( "" ) )
            {
                recordData = new RecordData();
                recordData.setRecordName( device.getRecordName() );
                recordData.setDeviceId( device.getID() );
                recordData.setDeviceModel( device.getModel() );
                recordData.setDeviceSerial( device.getSerial() );
                recordData.setDeviceVersionPreffix( device.getVersionPreffix() );
                recordData.setDeviceHardware( device.getHardware() );
                recordData.setDeviceFirmware( device.getFirmware() );
                recordData.setDateTimeFormat( device.getDateFormat(), device.getTimeFormat() );
            }
            ui_table.loadSavedData( recordData );
            loadFilesManagerPage();
        } );

        ui_btn_file_export.setOnClickListener( v ->
        {
            int selection = ui_liw_files.getCheckedItemPosition();
            if ( selection > -1 )
            {
                TableFilesAdapter adapter = (TableFilesAdapter) ui_liw_files.getAdapter();
                REC_ID_EXPORT = adapter.getItem( selection ).id;

                if ( Main.permissionToMemoryAccepted )
                {
                    export();
                }
                else
                {
                    ((Main) fragmentActivity).requestPermission( Main.REQUEST_MEMORY_REPORT_PERMISSION );
                }
            }
        } );

        ui_btn_file_delete.setOnClickListener( v ->
        {
            int selection = ui_liw_files.getCheckedItemPosition();
            if ( selection > -1 )
            {
                final Dialog dialog = new Dialog( fragmentActivity );
                dialog.requestWindowFeature( Window.FEATURE_NO_TITLE );
                dialog.setCanceledOnTouchOutside( false );
                dialog.setContentView( R.layout.dialog_yes_no_white );
                if ( dialog.getWindow() != null )
                {
                    dialog.getWindow().getAttributes().width = ViewPager.LayoutParams.MATCH_PARENT;
                }
                ((TextView) dialog.findViewById( R.id.dlg_title )).setText( R.string.warning );
                ((ImageView) dialog.findViewById( R.id.dlg_icon )).setImageResource( R.drawable.ic_warning );

                final TextView message = dialog.findViewById(R.id.message);
                message.setText(R.string.file_delete_all_confirm);
                dialog.findViewById(R.id.btn_no).setOnClickListener(v1 -> dialog.dismiss());
                dialog.findViewById(R.id.btn_yes).setOnClickListener(v12 ->
                {
                    TableFilesAdapter adapter = (TableFilesAdapter) ui_liw_files.getAdapter();
                    int recId = adapter.getItem( selection ).id;
                    database.deleteRecordData( recId );
                    if ( recordData != null && recordData.getRecordId() == recId )
                    {
                        recordData = null;
                        if ( !device.getRecordName().equals( "" ) )
                        {
                            recordData = new RecordData();
                            recordData.setRecordName( device.getRecordName() );
                            recordData.setDeviceId( device.getID() );
                            recordData.setDeviceModel( device.getModel() );
                            recordData.setDeviceSerial( device.getSerial() );
                            recordData.setDeviceVersionPreffix( device.getVersionPreffix() );
                            recordData.setDeviceHardware( device.getHardware() );
                            recordData.setDeviceFirmware( device.getFirmware() );
                            recordData.setDateTimeFormat( device.getDateFormat(), device.getTimeFormat() );
                        }
                    }
                    ui_table.loadSavedData( recordData );
                    loadFilesManagerPage();
                    dialog.dismiss();
                });

                dialog.show();
            }
        } );
    }

    private void loadFilesManagerPage()
    {
        REC_ID_EXPORT = -1;

        TableFilesAdapter adapter = (TableFilesAdapter) ui_liw_files.getAdapter();
        adapter.setDateTimeFormat( device.getDateFormat(), device.getTimeFormat() );
        adapter.add( database.getSavedList() );
        if ( recordData != null )
        {
            int selection = adapter.setOpenedRecordId( recordData.getRecordId() );
            ui_liw_files.setItemChecked( selection, true );
            if ( selection > -1 )
            {
                ui_btn_file_delete.setEnabled( true );
                ui_btn_file_close.setEnabled( true );
                ui_btn_file_export.setEnabled( true );
            }
            else
            {
                ui_btn_file_open.setEnabled( false );
                ui_btn_file_delete.setEnabled( false );
                ui_btn_file_close.setEnabled( false );
                ui_btn_file_export.setEnabled( false );
            }
        }
        else
        {
            adapter.setOpenedRecordId( -1 );
            ui_liw_files.setItemChecked( -1, true );
            ui_btn_file_delete.setEnabled( false );
            ui_btn_file_open.setEnabled( false );
            ui_btn_file_close.setEnabled( false );
            ui_btn_file_export.setEnabled( false );
        }
        adapter.repaint();
        ui_liw_files.invalidateViews();
    }

    private void createPageGeneralSetup()
    {
        ui_general_spn_auto_connection = getSpinner( R.id.spinnerAutoConnection , R.array.enable_disable     , VIEW_GENERAL );
        ui_general_spn_date_format     = getSpinner( R.id.spinnerDateFormat     , R.array.date_format        , VIEW_GENERAL );
        ui_general_spn_time_format     = getSpinner( R.id.spinnerTimeFormat     , R.array.time_format        , VIEW_GENERAL );
        if ( device.hasLangPtBr() )
        {
            ui_general_spn_language = getSpinner( R.id.spinnerLanguage, R.array.language_options , VIEW_GENERAL );
        }
        else
        {
            ui_general_spn_language = getSpinner( R.id.spinnerLanguage, R.array.bilanguage_options , VIEW_GENERAL );
        }

        ui_general_txw_free_memory     = getTV( R.id.device_memory   , VIEW_GENERAL );
        ui_general_txw_date            = getTV( R.id.device_date     , VIEW_GENERAL );
        ui_general_txw_time            = getTV( R.id.device_time     , VIEW_GENERAL );

        ImageButton ui_general_ibtn_erase_memory   = getIBTN( R.id.button_memory , VIEW_GENERAL );
        ImageButton ui_general_ibtn_date           = getIBTN( R.id.button_date   , VIEW_GENERAL );
        ImageButton ui_general_ibtn_time           = getIBTN( R.id.button_time   , VIEW_GENERAL );

        LinearLayout ui_lyt_gauge      = getLL( R.id.line_gauge      , VIEW_GENERAL );
        ui_lyt_gauge.setVisibility( View.GONE );

        ui_general_ibtn_erase_memory.setOnClickListener( v ->
        {
            final Dialog dialog = new Dialog( fragmentActivity );
            dialog.requestWindowFeature( Window.FEATURE_NO_TITLE );
            dialog.setCanceledOnTouchOutside( false );
            dialog.setContentView( R.layout.dialog_yes_no );
            if ( dialog.getWindow() != null )
            {
                dialog.getWindow().getAttributes().width = ViewPager.LayoutParams.MATCH_PARENT;
            }
            ((TextView) dialog.findViewById( R.id.dlg_title )).setText( R.string.erase_memory );
            ((ImageView) dialog.findViewById( R.id.dlg_icon )).setImageResource( R.drawable.ic_warning );

            final TextView message = dialog.findViewById( R.id.message );
            message.setText( R.string.erase_memory_confirm );

            final Button btnNo = dialog.findViewById( R.id.btn_no );
            btnNo.setOnClickListener( v1 ->  dialog.dismiss() );

            final Button btnYes = dialog.findViewById(R.id.btn_yes);
            btnYes.setOnClickListener( v12 ->
            {
                device.writeEraseMemory();
                dialog.dismiss();
            } );

            dialog.show();
        } );

        ui_general_ibtn_date.setOnClickListener( v -> pickDate() );

        ui_general_ibtn_time.setOnClickListener( v -> pickTime() );

        Button buttonSave = VIEW_GENERAL.findViewById( R.id.buttonSave );
        buttonSave.setOnClickListener( v ->
        {
            switch ( ui_general_spn_auto_connection.getSelectedItemPosition() )
            {
                case 0:
                    device.setAutoConnection( true );
                    break;
                case 1:
                    device.setAutoConnection( false );
                    break;
            }

            // DATE
            switch ( ui_general_spn_date_format.getSelectedItemPosition() )
            {
                case 0:
                    device.setDateFormat( Modbus.DATE_DD_MM_YY );
                    break;
                case 1:
                    device.setDateFormat( Modbus.DATE_MM_DD_YY );
                    break;
                default:
                    device.setDateFormat( Modbus.DATE_YYYY_MM_DD );
                    break;
            }

            switch ( ui_general_spn_time_format.getSelectedItemPosition() )
            {
                case 0:
                    device.setTimeFormat( Modbus.TIME_24H );
                    break;
                default:
                    device.setTimeFormat( Modbus.TIME_12H );
                    break;
            }

            switch ( ui_general_spn_language.getSelectedItemPosition() )
            {
                case 1:
                    device.setLanguage( Modbus.LANG_SPANISH );
                    break;
                case 2:
                    device.setLanguage( Modbus.LANG_PORTUGUESE );
                    break;
                default:
                    device.setLanguage( Modbus.LANG_ENGLISH );
                    break;
            }

            if ( device.getDate() != USER_DATE )
            {
                device.syncDate( USER_DATE );
            }
            device.writeSetup();

            ((Main) fragmentActivity).updateAutoConn( device.getAutoConnection() );
            ((Main) fragmentActivity).updateDateTimeFormat( device.getDateFormat(), device.getTimeFormat() );

            closeOverlay();
        } );

        Button buttonCancel = VIEW_GENERAL.findViewById( R.id.buttonCancel );
        buttonCancel.setOnClickListener( v -> closeOverlay() );
    }

    private void loadGeneralSetupPage()
    {
        ui_general_spn_auto_connection.setSelection( device.getAutoConnection() ? 0 : 1 );

        switch ( device.getDateFormat() )
        {
            case Modbus.DATE_DD_MM_YY:
                ui_general_spn_date_format.setSelection( 0 );
                break;
            case Modbus.DATE_MM_DD_YY:
                ui_general_spn_date_format.setSelection( 1 );
                break;
            default:
                ui_general_spn_date_format.setSelection( 2 );
                break;
        }

        switch ( device.getTimeFormat() )
        {
            case Modbus.TIME_24H:
                ui_general_spn_time_format.setSelection( 0 );
                break;
            default:
                ui_general_spn_time_format.setSelection( 1 );
                break;
        }

        switch ( device.getLanguage() )
        {
            case Modbus.LANG_SPANISH:
                ui_general_spn_language.setSelection( 1 );
                break;
            case Modbus.LANG_PORTUGUESE:
                ui_general_spn_language.setSelection( 2 );
                break;
            default:
                ui_general_spn_language.setSelection( 0 );
                break;
        }

        ui_general_txw_date.setText( FormatData.date( device.getDateFormat(), USER_DATE ) );
        ui_general_txw_time.setText( FormatData.time( device.getTimeFormat(), USER_DATE ) );
        ui_general_txw_free_memory.setText( device.getFreeMemory() );
    }

    private void createPageTestSetup()
    {
        ui_setup_spn_mode         = getSpinner( R.id.ui_spn_mode      , R.array.mpkxx5r_mode_auto_manual  , VIEW_SETUP );
        ui_setup_swt_bsg          = getSWT( R.id.ui_setup_swt_bsg, VIEW_SETUP );
        ui_setup_swt_timer        = getSWT( R.id.ui_setup_swt_timer, VIEW_SETUP );
        ui_setup_swt_auto_save    = getSWT( R.id.ui_swt_auto_save  , VIEW_SETUP );
        ui_setup_swt_save_here    = getSWT( R.id.ui_swt_save_here  , VIEW_SETUP );
        ui_setup_spn_fan          = getSpinner( R.id.ui_spn_fan       , R.array.enable_disable            , VIEW_SETUP );

        ui_setup_txw_auto_amp     = getTV( R.id.ui_txw_auto_amp       , VIEW_SETUP );
        ui_setup_txw_timer        = getTV( R.id.ui_txw_timer          , VIEW_SETUP );
        ui_setup_txw_record_name  = getTV( R.id.ui_txw_record_name    , VIEW_SETUP );

        ImageButton ui_setup_ibtn_auto_amp    = getIBTN( R.id.ui_ibtn_auto_amp    , VIEW_SETUP );
        ImageButton ui_setup_ibtn_timer       = getIBTN( R.id.ui_ibtn_timer       , VIEW_SETUP );
        ImageButton ui_setup_ibtn_record_name = getIBTN( R.id.ui_ibtn_record_name , VIEW_SETUP );

        ui_setup_lyt_auto_amp     = getLL( R.id.ui_lyt_auto_amp       , VIEW_SETUP );
        ui_setup_lyt_timer        = getLL( R.id.ui_lyt_timer          , VIEW_SETUP );
        ui_setup_lyt_fan          = getLL( R.id.ui_lyt_fan            , VIEW_SETUP );

        ui_setup_spn_mode.setOnItemSelectedListener( new AdapterView.OnItemSelectedListener()
        {
            @Override
            public void onItemSelected( AdapterView<?> parentView, View selectedItemView, int position, long id )
            {
                if ( position == DeviceOld.MODE_MANUAL ) // MANUAL
                {
                    ui_setup_lyt_auto_amp.setVisibility( View.GONE );
                }
                else // AUTO
                {
                    ui_setup_lyt_auto_amp.setVisibility( View.VISIBLE );
                }
            }

            @Override
            public void onNothingSelected( AdapterView<?> parentView )
            {
            }
        } );

        ui_setup_swt_timer.setOnCheckedChangeListener( ( compoundButton, b ) -> ui_setup_lyt_timer.setVisibility( b ? View.VISIBLE : View.GONE ) );

        ui_setup_ibtn_auto_amp.setOnClickListener( v ->
                pickNumber( ui_setup_txw_auto_amp, R.string.current_value_percent, "A", device.MIN_AMP, device.MAX_AMP, device.STEP_AMP) );

        ui_setup_ibtn_timer.setOnClickListener( v ->
                pickNumber( ui_setup_txw_timer, R.string.test_duration, "s", device.MIN_TIMER, device.MAX_TIMER, device.STEP_TIMER) );

        ui_setup_ibtn_record_name.setOnClickListener( v ->
                pickName( ui_setup_txw_record_name, R.string.record_name ) );

        Button buttonSave = VIEW_SETUP.findViewById( R.id.buttonSave );
        buttonSave.setOnClickListener( v ->
        {
            SAVE_IN_DEVICE = ui_setup_swt_save_here.isChecked();
            sharedPrefs.edit().putBoolean( PREF_SAVE_IN_DEVICE, SAVE_IN_DEVICE ).apply();

            int confAmp = Integer.parseInt( ui_setup_txw_auto_amp.getText().toString().replace( "A", "" ).trim() );
            if ( confAmp != device.getConfAmp() )
            {
                device.setConfAmp( confAmp );
                device.writeConfAmp();
            }

            int confTimer = Integer.parseInt( ui_setup_txw_timer.getText().toString().replace( "s", "" ).trim() );
            if ( confTimer != device.getConfTimer() )
            {
                device.setConfTimer( confTimer );
                device.writeConfTimer();
            }

            int     mode       = ui_setup_spn_mode.getSelectedItemPosition();
            boolean isTimer    = ui_setup_swt_timer.isChecked();
            boolean isBSG      = ui_setup_swt_bsg.isChecked();
            boolean isAutoSave = ui_setup_swt_auto_save.isChecked();
            boolean isFan      = ui_setup_spn_fan.getSelectedItemPosition() == 0;

            if ( device.getMode()       != mode ||
                 device.getTimer()      != isTimer ||
                 device.getBSG()        != isBSG ||
                 device.getAutoSave()   != isAutoSave ||
                 device.getCoolerAuto() != isFan )
            {
                device.setMode( mode );
                device.setTimer( isTimer );
                device.setBSG( isBSG );
                device.setAutoSave( isAutoSave );
                device.setCoolerAuto( isFan );
                device.writeSetup();
            }

            String recordName = ui_setup_txw_record_name.getText().toString();
            if ( !device.getRecordName().equals( recordName ) )
            {
                device.writeRecordName( recordName );
            }

            closeOverlay();
        } );

        Button buttonCancel = VIEW_SETUP.findViewById( R.id.buttonCancel );
        buttonCancel.setOnClickListener( v -> closeOverlay() );
    }

    private void loadTestSetupPage()
    {
        ui_setup_swt_save_here.setChecked( SAVE_IN_DEVICE );
        ui_setup_spn_mode.setSelection( device.getMode() );
        ui_setup_swt_timer.setChecked( device.getTimer() );
        ui_setup_swt_bsg.setChecked( device.getBSG() );
        ui_setup_swt_auto_save.setChecked( device.getAutoSave() );
        ui_setup_spn_fan.setSelection( device.getCoolerAuto() ? 0 : 1 );
        ui_setup_txw_auto_amp.setText( String.format( Locale.US, "%d A", device.getConfAmp() ) );
        ui_setup_txw_timer.setText( String.format( Locale.US, "%d s",device.getConfTimer() ) );
        ui_setup_txw_record_name.setText( device.getRecordName() );
    }

    /***********************************************************************************************
     * REGION Methods
     ***********************************************************************************************/

    public void createControlButtons( LinearLayout container )
    {
        final View commandButtons = View.inflate( fragmentActivity, R.layout.mpk257_control_buttons, null );

        commandButtons.setLayoutParams( new LinearLayout.LayoutParams( LinearLayout.LayoutParams.MATCH_PARENT, LinearLayout.LayoutParams.WRAP_CONTENT ) );
        container.removeAllViews();
        container.addView( commandButtons );

        ImageButton ui_ibtn_filter = commandButtons.findViewById( R.id.bt_filter );
        ui_btn_start = commandButtons.findViewById( R.id.bt_start );
        ui_btn_stop = commandButtons.findViewById( R.id.bt_stop );
        ui_ibtn_save = commandButtons.findViewById( R.id.bt_save );
        ui_ibtn_setup = commandButtons.findViewById( R.id.bt_setup );
        ui_ibtn_printer = commandButtons.findViewById( R.id.bt_printer );
        ui_ibtn_pwm_small_up = commandButtons.findViewById( R.id.bt_10 );
        ui_ibtn_pwm_small_down = commandButtons.findViewById( R.id.bt_0 );
        ui_ibtn_pwm_medium_up = commandButtons.findViewById( R.id.bt_30 );
        ui_ibtn_pwm_medium_down = commandButtons.findViewById( R.id.bt_20 );
        ui_ibtn_pwm_large_up = commandButtons.findViewById( R.id.bt_50 );
        ui_ibtn_pwm_large_down = commandButtons.findViewById( R.id.bt_40 );

        ui_ibtn_pwm_small_down.setOnClickListener( v -> device.decreaseSmallPWM() );
        ui_ibtn_pwm_small_up.setOnClickListener( v -> device.increaseSmallPWM() );
        ui_ibtn_pwm_medium_down.setOnClickListener( v -> device.decreaseMediumPWM() );
        ui_ibtn_pwm_medium_up.setOnClickListener( v -> device.increaseMediumPWM() );
        ui_ibtn_pwm_large_down.setOnClickListener( v -> device.decreaseLargePWM() );
        ui_ibtn_pwm_large_up.setOnClickListener( v -> device.increaseLargePWM() );

        ui_ibtn_pwm_small_down.setEnabled( false );
        ui_ibtn_pwm_small_up.setEnabled( false );
        ui_ibtn_pwm_medium_down.setEnabled( false );
        ui_ibtn_pwm_medium_up.setEnabled( false );
        ui_ibtn_pwm_large_down.setEnabled( false );
        ui_ibtn_pwm_large_up.setEnabled( false );

        ui_ibtn_setup.setOnClickListener( v ->
        {
            loadTestSetupPage();
            showOverlay( VIEW_SETUP );
        } );

        ui_btn_start.setOnClickListener( v ->
        {
            if ( !recordData.getRecordName().equals( device.getRecordName() ) && SAVE_IN_DEVICE )
            {
                dialogDiffRecordData();
            }
            else
            {
                dialogStartTest();
            }
        } );

        ui_btn_stop.setOnClickListener( v -> device.stop() );
        ui_ibtn_printer.setOnClickListener( v -> device.print() );
        ui_ibtn_save.setOnClickListener( v -> device.writeSave() );
        ui_ibtn_filter.setVisibility( View.GONE );

        ui_btn_start.setEnabled( false );
        ui_btn_stop.setEnabled( true );
        ui_btn_stop.setVisibility( View.GONE );

        ui_ibtn_save.setEnabled( false );
        ui_ibtn_printer.setEnabled( false );
        ui_ibtn_setup.setEnabled( false );
    }

    private void dialogDiffRecordData()
    {
        if ( ui_dlg_change_record_data == null )
        {
            ui_dlg_change_record_data = new Dialog( fragmentActivity );
            ui_dlg_change_record_data.requestWindowFeature( Window.FEATURE_NO_TITLE );
            ui_dlg_change_record_data.setCanceledOnTouchOutside( false );
            ui_dlg_change_record_data.setContentView( R.layout.dialog_yes_no_white );
            if ( ui_dlg_change_record_data.getWindow() != null ) ui_dlg_change_record_data.getWindow().getAttributes().width = ViewPager.LayoutParams.MATCH_PARENT;
            ((TextView) ui_dlg_change_record_data.findViewById( R.id.dlg_title )).setText( R.string.warning );
            ((ImageView) ui_dlg_change_record_data.findViewById( R.id.dlg_icon )).setImageResource( R.drawable.ic_warning );
            ((TextView) ui_dlg_change_record_data.findViewById( R.id.message )).setText( fragmentActivity.getResources().getString( R.string.diff_record_name ).toUpperCase() );

            final Button ui_btn_dlg_no = ui_dlg_change_record_data.findViewById( R.id.btn_no );
            ui_btn_dlg_no.setOnClickListener( v ->  ui_dlg_change_record_data.dismiss() );

            final Button ui_btn_dlg_yes = ui_dlg_change_record_data.findViewById( R.id.btn_yes );
            ui_btn_dlg_yes.setOnClickListener( v ->
            {
                dialogStartTest();
                ui_dlg_change_record_data.dismiss();
            } );
        }

        ui_dlg_change_record_data.show();
    }

    private void dialogStartTest()
    {
        if ( ui_dlg_start == null )
        {
            ui_dlg_start = new Dialog( fragmentActivity );
            ui_dlg_start.requestWindowFeature( Window.FEATURE_NO_TITLE );
            ui_dlg_start.setCanceledOnTouchOutside( false );
            ui_dlg_start.setContentView( R.layout.dialog_yes_no_white );
            if ( ui_dlg_start.getWindow() != null ) ui_dlg_start.getWindow().getAttributes().width = ViewPager.LayoutParams.MATCH_PARENT;
            ((TextView) ui_dlg_start.findViewById( R.id.dlg_title )).setText( R.string.md_warning_test_start );
            ((ImageView) ui_dlg_start.findViewById( R.id.dlg_icon )).setImageResource( R.drawable.ic_warning );
            ((TextView) ui_dlg_start.findViewById( R.id.message )).setText( fragmentActivity.getResources().getString( R.string.want_start_test ).toUpperCase() );

            final Button ui_btn_dlg_no = ui_dlg_start.findViewById( R.id.btn_no );
            ui_btn_dlg_no.setOnClickListener( v ->  ui_dlg_start.dismiss() );

            final Button ui_btn_dlg_yes = ui_dlg_start.findViewById( R.id.btn_yes );
            ui_btn_dlg_yes.setOnClickListener( v ->
            {
                device.start();
                ui_btn_start.setVisibility( View.GONE );
                ui_btn_stop.setVisibility( View.VISIBLE );
                ui_dlg_start.dismiss();
            } );
        }

        ui_dlg_start.show();
    }

    private void dialogFinished()
    {
        if ( ui_dlg_finished == null )
        {
            ui_dlg_finished = new Dialog( fragmentActivity );
            ui_dlg_finished.requestWindowFeature( Window.FEATURE_NO_TITLE );
            ui_dlg_finished.setCanceledOnTouchOutside( false );
            ui_dlg_finished.setContentView( R.layout.dialog_yes_no_white );
            if ( ui_dlg_finished.getWindow() != null ) ui_dlg_finished.getWindow().getAttributes().width = ViewPager.LayoutParams.MATCH_PARENT;
            ui_dlg_finished.findViewById( R.id.title_bar ).setVisibility( View.GONE );

            ((TextView) ui_dlg_finished.findViewById( R.id.message )).setText( R.string.md_test_finished );

            final Button ui_btn_dlg_no = ui_dlg_finished.findViewById( R.id.btn_no );
            ui_btn_dlg_no.setVisibility( View.GONE );

            final Button ui_btn_dlg_yes = ui_dlg_finished.findViewById( R.id.btn_yes );
            ui_btn_dlg_yes.setText( R.string.ok );
            ui_btn_dlg_yes.setOnClickListener( v -> ui_dlg_finished.dismiss() );
        }

        ui_dlg_finished.show();
    }

    private void dialogCancelled()
    {
        if ( ui_dlg_cancelled == null )
        {
            ui_dlg_cancelled = new Dialog( fragmentActivity );
            ui_dlg_cancelled.requestWindowFeature( Window.FEATURE_NO_TITLE );
            ui_dlg_cancelled.setCanceledOnTouchOutside( false );
            ui_dlg_cancelled.setContentView( R.layout.dialog_yes_no_white );
            if ( ui_dlg_cancelled.getWindow() != null ) ui_dlg_cancelled.getWindow().getAttributes().width = ViewPager.LayoutParams.MATCH_PARENT;
            ui_dlg_cancelled.findViewById( R.id.title_bar ).setVisibility( View.GONE );

            ((TextView) ui_dlg_cancelled.findViewById( R.id.message )).setText( R.string.test_cancelled );

            final Button ui_btn_dlg_no = ui_dlg_cancelled.findViewById( R.id.btn_no );
            ui_btn_dlg_no.setVisibility( View.GONE );

            final Button ui_btn_dlg_yes = ui_dlg_cancelled.findViewById( R.id.btn_yes );
            ui_btn_dlg_yes.setText( R.string.ok );
            ui_btn_dlg_yes.setOnClickListener( v -> ui_dlg_cancelled.dismiss() );
        }

        ui_dlg_cancelled.show();
    }

    private void dialogAutoPilot()
    {
        if ( ui_dlg_auto_pilot == null )
        {
            ui_dlg_auto_pilot = new Dialog( fragmentActivity );
            ui_dlg_auto_pilot.requestWindowFeature( Window.FEATURE_NO_TITLE );
            ui_dlg_auto_pilot.setCanceledOnTouchOutside( false );
            ui_dlg_auto_pilot.setContentView( R.layout.dialog_yes_no_white );
            if ( ui_dlg_auto_pilot.getWindow() != null ) ui_dlg_auto_pilot.getWindow().getAttributes().width = ViewPager.LayoutParams.MATCH_PARENT;
            ((TextView) ui_dlg_auto_pilot.findViewById( R.id.dlg_title )).setText( R.string.md_remote_control_locked );
            ((ImageView) ui_dlg_auto_pilot.findViewById( R.id.dlg_icon )).setImageResource( R.drawable.ic_warning );
            ((TextView) ui_dlg_auto_pilot.findViewById( R.id.message )).setText( fragmentActivity.getResources().getString( R.string.md_remote_control_locked_explained ) );

            final Button ui_btn_dlg_no = ui_dlg_auto_pilot.findViewById( R.id.btn_no );
            ui_btn_dlg_no.setVisibility( View.GONE );

            final Button ui_btn_dlg_yes = ui_dlg_auto_pilot.findViewById( R.id.btn_yes );
            ui_btn_dlg_yes.setText( R.string.ok );
            ui_btn_dlg_yes.setOnClickListener( v -> ui_dlg_auto_pilot.dismiss() );
        }

        ui_dlg_auto_pilot.show();
    }

    public void createSwipe()
    {
        PagerAdapter mPagerAdapter = new PagerAdapter( fragmentActivity.getSupportFragmentManager(), fragmentActivity.getBaseContext() );

        VIEW_PAGER = fragmentActivity.findViewById( R.id.pager );

        if ( VIEW_PAGER == null )
            return;

        VIEW_PAGER.setAdapter( mPagerAdapter );
        VIEW_PAGER.setOffscreenPageLimit( 4 );

        ui_main_screen = mPagerAdapter.getMainFragment();
        ui_table       = mPagerAdapter.table();

        //Create customs Tabs
        SlidingTabLayout mSlidingTabLayout =
                fragmentActivity.findViewById( R.id.sliding_tabs );
        mSlidingTabLayout.setDistributeEvenly( true );
        mSlidingTabLayout.setCustomTabView( R.layout.custom_tab, 0 );
        mSlidingTabLayout.setBackground(
                fragmentActivity.getResources().getDrawable( R.drawable.chamfer_black_top ) );
        mSlidingTabLayout.setCustomTabColorizer( new SlidingTabLayout.TabColorizer()
        {
            public int getIndicatorColor( int position )
            {
                return Color.GRAY;
            }

            public int getDividerColor( int position )
            {
                return Color.WHITE;
            }
        } );
        mSlidingTabLayout.setViewPager( VIEW_PAGER );
    }

    public void sendMessage( com.megabras.modbus.Message message )
    {
        ((Main) fragmentActivity).addMessage( message );
    }

    private void userMessage( String message, int color, boolean busy )
    {
        if ( !BOX_MESSAGE.equals( message ) || BOX_COLOR != color || BOX_BUSY != busy )
        {
            BOX_MESSAGE = message;
            BOX_COLOR = color;
            BOX_BUSY = busy;
            ((Main) fragmentActivity).message( message, color, busy );
        }
    }

    /***********************************************************************************************
     * REGION Permission
     ***********************************************************************************************/

    public void permissionCallback( int permission )
    {
        switch ( permission )
        {
            case Main.REQUEST_RECORD_AUDIO_PERMISSION:
                recordAudio();
                break;
            case Main.REQUEST_PICTURE_PERMISSION:
                takePicture();
                break;
            case Main.REQUEST_GPS_PERMISSION:
                getGPS();
                break;
            case Main.REQUEST_MEMORY_REPORT_PERMISSION:
                export();
                break;
            case Main.REQUEST_MEMORY_MAP_PERMISSION:
                map();
                break;
        }
    }

    /***********************************************************************************************
     * REGION Audio
     ***********************************************************************************************/

    private void recordAudio()
    {
        if ( recordData != null && recordData.getRecordId() > 0 )
        {
            database.createFolder( fragmentActivity, model );
            File location = database.getAudioFile( recordData.getRecordId() );
            audio.setFile( location );
            audio.showDialog( recordData.getRecordName() );
        }
    }

    /***********************************************************************************************
     * REGION Picture
     ***********************************************************************************************/

    private void takePicture()
    {
        if ( recordData != null && recordData.getRecordId() > 0 )
        {
            database.createFolder( fragmentActivity, model );
            File file = database.getPictureFile( recordData.getRecordId() );
            picture.showDialog( file );
        }
    }

    public void pictureStatus( final int status )
    {
        fragmentActivity.runOnUiThread( () ->
        {
            switch ( status )
            {
                case Picture.WAIT:
                    mCommonUI.changeButtonImage( R.id.bt_picture, R.drawable.ic_wait, false );
                    break;
                case Picture.FINISHED:
                    mCommonUI.changeButtonImage( R.id.bt_picture, R.drawable.ic_picture_states, true );
                    break;
            }
        } );
    }

    /***********************************************************************************************
     * REGION GPS
     ***********************************************************************************************/

    private void getGPS()
    {
        if ( recordData == null || recordData.getRecordId() <= 0 )
        {
            return;
        }

        if ( recordData.getGpsAccuracy() == 0 )
        {
            mGPS.getDataGPS();
        }
        else
        {
            String[] gps = new String[]{
                    recordData.getGpsLongitudeAsString(),
                    recordData.getGpsLatitudeAsString(),
                    recordData.getGpsAltitudeAsString(),
                    recordData.getGpsAccuracyAsString()
            };
            mGPS.dialogConfirmReplace( gps );
        }
    }

    public void GpsData( int which, String[] data )
    {
        switch ( which )
        {
            case GPS.RESULT:
                if ( data != null && recordData != null )
                {
                    recordData.setGpsLongitude( Double.parseDouble(data[0]) );
                    recordData.setGpsLatitude ( Double.parseDouble(data[1]) );
                    recordData.setGpsAltitude ( Double.parseDouble(data[2]) );
                    recordData.setGpsAccuracy ( Double.parseDouble(data[3]) );
                    database.saveOrUpdate( recordData );
                    ui_table.loadSavedData( recordData );
                }
                mCommonUI.changeButtonImage( ui_ibtn_gps, R.drawable.ic_gps_states, true );
                break;
            case GPS.WAIT:
                mCommonUI.changeButtonImage( ui_ibtn_gps, R.drawable.ic_wait, false );
                break;
            case GPS.CANCELLED:
                mCommonUI.changeButtonImage( ui_ibtn_gps, R.drawable.ic_gps_states, true );
                break;
        }
    }

    private void map()
    {
        if ( recordData != null && recordData.getRecordId() > 0 && recordData.getGpsAccuracy() > 0 )
        {
            MapData mapData = new MapData(
                    recordData.getRecordName(),
                    recordData.getGpsLongitude(),
                    recordData.getGpsLatitude(),
                    recordData.getGpsAltitudeAsString(),
                    recordData.getGpsAccuracyAsString()
            );
            map.showMap( database.getMapSnapshotFile( recordData.getRecordId() ), mapData );
        }
    }

    /***********************************************************************************************
     * REGION Report
     ***********************************************************************************************/

    private void creatDialogProgress()
    {
        if ( ui_dlg_progress == null )
        {
            ui_dlg_progress = new Dialog( fragmentActivity );
            ui_dlg_progress.requestWindowFeature( Window.FEATURE_NO_TITLE );
            ui_dlg_progress.setCanceledOnTouchOutside( false );
            ui_dlg_progress.setContentView( R.layout.progress_bar );
            ((TextView) ui_dlg_progress.findViewById( R.id.dlg_title )).setText( R.string.please_wait );
            ((TextView) ui_dlg_progress.findViewById( R.id.message )).setText( R.string.saving_data_in_pdf );
            ui_dlg_progress.findViewById( R.id.dlg_icon ).setVisibility( View.GONE );
        }
        ui_dlg_progress.show();
    }

    private void updateProgressBar( String progress )
    {
        ((TextView) ui_dlg_progress.findViewById( R.id.status )).setText( progress );
    }

    private void export()
    {
        if ( database == null || REC_ID_EXPORT < 1)
        {
            return;
        }

        RecordData recData = database.getRecordData( REC_ID_EXPORT );

        if ( recData != null )
        {
            recData.setDateTimeFormat( device.getDateFormat(), device.getTimeFormat() );
            creatDialogProgress();

            new Thread( () -> report.generate
                              (
                                  recData,
                                  database.getPictureFile( recData.getRecordId() ),
                                  database.getMapSnapshotFile( recData.getRecordId() )
                              ) ).start();
            closeOverlay();
        }
    }

    /***********************************************************************************************
     * REGION Handler
     ***********************************************************************************************/

    public static class mHandler extends Handler
    {
        private final WeakReference<Controller>
                mClass;

        static final int
                REPORT_UPDATE   = 0,
                REPORT_FINISHED = 1,
                REPORT_ERROR    = 2;

        public mHandler( Controller c )
        {
            mClass = new WeakReference<>( c );
        }

        @Override
        public void handleMessage( @NonNull android.os.Message msg )
        {
            final Controller c = mClass.get();

            switch( msg.what )
            {
                case mHandler.REPORT_UPDATE:
                    c.updateProgressBar( (String) msg.obj );
                    break;
                case mHandler.REPORT_FINISHED:
                    c.ui_dlg_progress.dismiss();
                    break;
                case mHandler.REPORT_ERROR:
                    c.ui_dlg_progress.dismiss();
                    break;
            }
        }
    }

    /***********************************************************************************************
     * REGION ???
     ***********************************************************************************************/

    public com.megabras.modbus.Message statusMessage()
    {
        return modbus.read.status( false );
    }

    public com.megabras.modbus.Message stopMessage()
    {
        return modbus.write.stop();
    }

    public void readGeneralConfig()
    {
        device.readFreeMemory();
        device.readyDate();
    }

    public void setMenuItemOptions( Menu menu, int optionsID )
    {
        if ( ui_menu_item_options == null )
        {
            ui_menu_item_options = menu.findItem( optionsID );
        }
    }

    /***********************************************************************************************
     * REGION CALLBACKs
     ***********************************************************************************************/

    public void stateChanged( int state )
    {
        switch ( state )
        {
            case BluetoothClassic.STATE_CONNECTED:
                device.syncDeviceInfo();
                ui_ibtn_pwm_small_down.setEnabled( false );
                ui_ibtn_pwm_small_up.setEnabled( false );
                ui_ibtn_pwm_medium_down.setEnabled( false );
                ui_ibtn_pwm_medium_up.setEnabled( false );
                ui_ibtn_pwm_large_down.setEnabled( false );
                ui_ibtn_pwm_large_up.setEnabled( false );
                ui_menu_item_options.setEnabled( false );
                break;
            case BluetoothClassic.STATE_CONNECTING:
                break;
            case BluetoothClassic.STATE_NOT_RESPONDING:
                break;
            case BluetoothClassic.STATE_FAIL:
                break;
            case BluetoothClassic.STATE_LOST:
            case BluetoothClassic.STATE_CLOSED:
                userMessage( fragmentActivity.getString( R.string.disconnected ), CommonUI.COLOR_BLACK, false );
                if ( ui_main_screen != null )
                {
                    ui_main_screen.disconnected();
                }
                device.offLine();
                ui_btn_start.setVisibility( View.VISIBLE );
                ui_btn_stop.setVisibility( View.GONE );
                ui_btn_start.setEnabled( false );
                ui_ibtn_printer.setEnabled( false );
                ui_ibtn_setup.setEnabled( false );
                ui_ibtn_save.setEnabled( false );
                ui_ibtn_pwm_small_down.setEnabled( false );
                ui_ibtn_pwm_small_up.setEnabled( false );
                ui_ibtn_pwm_medium_down.setEnabled( false );
                ui_ibtn_pwm_medium_up.setEnabled( false );
                ui_ibtn_pwm_large_down.setEnabled( false );
                ui_ibtn_pwm_large_up.setEnabled( false );
                ui_menu_item_options.setEnabled( false );
                ui_ibtn_file_manager.setEnabled( true );
                ui_ibtn_voice.setEnabled( true );
                ui_ibtn_picture.setEnabled( true );
                ui_ibtn_gps.setEnabled( true );
                ui_ibtn_map.setEnabled( true );
                closeOverlay();
                break;
        }
    }

    public void processResponse( int id, String response )
    {
        device.messageReceived( id, response );
    }

    public void addMessage( Message message )
    {
        sendMessage( message );
    }

    @SuppressLint("SetTextI18n")
    @Override
    public void update( Observable o, Object arg )
    {
        if ( o instanceof DeviceOld && arg instanceof DeviceOld.Changed )
        {
            DeviceOld.Changed whatChanged = (DeviceOld.Changed) arg;

            //Log.e( TAG, "UPDATE: " + whatChanged.name() );

            switch ( whatChanged )
            {
                case DEVICE_INFO:
                    device.syncDevices();
                    if ( device.getID() == 105 )
                    {
                        ui_ibtn_printer.setVisibility( View.VISIBLE );
                        ui_setup_lyt_fan.setVisibility( View.GONE );
                    }
                    else if ( device.getID() == 215 )
                    {
                        ui_ibtn_printer.setVisibility( View.GONE );
                        ui_setup_lyt_fan.setVisibility( View.VISIBLE );
                    }
                    createPageGeneralSetup();
                    //ui_menu_item_options.setEnabled( true );
                    break;
                case FREE_MEMORY:
                    ui_general_txw_free_memory.setText( device.getFreeMemory() );
                    break;
                case DATE:
                    USER_DATE = device.getDate();
                    loadGeneralSetupPage();
                    showOverlay( VIEW_GENERAL );
                    break;
                case READING_SETUP:
                    userMessage( fragmentActivity.getString( R.string.syncing ), CommonUI.COLOR_YELLOW, true );
                    break;
                case SETUP:
                    ui_main_screen.updateParameters( device );
                    loadTestSetupPage();
                    break;
                case RECORD_DATA:
                    if ( recordData == null || !recordData.getRecordName().equals( device.getRecordName() ) )
                    {
                        recordData = new RecordData();
                        recordData.setRecordName( device.getRecordName() );
                        recordData.setDeviceId( device.getID() );
                        recordData.setDeviceModel( device.getModel() );
                        recordData.setDeviceSerial( device.getSerial() );
                        recordData.setDeviceVersionPreffix( device.getVersionPreffix() );
                        recordData.setDeviceHardware( device.getHardware() );
                        recordData.setDeviceFirmware( device.getFirmware() );
                        recordData.setDateTimeFormat( device.getDateFormat(), device.getTimeFormat() );
                        ui_table.loadSavedData( recordData );
                    }
                    ui_main_screen.updateParameters( device );
                    ui_table.updateRecordData( recordData.getRecordName() );
                    loadTestSetupPage();
                    break;
                case CONF_AMP:
                    ui_main_screen.updateParameters( device );
                    loadTestSetupPage();
                    break;
                case CONF_TIMER:
                    ui_main_screen.updateParameters( device );
                    loadTestSetupPage();
                    break;
                case AUTO_PILOT_ON:
                    dialogAutoPilot();
                    ui_ibtn_save.setEnabled( false );
                    ui_ibtn_printer.setEnabled( false );
                    ui_ibtn_pwm_small_down.setEnabled( false );
                    ui_ibtn_pwm_small_up.setEnabled( false );
                    ui_ibtn_pwm_medium_down.setEnabled( false );
                    ui_ibtn_pwm_medium_up.setEnabled( false );
                    ui_ibtn_pwm_large_down.setEnabled( false );
                    ui_ibtn_pwm_large_up.setEnabled( false );
                    break;
                case CONFIG_CHANGED:
                    userMessage( fragmentActivity.getString( R.string.md_configurations_changed ), CommonUI.COLOR_YELLOW, true );
                    break;
                case BUSY:
                    userMessage( fragmentActivity.getString( R.string.wait ), CommonUI.COLOR_YELLOW, true );
                    ui_btn_start.setEnabled( false );
                    ui_ibtn_save.setEnabled( false );
                    ui_ibtn_printer.setEnabled( false );
                    ui_ibtn_setup.setEnabled( false );
                    ui_menu_item_options.setEnabled( false );
                    break;
                case TEST_CANCELLED:
                case TEST_FINISHED:
                case STANDY_BY:
                    if ( whatChanged == DeviceOld.Changed.TEST_FINISHED )
                    {
                        dialogFinished();
                    }
                    else if ( whatChanged == DeviceOld.Changed.TEST_CANCELLED )
                    {
                        dialogCancelled();
                    }
                    else
                    {
                        userMessage( fragmentActivity.getString( R.string.ready ), CommonUI.COLOR_GREEN, false );
                    }
                    ui_main_screen.resetGauge();
                    ui_btn_start.setVisibility( View.VISIBLE );
                    ui_btn_start.setEnabled( true );
                    ui_btn_stop.setVisibility( View.GONE );
                    ui_ibtn_save.setEnabled( true );
                    ui_ibtn_printer.setEnabled( true );
                    ui_ibtn_setup.setEnabled( true );
                    ui_ibtn_pwm_small_down.setEnabled( false );
                    ui_ibtn_pwm_small_up.setEnabled( false );
                    ui_ibtn_pwm_medium_down.setEnabled( false );
                    ui_ibtn_pwm_medium_up.setEnabled( false );
                    ui_ibtn_pwm_large_down.setEnabled( false );
                    ui_ibtn_pwm_large_up.setEnabled( false );
                    ui_menu_item_options.setEnabled( true );

                    ui_ibtn_file_manager.setEnabled( true );
                    ui_ibtn_gps.setEnabled( true );
                    ui_ibtn_picture.setEnabled( true );
                    ui_ibtn_voice.setEnabled( true );
                    ui_ibtn_map.setEnabled( true );
                    break;
                case TEST_STARTING:
                    userMessage( fragmentActivity.getString( R.string.md_warning_test_start ), CommonUI.COLOR_YELLOW, true );
                    ui_btn_start.setVisibility( View.GONE );
                    ui_btn_stop.setVisibility( View.VISIBLE );
                    ui_ibtn_setup.setEnabled( false );
                    ui_main_screen.reset();
                    ui_ibtn_file_manager.setEnabled( false );
                    ui_ibtn_gps.setEnabled( false );
                    ui_ibtn_picture.setEnabled( false );
                    ui_ibtn_voice.setEnabled( false );
                    ui_ibtn_map.setEnabled( false );
                    break;
                case TEST_RUNNING:
                    userMessage( fragmentActivity.getString( R.string.running ), CommonUI.COLOR_RED, true );
                    ui_btn_start.setVisibility( View.GONE );
                    ui_btn_stop.setVisibility( View.VISIBLE );
                    ui_ibtn_setup.setEnabled( false );
                    ui_menu_item_options.setEnabled( false );

                    if ( device.getMode() == DeviceOld.MODE_MANUAL && !device.isAutoPilot() )
                    {
                        ui_ibtn_pwm_small_down.setEnabled( true );
                        ui_ibtn_pwm_small_up.setEnabled( true );
                        ui_ibtn_pwm_medium_down.setEnabled( true );
                        ui_ibtn_pwm_medium_up.setEnabled( true );
                        ui_ibtn_pwm_large_down.setEnabled( true );
                        ui_ibtn_pwm_large_up.setEnabled( true );
                    }
                    ui_ibtn_file_manager.setEnabled( false );
                    ui_ibtn_gps.setEnabled( false );
                    ui_ibtn_picture.setEnabled( false );
                    ui_ibtn_voice.setEnabled( false );
                    ui_ibtn_map.setEnabled( false );
                    break;
                case RT_TEST_NUM:
                    recordData.newTestData( device.getRTTestNumber() );
                    recordData.setMode( device.getMode() );
                    recordData.setTimerEnabled( device.getTimer() );
                    recordData.setTimer( device.getConfTimer() );
                    recordData.setBsgEnabled( device.getBSG() );
                    ui_main_screen.updateMeasurement( device );
                    ui_table.updateTestNumber( device.getRTTestNumber() );
                    break;
                case RT_DATE:
                    recordData.setDate( device.getRTDate() );
                    ui_table.updateHeaderInfo( recordData.getLatstTestData() );
                    break;
                case RT_MEASUREMENT:
                    ui_main_screen.updateMeasurement( device );
                    break;
                case SAVING_MEASUREMENT:
                    userMessage( fragmentActivity.getString( R.string.saving ), CommonUI.COLOR_YELLOW, true );
                    break;
                case SAVED_MEASUREMENT:
                    if ( recordData.getLatstTestData() != null )
                    {
                        recordData.getLatstTestData().addMeasurement( device.getSavedMeasurement() );
                        ui_table.updateMeasurement( device.getSavedMeasurement(), device.getBSG(), true );
                        if ( SAVE_IN_DEVICE )
                        {
                            database.saveOrUpdate( recordData );
                        }
                    }
                    break;
            }
        }
    }

    /***********************************************************************************************
     * REGION Utilities
     ***********************************************************************************************/

    private TimePickerDialog.OnTimeSetListener timeListener = new TimePickerDialog.OnTimeSetListener()
    {
        @Override
        public void onTimeSet( TimePicker view, int hourOfDay, int minute )
        {
            if ( VIEW_GENERAL != null )
            {
                Calendar c = Calendar.getInstance();
                c.setTime( USER_DATE );
                c.set( Calendar.HOUR_OF_DAY, hourOfDay );
                c.set( Calendar.MINUTE, minute );
                c.set( Calendar.SECOND, 0 );
                USER_DATE = c.getTime();
                ui_general_txw_time.setText( FormatData.time( device.getTimeFormat(), USER_DATE ) );
            }
        }
    };

    private void pickTime()
    {
        Calendar calendar = new GregorianCalendar();
        calendar.setTime( USER_DATE );
        int     hour   = calendar.get( Calendar.HOUR_OF_DAY );
        int     minute = calendar.get( Calendar.MINUTE );
        boolean is24   = device.getTimeFormat() == Modbus.TIME_24H;
        new TimePickerDialog( fragmentActivity, timeListener, hour, minute, is24 ).show();
    }

    private DatePickerDialog.OnDateSetListener dateListener = new DatePickerDialog.OnDateSetListener()
    {
        public void onDateSet( DatePicker arg0, int year, int month, int day )
        {
            if ( VIEW_GENERAL != null )
            {
                String parts[] = ui_general_txw_time.getText().toString().split( ":" );
                int    hour    = Integer.parseInt( parts[0] );
                int    minutes = Integer.parseInt( parts[1] );

                Calendar c = Calendar.getInstance();
                c.set( year, month, day, hour, minutes );
                USER_DATE = c.getTime();
                ui_general_txw_date.setText( FormatData.date( device.getDateFormat(), USER_DATE ) );
            }
        }
    };

    private void pickDate()
    {
        Calendar calendar = new GregorianCalendar();
        calendar.setTime( USER_DATE );
        int year  = calendar.get( Calendar.YEAR );
        int month = calendar.get( Calendar.MONTH );
        int day   = calendar.get( Calendar.DAY_OF_MONTH );
        new DatePickerDialog( fragmentActivity, dateListener, year, month, day ).show();
    }

    private LinearLayout getLL( int layoutID, View view )
    {
        return (LinearLayout) view.findViewById( layoutID );
    }

    @SuppressWarnings("SameParameterValue")
    private ListView getLV( int layoutID, View view )
    {
        return (ListView) view.findViewById( layoutID );
    }

    private TextView getTV( int layoutID, View view )
    {
        return (TextView) view.findViewById( layoutID );
    }

    private ImageButton getIBTN( int layoutID, View view )
    {
        return (ImageButton) view.findViewById( layoutID );
    }

    private Switch getSWT( int layoutID, View view )
    {
        return (Switch) view.findViewById( layoutID );
    }

    private Button getBTN( int layoutID, View view )
    {
        return (Button) view.findViewById( layoutID );
    }

    private Spinner getSpinner( int spinnerID, int arrayID, View inflatedView) {
        ArrayAdapter<CharSequence> filter =
                ArrayAdapter.createFromResource(fragmentActivity, arrayID, android.R.layout.simple_spinner_item);
        filter.setDropDownViewResource(R.layout.simple_spinner_dropdown_item);

        final Spinner spinner = inflatedView.findViewById(spinnerID);
        spinner.setAdapter(filter);
        return spinner;
    }

    private void pickNumber( final TextView tv, int title, final String unity, final int min, final int max, final int step)
    {
        final Dialog dialog = new Dialog(fragmentActivity);
        dialog.requestWindowFeature(Window.FEATURE_NO_TITLE);
        dialog.setCanceledOnTouchOutside(false);
        dialog.setContentView(R.layout.main_dialog_number);
//        if (dialog.getWindow()!=null)
//            dialog.getWindow().getAttributes().width = WindowManager.LayoutParams.MATCH_PARENT;

        final EditText etNumber = dialog.findViewById(R.id.et_number);
        final TextView message  = dialog.findViewById(R.id.tv_message);
        final TextView warning  = dialog.findViewById(R.id.tv_warning);
        final TextView tvUnity  = dialog.findViewById(R.id.tv_unit);

        message.setText( fragmentActivity.getString(title) );
        tvUnity.setText(unity);
        etNumber.setText( tv.getText().toString().replace(unity, "").trim() );

        etNumber.setSelection(0, etNumber.getText().length());
        etNumber.requestFocus();
        final InputMethodManager imm = (InputMethodManager) fragmentActivity.getSystemService(Context.INPUT_METHOD_SERVICE);
        if ( imm != null )
        {
            imm.toggleSoftInput( InputMethodManager.SHOW_FORCED, InputMethodManager.HIDE_IMPLICIT_ONLY );
        }

        String warningMessage =
                fragmentActivity.getString(R.string.range)
                        + ": " + min + " " + unity + " - "
                        + max + " " + unity
                        + " ("+fragmentActivity.getString(R.string.step, step+" " + unity)+")";

        warning.setText( warningMessage );

        Button buttonSave = dialog.findViewById(R.id.bt_save);
        buttonSave.setOnClickListener(v ->
        {
            if( etNumber.getText().toString().equals("") )
                return;

            int number  = Integer.parseInt( etNumber.getText().toString().replace(unity,"").trim() );
            number      = (number/step)*step;

            if( number >= min && number <= max )
            {
                String n = number + " " + unity;
                tv.setText( n );
                if ( imm != null )
                {
                    imm.hideSoftInputFromWindow( etNumber.getWindowToken(), 0 );
                }
                dialog.dismiss();
            }
            else
            {
                warning.setTextColor(fragmentActivity.getResources().getColor(R.color.common_signin_btn_default_background));
            }
        });

        Button buttonCancel = dialog.findViewById(R.id.bt_cancel);
        buttonCancel.setOnClickListener(v ->
        {
            if ( imm != null )
            {
                imm.hideSoftInputFromWindow( etNumber.getWindowToken(), 0 );
            }
            dialog.dismiss();
        });

        dialog.show();
    }

    @SuppressWarnings("SameParameterValue")
    private void pickName( final TextView tv, int title )
    {
        final Dialog dialog = new Dialog( fragmentActivity );
        dialog.requestWindowFeature( Window.FEATURE_NO_TITLE );
        dialog.setCanceledOnTouchOutside( false );
        dialog.setContentView( R.layout.main_dialog_name );
        if ( dialog.getWindow() != null )
        {
            dialog.getWindow().getAttributes().width = WindowManager.LayoutParams.MATCH_PARENT;
        }

        final EditText ui_etx_name    = dialog.findViewById( R.id.et_name );
        final TextView ui_txw_message = dialog.findViewById( R.id.tv_message );

        ui_txw_message.setText( fragmentActivity.getString(title) );
        ui_etx_name.setText( tv.getText().toString() );
        ui_etx_name.setSelection(0, ui_etx_name.getText().length());
        ui_etx_name.requestFocus();

        final InputMethodManager imm = (InputMethodManager) fragmentActivity.getSystemService(Context.INPUT_METHOD_SERVICE);
        if ( imm != null )
        {
            imm.toggleSoftInput( InputMethodManager.SHOW_FORCED, InputMethodManager.HIDE_IMPLICIT_ONLY );
        }

        Button buttonSave = dialog.findViewById(R.id.bt_save);
        buttonSave.setOnClickListener(v ->
        {
            if( ui_etx_name.getText().toString().equals("") )
                return;

            tv.setText( ui_etx_name.getText() );
            if ( imm != null )
            {
                imm.hideSoftInputFromWindow( ui_etx_name.getWindowToken(), 0 );
            }
            dialog.dismiss();
        });

        Button buttonCancel = dialog.findViewById(R.id.bt_cancel);
        buttonCancel.setOnClickListener(v ->
        {
            if ( imm != null )
            {
                imm.hideSoftInputFromWindow( ui_etx_name.getWindowToken(), 0 );
            }
            dialog.dismiss();
        });

        dialog.show();
    }


    private void showOverlay( View view )
    {
        if ( OVERLAY != null && view != null )
        {
            OVERLAY.removeAllViews();
            OVERLAY.addView( view );
            OVERLAY.setVisibility( View.VISIBLE );
            ui_menu_item_options.setEnabled( false );
        }
    }

    private void closeOverlay()
    {
        if ( OVERLAY != null )
        {
            OVERLAY.removeAllViews();
            OVERLAY.setVisibility( View.GONE );
            ui_menu_item_options.setEnabled( true );
        }
    }
}
