package com.megabras.bluelogg;

import android.annotation.SuppressLint;
import android.app.Activity;
import android.os.Environment;
import android.util.Log;
import android.view.MotionEvent;
import android.view.View;
import android.widget.FrameLayout;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.OutputStream;
import java.math.RoundingMode;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;
import java.util.TimeZone;

public class MUtils {
	private final static String TAG = "MUtils";

	public static String removeSpecialChars(String name){
		name = name.replace(" ", "_");
		name = name.replace("|", "");
		name = name.replace("\\", "");
		name = name.replace("/", "");
		name = name.replace(":", "");
		name = name.replace("*", "");
		name = name.replace("?", "");
		name = name.replace("\"", "");
		name = name.replace("<", "");
		name = name.replace(">", "");
		name = name.replace(";", "");
		name = name.replace("+", "");
		name = name.replace("¨", "");
		name = name.replace("%", "");
		name = name.replace("$", "");
		name = name.replace("#", "");
		name = name.replace("'", "");
		return name;
	}
	
	public static String setCheckSum(String msg) {
		byte checksum = 0;
		if (msg.length() % 2 == 0) {
			for (int i = 0; i < msg.length(); i = i + 2) {
				checksum = (byte) (checksum + Integer.valueOf(
						msg.substring(i, i + 2), 16).byteValue());
			}
			checksum = (byte) ((255 - checksum) + 1);
			return (msg + String.format("%02X", checksum)).toUpperCase(Locale
					.getDefault());
		}
		return null;
	}
	
	public static boolean checkSum(String msg) {
		msg = msg.replace(Command.SF,"").replace(Command.EF,"");
		//CHECK THE PACK SIZE
		if(msg.length()<6){//Check if at least the message have de IP (01), Command type (03 or 10) and Pack size (00)
			return false;
		} else {
			String CMD_READ           = "03";
			String CMD_WRITE_SINGLE   = "06";
			String CMD_WRITE_MULTIPLE = "10";			
			String CMD = msg.substring(2,4);
			
			if(CMD.equals(CMD_WRITE_MULTIPLE) || CMD.equals(CMD_WRITE_SINGLE)){
				int packSizeInWriteResponse = 14;
				if(msg.length()!=packSizeInWriteResponse){
					return false;
				} 
			} else if(CMD.equals(CMD_READ)){
				int packSizeinBytes = getPackSize(msg);		
				int messageSize = 2 + 2 + 2 + (2*packSizeinBytes) + 2;//IP + CMD TYPE + PACK SIZE + MESSAGE (num of bytes to ASCII HEX) + CHECKSUM		
				if(msg.length() != messageSize){
					return false;
				}
			} else {
				return false;
			}			
		}		
		
		//CHECK THE CHECKSUM
		byte checksum = 0;
		if (msg.length() % 2 == 0) {
			for (int i = 0; i < msg.length(); i = i + 2) {
				byte temp = Integer.valueOf(msg.substring(i, i + 2), 16)
						.byteValue();
				checksum = (byte) (checksum + temp);
			}
			if (checksum == 0) {
				return true;
			}
		}
		return false;
	}
	
	private static int getPackSize(String msg){
		int packSize = 0;
		String hexPackSize = msg.substring(4,6);
		packSize = hexToInt(hexPackSize);
		return packSize;
	}
	
	public static String checkNull(String toCheck) {
		if (toCheck != null) {
			return toCheck;
		}
		return "";
	}
	
	public static String hexToFloat(String hex) {
		DecimalFormat df = new DecimalFormat("0.00");
		df.setRoundingMode(RoundingMode.DOWN);
		Long i = Long.parseLong(hex, 16);
		Float f = Float.intBitsToFloat(i.intValue());
		return df.format(f).replace(",", ".");
	}
	
	public static Float hexToFloatA(String hex) {		
		Long i = Long.parseLong(hex, 16);		
		Float f = Float.intBitsToFloat(i.intValue());		
		return f;
	}
	
	public static boolean[] byteToBit(byte[] bytes) {
		boolean[] bits = new boolean[bytes.length * 8];
		int index = 0;
		for (int a = bytes.length - 1; a >= 0; a--) {
			for (int i = 0; i < 8; i++) {
				if ((bytes[a] & (1 << i)) != 0) {
					bits[index] = true;
				} else {
					bits[index] = false;
				}
				index++;
			}
		}
		return bits;
	}
	
	public static boolean[] hexToBitArray(String s) {
		byte byte1 = Integer.valueOf(s.substring(0, 2), 16).byteValue();
		byte byte2 = Integer.valueOf(s.substring(2, 4), 16).byteValue();
		byte[] bytes = new byte[] { byte1, byte2 };		
		boolean[] bits = new boolean[bytes.length * 8];
		int index = 0;
		for (int a = bytes.length - 1; a >= 0; a--) {
			for (int i = 0; i < 8; i++) {
				if ((bytes[a] & (1 << i)) != 0) {
					bits[index] = true;
				} else {
					bits[index] = false;
				}
				index++;
			}
		}
		return bits;
	}
	
	public static String convertHexToString(String hex){		 
		  StringBuilder sb = new StringBuilder();
		  StringBuilder temp = new StringBuilder();
	 
		  //49204c6f7665204a617661 split into two characters 49, 20, 4c...
		  for( int i=0; i<hex.length()-1; i+=2 ){
	 
		      //grab the hex in pairs
		      String output = hex.substring(i, (i + 2));
		      //convert hex to decimal
		      int decimal = Integer.parseInt(output, 16);
		      //convert the decimal to character
		      sb.append((char)decimal);
	 
		      temp.append(decimal);
		  }
		  System.out.println("Decimal : " + temp.toString());
	 
		  return sb.toString();
	}

	/**
	 * Clean up the HEX message receveid from the device. Removes the device IP, function code, bytes quantity and CRC.
	 * Only to be used with the READ messages received.
	 *
	 * @param msg HEX message
	 */
	public static String cleanMsg(String msg){
		String cleanMsg;
		try{
			cleanMsg = msg.substring(6, msg.length()-2);	
		} catch (Exception e){
			cleanMsg = "";
		}        
        return cleanMsg;
    }

	public static void bitArrayToInt(boolean[] bits){
		boolean[] rev = new boolean[bits.length];
		for(int i=0;i<bits.length;i++){
			rev[(rev.length-1)-i] = bits[i];
		}
		int result = 0;
		for (int i = 0 ; i < bits.length; i++) {
			result = (result | toDigit(bits[i]))  << 1;
			Log.e(TAG,"R= " + result);
		}
		Log.e(TAG, "RESULT = " + Long.toString(result) );

		String ff = String.format("%04X", result)
						.toUpperCase(Locale.getDefault());
		Log.e(TAG,"IN HEX: " + ff);
//
//		//println(String.format("%x", bitSetInt));

//		StringBuilder builder = new StringBuilder();
//		for (int i = 0; i < rev.length; i++) {
//			if (rev[i]) builder.append("1"); else builder.append("0");
//		}
//		String r = builder.toString();
//		Log.e(TAG, "ss= " + r);
	}

	public static int bitArrayToInteger(boolean[] bits, int offset, int lenght){
		int result = 0;

		boolean[] reverse = new boolean[bits.length];
		for(int i=0;i<bits.length;i++){
			reverse[(reverse.length-1)-i] = bits[i];
		}

		String total = "";
		for(int i=0; i < reverse.length; i++){
			if(reverse[i])
				total += "1";
			else
				total += "0";
		}
		int realOffset = reverse.length - lenght - offset;
		String parcial = total.substring(realOffset, realOffset+lenght);
		result = Integer.parseInt(parcial, 2);
		return result;
	}

	static int toDigit(boolean b) { return b?1:0;}
	
	public static int hexToInt(String msg){
        return Integer.valueOf(msg, 16);
    }

	public static long getCurrentTime(){
		long currentTime;
		if(TimeZone.getDefault().inDaylightTime(new Date())){
			currentTime = ((new Date().getTime() + TimeZone.getDefault().getRawOffset() + TimeZone.getDefault().getDSTSavings()) / 1000L);
		} else {
			currentTime = ((new Date().getTime() + TimeZone.getDefault().getRawOffset()) / 1000L);
		}
		return currentTime;
	}

	public static String intToHexadecimal(int value, int lenght) {
		StringBuilder sb = new StringBuilder();
		sb.append(Integer.toHexString(value));
		while (sb.length() < lenght) {
			sb.insert(0, '0');
		}
		String hex = sb.toString();
		return hex.toUpperCase(Locale.getDefault());
	}

	public static String longToHexadecimal(long value, int lenght) {
		StringBuilder sb = new StringBuilder();
		sb.append(Long.toHexString(value));
		while (sb.length() < lenght) {
			sb.insert(0, '0');
		}
		String hex = sb.toString();
		return hex.toUpperCase(Locale.getDefault());
	}

	public static FrameLayout getOverlay(Activity activity, int overlayID, View inflatedView){
		FrameLayout overlay = (FrameLayout) activity.findViewById(overlayID);
		overlay.removeAllViews();
		overlay.addView(inflatedView);
		overlay.setVisibility(View.VISIBLE);
		// This is to prevent the ontouch propagate to others views
		overlay.requestDisallowInterceptTouchEvent(true);
		overlay.setOnTouchListener(new View.OnTouchListener(){
			public boolean onTouch(View v, MotionEvent event) {
				return true;
			}
		});
		return overlay;
	}

	@SuppressLint("ClickableViewAccessibility")
	public static FrameLayout getOverlay( Activity activity, int overlayID )
	{
		FrameLayout overlay = activity.findViewById( overlayID );
		overlay.setVisibility( View.VISIBLE );
		// This is to prevent the ontouch propagate to others views
		overlay.requestDisallowInterceptTouchEvent( true );
		overlay.setOnTouchListener( ( v, event ) -> true );
		return overlay;
	}

	@SuppressLint("ClickableViewAccessibility")
	public static FrameLayout getOverlayInstance( Activity activity, int overlayID )
	{
		FrameLayout overlay = activity.findViewById( overlayID );
		// This is to prevent the ontouch propagate to others views
		overlay.requestDisallowInterceptTouchEvent( true );
		overlay.setOnTouchListener( ( v, event ) -> true );
		return overlay;
	}


	public static void writeErrorLog(Activity activity, String msg){
		try {
			String folder = Environment.getExternalStorageDirectory().toString();
			File file = new File(folder + "/" + activity.getResources().getString(R.string.company) + "/" + "error.log");

			if(file.exists() && file.length() > 1000000){
				file.delete();
			}

			if (!file.exists()) {
				file.createNewFile();
			}

			FileWriter f;
			f = new FileWriter(file.toString(), true);
			f.write(msg);
			f.flush();
			f.close();
		} catch (Exception ignored){}
	}

	public static void log(int type, String cmd, String desc, boolean send) {
		DecimalFormat df = new DecimalFormat("00");
		String s = ">>";
		if(!send){
			s = "<<";
		}
		String t = s + df.format(type)  + " " + desc;
		String msg = String.format("%-27s", t) + ": " + cmd;

		if( !msg.contains("SHORT STATUS") ){
			Log.e("DEBUG", msg);
			if(!send){
				Log.e("CRLF ", "   ");
			}
		}
	}

	public static void error(Activity activity, String method, Exception error){
		try {
			String msg = "";
			SimpleDateFormat dateFormatter = new SimpleDateFormat("dd/MM/yy HH:mm:ss.SSS", Locale.getDefault());
			String company = activity.getResources().getString(R.string.company);
			String version = activity.getPackageManager().getPackageInfo(activity.getPackageName(), 0).versionName;
			String date = dateFormatter.format(new Date());
			StackTraceElement[] e = error.getStackTrace();

			msg += "DATE\t: \t" + date + "\r\n";
			msg += "BRAND\t: \t" + company + "\r\n";
			msg += "VERSION\t: \t" + version + "\r\n";
			msg += "CLASS\t: \t" + activity.getClass().getName() + "\r\n";
			msg += "METHOD\t: \t" + method + "\r\n";
			msg += "CAUSE\t: \t" + error.toString() + "\r\n";
			msg += "STACK: \r\n";

			for (StackTraceElement anE : e) {
				msg += anE.toString() + "\r\n";
			}
			msg += "\r\n";

			writeErrorLog(activity, msg);
		} catch (Exception e){
			Log.e("error log", e.toString());
		}
	}

	public static boolean backUp(File src, File dst){
		try{
//		final String inFileName = "/data/data/com.megabras.bluelogg/databases/"+DatabaseEntry.DATABASE_NAME;
//	    File dbFile = new File(inFileName);
			FileInputStream fis = new FileInputStream(src);

//			File folder = new File(Environment.getExternalStorageDirectory()+"/"+activity.getResources().getString(R.string.company));
//			if(!folder.exists()){
//				folder.mkdir();
//			}

			//String outFileName = folder.getAbsolutePath()+"/"+src.getName();

			// Open the empty db as the output stream
			OutputStream output = new FileOutputStream(dst.getAbsolutePath());
			// Transfer bytes from the inputfile to the outputfile
			byte[] buffer = new byte[1024];
			int length;
			while ((length = fis.read(buffer))>0){
				output.write(buffer, 0, length);
			}

			// Close the streams
			output.flush();
			output.close();
			fis.close();
			return dst.exists();
		}
		catch (Exception e){
			android.util.Log.e(TAG, e.getMessage());
			return false;
		}
	}
}
