package com.megabras.bluelogg.bluetooth;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.os.Environment;
import android.util.Log;

import java.io.File;
import java.util.ArrayList;
import java.util.Objects;

public final class Database extends SQLiteOpenHelper
{

    private final String
            TAG = "Database";

    private final String
            root = Environment.getExternalStorageDirectory().toString();

    private final Context
            context;

    private boolean
        isCreated = false;

    public Database( Context context )
    {
        super( context, Entry.DB_NAME, null, Entry.DB_VERSION );
        this.context = context;
    }

    void create()
    {
        String SQL = "CREATE TABLE IF NOT EXISTS " + Entry.TABLE_DEVICES + " (\n"
                + "\t" + Entry.ID           + " INTEGER PRIMARY KEY AUTOINCREMENT, \n"
                + "\t" + Entry.MODEL        + " TEXT   , \n"
                + "\t" + Entry.SERIAL       + " TEXT   , \n"
                + "\t" + Entry.MAC_ADDRESS  + " TEXT   , \n"
                + "\t" + Entry.BT_TYPE      + " INTEGER  \n"
                + ");";

        SQLiteDatabase db = this.getWritableDatabase();
        db.execSQL( SQL );
        db.execSQL( "PRAGMA foreign_keys=ON" );
        isCreated = true;
    }

    boolean alreadyExists()
    {
        File dbFile = context.getApplicationContext().getDatabasePath( Entry.DB_NAME );
        isCreated = dbFile.exists();
        return isCreated;
    }

    void insert( MBluetoothDevice mDevice )
    {
        if ( !isCreated )
        {
            return;
        }
        SQLiteDatabase db = this.getWritableDatabase();
        db.beginTransaction();
        ContentValues values = new ContentValues();

        values.put( Entry.MODEL      , mDevice.getName() );
        values.put( Entry.SERIAL     , mDevice.getSerial() );
        values.put( Entry.MAC_ADDRESS, mDevice.getAddress() );
        values.put( Entry.BT_TYPE    , mDevice.getDevice().getType() );

        db.insertWithOnConflict( Entry.TABLE_DEVICES, null, values, SQLiteDatabase.CONFLICT_IGNORE );
        db.setTransactionSuccessful();
        db.endTransaction();
    }

    void delete( String model, String serial )
    {
        if ( !isCreated )
        {
            return;
        }
        try
        {
            SQLiteDatabase db = this.getWritableDatabase();
            db.execSQL( "DELETE FROM " + Entry.TABLE_DEVICES + " WHERE (" + Entry.MODEL + "='" + model + "' AND " + Entry.SERIAL + "='" + serial + "');" );
        }
        catch ( Exception e )
        {
            Log.e( TAG, Objects.requireNonNull( e.getMessage() ) );
        }
    }

    ArrayList<SavedDevice> selectList()
    {
        if ( !isCreated )
        {
            return null;
        }
        try
        {
            ArrayList<SavedDevice> list = new ArrayList<>();
            SQLiteDatabase db = this.getReadableDatabase();
            Cursor c = db.rawQuery( "SELECT * FROM " + Entry.TABLE_DEVICES + " ORDER BY " + Entry.MODEL + ";", null );

            if ( c.moveToFirst() )
            {
                while ( !c.isAfterLast() )
                {
                    SavedDevice savedDevice = new SavedDevice();
                    int i = 0;
                    savedDevice.id     = c.getInt( i++ );
                    savedDevice.model  = c.getString( i++ );
                    savedDevice.serial = c.getString( i++ );
                    savedDevice.mac    = c.getString( i++ );
                    savedDevice.type   = c.getInt( i );
                    list.add( savedDevice );

                    c.moveToNext();
                }
            }
            c.close();

            return list;
        }
        catch ( Exception e )
        {
            return null;
        }
    }

    @Override public void onCreate(SQLiteDatabase db) {}
    @Override public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) { }

    public static abstract class Entry
    {
        static final int
                DB_VERSION = 1;

        static final String
                DB_NAME         = "devices_list.db",
                TABLE_DEVICES   = "device",

                // FIELDS
                ID              = "ID",
                MODEL           = "MODEL",
                SERIAL          = "SERIAL",
                MAC_ADDRESS     = "MAC",
                BT_TYPE         = "BT_TYPE";
    }

}

