package com.megabras.bluelogg.bluetooth;

import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.os.Handler;


public class DeviceBroadcastReceiver extends BroadcastReceiver {

	@SuppressWarnings("FieldCanBeLocal")
	private final String
			TAG 			= "DeviceBroadcastReceiver";

	public static final int
			STARTED  		= 1,
			FOUND    		= 2,
			FINISHED 		= 3,
			STATE_CHANGED 	= 4;

	private Handler
			handler;

	public void setHandler(Handler handler){
		this.handler = handler;
	}

	@Override
	public void onReceive(Context context, Intent intent) {
		try{
			String action = intent.getAction();
			if (BluetoothDevice.ACTION_BOND_STATE_CHANGED.equals(action)) {
				int state = intent.getIntExtra(BluetoothDevice.EXTRA_BOND_STATE, BluetoothDevice.ERROR);
				handler.obtainMessage(STATE_CHANGED, 0, 0, state).sendToTarget();
			}
			if (BluetoothAdapter.ACTION_DISCOVERY_STARTED.equals(action)) {
				handler.obtainMessage(STARTED, 0, 0, "").sendToTarget();
			}
			else if (BluetoothDevice.ACTION_FOUND.equals(action)) {
				BluetoothDevice device = intent.getParcelableExtra(BluetoothDevice.EXTRA_DEVICE);
				if (device!=null && device.getName()!=null)
					handler.obtainMessage(FOUND, 0, 0, device).sendToTarget();
			}
			else if (BluetoothAdapter.ACTION_DISCOVERY_FINISHED.equals(action)) {
				handler.obtainMessage(FINISHED, 0, 0, "").sendToTarget();
			}
		} catch (Exception e){
			for (StackTraceElement ste : e.getStackTrace())
				android.util.Log.e(TAG, ste.toString());
		}
	}
}
