package com.megabras.bluelogg.bluetooth;

import android.bluetooth.BluetoothDevice;
import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.ImageButton;
import android.widget.ImageView;
import android.widget.ProgressBar;
import android.widget.TextView;

import com.megabras.bluelogg.R;

import java.util.ArrayList;
import java.util.List;

import androidx.core.content.res.ResourcesCompat;

public class ListDevices extends BaseAdapter
{

	private final LayoutInflater
            inflater;

	private final Context
            context;

	private List<MBluetoothDevice>
            devices;

    private int
            progressBarIndex = -1,
            notBoundedFirstIndex = -1;

    static class ViewHolder
    {
        TextView    header;
        TextView    model;
        TextView    address;
        ImageView   icon;
        ImageButton button;
        ProgressBar progressBar;
    }

    ListDevices( Context context )
    {
        this.context = context;
        this.inflater = LayoutInflater.from( context );
    }

    void clear()
    {
        try
        {
            if ( devices == null || devices.size() <= 0 )
            {
                return;
            }

            devices.clear();
            notBoundedFirstIndex = -1;
            notifyDataSetChanged();
        }
        catch ( Exception ignored )
        {
        }
    }

    void resetNears()
    {
        try
        {
            if ( devices == null || devices.size() <= 0 )
            {
                return;
            }

            for ( MBluetoothDevice d : devices )
            {
                d.setIsNear( false );
            }
            notifyDataSetChanged();
        }
        catch ( Exception ignored )
        {
        }
    }

    public void add( ArrayList<MBluetoothDevice> deviceList )
    {
        if ( devices == null )
        {
            devices = new ArrayList<>();
        }

        for ( MBluetoothDevice bd : deviceList )
        {
            if ( !devices.contains( bd ) )
            {
                devices.add( bd );
            }
        }
        notifyDataSetChanged();
    }

    public int add( MBluetoothDevice device )
    {
        if ( device == null )
        {
            return 0;
        }

        if ( devices == null )
        {
            devices = new ArrayList<>();
        }

        int position = 0;

        if ( devices.size() > 0 && devices.contains( device ) )
        {
            for ( int i = 0; i < devices.size(); i++ )
            {
                if ( devices.get( i ).equals( device ) )
                {
                    devices.get( i ).setIsNear( true );
                    position = i;
                    break;
                }
            }
        }
        else
        {
            devices.add( device );
            position = devices.size() - 1;
        }

        notifyDataSetChanged();
        return position;
    }

    public int getCount()
    {
        if ( devices != null )
        {
            return devices.size();
        }
        else
        {
            return 0;
        }
    }

    public MBluetoothDevice getDevice( int position )
    {
        if ( devices == null || devices.size() <= 0 || position > devices.size() - 1 )
        {
            return null;
        }
        return devices.get( position );
    }

    public void remove( int position )
    {
        devices.remove( position );
        notBoundedFirstIndex = -1;
        notifyDataSetChanged();
    }

    public String getItem( int position )
    {
        return devices.get( position ).getName() + "\n" + devices.get( position ).getAddress();
    }

    public long getItemId( int position )
    {
        return position;
    }

    void showProgressBar( int position )
    {
        progressBarIndex = position;
        notifyDataSetChanged();
    }

    public View getView( int position, View view, ViewGroup parent )
    {
        View vi = view;
        ViewHolder holder;

        if (vi == null)
        {
            vi                 = inflater.inflate( R.layout.bluetooth_list_dev, parent, false );
            holder             = new ViewHolder();
            holder.header      = vi.findViewById( R.id.lbl_header );
            holder.model       = vi.findViewById( R.id.lbl_model );
            holder.address     = vi.findViewById( R.id.lbl_mac );
            holder.icon        = vi.findViewById( R.id.ico_dev );
            holder.button      = vi.findViewById( R.id.bt_unpair );
            holder.progressBar = vi.findViewById( R.id.pb_connecting );
            vi.setTag(holder);
        }
        else
        {
            holder = (ViewHolder) vi.getTag();
        }

        MBluetoothDevice dev = devices.get( position );
        String equipment     = devices.get( position ).getName() + " " + devices.get( position ).getSerial() + ( dev.getDevice().getType() == BluetoothDevice.DEVICE_TYPE_LE ? " (LE)" : "" );

        holder.model.setText( equipment );
        holder.address.setText( dev.getAddress() );

        if ( dev.getDevice().getType() == BluetoothDevice.DEVICE_TYPE_LE || dev.getDevice().getBondState() == BluetoothDevice.BOND_BONDED )
        {
            if ( position == 0 && holder.header != null )
            {
                holder.header.setVisibility( View.VISIBLE );
                holder.header.setText( context.getString( R.string.configured_devices ) );
            }

            holder.button.setVisibility( View.VISIBLE );
            if ( dev.isNear() )
            {
                holder.icon.setImageDrawable( ResourcesCompat.getDrawable( context.getResources(), R.drawable.ic_device_found, null ) );
            }
            else
            {
                holder.icon.setImageDrawable( ResourcesCompat.getDrawable( context.getResources(), R.drawable.ic_device_disabled, null ) );
            }
        }
        else
        {
            if ( notBoundedFirstIndex < 0 && holder.header != null )
            {
                notBoundedFirstIndex = position;
                holder.header.setVisibility( View.VISIBLE );
                holder.header.setText( context.getString( R.string.new_devices ) );
            }
            holder.icon.setImageDrawable( ResourcesCompat.getDrawable( context.getResources(), R.drawable.ic_device_found, null ) );
            holder.button.setVisibility( View.GONE );
        }

        // Is the element clicked
        if ( position == progressBarIndex )
        {
            holder.progressBar.setVisibility( View.VISIBLE );
        }
        else
        {
            holder.progressBar.setVisibility( View.GONE );
        }
        return vi;
    }
}