package com.megabras.bluelogg.extended;

import android.annotation.SuppressLint;
import android.app.Dialog;
import android.content.Context;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.drawable.ColorDrawable;
import android.graphics.drawable.Drawable;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.provider.Settings;
import android.util.Log;
import android.view.Gravity;
import android.view.View;
import android.view.Window;
import android.view.WindowManager;
import android.widget.Button;
import android.widget.ImageButton;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

import com.google.android.gms.common.ConnectionResult;
import com.google.android.gms.common.GoogleApiAvailability;
import com.google.android.gms.maps.CameraUpdateFactory;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.OnMapReadyCallback;
import com.google.android.gms.maps.SupportMapFragment;
import com.google.android.gms.maps.model.BitmapDescriptor;
import com.google.android.gms.maps.model.BitmapDescriptorFactory;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.LatLngBounds;
import com.google.android.gms.maps.model.Marker;
import com.google.android.gms.maps.model.MarkerOptions;
import com.megabras.bluelogg.R;

import java.io.File;
import java.io.FileOutputStream;
import java.util.ArrayList;

import androidx.core.content.ContextCompat;
import androidx.fragment.app.FragmentActivity;

public class GoogleMaps implements OnMapReadyCallback
{
    @SuppressWarnings({"FieldCanBeLocal", "unused"})
    private final String
            TAG = "GoogleMaps";

    private FragmentActivity
            activity;

    private GoogleMap
            map;

    private ArrayList<MapData>
            mapData;

    private Dialog
            mapDialog;

    private File
            snapshotFile;

    private int
            iconMap = 0;

    private final static int
            PLAY_SERVICES_RESOLUTION_REQUEST = 9000;

    /***********************************************************************************************
     * REGION Constructor
     ***********************************************************************************************/

    public GoogleMaps( FragmentActivity activity )
    {
        this.activity = activity;
    }

    /***********************************************************************************************
     * REGION Methods
     ***********************************************************************************************/

    public void showMap( File file, MapData mapData )
    {
        this.mapData = new ArrayList<>();
        this.mapData.add( mapData );
        selectAction( file, this.mapData );
    }

    public void showMap( File file, ArrayList<MapData> mapData )
    {
        this.mapData = mapData;
        selectAction( file, mapData );
    }

    private void selectAction( File file, ArrayList<MapData> mapData )
    {
        if ( checkBeforeShowMap() )
        {
            snapshotFile = file;
            if ( snapshotFile.exists() )
                previewSnapshot();
            else
                createMapDialog( mapData );
        }
    }

    private boolean checkBeforeShowMap()
    {
        if ( !isInternetOnline() )
        {
            askForWifi();
            return false;
        }

        GoogleApiAvailability apiAvailability = GoogleApiAvailability.getInstance();

        if ( apiAvailability.isGooglePlayServicesAvailable( activity ) != ConnectionResult.SUCCESS )
        {
            //"NEED UPDATE Google Play Services"
            apiAvailability
                    .getErrorDialog(activity, 10, PLAY_SERVICES_RESOLUTION_REQUEST)
                    .show();

            return false;
        }
        return true;
    }

    @Override
    public void onMapReady( GoogleMap googleMap )
    {
        map = googleMap;
        showMarkers( map, mapData, true );
    }

    private void createMapDialog( final ArrayList<MapData> mapData )
    {
        if ( mapDialog != null )
        {
            showMarkers( map, mapData, true );
            mapDialog.show();
            return;
        }

        mapDialog = new Dialog( activity, R.style.MyThemeCompat );
        mapDialog.requestWindowFeature( Window.FEATURE_NO_TITLE );
        mapDialog.setCanceledOnTouchOutside( false );
        mapDialog.setContentView( R.layout.dialog_map );
        if ( mapDialog.getWindow() != null )
        {
            mapDialog.getWindow().getAttributes().width = WindowManager.LayoutParams.MATCH_PARENT;
            mapDialog.getWindow().getAttributes().height = WindowManager.LayoutParams.MATCH_PARENT;
            mapDialog.getWindow().clearFlags( WindowManager.LayoutParams.FLAG_DIM_BEHIND );
        }

        ((TextView) mapDialog.findViewById( R.id.dlg_title )).setText( R.string.map );
        ((ImageView) mapDialog.findViewById( R.id.dlg_icon )).setImageResource( R.drawable.ic_map );

        final Toast toast = Toast.makeText(
                activity.getApplicationContext(),
                activity.getResources().getString( R.string.wait_map ),
                Toast.LENGTH_SHORT
        );
        toast.setGravity( Gravity.BOTTOM, 0, 120 );
        toast.show();

        final Button btnClose            = mapDialog.findViewById( R.id.btn_cancel );
        final ImageButton btnMap         = mapDialog.findViewById( R.id.btn_map );
        final ImageButton btnSatellite   = mapDialog.findViewById( R.id.btn_satellite );
        final ImageButton btnShowMarkers = mapDialog.findViewById( R.id.btn_show_markers );
        final ImageButton btnSnapshot    = mapDialog.findViewById( R.id.btn_snapshot );

        SupportMapFragment mapFragment = (SupportMapFragment) activity.getSupportFragmentManager().findFragmentById( R.id.map );
        if ( mapFragment != null )
        {
            mapFragment.getMapAsync( this );
        }

        final GoogleMap.SnapshotReadyCallback callback = snap ->
        {
            try
            {
                //shutter.start();
                FileOutputStream out = new FileOutputStream( snapshotFile );
                snap.compress( Bitmap.CompressFormat.JPEG, 100, out );
                mapDialog.dismiss();
                previewSnapshot();
            }
            catch ( Exception e )
            {
                e.printStackTrace();
            }
        };

        btnSnapshot.setOnClickListener( v -> map.snapshot( callback ) );
        btnShowMarkers.setOnClickListener( v -> markersDialog( map, mapData ) );
        btnClose.setOnClickListener( v -> mapDialog.dismiss() );
        mapDialog.findViewById( R.id.btn_close_dlg ).setOnClickListener( v -> mapDialog.dismiss() );

        btnMap.setOnClickListener( v ->
        {
            toast.show();
            btnMap.setVisibility( View.GONE );
            btnSatellite.setVisibility( View.VISIBLE );
            map.setMapType( GoogleMap.MAP_TYPE_NORMAL );
        } );

        btnSatellite.setOnClickListener( v ->
        {
            toast.show();
            btnMap.setVisibility( View.VISIBLE );
            btnSatellite.setVisibility( View.GONE );
            map.setMapType( GoogleMap.MAP_TYPE_SATELLITE );
        } );

        mapDialog.show();
    }

    @SuppressWarnings("ResultOfMethodCallIgnored")
    private void previewSnapshot()
    {
        final Dialog dialog = new Dialog( activity, R.style.MyThemeCompat );
        dialog.requestWindowFeature( Window.FEATURE_NO_TITLE );
        dialog.setCanceledOnTouchOutside( false );
        dialog.setContentView( R.layout.dialog_snapshot );
        if ( dialog.getWindow() != null )
        {
            dialog.getWindow().getAttributes().width = WindowManager.LayoutParams.MATCH_PARENT;
            dialog.getWindow().getAttributes().height = WindowManager.LayoutParams.MATCH_PARENT;
        }
        ((TextView) dialog.findViewById( R.id.dlg_title )).setText( activity.getString( R.string.snapshot ) );
        ((ImageView) dialog.findViewById(R.id.dlg_icon)).setImageResource(R.drawable.ic_picture);

        Button btnNewPic = dialog.findViewById( R.id.btn_new_picture );
        btnNewPic.setText( R.string.delete_snapshot );

        btnNewPic.setOnClickListener( v ->
        {
            if ( snapshotFile.exists() )
            {
                snapshotFile.delete();
            }
            dialog.dismiss();
            final Toast toast = Toast.makeText(
                    activity.getApplicationContext(),
                    activity.getResources().getString( R.string.snapshot_deleted ),
                    Toast.LENGTH_SHORT
            );
            toast.setGravity( Gravity.CENTER, 0, 0 );
            toast.show();
        } );

        ImageView img      = dialog.findViewById( R.id.img_preview );
        Bitmap    myBitmap = BitmapFactory.decodeFile( snapshotFile.getAbsolutePath() );
        img.setImageBitmap( myBitmap );

        dialog.findViewById( R.id.btn_cancel ).setOnClickListener( v -> dialog.dismiss() );
        dialog.findViewById( R.id.btn_close_dlg ).setOnClickListener( v -> dialog.dismiss() );

        dialog.show();
    }

    private void markersDialog( final GoogleMap map, final ArrayList<MapData> mapData )
    {
        final Dialog dialog = new Dialog( activity );
        dialog.requestWindowFeature( Window.FEATURE_NO_TITLE );
        dialog.setContentView( R.layout.dialog_markers );
        if ( dialog.getWindow() != null )
        {
            dialog.getWindow().getAttributes().width = WindowManager.LayoutParams.WRAP_CONTENT;
            dialog.getWindow().getAttributes().height = WindowManager.LayoutParams.WRAP_CONTENT;
            dialog.getWindow().clearFlags( WindowManager.LayoutParams.FLAG_DIM_BEHIND );
        }

        final Button btnTowerBig    = dialog.findViewById( R.id.btn_tower_big );
        final Button btnTowerMedium = dialog.findViewById( R.id.btn_tower_medium );
        final Button btnTowerSmall  = dialog.findViewById( R.id.btn_tower_small );
        final Button btnMarker      = dialog.findViewById( R.id.btn_marker );
        final Button btnNoMarker    = dialog.findViewById( R.id.btn_no_markers );

        btnTowerBig.setOnClickListener( v ->
        {
            iconMap = R.drawable.ic_tower;
            dialog.dismiss();
        } );

        btnTowerMedium.setOnClickListener( v ->
        {
            iconMap = R.drawable.ic_tower_m;
            dialog.dismiss();
        } );

        btnTowerSmall.setOnClickListener( v ->
        {
            iconMap = R.drawable.ic_tower_s;
            dialog.dismiss();
        } );

        btnMarker.setOnClickListener( v ->
        {
            iconMap = 0;
            dialog.dismiss();
        } );

        btnNoMarker.setOnClickListener( v ->
        {
            iconMap = -1;
            dialog.dismiss();
        } );

        dialog.setOnDismissListener( arg0 -> showMarkers( map, mapData, false ) );

        dialog.show();
    }

    private BitmapDescriptor bitmapDescriptorFromVector( int vectorResId )
    {
        Drawable vectorDrawable = ContextCompat.getDrawable( activity, vectorResId );
        if ( vectorDrawable != null )
        {
            vectorDrawable.setBounds( 0, 0, vectorDrawable.getIntrinsicWidth(), vectorDrawable.getIntrinsicHeight() );
            Bitmap bitmap = Bitmap.createBitmap( vectorDrawable.getIntrinsicWidth(), vectorDrawable.getIntrinsicHeight(), Bitmap.Config.ARGB_8888 );
            Canvas canvas = new Canvas( bitmap );
            vectorDrawable.draw( canvas );
            return BitmapDescriptorFactory.fromBitmap( bitmap );
        }
        else
        {
            return null;
        }
    }

    private void showMarkers( final GoogleMap map, ArrayList<MapData> mapData, final boolean move )
    {
        map.clear();
        LatLngBounds.Builder bc = new LatLngBounds.Builder();
        int index = 0;
        for ( MapData data : mapData )
        {
            if ( data.getLongitude() >= -180 || data.getLongitude() <= 180 )
            { // Longitude range is -180º and +180º
                LatLng            coord  = new LatLng( data.getLatitude(), data.getLongitude() );
                Paint             paint  = new Paint();
                int               stroke = 5;
                Paint.FontMetrics fm     = new Paint.FontMetrics();
                paint.getFontMetrics( fm );
                paint.setTextSize( 40 );

                float         w      = paint.measureText( data.getName() );
                float         h      = paint.getTextSize();
                Bitmap.Config conf   = Bitmap.Config.ARGB_8888;
                Bitmap        bmp    = Bitmap.createBitmap( (int) w + (stroke * 2), (int) h + (stroke * 2), conf );
                Canvas        canvas = new Canvas( bmp );

                paint.setTextAlign( Paint.Align.CENTER );
                paint.setAntiAlias( true );
                paint.setFakeBoldText( true );

                paint.setStyle( Paint.Style.STROKE );
                paint.setStrokeWidth( stroke );
                paint.setColor( Color.WHITE );
                canvas.drawText( data.getName(), w / 2 + stroke, h, paint );

                paint.setStyle( Paint.Style.FILL );
                paint.setStrokeWidth( 0 );
                paint.setColor( Color.BLACK );
                canvas.drawText( data.getName(), w / 2 + stroke, h, paint );

                map.setInfoWindowAdapter( new GoogleMap.InfoWindowAdapter()
                {
                    public View getInfoWindow( Marker arg0 )
                    {
                        return null;
                    }

                    public View getInfoContents( Marker arg0 )
                    {
                        String[] msg = arg0.getSnippet().split( "\n" );

                        @SuppressLint("InflateParams")
                        View v = activity.getLayoutInflater().inflate( R.layout.infoview, null );
                        ((TextView) v.findViewById( R.id.latitude )).setText( msg[0] );
                        ((TextView) v.findViewById( R.id.longitude )).setText( msg[1] );
                        ((TextView) v.findViewById( R.id.altitude )).setText( msg[2] );
                        ((TextView) v.findViewById( R.id.accuracy )).setText( msg[3] );
                        return v;
                    }
                } );

                String snippet =
                        data.getLatitude() + "\n" +
                        data.getLongitude() + "\n" +
                        data.getAltitude() + "\n" +
                        data.getAccuracy();

                MarkerOptions marker = new MarkerOptions();
                marker.draggable( true );
                marker.title( data.getName() );
                marker.position( coord );
                marker.snippet( snippet );

                if ( iconMap > 0 )
                {
                    marker.icon( bitmapDescriptorFromVector( iconMap ) );
                }

                if ( iconMap > -1 )
                {
                    map.addMarker( marker );
                    map.addMarker( new MarkerOptions()
                            .position( coord )
                            .anchor( 0.5f, 0.0f )
                            .snippet( snippet )
                            .icon( BitmapDescriptorFactory.fromBitmap( bmp ) ) );
                }

                bc.include( coord );
                if ( move )
                {
                    map.moveCamera( CameraUpdateFactory.newLatLngZoom( coord, 20 ) );
                }
                index++;
            } // if ( data.getLongitude() >= -180 || data.getLongitude() <= 180 )
            if ( index > 1 && move )
            {
                try
                {
                    map.moveCamera( CameraUpdateFactory.newLatLngBounds( bc.build(), 150 ) );
                }
                catch ( Exception e )
                {
                    for ( StackTraceElement ste : e.getStackTrace() )
                        Log.e( TAG, ste.toString() );
                }
            }
        }
    }

    /***********************************************************************************************
     * REGION Utilities
     ***********************************************************************************************/

    private boolean isInternetOnline()
    {
        ConnectivityManager connMgr     = (ConnectivityManager) activity.getSystemService( Context.CONNECTIVITY_SERVICE );
        NetworkInfo         networkInfo = null;
        if ( connMgr != null )
        {
            networkInfo = connMgr.getActiveNetworkInfo();
        }
        return ( networkInfo != null && networkInfo.isConnected() );
    }

    private void askForWifi()
    {
        final Dialog dialog = new Dialog( activity );
        dialog.requestWindowFeature( Window.FEATURE_NO_TITLE );
        dialog.setCanceledOnTouchOutside( false );
        dialog.setContentView( R.layout.dialog_yes_no );
        if ( dialog.getWindow() != null )
        {
            dialog.getWindow().getAttributes().width = WindowManager.LayoutParams.MATCH_PARENT;
            dialog.getWindow().clearFlags( WindowManager.LayoutParams.FLAG_DIM_BEHIND );
        }
        ((TextView) dialog.findViewById( R.id.dlg_title )).setText( R.string.no_internet );
        ((ImageView) dialog.findViewById( R.id.dlg_icon )).setImageResource( R.drawable.ic_wifi );

        final TextView message = dialog.findViewById( R.id.message );
        message.setText( R.string.no_internet_message );

        final Button btnYes = dialog.findViewById( R.id.btn_yes );
        btnYes.setText( R.string.open_settings );
        btnYes.setOnClickListener( v ->
        {
            Intent intent = new Intent( Settings.ACTION_SETTINGS );
            activity.startActivity( intent );
            dialog.dismiss();
        } );

        final Button btnNo = dialog.findViewById( R.id.btn_no );
        btnNo.setText( R.string.close );
        btnNo.setOnClickListener( v -> dialog.dismiss() );

        dialog.show();
    }
}
