package com.megabras.bluelogg.extended;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.List;

import android.annotation.SuppressLint;
import android.app.Activity;
import android.app.Dialog;
import android.content.DialogInterface;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Color;
import android.graphics.Matrix;
import android.graphics.Point;
import android.graphics.drawable.ColorDrawable;
import android.hardware.Camera;
import android.hardware.Camera.Parameters;
import android.hardware.Camera.PictureCallback;
import android.hardware.Camera.Size;
import android.os.Build;
import android.os.Looper;
import android.util.Log;
import android.view.Display;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;
import android.view.WindowManager;
import android.view.WindowManager.LayoutParams;
import android.widget.Button;
import android.widget.FrameLayout;
import android.widget.ImageButton;
import android.widget.ImageView;
import android.widget.TextView;

import com.megabras.bluelogg.R;

public class Picture
{
	@SuppressWarnings("FieldCanBeLocal")
	private final String
			TAG = "Picture";

	private File
			file;

	private android.media.MediaPlayer
			shutter;

	private Activity
			activity;

	private double
			scale;

	private PictureInterface
			parent;

	public static final int
			WAIT 		= 0,
			FINISHED    = 1;

	public static Picture getInstance()
	{
		if ( Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP )
		{
			return new PictureNewApi();
		}
		else
		{
			return new Picture();
		}
	}

	public void initialize( Activity activity, PictureInterface parent )
	{
		this.activity = activity;
		this.parent = parent;
		try
		{
			android.content.res.AssetFileDescriptor afd = activity.getAssets().openFd( "shutter.mp3" );
			shutter = new android.media.MediaPlayer();
			shutter.setDataSource( afd.getFileDescriptor(), afd.getStartOffset(), afd.getLength() );
			shutter.prepare();
		}
		catch ( IOException e )
		{
			e.printStackTrace();
		}
	}

	public void showDialog( final File picFile )
	{
		new Thread( () ->
		{
			Looper.prepare();
			parent.pictureStatus( Picture.WAIT );
			if ( picFile.exists() )
				previewPicture( "TEST", picFile, null );
			else
				takePicture( picFile, null );

			Looper.loop();
		} ).start();
	}

	public void showDialog( final File picFile, final DialogInterface.OnDismissListener onDismissListener )
	{
		new Thread( () ->
		{
			Looper.prepare();

			if ( picFile.exists() )
				previewPicture( "TEST", picFile, onDismissListener );
			else
				takePicture( picFile, onDismissListener );

			Looper.loop();
		} ).start();
	}

	private PictureCallback mPicture = new PictureCallback()
	{
		public void onPictureTaken( byte[] data, Camera camera )
		{
			if ( file == null )
			{
				return;
			}

			try
			{
				FileOutputStream fos = new FileOutputStream( file );
				Bitmap realImage = BitmapFactory.decodeByteArray( data, 0, data.length );

				if ( realImage.getWidth() > realImage.getHeight() )
				{
					realImage = rotate( realImage, 90 );
				}

				realImage.compress( Bitmap.CompressFormat.JPEG, 100, fos );
				fos.close();
			}
			catch ( FileNotFoundException e )
			{
				Log.e( "PictureCallback", "File not found: " + e.getMessage() );
			}
			catch ( IOException e )
			{
				Log.e( "PictureCallback", "Error accessing file: " + e.getMessage() );
			}
			catch ( Exception e )
			{
				Log.e( "PictureCallback", "Error accessing file: " + e.getMessage() );
			}
		}
	};

	private Camera getCameraInstance()
	{
		int numOfCameras = Camera.getNumberOfCameras();
		Camera c = null;
		try
		{
			if ( numOfCameras > 1 )
			{
				c = Camera.open(); // attempt to get a Camera instance
			}
			else
			{
				c = Camera.open( 0 );
			}
		}
		catch ( Exception e )
		{
			// Camera is not available (in use or does not exist)
			Log.e( "PICTURE", "getCameraInstance " + e.getMessage() );
		}
		return c; // returns null if camera is unavailable
	}
	
	@SuppressLint("SetTextI18n")
	public void takePicture( File picFile, final DialogInterface.OnDismissListener onDismissListener )
	{
		parent.pictureStatus( Picture.WAIT );
		this.file = picFile;
		final Dialog dialog = new Dialog( activity, R.style.MyThemeCompat );
		dialog.requestWindowFeature( Window.FEATURE_NO_TITLE );
		dialog.setCanceledOnTouchOutside( false );
		dialog.setContentView( R.layout.dialog_picture );
		if ( dialog.getWindow() != null )
		{
			dialog.getWindow().getAttributes().width = LayoutParams.MATCH_PARENT;
			dialog.getWindow().getAttributes().height = LayoutParams.MATCH_PARENT;
		}

		final Camera mCamera;
		CameraPreview mPreview;
		mCamera = getCameraInstance(); // 1 for frontal camera

		final Parameters params = mCamera.getParameters();

		if ( Camera.getNumberOfCameras() > 1 )
		{
			params.setRotation( 90 );
		}
		else
		{
			params.setRotation( 270 );
		}

		Display display = activity.getWindowManager().getDefaultDisplay();
		Point p = new Point();
		display.getSize( p );

		List<Size> pl = params.getSupportedPictureSizes();
		Size picSize;
		Size standard = pl.get( 0 );

		standard.width = 640;
		standard.height = 480;

		if ( pl.contains( standard ) )
		{
			picSize = standard;
		}
		else
		{
			if ( pl.get( 0 ).width < pl.get( pl.size() - 1 ).width )
			{
				picSize = pl.get( 0 );
			}
			else
			{
				picSize = pl.get( pl.size() - 1 );
			}
		}

		params.setPictureSize( picSize.width, picSize.height );

		for ( Size size : params.getSupportedPreviewSizes() )
		{
			if ( size.width > p.x )
			{
				params.setPreviewSize( size.width, size.height );
				scale = ((double) size.height / (double) size.width);
				if ( scale < 1 )
				{
					scale = 1 / scale;
				}
			}
		}

		params.setFocusMode( Parameters.FOCUS_MODE_AUTO );

		try
		{
			mCamera.setParameters( params );
		}
		catch ( Exception ce )
		{
			Log.e( TAG, "Set params error" );
		}

		mCamera.setDisplayOrientation( 90 );
		mPreview = new CameraPreview( activity, mCamera );

		final FrameLayout preview = dialog.findViewById( R.id.camera_preview );
		preview.addView( mPreview );

		dialog.setOnShowListener( dialog1 ->
		{
			ViewGroup.LayoutParams param = preview.getLayoutParams();
			param.height = (int) (preview.getWidth() * scale);
			preview.setLayoutParams( param );
			parent.pictureStatus( Picture.FINISHED );
		} );

		final Button captureButton = dialog.findViewById( R.id.button_capture );
		final Button btnPreview = dialog.findViewById( R.id.btn_preview );
		final Button btnClose = dialog.findViewById( R.id.btn_close );

		captureButton.setOnClickListener( v ->
				{
					mCamera.takePicture( null, null, mPicture );
					shutter.start();
					captureButton.setVisibility( View.GONE );
					btnPreview.setVisibility( View.VISIBLE );
				}
		);

		btnPreview.setOnClickListener( v ->
				{
					mCamera.startPreview();
					captureButton.setVisibility( View.VISIBLE );
					btnPreview.setVisibility( View.GONE );
				}
		);

		btnClose.setOnClickListener( v ->
		{
			if ( onDismissListener != null )
			{
				dialog.setOnDismissListener( onDismissListener );
			}
			dialog.dismiss();
		} );

		btnPreview.setVisibility( View.GONE );

		dialog.show();
	}
	
	@SuppressWarnings("ResultOfMethodCallIgnored")
	@SuppressLint("SetTextI18n")
	public void previewPicture( String obj, File files, final DialogInterface.OnDismissListener onDismissListener){
		parent.pictureStatus(Picture.WAIT);
		this.file = files;
		final Dialog dialog = new Dialog(activity, R.style.MyThemeCompat);
		dialog.requestWindowFeature(Window.FEATURE_NO_TITLE);
		dialog.setCanceledOnTouchOutside(false);
		dialog.setContentView(R.layout.dialog_picture_preview);
		if ( dialog.getWindow() != null )
		{
			dialog.getWindow().getAttributes().width = LayoutParams.MATCH_PARENT;
			dialog.getWindow().getAttributes().height = LayoutParams.MATCH_PARENT;
		}

		dialog.setOnShowListener( dialog1 -> parent.pictureStatus(Picture.FINISHED) );

		File imgFile;
		Button btnNewPic = dialog.findViewById(R.id.btn_new_picture);

		imgFile = file;
		btnNewPic.setOnClickListener( v ->
		{
			if ( file.exists() )
			{
				file.delete();
			}
			takePicture(file, onDismissListener);
			dialog.dismiss();
		} );

		ImageView img = dialog.findViewById(R.id.img_preview);
		Bitmap myBitmap = BitmapFactory.decodeFile(imgFile.getAbsolutePath());
		img.setImageBitmap(myBitmap);

		dialog.findViewById( R.id.btn_close ).setOnClickListener( v ->
		{
			if ( onDismissListener != null )
			{
				dialog.setOnDismissListener( onDismissListener );
			}
			dialog.dismiss();
		} );

		dialog.show();
	}

	@SuppressWarnings("SameParameterValue")
	Bitmap rotate( Bitmap bitmap, int degree )
	{
		int w = bitmap.getWidth();
		int h = bitmap.getHeight();

		Matrix mtx = new Matrix();
		mtx.postRotate( degree );

		return Bitmap.createBitmap( bitmap, 0, 0, w, h, mtx, true );
	}

	public void close()
	{
		try
		{
			if ( shutter != null )
			{
				shutter.release();
			}
		}
		catch ( Exception e )
		{
			Log.e( "Picture Close", e.toString() );
		}
	}

	public interface PictureInterface{
		void pictureStatus(final int status);
	}
	
}
