package com.megabras.bluelogg.extended;

import android.app.Activity;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.content.pm.ResolveInfo;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Rect;
import android.graphics.Typeface;
import android.net.Uri;
import android.os.Handler;
import android.text.Layout;
import android.text.StaticLayout;
import android.text.TextPaint;
import android.util.Log;

import com.megabras.bluelogg.R;
import com.megabras.modbus.MD15KVR.Measurement;
import com.megabras.modbus.MD15KVR.ResultDAI;
import com.megabras.modbus.MD15KVR.ResultPI;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import androidx.core.content.FileProvider;

@SuppressWarnings({"unused", "WeakerAccess"})
public class ReportProvider
{
    protected Activity
            activity;

    protected Canvas
            canvas;

    private final double
            TEXT_Y_ALIGN_FACTOR = 0.015; // This factor is not required, but I use him to REALLY vertical align strings

    protected final int
            MARGIN       = (int)(5 * 2.83465),
            BRAND_HEIGHT = 25;

    protected int
            CURSOR_X = 0,
            CURSOR_Y = 0,
            CURSOR_Y_BK,
            CURSOR_X_BK,
            PAPER_WIDTH,
            PAPER_HEIGHT;

    public static final int
            ALIGN_LEFT   = 0,
            ALIGN_CENTER = 1,
            ALIGN_RIGHT  = 2;

    protected File
            dir,
            tempDir;

    protected Handler
            handler;

    private Rect
            bounds,
            boundsBackup;

    private TextPaint
            paint;

    public ReportProvider(Activity activity)
    {
        this.activity = activity;
        bounds        = new Rect();
        boundsBackup  = new Rect();
        paint         = new TextPaint(TextPaint.ANTI_ALIAS_FLAG);
        paint.setTextSize(8);
        paint.setTextAlign(Paint.Align.LEFT);
        paint.setStrokeWidth(0.3f);
        paint.setColor(Color.BLACK);
        paint.setStrokeCap(Paint.Cap.ROUND);
    }

    public ReportProvider(Activity activity, File dir, File tempDir, Handler handler)
    {
        this(activity);
        this.handler  = handler;
        this.dir      = dir;
        this.tempDir  = tempDir;
    }

    public ReportProvider left(int value)
    {
        bounds.left = value;
        return this;
    }

    public ReportProvider right(int value)
    {
        bounds.right = value;
        return this;
    }

    public ReportProvider top(int value)
    {
        bounds.top = value;
        return this;
    }

    public ReportProvider bottom(int value)
    {
        bounds.bottom = value;
        return this;
    }

    protected Rect getBounds()
    {
        return bounds;
    }

    protected void drawSquare()
    {
        if (canvas==null || bounds==null || paint==null)
            return;

        canvas.drawLines(new float[]{bounds.left , bounds.top   , bounds.right, bounds.top   }, paint); // TOP
        canvas.drawLines(new float[]{bounds.left , bounds.top   , bounds.left , bounds.bottom}, paint); // LEFT
        canvas.drawLines(new float[]{bounds.right, bounds.top   , bounds.right, bounds.bottom}, paint); // RIGHT
        canvas.drawLines(new float[]{bounds.left , bounds.bottom, bounds.right, bounds.bottom}, paint); // BOTTOM
    }

    protected void drawMultilineText(String title, String text, boolean square)
    {
        if (canvas==null || bounds==null || paint==null)
            return;

        title = title.replaceAll(" ", "  ").toUpperCase();
        paint.setTypeface(Typeface.DEFAULT);
        paint.setTextSize(5f);
        float titleHeigh = -1 * paint.getFontMetrics().ascent / 2;
        canvas.drawText(title, bounds.left+3, 5, paint);

        paint.setTextSize(8f);

        text = text.replaceAll(" ", "  ");

        TextPaint mTextPaint = new TextPaint();
        mTextPaint.setTypeface(Typeface.DEFAULT);
        mTextPaint.setTextSize(8f);

        StaticLayout mTextLayout = new StaticLayout( text, mTextPaint, PAPER_WIDTH-8, Layout.Alignment.ALIGN_NORMAL, 1.2f, 0.0f, false);

        bounds.bottom = mTextLayout.getHeight() + 20;

        if ( square )
            this.drawSquare();

        canvas.translate(8, 10);
        mTextLayout.draw( canvas );
        canvas.translate(-8, -10);

        CURSOR_Y += bounds.height();
        canvas.translate(0, bounds.height());
    }

    protected void drawTitle(String title)
    {
        if (canvas==null || bounds==null || paint==null)
            return;

        this.drawSquare();

        title = title.replaceAll(" ", "  ");

        paint.setTypeface(Typeface.DEFAULT_BOLD);
        paint.setTextSize(14f);

        Rect boundsTitle = new Rect();
        paint.getTextBounds(title, 0, title.length(), boundsTitle);

        float centerX = (bounds.width() / 2f) - (boundsTitle.width() / 2f) + boundsTitle.left;
        float centerY = (bounds.height() / 2f) - ((paint.getFontMetrics().ascent + paint.getFontMetrics().descent) / 2f); // Ascent is negative, so this is (height/2) '+' ((ascent '-' descent)/2)
        centerY += centerY * TEXT_Y_ALIGN_FACTOR;

        canvas.drawText(title, centerX, centerY, paint);

        paint.setTypeface(Typeface.DEFAULT);
        paint.setTextSize(8f);

        CURSOR_Y += bounds.height();
        canvas.translate(0, bounds.height());
    }

    protected void drawItem(String title, String value)
    {
        if (canvas==null || bounds==null || paint==null)
            return;

        this.drawSquare();

        title = title.replaceAll(" ", "  ").toUpperCase();
        value = value.replaceAll(" ", "  ");

        paint.setTypeface(Typeface.DEFAULT);
        paint.setTextSize(5f);

        float titleHeigh = -1 * paint.getFontMetrics().ascent / 2;

        canvas.drawText(title, bounds.left+3, 5, paint);

        paint.setTextSize(8f);

        Rect boundsValue = new Rect();
        paint.getTextBounds(value, 0, value.length(), boundsValue);

        float centerX     = (bounds.width() / 2f) - (boundsValue.width() / 2f) + boundsValue.left;
//        float centerY     = bounds.height() / 2f;
//        float fontCenterY = -1 * ((paint.getFontMetrics().ascent + (paint.getFontMetrics().bottom)) / 2f);

        float centerY = (bounds.height() / 2f) - ((paint.getFontMetrics().ascent + paint.getFontMetrics().descent) / 2f); // Ascent is negative, so this is (height/2) '+' ((ascent '-' descent)/2)
        centerY += centerY * TEXT_Y_ALIGN_FACTOR;
        canvas.drawText(value, bounds.left+centerX, centerY + titleHeigh, paint);

        //canvas.drawText(value, bounds.left+centerX, centerY + fontCenterY + titleHeigh, paint);

        paint.setTypeface(Typeface.DEFAULT);
        paint.setTextSize(8f);
    }

    protected void drawItem(String title, String value, int align)
    {
        if (canvas==null || bounds==null || paint==null)
            return;

        this.drawSquare();

        title = title.replaceAll(" ", "  ").toUpperCase();
        value = value.replaceAll(" ", "  ");

        paint.setTypeface(Typeface.DEFAULT);
        paint.setTextSize(5f);

        float titleHeigh = -1 * paint.getFontMetrics().ascent / 2;

        canvas.drawText(title, bounds.left+3, 5, paint);

        paint.setTextSize(8f);

        Rect boundsValue = new Rect();
        paint.getTextBounds(value, 0, value.length(), boundsValue);

        float centerX;

        switch ( align )
        {
            case ALIGN_CENTER:
                centerX = (bounds.width() / 2f) - (boundsValue.width() / 2f) + boundsValue.left;
                break;
            default:
                centerX = 8;
                break;
        }
//        float centerY     = bounds.height() / 2f;
//        float fontCenterY = -1 * ((paint.getFontMetrics().ascent + (paint.getFontMetrics().bottom)) / 2f);

        float centerY = (bounds.height() / 2f) - ((paint.getFontMetrics().ascent + paint.getFontMetrics().descent) / 2f); // Ascent is negative, so this is (height/2) '+' ((ascent '-' descent)/2)
        centerY += centerY * TEXT_Y_ALIGN_FACTOR;
        canvas.drawText(value, bounds.left+centerX, centerY + titleHeigh, paint);

        //canvas.drawText(value, bounds.left+centerX, centerY + fontCenterY + titleHeigh, paint);

        paint.setTypeface(Typeface.DEFAULT);
        paint.setTextSize(8f);
    }

    protected void drawItem( String title, String value, int align, Typeface typeface, float fontSize )
    {
        if (canvas==null || bounds==null || paint==null)
            return;

        this.drawSquare();

        title = title.replaceAll(" ", "  ").toUpperCase();
        value = value.replaceAll(" ", "  ");

        paint.setTypeface( Typeface.DEFAULT );
        paint.setTextSize( 5f );

        float titleHeigh = -1 * paint.getFontMetrics().ascent / 2;

        canvas.drawText(title, bounds.left+3, 5, paint);

        paint.setTypeface( typeface );
        paint.setTextSize( fontSize );

        Rect boundsValue = new Rect();
        paint.getTextBounds(value, 0, value.length(), boundsValue);

        float centerX;

        switch ( align )
        {
            case ALIGN_CENTER:
                centerX = (bounds.width() / 2f) - (boundsValue.width() / 2f) + boundsValue.left;
                break;
            default:
                centerX = 8;
                break;
        }

        float centerY = (bounds.height() / 2f) - ((paint.getFontMetrics().ascent + paint.getFontMetrics().descent) / 2f); // Ascent is negative, so this is (height/2) '+' ((ascent '-' descent)/2)
        centerY += centerY * TEXT_Y_ALIGN_FACTOR;
        canvas.drawText(value, bounds.left+centerX, centerY + titleHeigh, paint);

        paint.setTypeface(Typeface.DEFAULT);
        paint.setTextSize(8f);
    }

    protected void drawLineItem( ArrayList<String> title, ArrayList<String> value, int align, Typeface typeface, float fontSize )
    {
        if (canvas==null || bounds==null || paint==null)
            return;

        saveBounds();

        if (title.size() != value.size())
            return;

        int width  = bounds.width() / title.size();
        int fill   = bounds.width() - (width*title.size());

        bounds.right = width;

        for (int i=0; i<title.size(); i++)
        {
            if (i==title.size()-1)
                bounds.right += fill;
            drawItem(title.get(i), value.get(i), align, typeface, fontSize);
            bounds.left  += width;
            bounds.right += width;
        }

        restoreBounds();

        CURSOR_Y += bounds.height();
        canvas.translate(0, bounds.height());
    }

    protected void drawLineItem(String title, String value, int align)
    {
        if (canvas==null || bounds==null || paint==null)
            return;

        drawItem(title, value, align);

        CURSOR_Y += bounds.height();
        canvas.translate(0, bounds.height());
    }

    protected void drawLineItem(ArrayList<String> title, ArrayList<String> value, int align)
    {
        if (canvas==null || bounds==null || paint==null)
            return;

        saveBounds();

        if (title.size() != value.size())
            return;

        int width  = bounds.width() / title.size();
        int fill   = bounds.width() - (width*title.size());

        bounds.right = width;

        for (int i=0; i<title.size(); i++)
        {
            if (i==title.size()-1)
                bounds.right += fill;
            drawItem(title.get(i), value.get(i), align);
            bounds.left  += width;
            bounds.right += width;
        }

        restoreBounds();

        CURSOR_Y += bounds.height();
        canvas.translate(0, bounds.height());
    }

    protected void drawLineItem(ArrayList<String> title, ArrayList<String> value)
    {
        if ( canvas == null || bounds == null || paint == null )
            return;

        saveBounds();

        if (title.size() != value.size())
            return;

        int width  = bounds.width() / title.size();
        int fill   = bounds.width() - (width*title.size());

        bounds.right = width;

        for (int i=0; i<title.size(); i++)
        {
            if (i==title.size()-1)
                bounds.right += fill;
            drawItem(title.get(i), value.get(i));
            bounds.left  += width;
            bounds.right += width;
        }

        restoreBounds();

        CURSOR_Y += bounds.height();
        canvas.translate(0, bounds.height());
    }

    protected void drawItem( String title, File file )
    {
        if ( canvas == null || bounds == null || paint == null )
        {
            return;
        }

        this.drawSquare();

        title = title.replaceAll(" ", "  ").toUpperCase();

        paint.setTypeface( Typeface.DEFAULT );
        paint.setTextSize( 5f );

        canvas.drawText( title, bounds.left + 3, 5, paint );

        paint.setTextSize( 8f );

        if ( file == null || !file.exists() )
        {
            return;
        }

        Bitmap bitmap = BitmapFactory.decodeFile( file.getAbsolutePath() );
        Rect src = new Rect( 0, 0, bitmap.getWidth(), bitmap.getHeight() );
        float factor = (float) bitmap.getWidth() / bitmap.getHeight();
        Rect dst = new Rect();
        dst.top    = bounds.top      + MARGIN;
        dst.bottom = bounds.height() - MARGIN;

        int w = (int)(factor * dst.height());

        int leftMargin = (bounds.width() - w) / 2;

        dst.left   = bounds.left + leftMargin;
        dst.right  = bounds.left + leftMargin + w;

        canvas.drawBitmap(bitmap, src, dst, paint);
    }

    protected void drawLineItem(String title, File file)
    {
        if (canvas==null || bounds==null || paint==null)
            return;

        drawItem(title, file);

        CURSOR_Y += bounds.height();
        canvas.translate(0, bounds.height());
    }

    protected void drawLineCompanyTestSite(com.megabras.bluelogg.em4058.EM4058test test, String selsite)
    {
        if ( canvas == null || bounds == null || paint == null )
            return;

        saveBounds();

        paint.setTypeface(Typeface.DEFAULT);

        int width;
        int fill;

        width  = bounds.width() / 2;
        fill   = bounds.width() - (width * 2);
        //width += width;

        bounds.right  = width;
        bounds.bottom = 70;

        this.drawSquare();

        paint.setTextSize(5f);
        canvas.drawText(activity.getString(R.string.firm).toUpperCase(), bounds.left+3, 5, paint);
        String[] company = test.getCompany();
        if (company!=null)
        {
            paint.setTextSize(8f);
            canvas.drawText(company[0]                                               , bounds.left+8, 20, paint);
            canvas.drawText(company[1]                                               , bounds.left+8, 30, paint);
            canvas.drawText(company[2] + " - " + company[3] + " - " + company[4], bounds.left+8, 40, paint);
            canvas.drawText(activity.getString(R.string.phone) + " " + company[5] + " / " + activity.getString(R.string.fax) + " " + company[6], bounds.left+8, 50, paint);
        }

        bounds.left  += width;
        bounds.right += width + fill;

        this.drawSquare();

        paint.setTextSize(5f);
        canvas.drawText(activity.getString(R.string.test_site).toUpperCase(), bounds.left+3, 5, paint);

        String[] testsite = test.getTestsiteFromFile(selsite);
        paint.setTextSize(8f);
        canvas.drawText( testsite[0]                                                , bounds.left+8, 20, paint);
        canvas.drawText( testsite[1]                                                , bounds.left+8, 30, paint);
        canvas.drawText( testsite[2]                                                , bounds.left+8, 40, paint);
        canvas.drawText(testsite[3] + " - " + testsite[4] + " - " + testsite[5], bounds.left+8, 50, paint);
        canvas.drawText(activity.getString(R.string.phone) + " " + testsite[6] + " / " + activity.getString(R.string.fax) + " " + testsite[7], bounds.left+8, 60, paint);

        CURSOR_Y += bounds.height();
        canvas.translate(0, bounds.height());

        restoreBounds();
    }

    protected void drawLineCompanyTestSite(com.megabras.bluelogg.tm25r.Database database, com.megabras.modbus.TM25R.TestData testData)
    {
        if ( canvas == null || bounds == null || paint == null )
            return;

        saveBounds();

        paint.setTypeface(Typeface.DEFAULT);

        int width;
        int fill;

        width  = bounds.width() / 2;
        fill   = bounds.width() - (width * 2);
        //width += width;

        bounds.right  = width;
        bounds.bottom = 70;

        this.drawSquare();

        paint.setTextSize(5f);
        canvas.drawText(activity.getString(R.string.firm).toUpperCase(), bounds.left+3, 5, paint);
        String[] company = database.getCompany();
        if (company!=null)
        {
            paint.setTextSize(8f);
            canvas.drawText(company[0]                                               , bounds.left+8, 20, paint);
            canvas.drawText(company[1]                                               , bounds.left+8, 30, paint);
            canvas.drawText(company[2] + " - " + company[3] + " - " + company[4], bounds.left+8, 40, paint);
            canvas.drawText(activity.getString(R.string.phone) + " " + company[5] + " / " + activity.getString(R.string.fax) + " " + company[6], bounds.left+8, 50, paint);
        }

        bounds.left  += width;
        bounds.right += width + fill;

        this.drawSquare();

        paint.setTextSize(5f);
        canvas.drawText(activity.getString(R.string.test_site).toUpperCase(), bounds.left+3, 5, paint);

        paint.setTextSize(8f);
        canvas.drawText(testData.getSiteName()                                              , bounds.left+8, 20, paint);
        canvas.drawText(testData.getSiteContact()                                               , bounds.left+8, 30, paint);
        canvas.drawText(testData.getSiteAddress()                                              , bounds.left+8, 40, paint);
        canvas.drawText(testData.getSiteState() + " - " + testData.getSiteCity() + " - " + testData.getSiteZipCode(), bounds.left+8, 50, paint);
        canvas.drawText(activity.getString(R.string.phone) + " " + testData.getSitePhone() + " / " + activity.getString(R.string.fax) + " " + testData.getSiteFax(), bounds.left+8, 60, paint);

        CURSOR_Y += bounds.height();
        canvas.translate(0, bounds.height());

        restoreBounds();
    }

    protected void drawLineImage( ArrayList<String> title, ArrayList<File> file )
    {
        if ( canvas == null || bounds == null || paint == null || title.size() != file.size() )
        {
            return;
        }

        saveBounds();

        int width;
        int fill;

        if ( title.size() == 2 )
        {
            width  = bounds.width() / 4;
            fill   = bounds.width() - (width * 4);
            width += width;
        }
        else
        {
            width = bounds.width() / title.size();
            fill  = bounds.width() - (width * title.size());
        }

        bounds.right = width;

        for (int i=0; i<title.size(); i++)
        {
            if (i==title.size()-1)
                bounds.right += fill;
            drawItem(title.get(i), file.get(i));
            bounds.left  += width;
            bounds.right += width;
        }

        restoreBounds();

        CURSOR_Y += bounds.height();
        canvas.translate(0, bounds.height());
    }

    protected void drawImage( File file )
    {
        if ( canvas == null || bounds == null || paint == null )
            return;

        Bitmap bitmap = BitmapFactory.decodeFile( file.getAbsolutePath() );

        Rect src   = new Rect( 0, 0, bitmap.getWidth(), bitmap.getHeight() );
        Rect dst   = new Rect();
        int margin = 5;
        float factor;

        float boundsFactor = (float) bounds.width() / bounds.height();
        float imageFactor  = (float) bitmap.getWidth() / bitmap.getHeight();

        if ( boundsFactor > 1 ) // Width is bigger than height
        {
            if ( boundsFactor > imageFactor )
            {
                dst.top    = bounds.top + margin;
                dst.bottom = bounds.height() - margin;

                int w = (int) (imageFactor * dst.height());
                int leftMargin = (bounds.width() - w) / 2;

                dst.left   = bounds.left + leftMargin;
                dst.right  = bounds.left + leftMargin + w;
            }
            else
            {
                dst.left   = bounds.left + margin;
                dst.right  = bounds.right - margin;

                int h = (int) (dst.width() / imageFactor);
                int topMargin = (bounds.height() - h) / 2;

                dst.top    = bounds.top + topMargin;
                dst.bottom = bounds.top + topMargin + h;
            }
        }
        else // Width is smaller than height
        {
            if ( boundsFactor > imageFactor )
            {
                dst.top    = bounds.top + margin;
                dst.bottom = bounds.height() - margin;

                int w = (int) (imageFactor * dst.height());
                int leftMargin = (bounds.width() - w) / 2;

                dst.left   = bounds.left + leftMargin;
                dst.right  = bounds.left + leftMargin + w;
            }
            else
            {
                dst.left   = bounds.left + margin;
                dst.right  = bounds.right - margin;

                int h = (int) (dst.width() / imageFactor);
                int topMargin = (bounds.height() - h) / 2;

                dst.top    = bounds.top + topMargin;
                dst.bottom = bounds.top + topMargin + h;
            }

            canvas.drawBitmap( bitmap, src, dst, paint );
        }
    }

//    protected void drawImage( File file )
//    {
//        if ( canvas == null || bounds == null || paint == null )
//            return;
//
//        Bitmap bitmap = BitmapFactory.decodeFile( file.getAbsolutePath() );
//
//        Rect src   = new Rect( 0, 0, bitmap.getWidth(), bitmap.getHeight() );
//        Rect dst   = new Rect();
//        int margin = 5;
//        float factor;
//
//        Log.e( this.getClass().getName(), "Bounds width : " + bounds.width() );
//        Log.e( this.getClass().getName(), "Bounds height: " + bounds.height() );
//        Log.e( this.getClass().getName(), "Image width  : " + bitmap.getWidth() );
//        Log.e( this.getClass().getName(), "Image height : " + bitmap.getHeight() );
//
//        if ( bounds.width() > bounds.height() )
//        {
//            factor     = (float) bitmap.getWidth() / bitmap.getHeight();
//            dst.top    = bounds.top + margin;
//            dst.bottom = bounds.height() - margin;
//            int w = (int)(factor * dst.height());
//            int leftMargin = (bounds.width() - w) / 2;
//            dst.left   = bounds.left + leftMargin;
//            dst.right  = bounds.left + leftMargin + w;
//            canvas.drawBitmap(bitmap, src, dst, paint);
//        }
//        else
//        {
//            factor     = (float) bitmap.getHeight() / bitmap.getWidth();
//            dst.left   = bounds.left + margin;
//            dst.right  = bounds.right - margin;
//
//            int h = (int) (factor * dst.width());
//            int topMargin = (bounds.height() - h) / 2;
//
//            dst.top    = bounds.top + topMargin;
//            dst.bottom = bounds.top + topMargin + h;
//
//            canvas.drawBitmap( bitmap, src, dst, paint );
//        }
//    }

    protected void drawLineImage( ArrayList<File> file )
    {
        if ( canvas == null || bounds == null || paint == null )
            return;

        saveBounds();

        int width;
        int fill;

        if ( file.size() == 2 )
        {
            width  = bounds.width() / 4;
            fill   = bounds.width() - (width*4);
            width += width;
        }
        else
        {
            width = bounds.width() / file.size();
            fill  = bounds.width() - (width*file.size());
        }

        bounds.right = width;

        for (int i=0; i<file.size(); i++)
        {
            if (i==file.size()-1)
                bounds.right += fill;
            if (file.get(i)!=null)
                drawImage( file.get( i ) );
            bounds.left  += width;
            bounds.right += width;
        }

        restoreBounds();

        CURSOR_Y += bounds.height();
        canvas.translate(0, bounds.height());
    }

    protected void drawRow(Typeface typeface, int[] widths, ArrayList<String> value)
    {
        if (canvas==null || bounds==null || paint==null)
            return;

        saveBounds();

        int widthTotal = 0;

        for ( int w : widths )
            widthTotal += w;

        int left = (PAPER_WIDTH - widthTotal) / 2;

        bounds.left = left;

        for ( int i = 0; i < value.size(); i++ )
        {
            bounds.right = bounds.left + widths[i];
            drawCell(typeface, value.get(i));
            bounds.left  += widths[i];
        }

        restoreBounds();

        CURSOR_Y += bounds.height();
        canvas.translate(0, bounds.height());
    }

    protected void drawBlankLine( int height )
    {
        if ( canvas == null )
            return;

        CURSOR_Y += height;
        canvas.translate( 0, height );
    }

    protected void drawRow(Typeface typeface, ArrayList<String> value)
    {
        if (canvas==null || bounds==null || paint==null)
            return;

        saveBounds();

        int width = bounds.width() / value.size();
        int fill  = bounds.width() - (width*value.size());

        bounds.right = width;

        for (int i=0; i<value.size(); i++)
        {
            if (i==value.size()-1)
                bounds.right += fill;
            drawCell(typeface, value.get(i));
            bounds.left  += width;
            bounds.right += width;
        }

        restoreBounds();

        CURSOR_Y += bounds.height();
        canvas.translate(0, bounds.height());
    }

    protected void drawHighlightedRow(Typeface typeface, ArrayList<String> value)
    {
        if (canvas==null || bounds==null || paint==null)
            return;

        float strokeBackup = paint.getStrokeWidth();
        paint.setStrokeWidth(strokeBackup*3);
        this.drawSquare();
        paint.setStrokeWidth(strokeBackup);


        this.drawRow(typeface, value);
    }

    protected boolean isHighlighted(Object obj, Measurement m)
    {
        if (obj==null || m==null)
            return false;
        else if (obj instanceof ResultDAI)
        {
            ResultDAI dai = (ResultDAI)obj;
            return dai.getResult() > 0 && dai.getTimeA() > 0 && dai.getTimeB() > 0 && (dai.getTimeA() == m.getElapsed() || dai.getTimeB() == m.getElapsed());
        }
        else if (obj instanceof ResultPI)
        {
            ResultPI pi = (ResultPI)obj;
            return pi.getResult() > 0 && pi.getTimeA() > 0 && pi.getTimeB() > 0 && ( (pi.getTimeA()*60) == m.getElapsed() || (pi.getTimeB()*60) == m.getElapsed());
        }
        return false;
    }

    protected void drawCell(Typeface typeface, String value)
    {
        if (canvas==null || bounds==null || paint==null)
            return;

        drawSquare();

        paint.setTypeface(typeface);
        paint.setTextSize(7f);

        Rect boundsValue = new Rect();
        paint.getTextBounds(value, 0, value.length(), boundsValue);

//        float centerY     = bounds.height() / 2f;
//        float fontCenterY = -1 * ((paint.getFontMetrics().ascent + (paint.getFontMetrics().bottom)) / 2f);
//        canvas.drawText(value, bounds.left+2, centerY + fontCenterY, paint);
//        float centerX = (bounds.width() / 2f) - (boundsValue.width() / 2f) + boundsValue.left;

        // Ascent is negative, so this is (height/2) '+' ((ascent '-' descent)/2)
        float centerY = (bounds.height() / 2f) - ((paint.getFontMetrics().ascent + paint.getFontMetrics().descent) / 2f);
        centerY += centerY * TEXT_Y_ALIGN_FACTOR;

        canvas.drawText(value, bounds.left+2, centerY, paint);

        paint.setTypeface(Typeface.DEFAULT);
        paint.setTextSize(8f);
    }

    protected void drawPageNumber(int currentPage, int maxPages, int align)
    {
        canvas.save();
        canvas.translate(-CURSOR_X, -CURSOR_Y);
        float TEXT_BK = paint.getTextSize();
        int COLOR_BK  = paint.getColor();
        int boxWidth  = 30;
        int boxHeight = 10;

        canvas.translate( MARGIN, MARGIN );

        int posY = PAPER_HEIGHT - boxHeight;

        canvas.translate(0, posY);

        canvas.drawRect(0, 0,boxWidth, boxHeight, paint);

        paint.setTextSize( 8f );
        paint.setColor( Color.WHITE );

        String text = currentPage + "/" + maxPages;

        float textHeight = paint.getFontMetrics().descent - paint.getFontMetrics().ascent; // ascent is negative
        float ascent     = -1 * paint.getFontMetrics().ascent;
        float fit        = (boxHeight - (textHeight * 3)) /2;

        if (fit < 0)
            fit = 0;

        fit += ascent;



        int posX = (int) ((boxWidth / 2) - (paint.measureText(text) / 2));

        canvas.drawText(text, posX,0+fit, paint);

        paint.setTextSize( TEXT_BK );
        paint.setColor( COLOR_BK );

        canvas.restore();
    }

    protected void drawGeneratedBy()
    {
        if ( canvas == null )
        {
            return;
        }
        canvas.save();
        canvas.translate(-CURSOR_X, -CURSOR_Y);
        float TEXT_BK = paint.getTextSize();
        paint.setTextSize(5f);
        //canvas.drawRect(0, 0,10, 10, paint);

        try
        {
            Bitmap brand = BitmapFactory.decodeResource(activity.getResources(), R.drawable.brand_report);
            Rect src     = new Rect(0, 0, brand.getWidth(), brand.getHeight());
            Rect dst     = new Rect();
            dst.top      = 0;
            dst.left     = 0;
            dst.bottom   = 20;
            float scale  = (float) (brand.getHeight() / dst.bottom);
            dst.right    = (int)   (brand.getWidth() / scale);

            String generatedBy = activity.getString(R.string.md_report_generated);
            String appName     = activity.getString(R.string.app_name);
            String version     = activity.getString(R.string.version) + ": " +
                    activity.getPackageManager().getPackageInfo(activity.getPackageName(), 0).versionName;

            float textHeight = paint.getFontMetrics().descent - paint.getFontMetrics().ascent; // ascent is negative
            float left       = dst.width() + 5;
            float ascent     = -1 * paint.getFontMetrics().ascent;
            float fit        = (dst.bottom - (textHeight * 3)) /2;

            if (fit < 0)
                fit = 0;

            fit += ascent;

            Rect boundsTitle   = new Rect();
            paint.getTextBounds(generatedBy, 0, generatedBy.length(), boundsTitle);

            float posX = PAPER_WIDTH  + MARGIN - (left + boundsTitle.width()) - 5;
            float posY = PAPER_HEIGHT + MARGIN - BRAND_HEIGHT;
            canvas.translate(posX, posY);

            canvas.drawBitmap(brand, src, dst, paint);

            canvas.drawText(generatedBy, left,0+fit           , paint);
            canvas.drawText(appName    , left,textHeight*1+fit, paint);
            canvas.drawText(version    , left,textHeight*2+fit, paint);
        }
        catch (Exception ignored)
        {
        }
        finally
        {
            paint.setTextSize(TEXT_BK);
            canvas.restore();
        }
    }

    private void saveBounds()
    {
        boundsBackup.top    = bounds.top;
        boundsBackup.left   = bounds.left;
        boundsBackup.right  = bounds.right;
        boundsBackup.bottom = bounds.bottom;
    }

    private void restoreBounds()
    {
        bounds.top    = boundsBackup.top;
        bounds.left   = boundsBackup.left;
        bounds.right  = boundsBackup.right;
        bounds.bottom = boundsBackup.bottom;
    }

    protected void moveCursor(float x, float y)
    {
        if ( canvas != null )
        {
            CURSOR_X += x;
            CURSOR_Y += y;
            canvas.translate( x, y );
        }
    }

    protected void saveCursor()
    {
        if ( canvas != null )
        {
            CURSOR_Y_BK = CURSOR_Y;
            CURSOR_X_BK = CURSOR_X;
            canvas.save();
        }
    }

    protected void resetCursor()
    {
        CURSOR_Y = 0;
        CURSOR_X = 0;
    }

    protected void restoreCursor()
    {
        if ( canvas != null )
        {
            CURSOR_Y = CURSOR_Y_BK;
            CURSOR_X = CURSOR_X_BK;
            canvas.restore();
        }
    }

    public void sendFile(File file)
    {
        Uri uri = FileProvider.getUriForFile(activity, activity.getApplicationContext().getPackageName(), file);

        Intent i = new Intent(Intent.ACTION_SEND);
        i.addFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION);
        i.setType("plain/text");
        i.setType("application/octet-stream");
        i.putExtra(Intent.EXTRA_SUBJECT, "Report");
        i.putExtra(Intent.EXTRA_STREAM, uri);

        String application;
        if (file.getName().toLowerCase().endsWith(".pdf"))
            application = "application/pdf";
        else
            application = "application/msword";

        Intent i2 = new Intent(Intent.ACTION_VIEW);
        i2.addFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION);
        i2.setDataAndType(uri, application);
        i2.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
        i2.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);

        Intent chooser = Intent.createChooser(i, "Send report...");
        chooser.putExtra(Intent.EXTRA_INITIAL_INTENTS, new Intent[] { i2 });
        try
        {
            List<ResolveInfo> resInfoList = activity.getPackageManager().queryIntentActivities(chooser, PackageManager.MATCH_DEFAULT_ONLY);

            for (ResolveInfo resolveInfo : resInfoList)
            {
                String packageName = resolveInfo.activityInfo.packageName;
                activity.grantUriPermission(packageName, uri, Intent.FLAG_GRANT_WRITE_URI_PERMISSION | Intent.FLAG_GRANT_READ_URI_PERMISSION);
            }

            activity.startActivity( chooser );
        }
        catch (android.content.ActivityNotFoundException e)
        {
            for (StackTraceElement ste : e.getStackTrace())
                android.util.Log.e("RE", ste.toString());
        }
    }

    @SuppressWarnings("ResultOfMethodCallIgnored")
    protected void deleteTempFolder()
    {
        if ( tempDir!=null && tempDir.exists() )
        {
            File[] children = tempDir.listFiles();
            for (File aChildren : children)
            {
                aChildren.delete();
            }
            tempDir.delete();
        }
    }
}
