package com.megabras.bluelogg.hp60kv;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.Color;
import android.graphics.Paint;
import android.os.Bundle;
import android.util.DisplayMetrics;
import android.util.Log;
import android.util.TypedValue;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.LinearLayout;
import android.widget.ListView;

import com.megabras.bluelogg.R;
import com.megabras.bluelogg.extended.Units;
import com.megabras.modbus.HP60KV.Device;
import com.megabras.modbus.HP60KV.Measurement;
import com.megabras.modbus.MUtils;

import org.achartengine.ChartFactory;
import org.achartengine.GraphicalView;
import org.achartengine.model.XYMultipleSeriesDataset;
import org.achartengine.model.XYSeries;
import org.achartengine.renderer.XYMultipleSeriesRenderer;
import org.achartengine.renderer.XYSeriesRenderer;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;

import androidx.annotation.NonNull;
import androidx.fragment.app.Fragment;

public class Charts extends Fragment
{
    @SuppressWarnings("unused")
    private final String
            TAG                 = "ChartFragment";

    public static final int
            VOLTAGE_CHART       = 0,
            CURRENT_CHART       = 1,
            RESISTANCE_CHART    = 2,
            TABLE               = 3;

    private int
            CHART_TYPE          = -1;

    @SuppressWarnings("FieldCanBeLocal")
    private final int
            MEASURE_GAP         = 5,
            MIN_TIME            = 5000;

    private XYSeries
            mXYSeries;

    private GraphicalView
            mChart;

    private double
            minY,
            maxY;

    private long
            POWER,
            CHART_CORRECTION    = 0;

    private XYMultipleSeriesRenderer
            mRenderer;

    private LinearLayout
            layout;

    private ListView
            list;

    private TableBaseAdapter
            adapter;

    private String
            chartTitle          = "";

    private Context
            context;

    protected final com.megabras.modbus.HP60KV.FormatData
            formatData;

    public Charts()
    {
        formatData = new com.megabras.modbus.HP60KV.FormatData();
    }

    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState)
    {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(new Date());
        calendar.set(Calendar.MILLISECOND, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        CHART_CORRECTION = calendar.getTimeInMillis();
        context = inflater.getContext();
        return inflater.inflate(R.layout.md10kvx_fgr_online, container, false);
    }

    @Override
    public void onStart()
    {
        super.onStart();
        createChart();
    }

    public void setChartType(int type, String chartTitle)
    {
        CHART_TYPE = type;
        this.chartTitle = chartTitle;
    }

    public int getChartType()
    {
        return CHART_TYPE;
    }

    private void createChart()
    {
        POWER = 0;
        minY  = Double.MAX_VALUE;
        maxY  = Double.MIN_VALUE;

        switch (CHART_TYPE)
        {
            case Charts.VOLTAGE_CHART:
                Chart(chartTitle,"V");
                break;
            case Charts.CURRENT_CHART:
                Chart(chartTitle,"A");
                break;
            case Charts.RESISTANCE_CHART:
                Chart(chartTitle,"GOhm");
                break;
            case Charts.TABLE:
                Table();
                break;
        }
    }

    void setDeviceID( int id )
    {
        formatData.setDeviceId( id );
    }

    public Bitmap getChartBitmap()
    {
        int gridColor   = mRenderer.getGridColor(0);
        int labelColor  = mRenderer.getLabelsColor();
        int yLabelColor = mRenderer.getYLabelsColor(0);
        int xLabelColor = mRenderer.getXLabelsColor();
        int axesColor   = mRenderer.getAxesColor();

        mRenderer.setGridColor(Color.argb(60, 0, 0, 0));
        mRenderer.setLabelsColor(Color.BLACK);
        mRenderer.setYLabelsColor(0, Color.BLACK);
        mRenderer.setXLabelsColor(Color.BLACK);
        mRenderer.setAxesColor(Color.BLACK);

        mChart.setDrawingCacheEnabled(true);
        Bitmap bitmap = mChart.getDrawingCache();

        mRenderer.setGridColor(gridColor);
        mRenderer.setLabelsColor(labelColor);
        mRenderer.setYLabelsColor(0, yLabelColor);
        mRenderer.setXLabelsColor(xLabelColor);
        mRenderer.setAxesColor(axesColor);
        return bitmap;
    }

    public void Table()
    {
        if( this.getView()!=null )
            layout = this.getView().findViewById(R.id.chart);
        else
            return;

        if( layout.getChildCount()>1 )
            return;

        list = new ListView( layout.getContext() );
        list.setDividerHeight(0);
        layout.addView(list);

        adapter = new TableBaseAdapter( layout.getContext() );
        list.setAdapter( adapter );
        list.setVisibility(View.VISIBLE);
    }

    public void setMode(String mode)
    {
        adapter.setMode(mode);
    }

    public void setPI(String pi)
    {
        adapter.setPI(pi);
    }

    public void setTestNumber(int testNumber)
    {
        showTable(true);
        adapter.setTestNumber(testNumber);
    }

    public void setDate(String date)
    {
        adapter.setDate(date);
    }

    void setTestResult(int result)
    {
        adapter.setResult(result);
    }

    public void showTable(boolean show)
    {
        if ( list!=null )
        {
            if(show)
                list.setVisibility(View.VISIBLE);
            else
                list.setVisibility(View.GONE);
        }
    }

    private void Chart(String chartTitle, String yTitle)
    {
        if(this.getView()!=null)
            layout = this.getView().findViewById(R.id.chart);
        else
            return;

        if(layout.getChildCount()>0)
            return;

        XYMultipleSeriesDataset mDataset = new XYMultipleSeriesDataset();
        mRenderer = new XYMultipleSeriesRenderer();
        XYSeriesRenderer mXYSeriesRenderer;
        mXYSeries = new XYSeries("Chart");

        mDataset.addSeries(mXYSeries);
        mXYSeriesRenderer = new XYSeriesRenderer();
        mXYSeriesRenderer.setColor(Color.RED);
        mXYSeriesRenderer.setFillPoints(true);
        mXYSeriesRenderer.setLineWidth(2);

        DisplayMetrics metrics = context.getResources().getDisplayMetrics();

        mRenderer.setLabelsTextSize(
                TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, context.getResources().getDimension(R.dimen.chart_label), metrics)
        );

        mRenderer.setChartTitleTextSize(
                TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, context.getResources().getDimension(R.dimen.chart_title), metrics)
        );

        mRenderer.setAxisTitleTextSize(
                TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, context.getResources().getDimension(R.dimen.chart_title), metrics)
        );

        mRenderer.addSeriesRenderer(mXYSeriesRenderer);
        mRenderer.setAntialiasing(false);
        mRenderer.setShowLegend(false);
        mRenderer.setYTitle(yTitle);
        mRenderer.setYLabelsAlign(Paint.Align.RIGHT);
        mRenderer.setXTitle("h:mm:ss");
        mRenderer.setChartTitle(chartTitle);
        mRenderer.setApplyBackgroundColor(true);
        mRenderer.setBackgroundColor(Color.TRANSPARENT);
        mRenderer.setMarginsColor(Color.argb(0x00, 0x01, 0x01, 0x01));
        mRenderer.setAxesColor(Color.TRANSPARENT);
        mRenderer.setShowGrid(true);
        mRenderer.setGridColor(Color.argb(60, 255, 255, 255));
        mRenderer.setYLabels(6);
        mRenderer.setXLabels(8);
        mRenderer.setXAxisMin(CHART_CORRECTION);
        mRenderer.setPanEnabled(false, false);
        mRenderer.setZoomEnabled(false, false);

        if (this.getActivity()!=null)
        {
            mRenderer.setMargins(new int[]
            {
                getActivity().getResources().getInteger(R.integer.chart_margin_top),
                getActivity().getResources().getInteger(R.integer.chart_margin_left),
                getActivity().getResources().getInteger(R.integer.chart_margin_bottom),
                getActivity().getResources().getInteger(R.integer.chart_margin_right),
            }); //top - left - bottom - right

            mChart = ChartFactory.getTimeChartView(
                    this.getActivity().getApplicationContext(),
                    mDataset,
                    mRenderer,
                    "H:mm:ss");


            layout.addView(mChart);
        }
    }

    private double resolution(double value, String unit)
    {
        if (value==Double.POSITIVE_INFINITY)
            return maxY*20;

        double r;
        if(POWER==0 && mRenderer!=null)
        {
            if (value>=Units.TERA)
            {
                r = value / Units.TERA;
                mRenderer.setYTitle("T"+unit);
                POWER = Units.TERA;
            }
            else if(value>=Units.GIGA)
            {
                r = value / Units.GIGA;
                mRenderer.setYTitle("G"+unit);
                POWER = Units.GIGA;
            }
            else if(value>=Units.MEGA)
            {
                r = value / Units.MEGA;
                mRenderer.setYTitle("M"+unit);
                POWER = Units.MEGA;
            }
            else if(value>Units.KILO)
            {
                r = value / Units.KILO;
                mRenderer.setYTitle("k"+unit);
                POWER = Units.KILO;
            }
            else if (value >= 1)
            {
                r = value;
                mRenderer.setYTitle(unit);
                POWER=1;
            }
            else if(value * Units.MILI>=1)
            {
                r = (value * Units.MILI);
                mRenderer.setYTitle("m"+unit);
                POWER = Units.MILI;
            }
            else if(value * Units.MICRO>=1)
            {
                r = (value * Units.MICRO);
                mRenderer.setYTitle("\u00B5"+unit);
                POWER = Units.MICRO;
            }
            else if (value * Units.NANO>=1)
            {
                r = (value * Units.NANO);
                mRenderer.setYTitle("n"+unit);
                POWER = Units.NANO;
            }
            else if (value * Units.PICO>=1)
            {
                r = (value * Units.PICO);
                mRenderer.setYTitle("p"+unit);
                POWER = Units.PICO;
            }
            else
            {
                r = 0;
            }
        }
        else
        {
            if(value >= 1)
            {
                r = (value / POWER);
            }
            else
            {
                r = (value * POWER);
            }
        }
        updateScaleY(r);
        return r;
    }

    private void updateScaleY(double newY)
    {
        if (newY<=0)
        {
            minY = 0;
            mRenderer.setYAxisMin(0);
            return;
        }

        if (newY < minY)
            minY = newY;

        if (newY > maxY)
            maxY = newY;

        double diff = maxY - minY;
        double perc = (diff*100)/maxY;
        perc = 100 - perc;
        perc = perc/100;
        double toAddMax = (maxY * perc)/2;
        double toAddMin = (maxY * perc)/2;
        double lowY     = minY - toAddMin;
        double upY      = maxY + toAddMax;

        if(lowY<0)
            lowY=0;

        mRenderer.setYAxisMax(upY);
        mRenderer.setYAxisMin(lowY);
    }

    // ##############################
    // ChartInterface
    // ##############################

    public void update(Measurement measurement, int firmware, boolean updateLast)
    {
        if ( !updateLast )
            if( measurement.getElapsed()%MEASURE_GAP!=0 )
                return;

        if( (measurement.getElapsed()*1000)<MIN_TIME )
            return;

        long   milliseconds = measurement.getElapsed()*1000;
        int    voltage      = measurement.getVoltage();
        double current      = measurement.getCurrent();
        double ohms         = measurement.getResistance();

        if( mXYSeries!=null && mXYSeries.getItemCount()>0 )
        {
            int index = mXYSeries.getItemCount()-1;
            if (mXYSeries.getX(index)==milliseconds+CHART_CORRECTION)
            {
                mXYSeries.remove(index);
            }
        }

        if(milliseconds>=MIN_TIME && mRenderer!=null)
        {
            mChart.setVisibility(View.VISIBLE);
            mRenderer.setAxesColor(Color.WHITE);
        }

        switch (CHART_TYPE)
        {
            case Charts.VOLTAGE_CHART:
                if (mXYSeries!=null)
                    mXYSeries.add( milliseconds+CHART_CORRECTION, resolution(voltage, "V") );
                break;
            case Charts.CURRENT_CHART:
                if (mXYSeries!=null)
                    mXYSeries.add( milliseconds+CHART_CORRECTION, resolution(current, "A") );
                break;
            case Charts.RESISTANCE_CHART:
                if (mXYSeries!=null)
                {
                    double ohm      = measurement.getResistance();
                    boolean[] stats = new boolean[16];
                    boolean[] st    = MUtils.intToBitArray(measurement.getStats());
                    System.arraycopy(st, 0, stats, 0, st.length);

                    if( stats[Device.ST_TWO_MEASURE_OVERRANGE] )
                        ohm = Double.POSITIVE_INFINITY;
                    else if ( stats[Device.ST_TWO_MEASURE_INVALID] || stats[Device.ST_TWO_MEASURE_WAIT] )
                        ohm = 0;

                    //addData(milliseconds, ohm);
                    mXYSeries.add(milliseconds + CHART_CORRECTION, resolution(ohm, "\u03A9"));
                }
                break;
            case Charts.TABLE:
                String stringMilliseconds = formatData.formatElapsedTime((short)(milliseconds/1000));
                String stringVoltage      = formatData.formatVoltage(voltage);
                String stringCurrent      = formatData.formatCurrentRT(current, firmware);
                String stringOhms         = formatData.formatResistance(ohms, measurement.getStats());
                if (adapter.getCount()>0)
                {
                    int index = adapter.getCount()-1;
                    if (adapter.getItem(index).getTime().equals(stringMilliseconds))
                        adapter.remove(index);
                }

                adapter.add(stringMilliseconds, stringVoltage, stringCurrent, stringOhms);
                adapter.repaint();
                break;
        }

        if(mChart!=null)
            mChart.repaint();
    }

    public void repaint()
    {
        switch (CHART_TYPE)
        {
            case Charts.VOLTAGE_CHART:
                mChart.repaint();
                break;
            case Charts.CURRENT_CHART:
                mChart.repaint();
                break;
            case Charts.RESISTANCE_CHART:
                mChart.repaint();
                break;
            case Charts.TABLE:
                adapter.repaint();
                break;
        }
    }

    void showResistance(boolean show)
    {
        if (CHART_TYPE==Charts.TABLE)
            adapter.showResistance(show);
    }

    public void setAllValues(ArrayList<Measurement> measurements, int firmware)
    {
        switch (CHART_TYPE)
        {
            case Charts.VOLTAGE_CHART:
                mXYSeries.clear();
                for ( Measurement m : measurements )
                {
                    mXYSeries.add( (m.getElapsed()*1000)+CHART_CORRECTION, resolution(m.getVoltage(), "V") );
                }
                mChart.setVisibility(View.VISIBLE);
                mRenderer.setAxesColor(Color.WHITE);
                mChart.repaint();
                break;
            case Charts.CURRENT_CHART:
                mXYSeries.clear();
                for ( Measurement m : measurements )
                {
                    double current = m.getCurrent();
                    mXYSeries.add( (m.getElapsed()*1000)+CHART_CORRECTION, resolution(current, "A") );
                }
                mChart.setVisibility(View.VISIBLE);
                mRenderer.setAxesColor(Color.WHITE);
                mChart.repaint();
                break;
            case Charts.RESISTANCE_CHART:
                mXYSeries.clear();
                for ( Measurement m : measurements )
                {
                    double ohm      = m.getResistance();
                    boolean[] stats = new boolean[16];
                    boolean[] st    = MUtils.intToBitArray(m.getStats());
                    System.arraycopy(st, 0, stats, 0, st.length);

                    if( stats[Device.ST_TWO_MEASURE_OVERRANGE] )
                        ohm = Double.POSITIVE_INFINITY;
                    else if ( stats[Device.ST_TWO_MEASURE_INVALID] || stats[Device.ST_TWO_MEASURE_WAIT] )
                        ohm = 0;

                    mXYSeries.add( (m.getElapsed()*1000)+CHART_CORRECTION, resolution(ohm, "\u03A9") );
                }
                mChart.setVisibility(View.VISIBLE);
                mRenderer.setAxesColor(Color.WHITE);
                mChart.repaint();
                break;
            case Charts.TABLE:
                adapter.clear();
                for ( Measurement m : measurements )
                {
                    String stringMilliseconds = formatData.formatElapsedTime (m.getElapsed());
                    String stringVoltage      = formatData.formatVoltage     (m.getVoltage());
                    String stringCurrent      = formatData.formatCurrentRT   (m.getCurrent(), firmware);
                    String stringOhms         = formatData.formatResistance  (m.getResistance(), m.getStats());

                    adapter.add(stringMilliseconds, stringVoltage, stringCurrent, stringOhms);
                }
                adapter.repaint();
                break;
        }
    }

    public void reset()
    {
        try
        {
            if (layout==null)
            {
                createChart();
            }
            if(CHART_TYPE==Charts.TABLE)
            {
                adapter.clear();
                adapter.repaint();
                layout.removeAllViews();
                layout.addView(list);
                showTable(false);
            }
            else
            {
                mXYSeries.clear();
                mChart.repaint();
                mChart.setVisibility(View.GONE);
                POWER = 0;
                minY = Double.MAX_VALUE;
                maxY = Double.MIN_VALUE;
            }
        }
        catch (Exception e)
        {
            Log.e("ERROR", e.getMessage()+"");
        }
    }
}
