package com.megabras.bluelogg.hp60kv;

import android.annotation.SuppressLint;
import android.app.DatePickerDialog;
import android.app.Dialog;
import android.app.TimePickerDialog;
import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.os.Handler;
import android.os.Message;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.Window;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.DatePicker;
import android.widget.FrameLayout;
import android.widget.ImageButton;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.TimePicker;

import com.megabras.DeviceList;
import com.megabras.bluelogg.extended.CommonUI;
import com.megabras.bluelogg.extended.FileDetails;
import com.megabras.bluelogg.MUtils;
import com.megabras.bluelogg.Main;
import com.megabras.bluelogg.R;
import com.megabras.bluelogg.bluetooth.BluetoothClassic;
import com.megabras.bluelogg.extended.Audio;
import com.megabras.bluelogg.extended.BooleanObservable;
import com.megabras.bluelogg.extended.GPS;
import com.megabras.bluelogg.extended.GoogleMaps;
import com.megabras.bluelogg.extended.IntegerObservable;
import com.megabras.bluelogg.extended.LogCat;
import com.megabras.bluelogg.extended.MapData;
import com.megabras.bluelogg.extended.Picture;
import com.megabras.bluelogg.extended.SlidingTabLayout;
import com.megabras.modbus.HP60KV.Device;
import com.megabras.modbus.HP60KV.Measurement;
import com.megabras.modbus.HP60KV.Protocol;
import com.megabras.modbus.HP60KV.TestDataAndroid;
import com.megabras.modbus.Modbus;

import java.io.File;
import java.lang.ref.WeakReference;
import java.text.DecimalFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;
import java.util.Observable;
import java.util.Observer;

import androidx.annotation.NonNull;
import androidx.asynclayoutinflater.view.AsyncLayoutInflater;
import androidx.fragment.app.FragmentActivity;
import androidx.viewpager.widget.ViewPager;

@SuppressWarnings({"FieldCanBeLocal", "WeakerAccess"})
@SuppressLint("SetTextI18n")
public class Controller extends com.megabras.bluelogg.Controller implements GPS.GpsInterface, Observer, Picture.PictureInterface
{
    /***********************************************************************************************
     * REGION Fields
     ***********************************************************************************************/

    @SuppressWarnings("unused")
    protected String
            TAG = "hp60kv.Controller";

    public static String
            model,
            serial;

    @SuppressWarnings("WeakerAccess")
    public static int
            //minFirmareVersion = 2,
            memoryMap,
            hardware,
            firmware,
            DEVICE_ID,
            FIRMWARE_OHMS      = 8,
            FIRMWARE_HIDE_OHMS = 10,
            FIRMWARE_DISCHARGE = 8;

    protected FragmentActivity
            activity;

    private Menu
            menu;

    private MenuItem
            menuOptions;

    private Helper
            helper;

    private ViewPager
            viewPager;

    private PagerAdapter
            pagerAdapter;

    private SlidingTabLayout
            tabLayout;

    private View
            pageTestSetup,
            pageGeneral;

    private int
            controlButtonsID    = R.layout.hp60kv_control_buttons,
            optionsID,
            timeFormat          = 0,
            selectedIndex       = -1;

    private LinearLayout
            toolBar;

    private Protocol
            modbus = new Protocol();

    private Button
            buttonStart,
            buttonStop,
            buttonVoltage;

    private ImageButton
            buttonSetup,
            buttonSave,
            buttonManage,
            buttonGPS,
            buttonPicture,
            buttonVoice,
            buttonMap;

    private boolean
            updated_overcurrent = false,
            initialConfig       = true,
            waitDischarge       = false;

    private CommonUI
            mCommonUI;

    private TestDataAndroid
            testData;

    private Picture
            picture;

    private GPS
            mGPS;

    private com.megabras.bluelogg.extended.Audio
            audio;

    private GoogleMaps
            map;

    private mHandler
            handler;

    private Dialog
            dlgProgress,
            dialogFinished;

    protected com.megabras.bluelogg.Main
            parent;

    private Device
            device = new Device();

    private BooleanObservable
            isTestRunning    = new BooleanObservable(1, true),
            isConnected      = new BooleanObservable(2, true),
            isTestStarting   = new BooleanObservable(3, false),
            isAutoPilot      = new BooleanObservable(4, false),
            isInternalError  = new BooleanObservable(5, false);

    private IntegerObservable
            messageID        = new IntegerObservable(1, 0);

    private Database
            database;

    @SuppressWarnings("unused")
    private Report
            report;

    private FrameLayout
            overlay;

    private int
            PACK_SIZE = 10;

    protected com.megabras.modbus.HP60KV.FormatData
            formatData;

    /***********************************************************************************************
     * REGION Constructor
     ***********************************************************************************************/

    @SuppressLint("CommitPrefEdits")
    public Controller(FragmentActivity fragmentActivity, CommonUI ui, String model, String serial)
    {
        memoryMap           = 0;
        hardware            = 0;
        firmware            = 0;
        helper              = new Helper(fragmentActivity);
        activity            = fragmentActivity;
        parent              = (com.megabras.bluelogg.Main)activity;
        Controller.model    = model;
        Controller.serial   = serial.trim();
        handler             = new mHandler(this);
        mCommonUI           = ui;
        picture             = Picture.getInstance();
        picture.initialize(fragmentActivity, this);
        mGPS                = new GPS(this, fragmentActivity);
        audio               = new Audio(fragmentActivity);
        map                 = new GoogleMaps(fragmentActivity);
        isTestRunning       .addObserver(this);
        isConnected         .addObserver(this);
        isTestStarting      .addObserver(this);
        isAutoPilot         .addObserver(this);
        isInternalError     .addObserver(this);
        messageID           .addObserver(this);
        device              .addObserver(this, Device.ID_OBSERVABLE_MEMORY);
        device              .addObserver(this, Device.ID_OBSERVABLE_MODE);
        String DB_NAME      = model + "_" + serial + "_REGISTERS.db";
        database            = new Database(activity, Controller.model, DB_NAME);
        report              = new Report(activity, database.getFolder(), database.getTemporaryFolder(), handler);
        overlay             = MUtils.getOverlay(activity, OVERLAY_ID);
        DEVICE_ID           = DeviceList.getIdByModel( model );
        LogCat.LEVEL = LogCat.LEVEL_NONE;
        pageTest();
        pageGeneral();
        formatData = new com.megabras.modbus.HP60KV.FormatData();
        formatData.setDeviceId( DEVICE_ID );
    }

    /***********************************************************************************************
     * REGION Implements Controller
     ***********************************************************************************************/

    public void createSwipe()
    {
        pagerAdapter = new PagerAdapter(activity.getSupportFragmentManager(), activity.getBaseContext());
        pagerAdapter.setDeviceID( DEVICE_ID );
        viewPager    = activity.findViewById(R.id.pager);

        if (viewPager == null)
            return;

        viewPager.setAdapter(pagerAdapter);
        viewPager.setOffscreenPageLimit(5);

        //Create customs Tabs
        tabLayout = activity.findViewById(R.id.sliding_tabs);
        tabLayout.setDistributeEvenly(true);
        tabLayout.setCustomTabView(R.layout.custom_tab, 0);
        tabLayout.setBackground(activity.getResources().getDrawable(R.drawable.chamfer_black_top));
        tabLayout.setCustomTabColorizer(new SlidingTabLayout.TabColorizer()
        {
            public int getIndicatorColor(int position) {
                return Color.GRAY;
            }

            public int getDividerColor(int position) {
                return Color.WHITE;
            }
        });
        tabLayout.setViewPager(viewPager);
    }

    public void createControlButtons(LinearLayout container)
    {
        final View commandButtons = View.inflate(activity, controlButtonsID, null);

        commandButtons.setLayoutParams(
                new LinearLayout.LayoutParams(
                        LinearLayout.LayoutParams.MATCH_PARENT,
                        (int)activity.getResources().getDimension(R.dimen.md15kvr_controls_height)));
        container.removeAllViews();
        container.addView(commandButtons);

        buttonStart = commandButtons.findViewById(R.id.bt_start);
        buttonStart.setOnClickListener(v ->
        {
            final Dialog dialog = new Dialog(activity);
            dialog.requestWindowFeature(Window.FEATURE_NO_TITLE);
            dialog.setCanceledOnTouchOutside(false);
            dialog.setCancelable(false);
            dialog.setContentView(R.layout.dialog_yes_no_white);
            if (dialog.getWindow()!=null)
                dialog.getWindow().getAttributes().width = ViewPager.LayoutParams.MATCH_PARENT;
            ((TextView) dialog.findViewById(R.id.dlg_title)).setText(R.string.md_warning_test_start);
            ((ImageView) dialog.findViewById(R.id.dlg_icon)).setImageResource(R.drawable.ic_warning);

            final TextView message = dialog.findViewById(R.id.message);
            message.setText(activity.getResources().getString(R.string.hipot_is_grounded).toUpperCase());

            final Button btnNo = dialog.findViewById(R.id.btn_no);
            btnNo.setOnClickListener(v1 -> dialog.dismiss());

            final Button btnYes = dialog.findViewById(R.id.btn_yes);
            btnYes.setOnClickListener(v12 -> {
                sendMessage( modbus.write.start() );
                buttonStart.setEnabled(false);
                dialog.dismiss();
            });

            dialog.show();
        });

        buttonStop = commandButtons.findViewById(R.id.bt_stop);
        buttonStop.setOnClickListener(v -> sendMessage( modbus.write.stop() ));

        final Controller controller = this;
        buttonVoltage = commandButtons.findViewById(R.id.bt_voltage);
        buttonVoltage.setOnClickListener(v -> helper.pickVoltage
                (
                        controller,
                        modbus,
                        device,
                        device.getVoltsMin(),
                        device.getVoltsMax(),
                        device.getVoltsStep()
                ));

        buttonVoltage.setEnabled( false );

        buttonSetup = commandButtons.findViewById(R.id.bt_setup);
        buttonSetup.setOnClickListener(v -> pageTestSetup());

        isTestRunning.set(false);
        isConnected.set(false);
    }

    public void setToolBarId(int toolBarId)
    {

        toolBar = activity.findViewById(toolBarId);
        toolBar.findViewById(R.id.bt_export).setVisibility(View.GONE);
        toolBar.findViewById(R.id.bt_details).setVisibility(View.GONE);
        buttonMap     = toolBar.findViewById(R.id.btn_map);
        buttonSave    = toolBar.findViewById(R.id.bt_save);
        buttonManage  = toolBar.findViewById(R.id.bt_manager);
        buttonGPS     = toolBar.findViewById(R.id.bt_gps);
        buttonPicture = toolBar.findViewById(R.id.bt_picture);
        buttonVoice   = toolBar.findViewById(R.id.bt_record);

        buttonSave.setOnClickListener(v ->
        {
            if (testData!=null)
            {
                database.save(testData);
                enableToolBar(true);
                buttonSave.setEnabled(false);
            }
        });

        buttonManage.setOnClickListener(v -> pageFilesManager());

        buttonPicture.findViewById(R.id.bt_picture).setOnClickListener(v ->
        {
            if (Main.permissionToPictureAccepted)
                takePicture();
            else
                ((Main)activity).requestPermission(Main.REQUEST_PICTURE_PERMISSION);
        });

        buttonGPS.setOnClickListener(v ->
        {
            if (Main.permissionToGPSAccepted)
                getGPS();
            else
                ((Main)activity).requestPermission(Main.REQUEST_GPS_PERMISSION);
        });

        buttonVoice.setOnClickListener(v ->
        {
            if (Main.permissionToRecordAccepted)
                recordAudio();
            else
                ((Main)activity).requestPermission(Main.REQUEST_RECORD_AUDIO_PERMISSION);
        });

        buttonMap.setOnClickListener(v ->
        {
            if(Main.permissionToMemoryAccepted)
                map();
            else
                ((Main)activity).requestPermission(Main.REQUEST_MEMORY_MAP_PERMISSION);
        });

        enableToolBar(false);
        buttonManage.setEnabled(true);


    }

    private void map()
    {
        if ( testData==null || testData.getGpsAcuraccy()==0 )
            return;

        database.createFolder(activity, model);

        MapData mapData = new MapData(
                testData.getTestNumber()+"",
                testData.getGpsLongitude(),
                testData.getGpsLatitude(),
                testData.getGpsAltitude()+"",
                testData.getGpsAcuraccy()+""
        );
        map.showMap(database.getMapSnapshotFile(testData.getSerial(),testData.getDate().getTime()+""), mapData);
    }

    public void onClose()
    {
        if (mGPS != null)
            mGPS.cancel();
        if (picture != null)
            picture.close();
    }

    public void setDateTimeFormat(int dateFormat, int timeFormat)
    {
        device.setDateFormat(dateFormat);
        device.setTimeFormat(timeFormat);
    }

    public void setAutoConn(boolean enable)
    {
        device.setAutoConnection(enable);
    }

    public boolean getAutoConn()
    {
        return device.getAutoConnection();
    }

    public void sendMessage(com.megabras.modbus.Message message)
    {
        parent.addMessage(message);
    }

    public com.megabras.modbus.Message statusMessage()
    {
        return modbus.read.status(2);
    }

//    public com.megabras.modbus.Message stopMessage()
//    {
//        return modbus.write.stop();
//    }

    public void readGeneralConfig()
    {
        messageID.set(R.string.read_general_params);
        if (menuOptions==null)
            menuOptions = menu.findItem(optionsID);

        menuOptions.setEnabled(false);
        sendMessage( modbus.read.date() );
        sendMessage( modbus.read.memoryPointer() );
        sendMessage( modbus.read.setup() );
    }

    public void setMenuItemOptions(Menu menu, int optionsID)
    {
        this.menu      = menu;
        this.optionsID = optionsID;

        if (menuOptions==null)
            menuOptions = menu.findItem(optionsID);

        if (pagerAdapter==null)
            return;
        pagerAdapter.resetAll();
    }

    private void userMessage(int stringID, int color, boolean busy)
    {
        String message = activity.getString(stringID);
        parent.message(message, color, busy);
    }

    private void enableToolBar(boolean enable)
    {
        if (toolBar==null)
            return;
        buttonPicture .setEnabled(enable);
        buttonGPS     .setEnabled(enable);
        buttonManage  .setEnabled(enable);
        buttonMap     .setEnabled(enable);
        buttonSave    .setEnabled(enable);
        buttonVoice   .setEnabled(enable);
    }

    private void enableToolBarWork(boolean enable)
    {
        if (toolBar==null)
            return;
        buttonPicture .setEnabled(enable);
        buttonGPS     .setEnabled(enable);
        buttonMap     .setEnabled(enable);
        buttonVoice   .setEnabled(enable);
    }

    private void enableCommandButtons(boolean enabled)
    {
        buttonStop    .setEnabled(enabled);
        buttonStart   .setEnabled(enabled);
        buttonSetup   .setEnabled(enabled);
//        if (device!=null && device.getOperationMode()==Protocol.MODE_MANUAL)
//            buttonVoltage .setEnabled(enabled);
    }

    /***********************************************************************************************
     * REGION Handler
     ***********************************************************************************************/

    public static class mHandler extends Handler
    {
        private final
                WeakReference<Controller> mClass;

        static final int
                REPORT_UPDATE   = 0,
                REPORT_FINISHED = 1,
                REPORT_ERROR    = 2;

        public mHandler(Controller c)
        {
            mClass = new WeakReference<>(c);
        }

        @Override
        public void handleMessage( @NonNull Message msg )
        {
            final Controller c = mClass.get();

            switch(msg.what)
            {
                case mHandler.REPORT_UPDATE:
                    c.updateProgressBar( (String) msg.obj );
                    break;
                case mHandler.REPORT_FINISHED:
                    c.dlgProgress.dismiss();
                    c.pagerAdapter.resetCharts();
                    c.pagerAdapter.showResistanceFields( firmware >= FIRMWARE_OHMS && firmware < FIRMWARE_HIDE_OHMS );
                    if ( c.testData!=null )
                    {
                        c.pagerAdapter.fillChartsAndTable(c.testData, c.testData.getFirmware(), c.isConnected.get());
                    }
                    break;
                case mHandler.REPORT_ERROR:
                    c.dlgProgress.dismiss();
                    c.pagerAdapter.resetCharts();
                    c.pagerAdapter.showResistanceFields( firmware >= FIRMWARE_OHMS && firmware < FIRMWARE_HIDE_OHMS );
                    if ( c.testData!=null )
                    {
                        c.pagerAdapter.fillChartsAndTable(c.testData, c.testData.getFirmware(), c.isConnected.get());
                    }
                    break;
            }
        }
    }

    /***********************************************************************************************
     * REGION Picture
     ***********************************************************************************************/

    public void takePicture()
    {
        if(database!=null && testData!=null)
        {
            database.createFolder(activity, model);
            File file = database.getPictureFile( testData.getSerial(), testData.getDate().getTime()+"" );
            picture.showDialog(file);
        }
    }

    public void pictureStatus(final int status)
    {
        activity.runOnUiThread(() -> {
            switch (status)
            {
                case Picture.WAIT:
                    mCommonUI.changeButtonImage(R.id.bt_picture, R.drawable.ic_wait, false);
                    break;
                case Picture.FINISHED:
                    mCommonUI.changeButtonImage(R.id.bt_picture, R.drawable.ic_picture_states, true);
                    break;
            }
        });
    }

    /***********************************************************************************************
     * REGION Permission
     ***********************************************************************************************/

    public void permissionCallback(int permission)
    {
        switch (permission)
        {
            case Main.REQUEST_RECORD_AUDIO_PERMISSION:
                recordAudio();
                break;
            case Main.REQUEST_PICTURE_PERMISSION:
                takePicture();
                break;
            case Main.REQUEST_GPS_PERMISSION:
                getGPS();
                break;
            case Main.REQUEST_MEMORY_REPORT_PERMISSION:
                export(selectedIndex);
                break;
            case Main.REQUEST_MEMORY_MAP_PERMISSION:
                map();
                break;
        }
    }

    /***********************************************************************************************
     * REGION GPS
     ***********************************************************************************************/

    private void getGPS()
    {
        if ( testData==null )
            return;

        if ( testData.getGpsAcuraccy()==0 )
            mGPS.getDataGPS();
        else
        {
            String[] gps = new String[]{
                    testData.getGpsLongitude()+"",
                    testData.getGpsLatitude()+"",
                    testData.getGpsAltitude()+"",
                    testData.getGpsAcuraccy()+""
            };
            mGPS.dialogConfirmReplace( gps );
        }
    }

    public void GpsData(int which, String[] data)
    {
        switch (which)
        {
            case GPS.RESULT:
                if ( data!=null && testData!=null)
                {
                    testData.setGpsLongitude( Double.parseDouble(data[0]) );
                    testData.setGpsLatitude ( Double.parseDouble(data[1]) );
                    testData.setGpsAltitude ( Double.parseDouble(data[2]) );
                    testData.setGpsAcuraccy ( Double.parseDouble(data[3]) );
                    database.updateGpsData( testData );
                }
                mCommonUI.changeButtonImage(buttonGPS, R.drawable.ic_gps_states, true);
                break;
            case GPS.WAIT:
                mCommonUI.changeButtonImage(buttonGPS, R.drawable.ic_wait, false);
                break;
            case GPS.CANCELLED:
                mCommonUI.changeButtonImage(buttonGPS, R.drawable.ic_gps_states, true);
                break;
        }
    }

    /***********************************************************************************************
     * REGION AUDIO
     ***********************************************************************************************/

    private void recordAudio()
    {
        if ( testData!=null )
        {
            database.createFolder(activity, model);
            File location  = database.getAudioFile(testData.getSerial(), testData.getDate().getTime()+"");
            audio.setFile( location );
            audio.showDialog(testData.getTestNumber()+"");
        }
    }

    /***********************************************************************************************
     * REGION Report
     ***********************************************************************************************/

    private void creatDialogProgress()
    {
        dlgProgress = new Dialog(activity);
        dlgProgress.requestWindowFeature(Window.FEATURE_NO_TITLE);
        dlgProgress.setCanceledOnTouchOutside(false);
        dlgProgress.setContentView(R.layout.progress_bar);
        ((TextView) dlgProgress.findViewById(R.id.dlg_title)).setText(R.string.please_wait);
        ((TextView) dlgProgress.findViewById(R.id.message)).setText(R.string.saving_data_in_pdf);
        dlgProgress.findViewById(R.id.dlg_icon).setVisibility(View.GONE);
    }

    private void updateProgressBar(String progress)
    {
        ((TextView)dlgProgress.findViewById(R.id.status)).setText(progress);
    }

    /***********************************************************************************************
     * REGION CALLBACKs
     ***********************************************************************************************/

    private TimePickerDialog.OnTimeSetListener timeListener = new TimePickerDialog.OnTimeSetListener()
    {
        @Override
        public void onTimeSet(TimePicker view, int hourOfDay, int minute)
        {
            if(pageGeneral!=null)
            {
                TextView time = pageGeneral.findViewById(R.id.device_time);
                Calendar c = Calendar.getInstance();
                c.setTime(device.getDate());
                c.set(Calendar.HOUR_OF_DAY, hourOfDay);
                c.set(Calendar.MINUTE, minute);
                c.set(Calendar.SECOND, 0);
                device.setDate( c.getTime() );
                time.setText(  formatData.time( device.getTimeFormat(), device.getDate() ) );
            }
        }
    };

    private void pickTime(int format)
    {
        Calendar calendar = new GregorianCalendar();
        calendar.setTime(device.getDate());
        int hour     = calendar.get(Calendar.HOUR_OF_DAY);
        int minute   = calendar.get(Calendar.MINUTE);
        boolean is24 = format==Modbus.TIME_24H;
        new TimePickerDialog(activity, timeListener, hour, minute, is24).show();
    }

    private DatePickerDialog.OnDateSetListener dateListener = new DatePickerDialog.OnDateSetListener()
    {
        public void onDateSet(DatePicker arg0, int year, int month, int day)
        {
            if ( pageGeneral!=null )
            {
                TextView date  = pageGeneral.findViewById(R.id.device_date);
                TextView time  = pageGeneral.findViewById(R.id.device_time);
                String parts[] = time.getText().toString().split(":");
                int hour       = Integer.parseInt(parts[0]);
                int minutes    = Integer.parseInt(parts[1]);

                Calendar c     = Calendar.getInstance();
                c.set(year, month, day, hour, minutes);
                device.setDate( c.getTime() );
                date.setText( formatData.date(device.getDateFormat(), device.getDate()) );
            }
        }
    };

    private void pickDate()
    {
        Calendar calendar = new GregorianCalendar();
        calendar.setTime(device.getDate());
        int year  = calendar.get(Calendar.YEAR);
        int month = calendar.get(Calendar.MONTH);
        int day   = calendar.get(Calendar.DAY_OF_MONTH);
        new DatePickerDialog(activity, dateListener, year, month, day).show();
    }

    /***********************************************************************************************
     * REGION Pages
     ***********************************************************************************************/

    private int pageTestSetupMODE;

    private void pageTest()
    {
        AsyncLayoutInflater inflater = new AsyncLayoutInflater(parent);
        inflater.inflate(R.layout.hp60kv_fgr_test_setup, overlay, (view, resid, parent) -> pageTestSetup = view);
    }

    private void pageTestSetup()
    {
        if ( pageTestSetup == null )
        {
            pageTest();
            return;
        }

        overlay.removeAllViews();
        overlay.addView(pageTestSetup);
        overlay.setVisibility(View.VISIBLE);

        final TextView tvMaxVoltage    = pageTestSetup.findViewById(R.id.tvMaxVoltage);
        final TextView tvManualVoltage = pageTestSetup.findViewById(R.id.tvManualVoltage);
        final TextView tvMaxCurrent    = pageTestSetup.findViewById(R.id.tv_max_current);
        final TextView tvTimer         = pageTestSetup.findViewById(R.id.tv_timer);
        final TextView tvRampUp        = pageTestSetup.findViewById(R.id.tv_ramp_up);
        final TextView tvRampDown      = pageTestSetup.findViewById(R.id.tv_ramp_down);

        // MAX VOLTAGE
        ImageButton buttonSetMaxVoltage = pageTestSetup.findViewById(R.id.bt_set_max_voltage);
        buttonSetMaxVoltage.setOnClickListener(v -> helper.pickNumber
                (
                        tvMaxVoltage,
                        R.string.max_voltage,
                        "V",
                        device.getSetupVoltsMin(),
                        device.getVoltsMax(),
                        device.getVoltsStep()
                ));

        // MANUAL
        ImageButton buttonSetVoltage = pageTestSetup.findViewById(R.id.bt_set_voltage);
        buttonSetVoltage.setOnClickListener(v -> helper.pickNumber
                (
                        tvManualVoltage,
                        R.string.voltage,
                        "V",
                        device.getSetupVoltsMin(),
                        device.getVoltsMax(),
                        device.getVoltsStep()
                ));

        // MAX CURRENT
        ImageButton buttonMaxCurrent = pageTestSetup.findViewById(R.id.ibt_max_current);
        if ( firmware >= 8 || DEVICE_ID != DeviceList.ID_HP60KV )
        {
            buttonMaxCurrent.setOnClickListener(v -> helper.pickNumberFloat
            (
                    tvMaxCurrent,
                    R.string.max_leakage,
                    device.getAmpsUnity(),
                    device.getAmpsMin() / 1000D,
                    device.getAmpsMax() / 1000D
            ));
        }
        else
        {
            buttonMaxCurrent.setOnClickListener(v -> helper.pickNumber
            (
                    tvMaxCurrent,
                    R.string.max_leakage,
                    device.getAmpsUnity(),
                    device.getAmpsMin(),
                    device.getAmpsMax(),
                    device.getAmpsStep()
            ));
        }

        // TIMER
        ImageButton buttonTimer = pageTestSetup.findViewById(R.id.ibt_timer);
        buttonTimer.setOnClickListener(v -> helper.pickTimerWithStep(tvTimer, device.DEVICE_TIMER_MIN, device.DEVICE_TIMER_MAX, device.DEVICE_STEP_TIMER));

        // RAMP UP
        ImageButton buttonRamUp = pageTestSetup.findViewById(R.id.ibt_ramp_up);
        buttonRamUp.setOnClickListener(v -> helper.pickNumber
                (
                        tvRampUp,
                        R.string.ramp_up,
                        "V/s",
                        device.DEVICE_RAMP_MIN,
                        device.DEVICE_RAMP_MAX,
                        device.DEVICE_STEP_RAMP
                ));

        // RAMP UP
        ImageButton buttonRampDown = pageTestSetup.findViewById(R.id.ibt_ramp_down);
        buttonRampDown.setOnClickListener(v -> helper.pickNumber
                (
                        tvRampDown,
                        R.string.ramp_down,
                        "V/s",
                        device.DEVICE_RAMP_MIN,
                        device.DEVICE_RAMP_MAX,
                        device.DEVICE_STEP_RAMP
                ));

        // MODE SELECTION
        final String[] modes = new String[]
                {
                        activity.getString(R.string.Manual),
                        "Auto",
                        activity.getString(R.string.Ramp)
                };

        final Spinner
                spMode    = helper.getSpinner(R.id.spMode            , modes                 , pageTestSetup),
                spLeakage = helper.getSpinner(R.id.spEnableMaxCurrent, R.array.enable_disable, pageTestSetup);

        spLeakage.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                if (position==0)
                    pageTestSetup.findViewById(R.id.lineMaxCurrent).setVisibility(View.VISIBLE);
                else
                    pageTestSetup.findViewById(R.id.lineMaxCurrent).setVisibility(View.GONE);
            }

            @Override
            public void onNothingSelected(AdapterView<?> parent) {}
        });

        spMode.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener()
        {
            @Override
            public void onItemSelected(AdapterView<?> parent, View view, int position, long id)
            {
                pageTestSetupMODE = position;
                int maxLeakage = 0;
                switch (pageTestSetupMODE)
                {
                    case Device.MODE_MANUAL:
                        pageTestSetup.findViewById(R.id.lineMaxVoltage).setVisibility(View.VISIBLE);
                        pageTestSetup.findViewById(R.id.lineManual).setVisibility(View.GONE);
                        pageTestSetup.findViewById(R.id.lineTimer).setVisibility(View.GONE);
                        pageTestSetup.findViewById(R.id.lineRampUp).setVisibility(View.GONE);
                        pageTestSetup.findViewById(R.id.lineRampDown).setVisibility(View.GONE);
                        tvMaxVoltage.setText( device.getParamManual().getMaxVoltage()+" V" );
                        maxLeakage = device.getParamManual().getMaxCurrent();
                        break;
                    case Device.MODE_AUTO:
                        pageTestSetup.findViewById(R.id.lineMaxVoltage).setVisibility(View.GONE);
                        pageTestSetup.findViewById(R.id.lineManual).setVisibility(View.VISIBLE);
                        pageTestSetup.findViewById(R.id.lineTimer).setVisibility(View.VISIBLE);
                        pageTestSetup.findViewById(R.id.lineRampUp).setVisibility(View.GONE);
                        pageTestSetup.findViewById(R.id.lineRampDown).setVisibility(View.GONE);
                        tvManualVoltage.setText( device.getParamSVT().getVoltage()+" V" );
                        maxLeakage = device.getParamSVT().getMaxCurrent();
                        tvTimer.setText( formatData.formatElapsedTime(device.getParamSVT().getDuration()) );
                        break;
                    case Device.MODE_RAMP:
                        pageTestSetup.findViewById(R.id.lineMaxVoltage).setVisibility(View.GONE);
                        pageTestSetup.findViewById(R.id.lineManual).setVisibility(View.VISIBLE);
                        pageTestSetup.findViewById(R.id.lineTimer).setVisibility(View.VISIBLE);
                        pageTestSetup.findViewById(R.id.lineRampUp).setVisibility(View.VISIBLE);
                        pageTestSetup.findViewById(R.id.lineRampDown).setVisibility(View.VISIBLE);
                        tvManualVoltage.setText( device.getParamRamp().getVoltage()+" V" );
                        maxLeakage = device.getParamRamp().getMaxCurrent();
                        tvTimer.setText( formatData.formatElapsedTime(device.getParamRamp().getDuration()) );
                        tvRampUp.setText( device.getParamRamp().getRampUpVoltage()+" V/s" );
                        tvRampDown.setText( device.getParamRamp().getRampDownVoltage()+" V/s" );
                        break;
                }

                tvMaxCurrent.setText( formatData.formatCurrentConfig( maxLeakage, firmware ) );

                if ( maxLeakage > 0 )
                {
                    spLeakage.setSelection(0);
                }
                else
                {
                    spLeakage.setSelection(1);
                    tvMaxCurrent.setText( formatData.formatCurrentConfig( device.getAmpsMin(), firmware ) );
                }
            }

            @Override public void onNothingSelected(AdapterView<?> parent) {}
        });
        pageTestSetupMODE = device.getOperationMode();
        spMode.setSelection( pageTestSetupMODE );

        // ***** SAVE ***** //
        Button buttonSave = pageTestSetup.findViewById( R.id.buttonSave );
        buttonSave.setOnClickListener( v ->
        {
            // SAVE MODE
            if ( spMode.getSelectedItemPosition() != device.getOperationMode() )
            {
                device.setOperationMode( spMode.getSelectedItemPosition() );
                sendMessage( modbus.write.mode( spMode.getSelectedItemPosition() ) );
            }

            boolean leakageEnable = spLeakage.getSelectedItemPosition() == 0;

            int newMaxLeakage;
            if ( leakageEnable )
            {
                if ( firmware >= 8 || DEVICE_ID != DeviceList.ID_HP60KV )
                {
                    String sNumber = tvMaxCurrent.getText().toString()
                            .replace( device.getAmpsUnity(), "" )
                            .replace( ",", "." )
                            .trim();
                    double number = Double.parseDouble( sNumber );
                    newMaxLeakage = (int) (number * 1000);
                }
                else
                {
                    newMaxLeakage = Integer.parseInt( tvMaxCurrent.getText().toString().replace( device.getAmpsUnity(), "" ).trim() );
                }
            }
            else
            {
                newMaxLeakage = 0;
            }

            // SAVE PARAMS
            switch ( pageTestSetupMODE )
            {
                case Device.MODE_MANUAL:
                    device.getParamManual().setMaxVoltage( Integer.parseInt( tvMaxVoltage.getText().toString().replace( "V", "" ).trim() ) );
                    device.getParamManual().setMaxCurrent( newMaxLeakage );
                    sendMessage( modbus.write.paramsManual( device ) );
                    break;
                case Device.MODE_AUTO:
                    device.getParamSVT().setVoltage( Integer.parseInt( tvManualVoltage.getText().toString().replace( "V", "" ).trim() ) );
                    device.getParamSVT().setMaxCurrent( newMaxLeakage );
                    String[] parts = tvTimer.getText().toString().split( ":" );
                    int minutes = Integer.parseInt( parts[0] ) * 60;
                    int seconds = Integer.parseInt( parts[1] );
                    device.getParamSVT().setDuration( minutes + seconds );
                    sendMessage( modbus.write.paramsSVT( device ) );
                    break;
                case Device.MODE_RAMP:
                    device.getParamRamp().setVoltage( Integer.parseInt( tvManualVoltage.getText().toString().replace( "V", "" ).trim() ) );
                    device.getParamRamp().setMaxCurrent( newMaxLeakage );
                    String[] partsR = tvTimer.getText().toString().split( ":" );
                    int minutesR = Integer.parseInt( partsR[0] ) * 60;
                    int secondsR = Integer.parseInt( partsR[1] );
                    device.getParamRamp().setDuration( minutesR + secondsR );
                    device.getParamRamp().setRampUpVoltage( Integer.parseInt( tvRampUp.getText().toString().replace( "V/s", "" ).trim() ) );
                    device.getParamRamp().setRampDownVoltage( Integer.parseInt( tvRampDown.getText().toString().replace( "V/s", "" ).trim() ) );
                    sendMessage( modbus.write.paramsRamp( device ) );
                    break;
            }
            messageID.set( R.string.updating );
            helper.closeOverlay( overlay );
        } );

        Button buttonCancel = pageTestSetup.findViewById(R.id.buttonCancel);
        buttonCancel.setOnClickListener(v ->
        {
            messageID.set(R.string.ready);
            helper.closeOverlay(overlay);
        });
    }

    private void pageGeneral()
    {
        AsyncLayoutInflater inflater = new AsyncLayoutInflater(parent);
        inflater.inflate(R.layout.hp60kv_fgr_general_setup, overlay, (view, resid, parent) -> pageGeneral = view);
    }

    private void pageGeneralSetup()
    {
        if (pageGeneral==null)
        {
            pageGeneral();
            return;
        }

        overlay.removeAllViews();
        overlay.addView(pageGeneral);
        overlay.setVisibility(View.VISIBLE);

        LinearLayout ui_ll_buzzer = pageGeneral.findViewById(R.id.ui_ll_buzzer);
        if ( firmware >= 8  || DEVICE_ID != DeviceList.ID_HP60KV )
        {
            ui_ll_buzzer.setVisibility( View.GONE );
        }
        else
        {
            ui_ll_buzzer.setVisibility( View.VISIBLE );
        }

        final TextView
                date   = pageGeneral.findViewById(R.id.device_date),
                time   = pageGeneral.findViewById(R.id.device_time),
                memory = pageGeneral.findViewById(R.id.device_memory);

        final Spinner
                spinnerBuzzer     = helper.getSpinner(R.id.spBuzzer,              R.array.hp60kv_buzzer,    pageGeneral),
                spinnerConnect    = helper.getSpinner(R.id.spinnerAutoConnection, R.array.enable_disable,   pageGeneral),
                spinnerDateFormat = helper.getSpinner(R.id.spinnerDateFormat,     R.array.date_format,      pageGeneral),
                spinnerTimeFormat = helper.getSpinner(R.id.spinnerTimeFormat,     R.array.time_format,      pageGeneral),
                spinnerLanguage   = helper.getSpinner(R.id.spinnerLanguage,       R.array.language_options, pageGeneral);

        date.setText  ( formatData.date(device.getDateFormat(), device.getDate()) );
        time.setText  ( formatData.time(device.getTimeFormat(), device.getDate()) );
        String free = device.getFreeMemory() + " %";
        memory.setText( free );
        pageGeneral.findViewById(R.id.line_gauge).setVisibility(View.GONE);

        ImageButton btEraseMem = pageGeneral.findViewById(R.id.button_memory);
        btEraseMem.setOnClickListener(v -> {

            final Dialog dialog = new Dialog(activity);
            dialog.requestWindowFeature(Window.FEATURE_NO_TITLE);
            dialog.setCanceledOnTouchOutside(false);
            dialog.setContentView(R.layout.dialog_yes_no_white);
            if (dialog.getWindow()!=null)
                dialog.getWindow().getAttributes().width = ViewPager.LayoutParams.MATCH_PARENT;
            ((TextView) dialog.findViewById(R.id.dlg_title)).setText(R.string.erase_memory);
            ((ImageView) dialog.findViewById(R.id.dlg_icon)).setImageResource(R.drawable.ic_warning);

            final TextView message = dialog.findViewById(R.id.message);
            message.setText(R.string.erase_memory_confirm);

            final Button btnNo = dialog.findViewById(R.id.btn_no);
            btnNo.setOnClickListener(v1 -> dialog.dismiss());

            final Button btnYes = dialog.findViewById(R.id.btn_yes);
            btnYes.setOnClickListener(v12 -> {
                sendMessage( modbus.write.memoryErase() );
                messageID.set(R.string.erase_memory);
                memory.setText( activity.getString(R.string.wait) );
                dialog.dismiss();
            });

            dialog.show();
        });

        if(device.getAutoConnection()){
            spinnerConnect.setSelection(0);
        } else {
            spinnerConnect.setSelection(1);
        }

        if ( device.getBuzzerStatus() )
            spinnerBuzzer.setSelection(0);
        else
            spinnerBuzzer.setSelection(1);

        switch (device.getDateFormat()){
            case Modbus.DATE_DD_MM_YY:
                spinnerDateFormat.setSelection(0);
                break;
            case Modbus.DATE_MM_DD_YY:
                spinnerDateFormat.setSelection(1);
                break;
            default:
                spinnerDateFormat.setSelection(2);
                break;
        }

        switch (device.getTimeFormat()){
            case Modbus.TIME_24H:
                spinnerTimeFormat.setSelection(0);
                break;
            default:
                spinnerTimeFormat.setSelection(1);
                break;
        }

        switch (device.getLanguage()){
            case Modbus.LANG_ENGLISH:
                spinnerLanguage.setSelection(0);
                break;
            case Modbus.LANG_SPANISH:
                spinnerLanguage.setSelection(1);
                break;
            default:
                spinnerLanguage.setSelection(2);
                break;
        }

        spinnerDateFormat.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                switch (position){
                    case 0:
                        date.setText( formatData.date(Modbus.DATE_DD_MM_YY, device.getDate()) );
                        break;
                    case 1:
                        date.setText( formatData.date(Modbus.DATE_MM_DD_YY, device.getDate()) );
                        break;
                    default:
                        date.setText( formatData.date(Modbus.DATE_YYYY_MM_DD, device.getDate()) );
                        break;
                }
            }

            @Override
            public void onNothingSelected(AdapterView<?> parent) {}
        });

        spinnerTimeFormat.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {

                switch (position){
                    case 0:
                        time.setText( formatData.time(Modbus.TIME_24H, device.getDate()) );
                        timeFormat = Modbus.TIME_24H;
                        break;
                    default:
                        time.setText( formatData.time(Modbus.TIME_12H, device.getDate()) );
                        timeFormat = Modbus.TIME_12H;
                        break;
                }
            }

            @Override
            public void onNothingSelected(AdapterView<?> parent) {}
        });

        ImageButton buttonDate = pageGeneral.findViewById(R.id.button_date);
        buttonDate.setOnClickListener(v -> pickDate());

        ImageButton buttonTime = pageGeneral.findViewById(R.id.button_time);
        buttonTime.setOnClickListener(v -> pickTime(timeFormat));

        Button buttonSave = pageGeneral.findViewById(R.id.buttonSave);
        buttonSave.setOnClickListener(v -> {
            switch (spinnerConnect.getSelectedItemPosition()) {
                case 0:
                    device.setAutoConnection(true);
                    break;
                case 1:
                    device.setAutoConnection(false);
                    break;
            }

            boolean buzzer;
            buzzer = spinnerBuzzer.getSelectedItemPosition() == 0;

            int dateFormat, timeFormat, language;
            // DATE
            switch (spinnerDateFormat.getSelectedItemPosition()) {
                case 0:
                    dateFormat = Modbus.DATE_DD_MM_YY;
                    break;
                case 1:
                    dateFormat = Modbus.DATE_MM_DD_YY;
                    break;
                default:
                    dateFormat = Modbus.DATE_YYYY_MM_DD;
                    break;
            }

            switch (spinnerTimeFormat.getSelectedItemPosition()) {
                case 0:
                    timeFormat = Modbus.TIME_24H;
                    break;
                default:
                    timeFormat = Modbus.TIME_12H;
                    break;
            }

            switch (spinnerLanguage.getSelectedItemPosition()) {
                case 0:
                    language = Modbus.LANG_ENGLISH;
                    break;
                case 1:
                    language = Modbus.LANG_SPANISH;
                    break;
                default:
                    language = Modbus.LANG_PORTUGUESE;
                    break;
            }

            device.setDateFormat(dateFormat);
            device.setTimeFormat(timeFormat);
            device.setLanguage(language);
            device.setBuzzerStatus(buzzer);

            messageID.set(R.string.updating);
            sendMessage( modbus.write.date( device.getDate() ) );
            sendMessage( modbus.write.setup(device) );
            ((Main)activity).updateAutoConn(device.getAutoConnection());
            ((Main)activity).updateDateTimeFormat(dateFormat, timeFormat);
            helper.closeOverlay(overlay);
        });

        Button buttonCancel = pageGeneral.findViewById(R.id.buttonCancel);
        buttonCancel.setOnClickListener(v -> {
            messageID.set(R.string.ready);
            menuOptions.setEnabled(true);
            helper.closeOverlay(overlay);
        });
    }

    @SuppressLint("ClickableViewAccessibility")
    private void pageFilesManager()
    {
        AsyncLayoutInflater inflater = new AsyncLayoutInflater(parent);
        inflater.inflate(R.layout.mpk257_files_manager, overlay, (view, resid, parent) -> pageFiles(view, overlay));
    }

    private void pageFiles(final View view, final FrameLayout overlay)
    {
        overlay.removeAllViews();
        overlay.addView(view);
        overlay.setVisibility(View.VISIBLE);

        final ListView list  = view.findViewById(R.id.list_savedfiles);
        DecimalFormat number = new DecimalFormat("00000");

        final Button
                btDelete = view.findViewById(R.id.bt_delete),
                btOpen   = view.findViewById(R.id.bt_open),
                btNew    = view.findViewById(R.id.bt_new),
                btRename = view.findViewById(R.id.bt_rename),
                btClose  = view.findViewById(R.id.bt_close),
                btExport = view.findViewById(R.id.bt_export);

        btDelete.setEnabled(false);
        btOpen.setEnabled(false);
        btClose.setEnabled(false);
        btExport.setEnabled(false);

        btNew.setVisibility(View.GONE);
        btRename.setVisibility(View.GONE);

        final List<FileDetails> fileList = database.getSavedFilesList(serial, device.getDateFormat(), device.getTimeFormat());
        int selectedListIndex = -1;

        final String[] nameList;
        if ( fileList!=null && fileList.size()>0 )
        {
            nameList = new String[fileList.size()];
            for(int i=0; i<fileList.size(); i++)
            {
                nameList[i] = activity.getString(R.string.test_number) + " " + number.format( Integer.parseInt(fileList.get(i).INDEX) ) + " - " + fileList.get(i).DATE;

                // Already opened
                if ( testData!=null &&
                     testData.getSerial().equals(fileList.get(i).SERIAL) &&
                     (testData.getTestNumber()+"").equals(fileList.get(i).INDEX) &&
                        formatData.getDateFormatted(testData.getDate(), device.getDateFormat(), device.getTimeFormat()).equals(fileList.get(i).DATE))
                {
                    selectedListIndex = i;
                    nameList[i] += "  \u2705"; // ✅⚐ ✓✅◄ ☚ \u2611 \u261C \uD83D\uDCC2";
                    btOpen.setEnabled( false );
                }
            }

            final ArrayAdapter<String> adapter = new ArrayAdapter<>(activity,
                    android.R.layout.simple_list_item_activated_1, android.R.id.text1, nameList);

            list.setAdapter    (adapter);
            list.setChoiceMode (ListView.CHOICE_MODE_SINGLE);
            list.setItemChecked(selectedListIndex, true);
            list.setSelection  (selectedListIndex);

            if(selectedListIndex<0)
            {
                btDelete.setEnabled(false);
                btOpen.setEnabled(false);
                btClose.setEnabled(false);
                btExport.setEnabled(false);
            }
            else
            {
                btDelete.setEnabled(true);
                btOpen.setEnabled(false);
                btClose.setEnabled(true);
                btExport.setEnabled(true);
            }

            list.setOnItemClickListener((parent, view1, position, id) -> {
                btDelete.setEnabled(true);
                btExport.setEnabled(true);
                btOpen.setEnabled(true);
                btClose.setEnabled(false);
                if ( fileList.size()>0 )
                {
                    if ( testData!=null )
                    {
                        // Already opened
                        if ( (testData.getTestNumber()+"").equals( fileList.get(position).INDEX ) &&
                                formatData.getDateFormatted(testData.getDate(), device.getDateFormat(), device.getTimeFormat())
                                        .equals(fileList.get(position).DATE) )
                        {
                            btOpen.setEnabled(false);
                            btClose.setEnabled(true);
                        }
                    }
                }
            });
        }
        else
        {
            list.setAdapter(null);
        }

        btDelete.setOnClickListener(v ->
        {
            final Dialog dialog = new Dialog(activity);
            dialog.requestWindowFeature(Window.FEATURE_NO_TITLE);
            dialog.setCanceledOnTouchOutside(false);
            dialog.setContentView(R.layout.dialog_yes_no_white);
            if (dialog.getWindow()!=null)
                dialog.getWindow().getAttributes().width = ViewPager.LayoutParams.MATCH_PARENT;
            ((TextView) dialog.findViewById(R.id.dlg_title)).setText(R.string.erase);
            ((ImageView) dialog.findViewById(R.id.dlg_icon)).setImageResource(R.drawable.ic_warning);

            final TextView message = dialog.findViewById(R.id.message);
            message.setText(R.string.file_delete_all_confirm);

            dialog.findViewById(R.id.btn_no).setOnClickListener(v1 -> dialog.dismiss());

            dialog.findViewById(R.id.btn_yes).setOnClickListener(v12 ->
            {
                int selectedIndex = list.getCheckedItemPosition();
                if(selectedIndex > -1)
                {
                    if ( fileList==null )
                        return;

                    database.deleteTable( fileList.get(selectedIndex).FILENAME );
                    if ( testData!=null && testData.getSerial().equals(fileList.get(selectedIndex).SERIAL) && (testData.getTestNumber()+"").equals(fileList.get(selectedIndex).INDEX) )
                    {
                        testData=null;
                        enableToolBarWork(false);
                        pagerAdapter.resetAll(isConnected.get(), device, firmware);
                    }
                }
                dialog.dismiss();
                helper.closeOverlay(overlay);
                pageFiles(view, overlay);
            });

            dialog.show();
        });

        btOpen.setOnClickListener(v ->
        {
            int selectedIndex = list.getCheckedItemPosition();
            if ( selectedIndex>-1 )
            {
                testData=null;

                if ( fileList==null )
                    return;

                testData = database.getResult( fileList.get(selectedIndex).FILENAME );

                if ( testData!=null && testData.getDevice()!=null )
                {
                    testData.setDevice(device);
                }

                if ( testData==null )
                    return;

                enableToolBarWork(true);

                device.setFirmware( testData.getFirmware() );
                device.switchId( DeviceList.getIdByModel( model ) );
                pagerAdapter.showChartResistance(testData.getFirmware() >= FIRMWARE_OHMS && testData.getFirmware() < FIRMWARE_HIDE_OHMS);
                viewPager.setAdapter(pagerAdapter);
                tabLayout.setViewPager(viewPager);
                pagerAdapter.resetAll(isConnected.get(), device, firmware);
                pagerAdapter.showResistanceFields(testData.getFirmware() >= FIRMWARE_OHMS && testData.getFirmware() < FIRMWARE_HIDE_OHMS);
                pagerAdapter.fillChartsAndTable(testData, testData.getFirmware(), isConnected.get());
                pagerAdapter.setPITable(testData.getPI());
                viewPager.setCurrentItem(pagerAdapter.getCount()-1, true);
                buttonSave.setEnabled(false);
            }
            helper.closeOverlay(overlay);
        });

        btClose.setOnClickListener(v ->
        {
            list.clearChoices();
            testData=null;
            enableToolBarWork(false);
            pagerAdapter .resetAll(isConnected.get(), device, firmware);
            btClose      .setEnabled(false);
            btOpen       .setEnabled(false);
            btDelete     .setEnabled(false);
            helper       .closeOverlay(overlay);
            pageFiles(view, overlay);
        });

        btExport.setOnClickListener(v -> {
            selectedIndex = list.getCheckedItemPosition();
            if(Main.permissionToMemoryAccepted)
                export(selectedIndex);
            else
                ((Main)activity).requestPermission(Main.REQUEST_MEMORY_REPORT_PERMISSION);

        });

        view.findViewById(R.id.bt_cancel).setOnClickListener(v -> helper.closeOverlay(overlay));
    }

    private void export(int index)
    {
        if( index > -1 )
        {
            final List<FileDetails> fileList = database.getSavedFilesList(serial, device.getDateFormat(), device.getTimeFormat());
            if ( fileList == null )
                return;

            final TestDataAndroid exportData = database.getResult( fileList.get(index).FILENAME );
            if ( exportData == null )
                return;

            exportData.getDevice().setDateFormat(device.getDateFormat());
            exportData.getDevice().setTimeFormat(device.getTimeFormat());
            pagerAdapter.showChartResistance( exportData.getFirmware() >= FIRMWARE_OHMS && exportData.getFirmware() < FIRMWARE_HIDE_OHMS );
            viewPager.setAdapter( pagerAdapter );
            tabLayout.setViewPager( viewPager );
            pagerAdapter.fillChartsAndTable(exportData, exportData.getFirmware(), isConnected.get());

            if( dlgProgress == null )
                creatDialogProgress();
            dlgProgress.show();

            new Thread(() ->
            {
                if ( pagerAdapter!=null )
                {
                    try
                    {
                        Thread.sleep(500);
                    }
                    catch (InterruptedException e)
                    {
                        e.printStackTrace();
                    }

                    File voltage    = helper.saveChartImage(pagerAdapter.chartVoltage(), database.getTemporaryFolder());
                    File current    = helper.saveChartImage(pagerAdapter.chartCurrent(), database.getTemporaryFolder());
                    File ohms       = null;


                    if ( exportData.getFirmware() >= FIRMWARE_OHMS && exportData.getFirmware() < FIRMWARE_HIDE_OHMS )
                    {
                        ohms = helper.saveChartImage( pagerAdapter.chartOhms(), database.getTemporaryFolder() );
                    }

                    report.saveDocReport(exportData,
                            database.getPictureFile(exportData.getSerial()    , exportData.getDate().getTime()+""),
                            database.getMapSnapshotFile(exportData.getSerial(), exportData.getDate().getTime()+""),
                            voltage,
                            current,
                            ohms
                    );
                }
            }).start();
            closeOverlay(activity);
        }
    }

    /***********************************************************************************************
     * REGION State machine
     ***********************************************************************************************/

    public void stateChanged(int state)
    {
        switch (state)
        {
            case BluetoothClassic.STATE_CONNECTED:
                isConnected.set(true);
                break;
            case BluetoothClassic.STATE_CONNECTING:
                break;
            case BluetoothClassic.STATE_NOT_RESPONDING:
                break;
            case BluetoothClassic.STATE_FAIL:
                isConnected.set(false);
                break;
            case BluetoothClassic.STATE_LOST:
                isConnected.set(false);
                break;
            case BluetoothClassic.STATE_CLOSED:
                isConnected.set(false);
                break;
        }
    }

    @Override
    public void update(Observable observable, Object arg1)
    {
        if (observable instanceof BooleanObservable)
        {
            BooleanObservable bol = (BooleanObservable) observable;
            if (bol.ID == isConnected.ID)
            {
                if (bol.get())
                {
                    messageID.set(R.string.read_general_params);
                    initialConfig = true;
                    sendMessage(modbus.read.memoryMap());
                }
                else
                {
                    helper.closeOverlay(overlay);
                    if ( buttonManage != null )
                        buttonManage.setEnabled( true );
                    enableCommandButtons(false);
                    if (pagerAdapter!=null && device!=null)
                        pagerAdapter.resetMainScreen(isConnected.get(), device, firmware);
                    if (dialogFinished!=null && dialogFinished.isShowing())
                    {
                        ((Button)dialogFinished.findViewById(R.id.btn_yes)).setText(R.string.ok);
                        dialogFinished.findViewById(R.id.btn_yes).setEnabled(true);
                    }
                }
            }
            else if (bol.ID == isTestStarting.ID)
            {
                if (bol.get())
                {
                    sendMessage(modbus.read.testNumber());
                    pagerAdapter.resetAll(isConnected.get(), device, firmware);
                    buttonStart.setVisibility(View.GONE);
                    buttonStop.setVisibility(View.VISIBLE);
                    buttonSetup.setEnabled(false);
                    enableToolBar(false);
                    messageID.set(R.string.md_warning_test_start);
                    if (menuOptions!=null)
                        menuOptions.setEnabled(false);
                }
                else
                {
                    if (!isTestRunning.get() && buttonStop.getVisibility()==View.VISIBLE) // Test interrupted before Flag Test Running
                    {
                        if (!isAutoPilot.get())
                        {
                            buttonStart.setEnabled(true);
                        }
                        isTestRunning.update(false);
                        dialogTestFinished();
                        if (menuOptions!=null)
                            menuOptions.setEnabled(true);
                    }
                }
            }
            else if (bol.ID == isTestRunning.ID)
            {
                if (bol.get())
                {
                    if (pagerAdapter!=null)
                        pagerAdapter.hidePI();
                    buttonStart.setVisibility(View.GONE);
                    buttonStop.setVisibility(View.VISIBLE);
                    buttonSetup.setEnabled(false);
                    if ( device.getOperationMode() == Device.MODE_MANUAL && !isAutoPilot.get() )
                        buttonVoltage.setEnabled(true);
                    enableToolBar(false);
                    if (menuOptions!=null)
                        menuOptions.setEnabled(false);
                }
                else
                {
                    buttonStart.setVisibility(View.VISIBLE);
                    buttonStop.setVisibility(View.GONE);
                    buttonVoltage.setEnabled(false);
                    if (!isAutoPilot.get())
                        buttonSetup.setEnabled(true);
                    else
                        messageID.set(R.string.auto_pilot);

                    if (buttonManage!=null)
                        buttonManage.setEnabled(true);
                    if ( buttonSave!=null )
                    {
                        if ( testData!=null && testData.getMeasurement().size()>0 )
                        {
                            buttonSave.setEnabled(true);
                        }
                        else
                            buttonSave.setEnabled(false);
                    }
                    if (menuOptions!=null)
                        menuOptions.setEnabled(true);
                }
            }
            else if ( bol.ID == isAutoPilot.ID )
            {
                if ( bol.get() )
                {
                    if (device!=null)
                        helper.warnAboutLock(device.getStatusOne(Device.ST_ONE_SPARK_DETECTED));
                    else
                        helper.warnAboutLock(false);
                    messageID.set(R.string.auto_pilot);
                    enableCommandButtons(false);
                    buttonStop.setEnabled(true);
                    buttonVoltage.setEnabled(false);
                }
                else
                {
                    buttonStart.setEnabled(true);
                    buttonSetup.setEnabled(true);
//                    if (device!=null && device.getOperationMode()==Protocol.MODE_MANUAL)
//                        buttonVoltage.setEnabled(true);
                }
            }
            else if ( bol.ID == isInternalError.ID )
            {
                if ( bol.get() )
                {
                    helper.warnInternalError();
                    buttonStart.setEnabled( false );
                    buttonSetup.setEnabled( false );
//                    if ( device!=null && device.getOperationMode()==Protocol.MODE_MANUAL )
//                        buttonVoltage.setEnabled( false );
                }
                else
                {
                    buttonStart.setEnabled( true );
                    buttonSetup.setEnabled( true );
//                    if ( device!=null && device.getOperationMode() == Protocol.MODE_MANUAL )
//                        buttonVoltage.setEnabled( true );
                }
            }
        }
        else if (observable instanceof com.megabras.modbus.IntegerObservable)
        {
            com.megabras.modbus.IntegerObservable ob = (com.megabras.modbus.IntegerObservable) observable;
            switch (ob.ID)
            {
                case Device.ID_OBSERVABLE_MEMORY:
                    if ( pageGeneral!=null )
                        ((TextView) pageGeneral.findViewById(R.id.device_memory)).setText( device.getFreeMemory()+"%" );
                    break;
                case Device.ID_OBSERVABLE_MODE:
                    pagerAdapter.resetMainScreen(isConnected.get(), device, firmware);
//                    if (device.getOperationMode()==Protocol.MODE_MANUAL)
//                        buttonVoltage.setEnabled(true);
//                    else
//                        buttonVoltage.setEnabled(false);
//                    break;
            }
        }
        else if (observable instanceof IntegerObservable)
        {
            IntegerObservable ob = (IntegerObservable) observable;
            if (ob.ID == messageID.ID)
            {
                switch (ob.get())
                {
                    case R.string.read_general_params:
                        userMessage(ob.get(), CommonUI.COLOR_YELLOW, true);
                        break;
                    case R.string.firmware_was_read:
                        userMessage(ob.get(), CommonUI.COLOR_YELLOW, true);
                        break;
                    case R.string.memory_map_was_read:
                        userMessage(ob.get(), CommonUI.COLOR_YELLOW, true);
                        break;
                    case R.string.md_test_running:
                        userMessage(ob.get(), CommonUI.COLOR_RED, true);
                        break;
                    case R.string.md_warning_test_start:
                        userMessage(ob.get(), CommonUI.COLOR_YELLOW, true);
                        break;
                    case R.string.md_configurations_changed:
                        userMessage(ob.get(), CommonUI.COLOR_YELLOW, true);
                        break;
                    case R.string.operation_mode_was_read:
                        userMessage(ob.get(), CommonUI.COLOR_YELLOW, true);
                        break;
                    case R.string.open_port:
                        userMessage(ob.get(), CommonUI.COLOR_GREEN, false);
                        break;
                    case R.string.ready:
                        userMessage(ob.get(), CommonUI.COLOR_GREEN, false);
                        break;
                    case R.string.md_test_finished:
                        userMessage(ob.get(), CommonUI.COLOR_GREEN, false);
                        break;
                    case R.string.test_cancelled:
                        userMessage(ob.get(), CommonUI.COLOR_YELLOW, false);
                        break;
                    case R.string.test_voltage_was_read:
                        userMessage(ob.get(), CommonUI.COLOR_YELLOW, true);
                        break;
                    case R.string.updating:
                        userMessage(ob.get(), CommonUI.COLOR_YELLOW, true);
                        break;
                    case R.string.generation_lock:
                        userMessage(ob.get(), CommonUI.COLOR_YELLOW, false);
                        break;
                    case R.string.emergency_stop:
                        userMessage(ob.get(), CommonUI.COLOR_YELLOW, false);
                        break;
                    case R.string.spark_detection:
                        userMessage(ob.get(), CommonUI.COLOR_YELLOW, false);
                        break;
                    case R.string.service_center:
                        userMessage(ob.get(), CommonUI.COLOR_YELLOW, false);
                        break;
                    case R.string.auto_pilot:
                        userMessage(ob.get(), CommonUI.COLOR_YELLOW, false);
                        break;
                }
            }
        }
    }

    private void dialogTestFinished()
    {
        String results;

        if ( device.getStatusOne(6) && device.getStatusOne(7) )
            results = activity.getString(R.string.result).toUpperCase() + " : " + activity.getString(R.string.pass);
        else if ( !device.getStatusOne(6) && device.getStatusOne(7) )
            results = activity.getString(R.string.result).toUpperCase() + " : " + activity.getString(R.string.fail);
        else if ( device.getStatusOne(6) ) // else if ( statusOne[2] && !statusOne[3] )
            results = activity.getString(R.string.result).toUpperCase() + " : " + activity.getString(R.string.aborted);
        else
            results = activity.getString(R.string.result).toUpperCase() + " : " + " - - - ";

        if ( device.getStatusOne(4) )
            results += "\r\n" + activity.getString(R.string.reason).toUpperCase() + " : " + activity.getString(R.string.spark_detection).toUpperCase();
        else if (device.getStatusOne(5))
            results += "\r\n" + activity.getString(R.string.reason).toUpperCase() + " : " + activity.getString(R.string.overcurrent).toUpperCase();

        if ( dialogFinished == null )
        {
            dialogFinished = new Dialog( activity );
            dialogFinished.requestWindowFeature( Window.FEATURE_NO_TITLE );
            dialogFinished.setCanceledOnTouchOutside( false );
            dialogFinished.setCancelable( false );
            dialogFinished.setContentView( R.layout.dialog_message_white );
            if ( dialogFinished.getWindow() != null )
            {
                dialogFinished.getWindow().setBackgroundDrawable( new ColorDrawable( Color.TRANSPARENT ) );
            }
            ((TextView) dialogFinished.findViewById( R.id.dlg_title )).setText( R.string.md_test_finished );
            ((ImageView) dialogFinished.findViewById( R.id.dlg_icon )).setImageResource( R.drawable.ic_warning );
            ((TextView) dialogFinished.findViewById( R.id.observation )).setText( "" );
            dialogFinished.findViewById( R.id.ui_iv_warning ).setVisibility( View.GONE );
            dialogFinished.findViewById( R.id.btn_yes ).setOnClickListener( v -> dialogFinished.dismiss() );
            dialogFinished.findViewById( R.id.btn_close_dlg ).setVisibility( View.GONE );
        }
        ((TextView) dialogFinished.findViewById( R.id.message )).setText( results );
        dialogFinished.show();
    }

    public void processResponse(int id, String rx)
    {
        switch (id)
        {
            case Protocol.ID_REAL_TIME_READ:
                if (!isTestStarting.get() && !isTestRunning.get() && !isAutoPilot.get())
                    messageID.set(R.string.ready);

                device.setStatusOne( modbus.parse.statusOne(rx) );
                device.setStatusTwo( modbus.parse.statusTwo(rx) );
                pagerAdapter.updateStatus( modbus.parse.statusOne(rx) );

                if ( device.getStatusOne(Device.ST_ONE_CONFIG_CHANGED) )
                {
                    helper.closeOverlay(overlay);
                    messageID.set( R.string.md_configurations_changed );
                    sendMessage( modbus.write.resetConfigChanged() );
                }

                if ( device.getStatusOne(Device.ST_ONE_GENERATION_LOCK) )
                {
                    messageID.set( R.string.generation_lock );
                    if (buttonStart.isEnabled())
                        buttonStart.setEnabled(false);
                }
                else
                {
                    if (!buttonStart.isEnabled() && updated_overcurrent)
                        buttonStart.setEnabled(true);
                    if (!buttonStart.isEnabled() && !isTestRunning.get() && !isAutoPilot.get() && !isInternalError.get())
                        buttonStart.setEnabled(true);
                }

                if ( device.getStatusOne(Device.ST_ONE_EMERGENCY_STOP) )
                {
                    messageID.set( R.string.emergency_stop );
                }

                if ( device.getStatusOne(Device.ST_ONE_INTERNAL_PROTECTION) )
                {
                    messageID.set( R.string.service_center );
                }

                if ( device.getStatusOne(Device.ST_ONE_IP_CALCULATION) &&
                     firmware >= FIRMWARE_OHMS &&
                     firmware < FIRMWARE_HIDE_OHMS &&
                     testData!=null &&
                     testData.getPI()<0 &&
                     testData.getMeasurement().size()>0 &&
                     testData.getMeasurementLast().getElapsed()>=600)
                {
                    sendMessage( modbus.read.pi() );
                }

                PACK_SIZE = 10;
                if ( isTestRunning.get() )
                {
                    if ( testData == null )
                    {
                        isTestStarting.set( true );
                        break;
                    }

                    if ( device.getOperationMode() == Device.MODE_RAMP )
                        PACK_SIZE = 14;

                    if ( rx.length()>=(PACK_SIZE*4) ) // This is to be sure that it's not a SHORT STATUS message
                    {
                        Measurement m = modbus.parse.statusMeasurement(rx);
                        pagerAdapter.updateMainScreen(m, device, firmware);
                        int initialTime = 15;
                        if ( device.getOperationMode() == Device.MODE_RAMP )
                            initialTime = 0;
                        if (m.getElapsed()>=initialTime && m.getElapsed()%5==0)
                        {
                            if(testData.getMeasurement().size()>0)
                            {
                                if ( testData.getMeasurementLast().getElapsed()==m.getElapsed() )
                                    testData.getMeasurement().remove( testData.getMeasurement().size()-1 );
                            }
                            // Prevent last voltage peak in Ramp mode
                            if (device.getOperationMode()==Device.MODE_RAMP && device.getRampStage()==Device.RAMP_FINISHED)
                            {
                                if (m.getVoltage()<testData.getMeasurementLast().getVoltage() || m.getCurrent()<testData.getMeasurementLast().getCurrent())
                                {
                                    testData.addMeasument(m);
                                    pagerAdapter.updateChartsAndTable(m, testData.getFirmware(), false);
                                }
                            }
                            else
                            {
                                testData.addMeasument(m);
                                pagerAdapter.updateChartsAndTable(m, testData.getFirmware(), false);
                            }
                        }
                    }

                    messageID.set( R.string.md_test_running );
                    sendMessage( modbus.read.status(PACK_SIZE) );
                    updated_overcurrent = false;
                }
                else
                {
                    if ( rx.length()>=(PACK_SIZE*4) )
                    {
                        Measurement m = modbus.parse.statusMeasurement(rx);
                        pagerAdapter.updateMainScreen(m, device, firmware);

                        String result = "";

                        result += device.getStatusOne(7) ? "1" : "0";
                        result += device.getStatusOne(6) ? "1" : "0";
                        result += device.getStatusOne(5) ? "1" : "0";
                        result += device.getStatusOne(4) ? "1" : "0";

                        testData.setTestResult( Integer.parseInt(result, 2) );
                        pagerAdapter.setTestResult( Integer.parseInt(result, 2) );

                        if ( m.getElapsed()>=5) // LAST MEASUREMENT, AFTER STOPPED TEST
                        {
                            if (device.getOperationMode()!=Device.MODE_RAMP || testData.getStatusSpark() || testData.getStatusOvercurrent())
                            {
                                if (testData.getMeasurement().size() > 0)
                                {
                                    if (testData.getMeasurementLast().getElapsed() == m.getElapsed())
                                        testData.getMeasurement().remove(testData.getMeasurement().size() - 1);
                                }
                                testData.addMeasument(m);
                                pagerAdapter.updateChartsAndTable(m, testData.getFirmware(), true);
                            }
                        }

                        if (device.getStatusOne(Device.ST_ONE_OVERCURRENT) && !updated_overcurrent)
                        {
                            pagerAdapter.waitonDisplay();
                            buttonStart.setEnabled(false);
                            handler.postDelayed(() ->
                            {
                                sendMessage(modbus.read.status(PACK_SIZE));
                                updated_overcurrent=true;
                            }, 2500);
                        }
                        else
                        {
                            dialogTestFinished();
                            if (!isAutoPilot.get())
                                buttonStart.setEnabled(true);
                            // This is in the case of a test FAIL right in the beginning
                            if (testData!=null && testData.getMeasurement().size()>0 && !buttonSave.isEnabled())
                                buttonSave.setEnabled(true);
                        }

                        if ( firmware >= FIRMWARE_DISCHARGE )
                        {
                            waitDischarge = true;
                            if ( dialogFinished != null )
                            {
                                dialogFinished.findViewById( R.id.ui_iv_warning ).setVisibility( View.VISIBLE );
                                ((TextView) dialogFinished.findViewById( R.id.dlg_title )).setText( R.string.discharging_potential );
                                ((Button) dialogFinished.findViewById( R.id.btn_yes )).setText( R.string.wait );
                                dialogFinished.findViewById( R.id.btn_yes ).setEnabled( false );
                            }
                        }
                    }
                    if ( waitDischarge )
                    {
                        handler.postDelayed( () -> sendMessage( modbus.read.voltageDischarge() ), 3000 );
                        dialogTestFinished();
                    }
                }
                isTestRunning.set   ( device.getStatusOne(Device.ST_ONE_RUNNING)        );
                isTestStarting.set  ( device.getStatusOne(Device.ST_ONE_STARTING)       );
                isAutoPilot.set     ( device.getStatusOne(Device.ST_ONE_AUTO_PILOT)     );
                isInternalError.set ( device.getStatusTwo(Device.ST_TWO_INTERNAL_ERROR) );
                break;
            case Protocol.ID_DISCHARGE_READ:
                double discharge = modbus.parse.voltageDischarge(rx);
                if ( discharge > 100 )
                {
                    sendMessage(modbus.read.voltageDischarge());
                    if ( dialogFinished != null )
                    {
                        String result = activity.getString(R.string.potential).toUpperCase() + ": " + formatData.dischargeVoltage(discharge);
                        ((TextView) dialogFinished.findViewById(R.id.observation)).setText(result);

                        if (dialogFinished.findViewById(R.id.ui_iv_warning).getVisibility()!=View.VISIBLE)
                           dialogFinished.findViewById(R.id.ui_iv_warning).setVisibility(View.VISIBLE);
                        else
                           dialogFinished.findViewById(R.id.ui_iv_warning).setVisibility(View.INVISIBLE);
                    }
                }
                else
                {
                    waitDischarge = false;
                    if ( dialogFinished != null )
                    {
                        String result = activity.getString(R.string.potential).toUpperCase() + ": " + "< 100 V" ;
                        ((TextView) dialogFinished.findViewById(R.id.observation)).setText(result);
                        dialogFinished.findViewById(R.id.ui_iv_warning).setVisibility(View.INVISIBLE);
                        ((TextView) dialogFinished.findViewById(R.id.dlg_title)).setText(R.string.md_test_finished);
                        ((Button)dialogFinished.findViewById(R.id.btn_yes)).setText(R.string.ok);
                        dialogFinished.findViewById(R.id.btn_yes).setEnabled(true);
                    }
                }
                break;
            case Protocol.ID_PI_READ:
                double pi = modbus.parse.pi(rx);
                if (pagerAdapter!=null)
                    pagerAdapter.setPI(pi);
                if(testData!=null)
                    testData.setPI(pi);
                break;
            case Protocol.ID_MEMORY_MAP_READ:
                messageID.set(R.string.memory_map_was_read);
                memoryMap = modbus.parse.memoryMap(rx);
                Modbus.setRevisionMap(memoryMap);
                sendMessage(modbus.read.hardware());
                break;
            case Protocol.ID_HARDWARE_READ:
                hardware = modbus.parse.hardware(rx);
                sendMessage(modbus.read.firmware());
                break;
            case Protocol.ID_FIRMWARE_READ:
                messageID.set(R.string.firmware_was_read);
                firmware = modbus.parse.firmware(rx);
                device.setFirmware( firmware );
                device.switchId( DeviceList.getIdByModel( model ) );
                pagerAdapter.showChartResistance(firmware >= FIRMWARE_OHMS && firmware < FIRMWARE_HIDE_OHMS);
                viewPager.setAdapter(pagerAdapter);
                tabLayout.setViewPager(viewPager);
                pagerAdapter.resetAll(isConnected.get(), device, firmware);
                pagerAdapter.showResistanceFields(firmware >= FIRMWARE_OHMS  && firmware < FIRMWARE_HIDE_OHMS);

                if (testData!=null)
                    pagerAdapter.fillChartsAndTable(testData, firmware, isConnected.get());

                if ( DEVICE_ID == DeviceList.ID_HP60KV && firmware < Device.FIRMWARE_MIN_VERSION )
                {
                    final Dialog dialog = new Dialog(activity);
                    dialog.requestWindowFeature(Window.FEATURE_NO_TITLE);
                    dialog.setCanceledOnTouchOutside(false);
                    dialog.setContentView(R.layout.dialog_yes_no_white);
                    if (dialog.getWindow()!=null)
                        dialog.getWindow().getAttributes().width = ViewPager.LayoutParams.MATCH_PARENT;
                    ((TextView) dialog.findViewById(R.id.dlg_title)).setText(R.string.warning);
                    ((ImageView) dialog.findViewById(R.id.dlg_icon)).setImageResource(R.drawable.ic_warning);

                    final TextView message = dialog.findViewById(R.id.message);
                    message.setText( parent.getString(R.string.equipment_firmware_update, parent.getString(R.string.company)));

                    Button buttonNo = dialog.findViewById(R.id.btn_no);
                    buttonNo.setVisibility(View.GONE);

                    final Button buttonYes = dialog.findViewById(R.id.btn_yes);
                    buttonYes.setText(R.string.ok);
                    buttonYes.setOnClickListener(v -> dialog.dismiss());

                    dialog.show();

                    parent.closeConnection();
                }
                else
                {
                    sendMessage(modbus.read.voltage());
                    sendMessage(modbus.read.manualParams());
                    sendMessage(modbus.read.svtParams());
                    sendMessage(modbus.read.rampParams());
                    sendMessage(modbus.read.setup());
                }
                break;
            case Protocol.ID_MEM_POINTER_READ:
                device.setFreeMemory( modbus.parse.memoryFree(rx) );
                break;
            case Protocol.ID_MEM_POINTER_WRITE:
                sendMessage( modbus.read.memoryPointer() );
                break;
            case Protocol.ID_DATE_READ:
                device.setDate( modbus.parse.date( rx ) );
                break;
            case Protocol.ID_DATE_WRITE:
                break;
            case Protocol.ID_GENERAL_SETUP_READ:
                messageID.set(R.string.operation_mode_was_read);
                modbus.parse.setup(device, rx);
                if ( !initialConfig )
                    pageGeneralSetup();
                else
                {
                    initialConfig = false;
                    enableCommandButtons(true);
                }
                pagerAdapter.setTestParameters(device, firmware);
                break;
            case Protocol.ID_GENERAL_SETUP_WRITE:
                messageID.set(R.string.ready);
                menuOptions.setEnabled(true);
                break;
            case Protocol.ID_TEST_VOLTAGE_READ:
                messageID.set(R.string.test_voltage_was_read);
                android.util.Log.e( TAG, "V = " + rx );
                android.util.Log.e( TAG, "V = " + modbus.parse.voltage(rx) );
                device.setTestVoltage(modbus.parse.voltage(rx));
                break;
            case Protocol.ID_MANUAL_PARAMS_READ:
                messageID.set(R.string.manual_parameters_was_read);
                device.setParamManual(modbus.parse.manualParams(rx));
                break;
            case Protocol.ID_SVT_PARAMS_READ:
                messageID.set(R.string.auto_parameters_was_read);
                device.setParamSVT(modbus.parse.svtParams(rx));
                break;
            case Protocol.ID_RAMP_PARAMS_READ:
                messageID.set(R.string.ramp_parameters_was_read);
                device.setParamRamp(modbus.parse.rampParams(rx));
                break;
            case Protocol.ID_PARAMS_MANUAL_WRITE:
                pagerAdapter.setTestParameters(device, firmware);
                break;
            case Protocol.ID_PARAMS_SVT_WRITE:
                pagerAdapter.setTestParameters(device, firmware);
                break;
            case Protocol.ID_PARAMS_RAMP_WRITE:
                pagerAdapter.setTestParameters(device, firmware);
                break;
            case Protocol.ID_TEST_VOLTAGE_WRITE:
                pagerAdapter.setTestParameters(device, firmware);
                break;
            case Protocol.ID_TEST_NUMBER_READ:
                testData = new TestDataAndroid(model, serial);
                testData.setMemoryMap(memoryMap);
                testData.setHardware(hardware);
                testData.setFirmware(firmware);
                testData.setDevice(device);
                testData.setDate(new Date());
                testData.setTestNumber(modbus.parse.testNumber(rx));
                pagerAdapter.resetMainScreen(isConnected.get(), device, firmware);
                pagerAdapter.setTestNumber(testData.getTestNumber());
                String dateTime = formatData.getDateFormatted
                                  (
                                      testData.getDate(),
                                      device.getDateFormat(),
                                      device.getTimeFormat()
                                  );
                pagerAdapter.setTestDate(dateTime);
                pagerAdapter.setTestMode(activity.getResources()
                        .getStringArray(R.array.hp60kv_op_modes)[device.getOperationMode()]);
                device.setTestVoltage(0);
                pagerAdapter.setTestParameters(device, firmware);
                break;
            case Protocol.ID_CONFIG_CHANGE_WRITE:
                initialConfig = true;
                sendMessage( modbus.read.voltage()      );
                sendMessage( modbus.read.manualParams() );
                sendMessage( modbus.read.svtParams()    );
                sendMessage( modbus.read.rampParams()   );
                sendMessage( modbus.read.setup()        );
                break;
        }
    }
}
