package com.megabras.bluelogg.md10kvx;

import android.annotation.SuppressLint;
import android.app.Dialog;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothSocket;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.graphics.Bitmap;
import android.graphics.Bitmap.CompressFormat;
import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.graphics.drawable.Drawable;
import android.graphics.drawable.LayerDrawable;
import android.os.Bundle;
import android.os.CountDownTimer;
import android.os.Environment;
import android.os.Handler;
import android.os.Message;
import android.os.SystemClock;
import android.text.SpannableString;
import android.text.SpannableStringBuilder;
import android.text.style.AbsoluteSizeSpan;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.MotionEvent;
import android.view.View;
import android.view.Window;
import android.view.WindowManager.LayoutParams;
import android.widget.Button;
import android.widget.Chronometer;
import android.widget.ImageButton;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;

import com.megabras.bluelogg.Command;
import com.megabras.bluelogg.MUtils;
import com.megabras.bluelogg.R;
import com.megabras.bluelogg.RecordAudio;
import com.megabras.bluelogg.RecordEventListener;
import com.megabras.bluelogg.bluetooth.EnableBluetooth;
import com.megabras.bluelogg.extended.GPS;
import com.megabras.bluelogg.extended.GPS.GpsInterface;
import com.megabras.bluelogg.extended.Picture;
import com.megabras.bluelogg.extended.SlidingTabLayout;
import com.megabras.bluelogg.extended.Units;
import com.megabras.bluelogg.md10kvx.MD10KVxTest.Result;

import java.io.File;
import java.io.FileOutputStream;
import java.lang.ref.WeakReference;
import java.math.RoundingMode;
import java.text.DateFormat;
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.LinkedList;
import java.util.Locale;
import java.util.Queue;
import java.util.UUID;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;
import androidx.core.app.ActivityCompat;
import androidx.core.content.ContextCompat;
import androidx.core.content.res.ResourcesCompat;
import androidx.viewpager.widget.ViewPager;

@SuppressWarnings("SetTextI18n,FieldCanBeLocal")
public class MD10KVx extends AppCompatActivity implements GpsInterface, RecordEventListener, Picture.PictureInterface {

	/***********************************************************************************************
	 * REGION FIELDS
	 ***********************************************************************************************/

	private final String
			TAG = "MD10KVx";

	private String
			model,
			mac,
			serial,
			rx = "";

	private final UUID
			UUID_SPP = UUID.fromString("00001101-0000-1000-8000-00805F9B34FB");

	private Menu
			menu;

	private MD10KVxConnectThread
			trd;

	private BluetoothSocket
			socket;

	private Runnable
			runFIFO,
			rxTimeOut,
			checkConnection;

	private Queue<Command>
			FIFO = new LinkedList<>(); //First In First Out

	private SpannableStringBuilder
			startSpanString;

	private boolean
			idle = true,
			testStopped = false,
			testRunning = false,
			printerEnabled = false,
			autoConn = true,
			isSvt = false,
			showSetupDialog = false,
			forcedToStop = false,
			warnedOnce = false,
			ignoreChange = false,
			showChart = true,
			showlog  = true;

	private int
			timeoutCount = 0,
			rxTries = 0;

	private final int
			RX_TIMEOUT    = 50,
			//TIMEOUT_TRIES = 5,
			TIMEOUT_TRIES = 30,
			RUNFIFO_TIME  = 50,
			PING_TIME     = 500;

	private Button button_start;
	private Button button_stop;
	private MD10KVxChartInterface chartResistance;
	private MD10KVxChartInterface chartVoltage;
	private MD10KVxChartInterface chartCurrent;
	private MD10KVxChartInterface table;
	//private long millisec = 0;
	private long logmillisec = 0;
	private long lastElapsedTime = 0;
	private MD10KVxUI ui;
	private final mHandler handler = new mHandler(this);
	private short capIsready = 0;
	private short capIsreadyTries = 0;
	private MD10KVxTest test;
	private float lastVoltage = 0;
	private float lastResistance = 0;
	private float lastCurrent = 0;
	private GPS gps;

	private Picture
			picture;

	private RecordAudio
			audio;

	private Dialog
			dlgAudio;

//	private Report
//			report;

	private ReportPDF
			reportPDF;

	private File
			rootDir,
			dir,
			tempDir;

	public static final int
			INITIAL_VALUE = 5000,
			MEASURE_GAP = 5000;

	private boolean checkedFrameType = false;


	public static boolean
			permissionToRecordAccepted  = false,
			permissionToPictureAccepted = false,
			permissionToGPSAccepted     = false,
			permissionToMemoryAccepted  = false;

	public static String []
			permissionsMemory   = {android.Manifest.permission.READ_EXTERNAL_STORAGE, android.Manifest.permission.WRITE_EXTERNAL_STORAGE},
			permissionsAudio    = {android.Manifest.permission.READ_EXTERNAL_STORAGE, android.Manifest.permission.WRITE_EXTERNAL_STORAGE, android.Manifest.permission.RECORD_AUDIO},
			permissionsPicture  = {android.Manifest.permission.READ_EXTERNAL_STORAGE, android.Manifest.permission.WRITE_EXTERNAL_STORAGE, android.Manifest.permission.CAMERA},
			permissionsGPS      = {android.Manifest.permission.ACCESS_COARSE_LOCATION, android.Manifest.permission.ACCESS_FINE_LOCATION};

	public static final int
			REQUEST_RECORD_AUDIO_PERMISSION  = 200,
			REQUEST_PICTURE_PERMISSION       = 201,
			REQUEST_GPS_PERMISSION           = 202,
			REQUEST_MEMORY_PERMISSION        = 203,
			REQUEST_MEMORY_REPORT_PERMISSION = 204,
			REQUEST_MEMORY_MAP_PERMISSION    = 205;

	private void checkPermissions()
	{
		if ( ContextCompat.checkSelfPermission( this, android.Manifest.permission.READ_EXTERNAL_STORAGE ) == PackageManager.PERMISSION_GRANTED &&
				ContextCompat.checkSelfPermission( this, android.Manifest.permission.WRITE_EXTERNAL_STORAGE ) == PackageManager.PERMISSION_GRANTED )
		{
			permissionToMemoryAccepted = true;
		}
		if ( ContextCompat.checkSelfPermission( this, android.Manifest.permission.READ_EXTERNAL_STORAGE ) == PackageManager.PERMISSION_GRANTED &&
				ContextCompat.checkSelfPermission( this, android.Manifest.permission.WRITE_EXTERNAL_STORAGE ) == PackageManager.PERMISSION_GRANTED &&
				ContextCompat.checkSelfPermission( this, android.Manifest.permission.CAMERA ) == PackageManager.PERMISSION_GRANTED )
		{
			permissionToPictureAccepted = true;
		}
		if ( ContextCompat.checkSelfPermission( this, android.Manifest.permission.READ_EXTERNAL_STORAGE ) == PackageManager.PERMISSION_GRANTED &&
				ContextCompat.checkSelfPermission( this, android.Manifest.permission.WRITE_EXTERNAL_STORAGE ) == PackageManager.PERMISSION_GRANTED &&
				ContextCompat.checkSelfPermission( this, android.Manifest.permission.RECORD_AUDIO ) == PackageManager.PERMISSION_GRANTED )
		{
			permissionToRecordAccepted = true;
		}
		if ( ContextCompat.checkSelfPermission( this, android.Manifest.permission.ACCESS_COARSE_LOCATION ) == PackageManager.PERMISSION_GRANTED &&
				ContextCompat.checkSelfPermission( this, android.Manifest.permission.ACCESS_FINE_LOCATION ) == PackageManager.PERMISSION_GRANTED )
		{
			permissionToGPSAccepted = true;
		}
	}

	@Override
	public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
		super.onRequestPermissionsResult(requestCode, permissions, grantResults);
		switch (requestCode){
			case REQUEST_RECORD_AUDIO_PERMISSION:
				permissionToRecordAccepted  = (grantResults[0] == PackageManager.PERMISSION_GRANTED && grantResults[1] == PackageManager.PERMISSION_GRANTED && grantResults[2] == PackageManager.PERMISSION_GRANTED);
				if (permissionToRecordAccepted)
					permissionCallback(requestCode);
				break;
			case REQUEST_PICTURE_PERMISSION:
				permissionToPictureAccepted  = (grantResults[0] == PackageManager.PERMISSION_GRANTED && grantResults[1] == PackageManager.PERMISSION_GRANTED && grantResults[2] == PackageManager.PERMISSION_GRANTED);
				if (permissionToPictureAccepted)
					permissionCallback(requestCode);
				break;
			case REQUEST_GPS_PERMISSION:
				permissionToGPSAccepted = (grantResults[0] == PackageManager.PERMISSION_GRANTED && grantResults[1] == PackageManager.PERMISSION_GRANTED);
				if (permissionToGPSAccepted)
					permissionCallback(requestCode);
				break;
			case REQUEST_MEMORY_PERMISSION:
				permissionToMemoryAccepted = (grantResults[0] == PackageManager.PERMISSION_GRANTED && grantResults[1] == PackageManager.PERMISSION_GRANTED);
				if (permissionToMemoryAccepted)
					permissionCallback(requestCode);
				break;
			case REQUEST_MEMORY_REPORT_PERMISSION:
				permissionToMemoryAccepted = (grantResults[0] == PackageManager.PERMISSION_GRANTED && grantResults[1] == PackageManager.PERMISSION_GRANTED);
				if (permissionToMemoryAccepted)
					permissionCallback(requestCode);
				break;
			case REQUEST_MEMORY_MAP_PERMISSION:
				permissionToMemoryAccepted = (grantResults[0] == PackageManager.PERMISSION_GRANTED && grantResults[1] == PackageManager.PERMISSION_GRANTED);
				if (permissionToMemoryAccepted)
					permissionCallback(requestCode);
				break;
		}
	}

	public void requestPermission(int permission)
	{
		String[] list = null;
		switch (permission)
		{
			case REQUEST_RECORD_AUDIO_PERMISSION:
				list = permissionsAudio;
				break;
			case REQUEST_PICTURE_PERMISSION:
				list = permissionsPicture;
				break;
			case REQUEST_GPS_PERMISSION:
				list = permissionsGPS;
				break;
			case REQUEST_MEMORY_PERMISSION:
				list = permissionsMemory;
				break;
			case REQUEST_MEMORY_REPORT_PERMISSION:
				list = permissionsMemory;
				break;
			case REQUEST_MEMORY_MAP_PERMISSION:
				list = permissionsMemory;
				break;
		}
		if (list!=null)
			ActivityCompat.requestPermissions(this, list, permission);
	}

	public void permissionCallback(int permission)
	{
		switch (permission)
		{
			case REQUEST_RECORD_AUDIO_PERMISSION:
				recordAudio();
				break;
			case REQUEST_PICTURE_PERMISSION:
				picture();
				break;
			case REQUEST_GPS_PERMISSION:
				gps();
				break;
			case REQUEST_MEMORY_REPORT_PERMISSION:
				//report.createFolder();
				this.createFolder();
				export();
				break;
			case REQUEST_MEMORY_PERMISSION:
				break;
			case REQUEST_MEMORY_MAP_PERMISSION:
				break;
		}
	}


	@Override
	protected void onCreate( Bundle savedInstanceState )
	{
		super.onCreate( savedInstanceState );
		try
		{
			chooseLayoutToInflate();
			keepTheScreenOn();

			checkPermissions();

			gps = new GPS( this );
			picture = Picture.getInstance();
			picture.initialize( this, this );
			audio = new RecordAudio();
			audio.setRecordEventListener( this );
			loadExtras();
//			report = new Report( this, handler, model );
			loadRunnables();
			showChart();
			createSwipe();
			loadButtonsFuncions();
			test = new MD10KVxTest( this );
			ui.enableSaveButton( false, test );
			ui.enableGpsButton( false );
			ui.enableRecordButton( false );
			ui.enablePictureButton( false );
			loadDecimalFormats();
			checkedFrameType = false;


			if ( permissionToMemoryAccepted )
			{
				this.createFolder();
			}

			if ( getSupportActionBar() != null )
			{
				getSupportActionBar().setDisplayShowHomeEnabled( true );
				getSupportActionBar().setIcon( R.drawable.brand_small );
				getSupportActionBar().setDisplayShowTitleEnabled( false );
				getSupportActionBar().setElevation( 0 );
			}

			try
			{
				startSpanString = new SpannableStringBuilder();
				SpannableString stringTop = new SpannableString( getResources().getString( R.string.start_keep_pressed ) + "\n" );
				stringTop.setSpan( new AbsoluteSizeSpan( getResources().getDimensionPixelSize( R.dimen.start_stop_text ), false ), 0, stringTop.length(), 0 );
				startSpanString.append( stringTop );

				SpannableString stringBottom = new SpannableString( getResources().getString( R.string.hold_pressed ) );
				stringBottom.setSpan( new AbsoluteSizeSpan( getResources().getDimensionPixelSize( R.dimen.bt_start_bottom ), false ), 0, stringBottom.length(), 0 );
				startSpanString.append( stringBottom );
				((Button) findViewById( R.id.bt_start )).setText( startSpanString, TextView.BufferType.SPANNABLE );
			}
			catch ( Exception ignored )
			{
			}
		}
		catch ( Exception e )
		{
			error( "onCreate", e );
		}
	}

	private void chooseLayoutToInflate() {
		setContentView( R.layout.md10kvx_main );
//		Display display = getWindowManager().getDefaultDisplay();
//		Point p = new Point();
//		display.getSize(p);
//		int screenWidth = p.x;
//
//		if (screenWidth < 540) {
//			setContentView(R.layout.md10kvx_main_480p);
//		} else {
//			setContentView(R.layout.md10kvx_main);
//		}
	}

	@Override
	protected void onStart() {
		super.onStart();
	}

	@Override
	protected void onDestroy()
	{
		super.onDestroy();
		try
		{
			if ( socket != null )
			{
				socket.close();
			}
			if ( gps != null )
			{
				gps.cancel();
			}
			if ( picture != null )
			{
				picture.close();
			}
		}
		catch ( Exception e )
		{
			error( "onDestroy", e );
		}
	}

	@Override
	public void onBackPressed() {
		ui.confirmExit(this);
	}

	@Override
	public void onPause() {
		super.onPause();
		if (this.testRunning) {
			testStopped = true;
			forcedToStop = true;
		}
	}

	@Override
	public void onResume() {
		super.onResume();
		if (forcedToStop) {
			ui.warnAboutPause();
			forcedToStop = false;
		}
	}

	/**
	 * Create the header menu
	 */
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		try {
			getMenuInflater().inflate(R.menu.md10kvx_menu, menu);
			this.menu = menu;
			changeAlpha(1, false);
			changeAlpha(2, false);
			changeAlpha(3, false);
			isConnected(false);
			ui.messageBox("", MD10KVxUI.COLOR_BLACK, false);
			//calculateFontSize();
			if (autoConn) {
				this.connecting();
			}
			return true;
		} catch (Exception e) {
			error("onCreateOptionsMenu", e);
			return false;
		}
	}

//	public Dialog getDialogModal() {
//		Dialog dialog = new Dialog(this);
//		dialog.requestWindowFeature(Window.FEATURE_NO_TITLE);
//		dialog.getWindow().getAttributes().height = LayoutParams.WRAP_CONTENT;
//		dialog.getWindow().getAttributes().width = LayoutParams.WRAP_CONTENT;
//		dialog.getWindow().setBackgroundDrawable(new ColorDrawable(Color.TRANSPARENT));
//		dialog.setCanceledOnTouchOutside(false);
//		return dialog;
//	}

//	private void calculateFontSize() {
//		try {
//			TableRow container =
//					this.findViewById(R.id.container_resistance);
//			TextView label =
//					this.findViewById(R.id.lb_resistance);
//
//			String text = "000.00 MO";
//			label.setText(text);
//			label.measure(MeasureSpec.UNSPECIFIED, label.getHeight());
//
//			for (int i = 1; label.getMeasuredWidth() < container.getWidth() - 10 && label.getMeasuredHeight() < container.getHeight() - 5; i++) {
//				label.setTextSize(TypedValue.COMPLEX_UNIT_PX, label.getTextSize() + 1);
//				label.measure(MeasureSpec.UNSPECIFIED, label.getHeight());
//
//				//float r = label.getMeasuredHeight() / label.getTextSize();
//				//lo("RELATION: " + label.getMeasuredHeight() + " - " + container.getHeight());
//				if (i > 100) {
//					break;
//				}
//			}
//
//			label.setText("- - -");
//		} catch (Exception e) {
//			e.printStackTrace();
//		}
//	}

	@Override
	public boolean onOptionsItemSelected( @NonNull MenuItem item )
	{
		try
		{
			switch ( item.getItemId() )
			{
				case R.id.connect:
					this.connecting();
					return true;
				case R.id.disconnect:
					this.disconnect();
					return true;
				case R.id.changedevice:
					this.selectAnotherDevice();
					return true;
				default:
					return super.onOptionsItemSelected( item );
			}
		}
		catch ( Exception e )
		{
			error( "onOptionsItemSelected", e );
			return false;
		}
	}


	private void loadExtras() {
		try {
			Bundle extras = getIntent().getExtras();
			if (extras != null) {
				mac = MUtils.checkNull(extras.getString("MAC"));
				serial = MUtils.checkNull(extras.getString("SERIAL"));
				model = MUtils.checkNull(extras.getString("MODEL"));
			}
			if(mac.equals("")){
				selectAnotherDevice();
			}
			ui = new MD10KVxUI(this);
			ui.display().clearAll();
			ui.display().model(model + " " + serial);
		} catch (Exception e) {
			error("loadExtras", e);
		}
	}

	private void keepTheScreenOn()
	{
		try
		{
			getWindow().addFlags( LayoutParams.FLAG_KEEP_SCREEN_ON );
		}
		catch ( Exception e )
		{
			error( "keepTheScreenOn", e );
		}
	}

	@Override
	public void onStop() {
		super.onStop();
	}


	// ######################
	//  COMUNICATION
	// ######################


	public static class mHandler extends Handler
	{
		private final WeakReference<MD10KVx>
				mActivity;

		static final int
				MESSAGE_DISCONNECTED = -1,
				MESSAGE_READ         = 1,
				MESSAGE_INFO         = 2,
				MESSAGE_INFO_BUSY    = 3,
				MESSAGE_REPORT_DONE  = 4,
				REPORT_UPDATE        = 5;

		public mHandler(MD10KVx activity) {
			mActivity = new WeakReference<>(activity);
		}

		@Override
		public void handleMessage( @NonNull Message msg )
		{
			MD10KVx activity = mActivity.get();
			if ( activity != null )
			{
				activity.statusBlink();
				if ( msg.what == MESSAGE_READ )
				{
					activity.rxTimeOutStart();
					activity.rx += (String) msg.obj;
				}
				if ( msg.what == MESSAGE_INFO )
				{
					activity.ui.messageBox( (String) msg.obj, msg.arg1, false );
				}
				if ( msg.what == MESSAGE_INFO_BUSY )
				{
					activity.ui.messageBox( (String) msg.obj, msg.arg1, true );
				}
				if ( msg.what == MESSAGE_DISCONNECTED )
				{
					activity.disconnect();
				}
				if ( msg.what == MESSAGE_REPORT_DONE )
				{
					activity.ui.changeButtonImage( R.id.bt_export, R.drawable.ic_export, true );
				}
			}
		}
	}

//	public Handler getHanler() {
//		return handler;
//	}

//	public void addCommand(Command cmd) {
//		FIFO.add(cmd);
//	}

	public String openConnection( String mac )
	{
		try
		{
			BluetoothAdapter adaptador = BluetoothAdapter.getDefaultAdapter();
			adaptador.cancelDiscovery();
			BluetoothDevice device = adaptador.getRemoteDevice( mac );

			if ( socket != null && socket.isConnected() )
			{
				socket.close();
			}

			try
			{
				socket = device.createInsecureRfcommSocketToServiceRecord( UUID_SPP );
			}
			catch ( Exception e )
			{
				socket = device.createRfcommSocketToServiceRecord( UUID_SPP );
			}

			socket.connect();

			trd = new MD10KVxConnectThread();
			trd.ConnectedThread( socket, handler );
			trd.setDaemon( true );
			trd.start();
		}
		catch ( Exception e )
		{
			return (e.getMessage() != null) ? e.getMessage().toUpperCase( Locale.ENGLISH ) : "Connection error";
		}
		return "OK";
	}

	private int timeToWait = 0;

	private void connecting() {
		menu.findItem(R.id.connect).setEnabled(false);
		ui.messageBox(R.string.md_connecting, MD10KVxUI.COLOR_YELLOW, true);
		new Thread( () ->
		{
			try
			{
				Thread.sleep( timeToWait );
			}
			catch ( InterruptedException ignored )
			{
			}

			final String statConn = openConnection( mac );

			runOnUiThread( () ->
			{
				try {
					if (statConn.equals("OK")) {
						isConnected(true);
						enableCommandButtons(false);
						Thread.sleep(600);//Pause needed by the chinese bluetooth module
						FIFO.add(Command.MODEL());
						FIFO.add(Command.MD10KVx.TIMEOUT(2500));
						FIFO.add(Command.MD10KVx.BUZZER(200));
						FIFO.add(Command.MD10KVx.READ_SETUP());
						timeToWait = 0;
					} else {
						isConnected(false);
						if (statConn.contains("TIMEOUT")) {
							timeToWait = 9000;
						}
						ui.messageBox(R.string.md_not_connected, MD10KVxUI.COLOR_YELLOW, false);
						//log("CONNECTION: " + statConn);
					}
				} catch (Exception ignored) {
					//log("CONNECTION: " + e.getMessage());
				}
			} );
		} ).start();
	}

	private void isConnected(boolean conected) {
		if (conected) {
			handler.removeCallbacks(runFIFO);
			handler.postDelayed(runFIFO, RUNFIFO_TIME);
			handler.removeCallbacks(checkConnection);
			handler.postDelayed(checkConnection, PING_TIME);
			rx = "";
			rxTries = 0;
			testRunning = false;
		} else {
			handler.removeCallbacks(checkConnection);
			handler.removeCallbacks(rxTimeOut);
			handler.removeCallbacks(runFIFO);
			timeoutCount = 0;
			idle = true;
			testStopped = false;
			FIFO.clear();
			rx = "";
			//ui.battery(Integer.valueOf("0", 16));
		}
		changeAlpha( 1, conected );
		menu.findItem( R.id.connect ).setEnabled( true );
		menu.findItem( R.id.connect ).setVisible( !conected );
		menu.findItem( R.id.disconnect ).setVisible( conected );
		menu.findItem( R.id.changedevice ).setVisible( !conected );
		enableCommandButtons( conected );
	}

	protected void enableCommandButtons(boolean enable) {
		findViewById(R.id.bt_printer).setEnabled(enable);
		findViewById(R.id.bt_start).setEnabled(enable);
		findViewById(R.id.bt_stop).setEnabled(true);
		findViewById(R.id.bt_setup).setEnabled(enable);
	}

	protected void blockControl(boolean block) {
		if (block && !warnedOnce) {
			ui.warnAboutLock();
		}
		warnedOnce = block;
		findViewById(R.id.bt_printer).setEnabled(!block);
		findViewById(R.id.bt_start).setEnabled(!block);
		findViewById(R.id.bt_setup).setEnabled(!block);
	}

	public void disconnect() {
		try {
			if (trd != null) {
				trd.cancel();
			}
			if (socket != null && socket.isConnected()) {
				socket.close();
			}
			ui.messageBox(R.string.md_disconnected, MD10KVxUI.COLOR_BLACK, false);
			findViewById(R.id.bt_start).setEnabled(false);
			findViewById(R.id.bt_start).setVisibility(View.VISIBLE);
			findViewById(R.id.bt_stop).setVisibility(View.GONE);

			if (table.getTableSize() > 0 && test.getTestDate().equals("")) {
				ui.enableSaveButton(true, test);
				//ui.enableExportButton(false);
			}
			ui.enableManagerButton(true);

			rx = "";
			//log("disconn");
			this.isConnected(false);
		} catch (Exception e) {
			error("Disconnect", e);
		}
	}

	private void loadRunnables()
	{
		FIFO = new LinkedList<>();

		runFIFO = new Runnable()
		{
			public void run()
			{
				if ( idle && !FIFO.isEmpty() )
				{
					idle = false;
					if ( testStopped )
					{
						FIFO.clear();
						FIFO.add( Command.MD10KVx.STOP() );
					}

					statusBlink();

					rx = "";

					Command command = FIFO.peek();
					if ( command != null )
					{
						trd.write( (Command.SF + command.getCommand() + Command.EF).getBytes() );
						rxTimeOutStart();
						//log( "TX> " + command.getDescription() + " > " + command.getCommand() );
					}
				}
				handler.postDelayed( this, RUNFIFO_TIME );
			}
		};

		rxTimeOut = () ->
		{
			rx = rx.replace(":", "");
			rx = rx.replace("\r\n", "");

			if ( MUtils.checkSum( rx ) )
			{
				checkedFrameType = true;
				timeoutCount = 0;
				try
				{
					rxTries = -1;
					processProtocol( rx );
				}
				catch ( Exception e )
				{
					rx = e.toString();
					processProtocol( rx );
				}
			}
			else
			{
				if ( rxTries == 10 || rxTries == 15 || rxTries == 20 )
				{
					retryLastCommand();
				}
				if ( rxTries > 60 )
				{
					processProtocol( "TIMEOUT:" + rx );
					rxTries = -1;
				}
				if ( rxTries != -1 )
				{
					rxTimeOutStart();
				}
			}
			rxTries++;
		};

		checkConnection = new Runnable() {
			public void run() {
				if ( timeoutCount > 1 && timeoutCount <= TIMEOUT_TRIES )
				{
					Command command = FIFO.peek();
					if ( !FIFO.isEmpty() && command != null && command.getType() == Command.MODEL().getType() && !checkedFrameType )
					{
						Command.enableFrame();
						retryLastCommand();
					}
					else
					{
						if ( FIFO.isEmpty() )
							FIFO.add( Command.MD10KVx.SHORT_STATUS() );
					}
				}
				if ( timeoutCount > TIMEOUT_TRIES )
				{
					ui.messageBox( R.string.md_disconnected, MD10KVxUI.COLOR_YELLOW, false );
					//if(!trd.isConnected())
					disconnect();
					timeoutCount = 0;
					timeToWait = 9000;
					return;
				}
				timeoutCount++;
				handler.postDelayed(this, PING_TIME);
			}
		};

		final MD10KVx parent = this;

		new Runnable()
		{
			public void run()
			{
				File resistanceImage = saveChartImage( chartResistance );
				File voltageImage    = saveChartImage( chartVoltage );
				File currentImage    = saveChartImage( chartCurrent );

				//report.saveDocReport( test, resistanceImage, voltageImage, currentImage );
				reportPDF.generate( parent, test, test.getPictureName(), null, new File[]{ resistanceImage, voltageImage, currentImage } );
			}
		};
	}

	public void addCommand( Command cmd )
	{
		FIFO.add( cmd );
	}

	private void retryLastCommand()
	{
		idle = true;
		//log( "RETRY LAST COMMAND" );
	}

	private void statusBlink()
	{
		changeAlpha( 3, true );
		handler.postDelayed( () -> changeAlpha( 3, false ), 100 );
	}

	private void rxTimeOutStart()
	{
		handler.removeCallbacks( rxTimeOut );
		handler.postDelayed( rxTimeOut, RX_TIMEOUT );
	}

	public Float getSimpleVoltage( float v )
	{
		Float  result;
		String vString = resolutionV( v / Units.KILO );
		float  vKilo   = Float.parseFloat( vString );
		result = vKilo * Units.KILO;
		return result;
	}

	private boolean check( Command c, String message )
	{
		Command command = FIFO.peek();
		boolean result = ( command != null && c.getType() == command.getType() && message.length() == command.getResponseSize() );
		if ( result )
			FIFO.remove();
		return result;
	}

	@SuppressWarnings({"RedundantCast", "ResultOfMethodCallIgnored", "StatementWithEmptyBody"})
	private void processProtocol(String msg) {
		Log.e( TAG, msg );
		try {
//			Command command = FIFO.peek();
//			if ( command != null )
//			{
//				log( "RX> " + command.getDescription() + " < " + msg + "\r\n  " );
//			}
			if ( check( Command.MD10KVx.PRINTER(), msg ) )
			{
				printerEnabled = !printerEnabled;
				printer( printerEnabled );
			}
			else if ( check( Command.MD10KVx.FILTER(), msg ) )
			{
				//RECEIVED FILTER
			}
			else if ( check( Command.MODEL(), msg ) )
			{
				//RECEIVED MODEL
			}
			else if ( check( Command.MD10KVx.START(), msg ) )
			{
				FIFO.add( Command.MD10KVx.TEST_NUMBER() );
			}
			else if ( check( Command.MD10KVx.TEST_NUMBER(), msg ) )
			{
				FIFO.add( Command.MD10KVx.FULL_STATUS() );
				this.resetCharts();
				test.Clear();
				ui.display().clear();
				button_start.setVisibility( View.GONE );
				button_stop.setVisibility( View.VISIBLE );
				ui.enableSaveButton( false, test );
				//ui.enableExportButton( false );
				ui.enableGpsButton( false );
				ui.enableRecordButton( false );
				ui.enablePictureButton( false );
				ui.enableManagerButton( false );
				ui.messageBox( R.string.md_test_running, MD10KVxUI.COLOR_RED, true );
				int testNumber = Integer.valueOf( msg.substring( 6, 10 ), 16 );
				ui.display().testNumber( testNumber + "" );
				lastElapsedTime = 0;
			}
			else if ( check(Command.MD10KVx.SHORT_STATUS(), msg) )
			{
				String status  = MUtils.cleanMsg(msg);
				boolean[] bits = MUtils.hexToBitArray(status.substring(0, 4));
				String battery = status.substring(4, 8);
				ui.battery(Integer.valueOf(battery, 16));

				if ( bits[0] )
				{
					if ( !testRunning )
						FIFO.add( Command.MD10KVx.TEST_NUMBER() );
					testRunning = true;
				}
				else
				{
					if ( !testRunning )
					{
						button_start.setVisibility( View.VISIBLE );
						button_stop.setVisibility( View.GONE );
					}
				}

				if ( bits[1] )
				{
					printerEnabled = true;
					printer( true );
				}
				else
				{
					printerEnabled = false;
					printer( false );
				}

				if ( bits[2] )
				{
					Command.MD10KVx.SEL_FILTER( 1 );
				}
				else
				{
					Command.MD10KVx.SEL_FILTER( 0 );
				}

				if ( bits[12] )
				{
					FIFO.add( Command.MD10KVx.RESET_EXTERNAL() );
					if ( !ignoreChange )
						ui.setupChangedDirectly( this );
					ignoreChange = false;
				}

				//1 - Operando pelo aparelho; 0 - Operacao remota;
				blockControl(bits[13]);
			}
			else if ( check(Command.MD10KVx.FULL_STATUS(), msg) )
			{
				short WORD_SIZE 		= 4;//In this case, 4 chars
				short POINTER_SIZE 		= 0;
				short STATUS_SIZE 		= (short) (WORD_SIZE);
				short BATERY_SIZE 		= (short) (WORD_SIZE);
				short MINUTES_SIZE 		= (short) (WORD_SIZE);
				short SECONDS_SIZE 		= (short) (WORD_SIZE);
				short RESISTANCE_SIZE 	= (short) (2 * WORD_SIZE);
				short VOLTAGE_SIZE 		= (short) (2 * WORD_SIZE);
				short CURRENT_SIZE 		= (short) (2 * WORD_SIZE);
				short CAPACITANCE_SIZE 	= (short) (WORD_SIZE);
				short RESERVED_SIZE 	= (short) (WORD_SIZE);
				short PI_SIZE 			= (short) (2 * WORD_SIZE);
				short SVT_SIZE 			= (short) (2 * WORD_SIZE);
				short DAI_SIZE 			= (short) (2 * WORD_SIZE);

				String msgCleaned = MUtils.cleanMsg(msg);

				boolean[] status	 = MUtils.hexToBitArray(msgCleaned.substring(POINTER_SIZE, POINTER_SIZE + STATUS_SIZE));
				POINTER_SIZE 		+= STATUS_SIZE;
				String battery		 = msgCleaned.substring(POINTER_SIZE, POINTER_SIZE + BATERY_SIZE);
				POINTER_SIZE 		+= BATERY_SIZE;
				String minutes		 = msgCleaned.substring(POINTER_SIZE, POINTER_SIZE + MINUTES_SIZE);
				POINTER_SIZE 		+= MINUTES_SIZE;
				String seconds		 = msgCleaned.substring(POINTER_SIZE, POINTER_SIZE + SECONDS_SIZE);
				POINTER_SIZE 		+= SECONDS_SIZE;
				String resistance    = msgCleaned.substring(POINTER_SIZE, POINTER_SIZE + RESISTANCE_SIZE);
				POINTER_SIZE 		+= RESISTANCE_SIZE;
				String voltage		 = msgCleaned.substring(POINTER_SIZE, POINTER_SIZE + VOLTAGE_SIZE);
				POINTER_SIZE 		+= VOLTAGE_SIZE;
				String current		 = msgCleaned.substring(POINTER_SIZE, POINTER_SIZE + CURRENT_SIZE);
				POINTER_SIZE 		+= CURRENT_SIZE;
				//CAPACITANCE
				msgCleaned.substring(POINTER_SIZE, POINTER_SIZE + CAPACITANCE_SIZE);
				POINTER_SIZE 		+= CAPACITANCE_SIZE;
				//Reserved
				msgCleaned.substring(POINTER_SIZE, POINTER_SIZE + RESERVED_SIZE);
				POINTER_SIZE 		+= RESERVED_SIZE;
				String pi			 = msgCleaned.substring(POINTER_SIZE, POINTER_SIZE + PI_SIZE);
				POINTER_SIZE 		+= PI_SIZE;
				String svt			 = msgCleaned.substring(POINTER_SIZE, POINTER_SIZE + SVT_SIZE);
				POINTER_SIZE 		+= SVT_SIZE;
				String dai 			 = msgCleaned.substring(POINTER_SIZE, POINTER_SIZE + DAI_SIZE);
				//POINTER_SIZE += DAI_SIZE;

				ui.battery(Integer.valueOf(battery, 16));
				String mmss =
						String.format(Locale.US, "%02d", MUtils.hexToInt(minutes))
								+ ":"
								+ String.format(Locale.US, "%02d", MUtils.hexToInt(seconds));
				ui.display().elapsedTime(mmss);

				final float r 	= Float.valueOf(MUtils.hexToFloat(resistance));
				resistance 		= resolutionR(r);

				final float v 	= Float.valueOf(MUtils.hexToFloat(voltage));
				voltage 		= resolutionV(v / Units.KILO) + " kV";

				final float a 	= MUtils.hexToFloatA(current);
				current 		= resolutionA(a);

				resolutionA(getSimpleVoltage(v) / r);

				final long elapsedTime   = ((MUtils.hexToInt(minutes) * 60) + MUtils.hexToInt(seconds)) * 1000;
				final boolean overrangeR = resistance.startsWith(">");
				final boolean overrangeA = current.equals("0 nA");

				if (overrangeR || overrangeA) {
					resistance = ">10T\u03A9";
					current = "0 nA";
				}

				ui.display().resistance(resistance);
				ui.display().current(current);

				if (lastElapsedTime < elapsedTime && lastElapsedTime >= INITIAL_VALUE && (lastElapsedTime) % MEASURE_GAP == 0) {
					chartResistance.update(this, lastElapsedTime, lastResistance, lastVoltage, lastCurrent, true);
					chartVoltage.update(this, lastElapsedTime, lastResistance, lastVoltage, lastCurrent, true);
					chartCurrent.update(this, lastElapsedTime, lastResistance, lastVoltage, lastCurrent, true);
					table.update(this, lastElapsedTime, lastResistance, lastVoltage, lastCurrent, true);
					test.addResults(lastElapsedTime + "", lastResistance + "", lastVoltage + "", (getSimpleVoltage(lastVoltage) / lastCurrent) + "");
				}

				lastElapsedTime = elapsedTime;
				lastResistance = r;
				lastCurrent = a;

				//Bit 0 (Running): 1 - Ensaio em andamento; 0 - Nao esta realizando ensaio.
				if (status[0]) {
					//This ckecking is necessary because of CAPACITANCE calculation (there are a voltage decay)
					if (v > 10) {
						ui.display().voltage(voltage);
						lastVoltage = v;
					}
					testRunning = true;
					button_start.setVisibility(View.GONE);
					button_stop.setVisibility(View.VISIBLE);
					ui.messageBox(R.string.md_test_running, MD10KVxUI.COLOR_RED, true);
					FIFO.add(Command.MD10KVx.FULL_STATUS());
				} else {
					button_start.setVisibility(View.VISIBLE);
					button_stop.setVisibility(View.GONE);
					if (testRunning) {
						ui.messageBox(R.string.md_calculating_capacitance,
								MD10KVxUI.COLOR_YELLOW, true);
						FIFO.add(Command.MD10KVx.CAPACITANCE());
					}
				}

				//Bit 1 (Printer): 1 - Impressao habilitada; 0 - Impressao desabilitada;
				if (status[1]) {
					printerEnabled = true;
					printer(true);
				} else {
					printerEnabled = false;
					printer(false);
				}

				//Bit 2 (Filter): 1 - Filtro de medicoes ligado; 0 - Filtro desligado;
				if (status[2]) {
					Command.MD10KVx.SEL_FILTER(1);
				} else {
					Command.MD10KVx.SEL_FILTER(0);
				}

				//Bit 5 (SVT/PI): 1 - Valor de SVT/PI disponivel; 0 - indisponivel;
				if (status[5]) {
					if (isSvt) {
						ui.display().svt(checkPiDaiSvt(svt));
					} else {
						ui.display().pi(checkPiDaiSvt(pi));
					}
				}

				//Bit 6 (DAI): 1 - Valor de DAI disponivel; 0 - indisponivel;
				if (status[6]) {
					ui.display().dai(checkPiDaiSvt(dai));
				}

				if (status[12]) {
					FIFO.add(Command.MD10KVx.RESET_EXTERNAL());
					FIFO.add(Command.MD10KVx.READ_SETUP());
				}

				//1 - Operando pelo aparelho; 0 - Operacao remota;
				blockControl(status[13]);
			}
			else if ( check(Command.MD10KVx.STOP(), msg) ) {
				testStopped = false;
				FIFO.add(Command.MD10KVx.CAPACITANCE());
			}
			else if ( check(Command.MD10KVx.CAPACITANCE(), msg) ) {
				short WORD_SIZE        = 4;//In this case, 4 chars
				short POINTER_SIZE     = 0;
				short STATUS_SIZE      = (short) (WORD_SIZE);
				short BATERY_SIZE      = (short) (WORD_SIZE);
				short MINUTES_SIZE     = (short) (WORD_SIZE);
				short SECONDS_SIZE     = (short) (WORD_SIZE);
				short RESISTANCE_SIZE  = (short) (2 * WORD_SIZE);
				short VOLTAGE_SIZE     = (short) (2 * WORD_SIZE);
				short CURRENT_SIZE     = (short) (2 * WORD_SIZE);
				short CAPACITANCE_SIZE = (short) (WORD_SIZE);

				String msgCleaned = MUtils.cleanMsg(msg);

				boolean[] status = MUtils.hexToBitArray(msgCleaned.substring(POINTER_SIZE, POINTER_SIZE + STATUS_SIZE));
				POINTER_SIZE += STATUS_SIZE;
				String battery = msgCleaned.substring(POINTER_SIZE, POINTER_SIZE + BATERY_SIZE);
				POINTER_SIZE += BATERY_SIZE;
				String minutes = msgCleaned.substring(POINTER_SIZE, POINTER_SIZE + MINUTES_SIZE);
				POINTER_SIZE += MINUTES_SIZE;
				String seconds = msgCleaned.substring(POINTER_SIZE, POINTER_SIZE + SECONDS_SIZE);
				POINTER_SIZE += SECONDS_SIZE;
				String resistance = msgCleaned.substring(POINTER_SIZE, POINTER_SIZE + RESISTANCE_SIZE);
				POINTER_SIZE += RESISTANCE_SIZE;
				String voltage = msgCleaned.substring(POINTER_SIZE, POINTER_SIZE + VOLTAGE_SIZE);
				POINTER_SIZE += VOLTAGE_SIZE;
				String current = msgCleaned.substring(POINTER_SIZE, POINTER_SIZE + CURRENT_SIZE);
				POINTER_SIZE += CURRENT_SIZE;
				String capacitance = msgCleaned.substring(POINTER_SIZE, POINTER_SIZE + CAPACITANCE_SIZE);

				ui.battery(Integer.valueOf(battery, 16));
				String mmss = String.format(Locale.US, "%02d", MUtils.hexToInt(minutes))
						+ ":" + String.format(Locale.US, "%02d", MUtils.hexToInt(seconds));
				ui.display().elapsedTime(mmss);

				float r = Float.valueOf(MUtils.hexToFloat(resistance));
				resistance = resolutionR(r);
				ui.display().resistance(resistance);

				float v = Float.valueOf(MUtils.hexToFloat(voltage));
				voltage = resolutionV(v / Units.KILO) + " kV";

				float a = MUtils.hexToFloatA(current);
				current = resolutionA(a);
				ui.display().current(current);

				if (lastElapsedTime >= INITIAL_VALUE && (lastElapsedTime) % MEASURE_GAP == 0) {
					chartResistance.update(this, lastElapsedTime, lastResistance, lastVoltage, lastCurrent, true);
					chartVoltage.update(this, lastElapsedTime, lastResistance, lastVoltage, lastCurrent, true);
					chartCurrent.update(this, lastElapsedTime, lastResistance, lastVoltage, lastCurrent, true);
					table.update(this, lastElapsedTime, lastResistance, lastVoltage, lastCurrent, true);
					test.addResults(lastElapsedTime + "", lastResistance + "", lastVoltage + "", (getSimpleVoltage(lastVoltage) / lastCurrent) + "");
				}

				lastElapsedTime = 0;

				//Bit 0 (Running): 1 - Ensaio em andamento; 0 - Nao esta realizando ensaio.
				if (status[0]) {
					testRunning = true;
					button_start.setVisibility(View.GONE);
					button_stop.setVisibility(View.VISIBLE);
					ui.messageBox(R.string.md_test_running, MD10KVxUI.COLOR_RED, true);
					FIFO.add(Command.MD10KVx.FULL_STATUS());
				} else {
					button_start.setEnabled(false);
					button_start.setVisibility(View.VISIBLE);
					button_stop.setVisibility(View.GONE);
					if (testRunning) {
						ui.messageBox(getResources().getString(R.string.md_calculating_capacitance_v) + voltage, MD10KVxUI.COLOR_YELLOW, true);
						FIFO.add(Command.MD10KVx.CAPACITANCE());
					}
				}

				if (v < 45) {
					capIsready++;
					if (capIsready >= 15) {
						testRunning = false;
						capIsready = -1;
						ui.messageBox(R.string.md_test_finished, MD10KVxUI.COLOR_GREEN, false);
						button_start.setEnabled(true);
						if (table.getTableSize() > 0)
							ui.enableSaveButton(true, test);
						ui.enableManagerButton(true);
					}
				} else {
					capIsready = 0;
					capIsreadyTries = 0;
				}

				//Bit 4 (Capacitancia): 1 - Valor de capacitancia disponivel; 0 - indisponivel;
				if (status[4]) {
					//testRunning = false;
					int icap = Integer.valueOf(capacitance, 16);
					if (icap <= 0 && capIsreadyTries <= 5) {
						capIsreadyTries++;
					} else {
						ui.display().capacitance(resolutionF(icap));
						ui.messageBox(R.string.md_test_finished, MD10KVxUI.COLOR_GREEN, false);
						button_start.setEnabled(true);
						testRunning = false;
						if (table.getTableSize() > 0) {
							ui.enableSaveButton(true, test);
						}
						ui.enableManagerButton(true);
					}
				}
			}
			else if ( check(Command.MD10KVx.RESET_EXTERNAL(), msg) ) {
				//RESET EXTERNAL
			}
			else if ( check(Command.MD10KVx.TIMEOUT(0), msg) ) {
				ui.messageBox(R.string.md_checkings_1of3, MD10KVxUI.COLOR_YELLOW, true);
			}
			else if ( check(Command.MD10KVx.BUZZER(0), msg) ) {
				ui.messageBox(R.string.md_checkings_2of3, MD10KVxUI.COLOR_YELLOW, true);
			}
			else if ( check(Command.MD10KVx.READ_SETUP(), msg) ) {
				String setup    = MUtils.cleanMsg(msg);
				int testVoltage = MUtils.hexToInt(setup.substring(0, 4));
				int opMode      = MUtils.hexToInt(setup.substring(4, 8));
				int lim         = MUtils.hexToInt(setup.substring(8, 12));
				ui.display().setupVoltage(testVoltage * 25 + " V");
				ui.display().setupLim(Command.MD10KVx.LIM(lim));

				if (opMode > 2)
					ui.display().setupTimer(Command.MD10KVx.OPMODE(opMode));
				else
					ui.display().setupTimer("- - -");

				if (opMode < 3)
					ui.display().setupSvt(Command.MD10KVx.OPMODE(opMode));
				else
					ui.display().setupSvt("- - -");

				isSvt = opMode == 1 || opMode == 2;

				Command.MD10KVx.SEL_VOLTAGE(testVoltage * 25);
				Command.MD10KVx.SEL_OPMODE(opMode);
				Command.MD10KVx.SEL_LIM(lim);

				if ( showSetupDialog )
				{
					ui.createSet( this, testRunning );
					showSetupDialog = false;
				}
				else
				{
					if ( !testRunning )
					{
						ui.messageBox( R.string.md_equipment_ready, MD10KVxUI.COLOR_GREEN, false );
					}
				}
				if ( !testRunning )
				{
					this.enableCommandButtons( true );
				}
			}
			else if ( check(Command.MD10KVx.BUZZER(), msg) ) {
				//BUZZER RECEIVED
			}
			else if ( check(Command.MD10KVx.WRITE_SETUP(0, 0, 0), msg) )
				FIFO.add(Command.MD10KVx.READ_SETUP());


			//FIFO.remove();
			idle = true;
			rx   = "";
		} catch (Exception e) {
			error("processProtocol", e);
		}
	}


	// ############################################
	// INTERFACE WITH USER 
	// ############################################

	@SuppressLint("ClickableViewAccessibility")
	private void loadButtonsFuncions() {
		ImageButton button_printer = findViewById(R.id.bt_printer);
		button_printer.setOnClickListener( v -> FIFO.add(Command.MD10KVx.PRINTER()) );

		button_stop = findViewById(R.id.bt_stop);
		button_stop.setVisibility(View.GONE);
		button_stop.setOnClickListener( v -> testStopped = true );

		ImageButton btSetup = findViewById(R.id.bt_setup);
		btSetup.setOnClickListener( v ->
		{
			showSetupDialog = true;
			FIFO.add(Command.MD10KVx.READ_SETUP());
		} );

		final CountDown mCountDownTimer = new CountDown(3000, 100);

		button_start = findViewById(R.id.bt_start);

		button_start.setOnTouchListener( ( v, event ) ->
		{
			Button bc = (Button) v;
			switch ( event.getAction() )
			{
				case MotionEvent.ACTION_DOWN:
					mCountDownTimer.cancel();
					mCountDownTimer.start();
					bc.setPressed( true );
					bc.setText( getString( R.string.starting ).toUpperCase() );
					break;
				case MotionEvent.ACTION_UP:
					v.performClick();
					mCountDownTimer.cancel();
					if ( !mCountDownTimer.isFinished() )
					{
						ui.messageBox( R.string.md_hold_start, MD10KVxUI.COLOR_YELLOW, false );
					}
					((Button) findViewById( R.id.bt_start )).setText( startSpanString, TextView.BufferType.SPANNABLE );
					bc.setPressed( false );
					break;
				case MotionEvent.ACTION_OUTSIDE:
					v.performClick();
					mCountDownTimer.cancel();
					if ( !mCountDownTimer.isFinished() )
					{
						ui.messageBox( R.string.md_hold_start, MD10KVxUI.COLOR_YELLOW, false );
					}
					((Button) findViewById( R.id.bt_start )).setText( startSpanString, TextView.BufferType.SPANNABLE );
					bc.setPressed( false );
					break;
			}
			return true;
		} );
	}


	private void resetCharts() {
		createSwipe();
	}

	public void clearAll() {
		resetCharts();
		test.Clear();
		ui.display().clear();

		ui.enableSaveButton(false, test);
		//ui.enableExportButton(false);
		//ui.enableGpsButton(false);
		ui.changeButtonImage(R.id.bt_gps, R.drawable.ic_gps_small, true);
		ui.enableRecordButton(false);
		ui.enablePictureButton(false);
	}

	/**
	 * Create CountDownTimer
	 */
	public class CountDown extends CountDownTimer {
		long time;
		int buzz = 0;
		boolean finished = false;

		CountDown( long time, long tick )
		{
			super( time, tick );
			this.time = time;
		}

		public void onTick( long millisUntilFinished )
		{
			if ( !button_start.isPressed() )
			{
				ui.messageBox( getResources().getString( R.string.md_hold_start_by ) + (time / 1000) + "s", MD10KVxUI.COLOR_YELLOW, false );
				this.cancel();
			}
			else
			{
				if ( buzz >= 6 || buzz == 0 )
				{
					FIFO.add( Command.MD10KVx.BUZZER() );
					buzz = 1;
				}
				ui.messageBox( getResources().getString( R.string.md_start_in ) + (millisUntilFinished) + "ms", MD10KVxUI.COLOR_YELLOW, true );
				if ( showChart )
				{
					showChart();
				}
			}
			buzz++;
			finished = false;
		}

		public void onFinish() {
			if (button_start.isPressed()) {
				ui.messageBox(R.string.md_warning_test_start, MD10KVxUI.COLOR_RED, true);
				button_start.setPressed(false);
				button_start.setVisibility(View.GONE);
				button_stop.setVisibility(View.VISIBLE);
				button_stop.setPressed(false);
				FIFO.add(Command.MD10KVx.START());
				testRunning = true;
				finished = true;
			}
		}

		public boolean isFinished() {
			return finished;
		}
	}

//	private void printer(boolean enabled) {
//		ImageButton btPrinter = findViewById(R.id.bt_printer);
//		Rect bounds = btPrinter.getCompoundDrawables()[0].getBounds();
//		Drawable img;
//
//		if (enabled) {
//			//img = getResources().getDrawable(R.drawable.led_drawable_on);
//			img = ResourcesCompat.getDrawable(getResources(), R.drawable.led_drawable_on, null);
//		} else {
//			//img = getResources().getDrawable(R.drawable.led_drawable_off);
//			img = ResourcesCompat.getDrawable(getResources(), R.drawable.led_drawable_off, null);
//		}
//		if(img!=null){
//			img.setBounds(bounds);
//			btPrinter.setCompoundDrawables(img, null, null, null);
//		}
//	}

	private void printer( boolean enabled )
	{
		Drawable img;

		if ( enabled )
		{
			img = ResourcesCompat.getDrawable( getResources(), R.drawable.ic_printer_on_states, null );
		}
		else
		{
			img = ResourcesCompat.getDrawable( getResources(), R.drawable.ic_printer_off_states, null );
		}

		ImageButton btPrinter = findViewById( R.id.bt_printer );
		if ( btPrinter != null && img != null )
		{
			btPrinter.setImageDrawable( img );
		}
	}

	/*********************************************
	 * REG RECORD AUDIO
	 **********************************************/

	@SuppressWarnings("SetTextI18n")
	public void recording(boolean isRecording) {
		TextView duration = dlgAudio.findViewById(R.id.txt_duration);
		Chronometer chrono = dlgAudio.findViewById(R.id.chrono);
		ImageButton btnClose = dlgAudio.findViewById(R.id.btn_close_dlg);
		ImageButton btnRecord = dlgAudio.findViewById(R.id.btn_record);
		ImageButton btnStop = dlgAudio.findViewById(R.id.btn_stop_recording);
		ImageButton btnPlay = dlgAudio.findViewById(R.id.btn_play);

		if (isRecording) {
			enableButton(btnClose, false);
			enableButton(btnPlay, false);
			enableButton(btnRecord, false, false);
			enableButton(btnStop, true, true);
			chrono.setBase(SystemClock.elapsedRealtime());
			chrono.setOnChronometerTickListener( chronometer ->
			{
				CharSequence mm = chronometer.getText().subSequence(0, 2);
				CharSequence ss = chronometer.getText().subSequence(3, 5);
				if (mm.equals("01") && ss.equals("00")) {
					chronometer.setText("60/60s");
				} else {
					chronometer.setText(ss + "/60s");
				}
			} );
			chrono.start();
		} else {
			enableButton(btnClose, true);
			enableButton(btnPlay, true);
			enableButton(btnRecord, true, true);
			enableButton(btnStop, false, false);
			chrono.stop();
			chrono.setText("00/60s");
			int seconds = audio.getAudioLenght() / 1000;
			duration.setText(seconds + " s");
		}
	}

	@SuppressWarnings("SetTextI18n")
	public void playing(boolean isPlaying) {
		Chronometer chrono = dlgAudio.findViewById(R.id.chrono);
		ImageButton btnClose = dlgAudio.findViewById(R.id.btn_close_dlg);
		ImageButton btnRecord = dlgAudio.findViewById(R.id.btn_record);
		ImageButton btnPlay = dlgAudio.findViewById(R.id.btn_play);
		ImageButton btnStop = dlgAudio.findViewById(R.id.btn_stop);

		if (isPlaying) {
			enableButton(btnClose, false);
			enableButton(btnRecord, false);
			enableButton(btnPlay, false, false);
			enableButton(btnStop, true, true);
			chrono.setBase(SystemClock.elapsedRealtime());
			chrono.setOnChronometerTickListener( chronometer ->
			{
				CharSequence mm = chronometer.getText().subSequence(0, 2);
				CharSequence ss = chronometer.getText().subSequence(3, 5);
				if (mm.equals("01") && ss.equals("00")) {
					chronometer.setText("60/60s");
				} else {
					chronometer.setText(ss + "/60s");
				}
			} );
			chrono.start();
		} else {
			enableButton(btnClose, true);
			enableButton(btnRecord, true);
			enableButton(btnPlay, true, true);
			enableButton(btnStop, false, false);
			chrono.stop();
			chrono.setText("00/60s");
		}
	}

	public void record(View v) {
		if (permissionToRecordAccepted)
			recordAudio();
		else
			requestPermission(REQUEST_RECORD_AUDIO_PERMISSION);
	}

	public void recordAudio() {
		if (!test.getTestNum().equals("")) {
			if (dlgAudio == null) {
				dlgAudio = createAudioDialog();
			}

			File file = test.getAudioName();
			audio.setAudioFile(file.getAbsolutePath());

			Chronometer chrono = dlgAudio.findViewById(R.id.chrono);
			TextView duration = dlgAudio.findViewById(R.id.txt_duration);
			ImageButton btnPlay = dlgAudio.findViewById(R.id.btn_play);
			//TextView objectName = (TextView)    dlgAudio.findViewById(R.id.txt_object_name);
			TextView testNumber = dlgAudio.findViewById(R.id.txt_test_number);
			testNumber.setText(test.getTestNum());

			if (audio.fileExists()) {
				int seconds = audio.getAudioLenght() / 1000;
				String text = seconds + " s";
				duration.setText(text);
				enableButton(btnPlay, true);
			} else {
				duration.setText("");
				chrono.setBase(SystemClock.elapsedRealtime());
				enableButton(btnPlay, false);
			}
			dlgAudio.show();
		}
	}

	@SuppressWarnings("SetTextI18n")
	private Dialog createAudioDialog()
	{
		final Dialog dialog = new Dialog( this );
		dialog.requestWindowFeature( Window.FEATURE_NO_TITLE );
		dialog.setCanceledOnTouchOutside( false );
		dialog.setContentView( R.layout.dialog_audio );
		if ( dialog.getWindow() != null )
		{
			dialog.getWindow().setBackgroundDrawable( new ColorDrawable( Color.TRANSPARENT ) );
		}

		((TextView) dialog.findViewById( R.id.dlg_title )).setText( R.string.voice_annotation );
		((ImageView) dialog.findViewById( R.id.dlg_icon )).setImageResource( R.drawable.ic_audio_gray );

		final ImageButton btnRecord = dialog.findViewById( R.id.btn_record );
		final ImageButton btnStopRecord = dialog.findViewById( R.id.btn_stop_recording );
		final ImageButton btnPlay = dialog.findViewById( R.id.btn_play );
		final ImageButton btnStop = dialog.findViewById( R.id.btn_stop );
		final Chronometer chrono = dialog.findViewById( R.id.chrono );

		dialog.findViewById( R.id.line_object ).setVisibility( View.GONE );

		//dialog.findViewById( R.id.btn_close ).setOnClickListener( vi -> dialog.dismiss() );
		dialog.findViewById( R.id.btn_close_dlg ).setOnClickListener( vi -> dialog.dismiss() );

		btnRecord.setOnClickListener( v ->
		{
			chrono.setVisibility( View.VISIBLE );
			audio.startRecording();
		} );

		btnStopRecord.setOnClickListener( v -> audio.stopRecording() );

		btnPlay.setOnClickListener( v -> audio.startPlaying() );

		btnStop.setOnClickListener( v -> audio.stopPlaying() );

		dialog.setOnShowListener( dialog1 -> chrono.setText( "00/60s" ) );

		return dialog;
	}

	private void enableButton(Button btn, boolean enable) {
		int color = enable ? Color.WHITE : Color.DKGRAY;
		btn.setEnabled(enable);
		btn.setTextColor(color);
	}

	private void enableButton(ImageButton btn, boolean enable) {
		float alpha = enable ? (float) 1.0 : (float) 0.2;
		btn.setEnabled(enable);
		btn.setAlpha(alpha);
	}

	private void enableButton(ImageButton btn, boolean enable, boolean hide) {
		float alpha = enable ? (float) 1.0 : (float) 0.2;
		int visibility = hide ? View.VISIBLE : View.GONE;
		btn.setVisibility(visibility);
		btn.setEnabled(enable);
		btn.setAlpha(alpha);
	}

	/*********************************************
	 * REG GPS INTERFACE
	 **********************************************/

	public void getGpsData(View v) {
		if(permissionToGPSAccepted)
			gps();
		else
			requestPermission(REQUEST_GPS_PERMISSION);
	}

	public void gps() {
		if (table.getTableSize() == 0) {
			return;
		}
		if (test.getLatitude().equals("")) {
			gps.getDataGPS();
		} else {
			String[] data = new String[4];
			data[0] = test.getLongitude();
			data[1] = test.getLatitude();
			data[2] = test.getAltitude();
			data[3] = test.getAccuracy();

			gps.dialogConfirmReplace(data);
		}
	}

	public void GpsData(int which, String[] data) {
		switch (which) {
			case GPS.RESULT:
				if (data != null) {
					test.setLongitude(data[0]);
					test.setLatitude(data[1]);
					test.setAltidude(data[2]);
					test.setAccuracy(data[3]);
					test.updateGpsData();
					ui.changeButtonImage(R.id.bt_gps, R.drawable.ic_gps_small_details, true);
				}
				break;
			case GPS.WAIT:
				ui.changeButtonImage(R.id.bt_gps, R.drawable.ic_wait, false);
				break;
			case GPS.CANCELLED:
				if (test.getLatitude().equals("")) {
					ui.changeButtonImage(R.id.bt_gps, R.drawable.ic_gps_small, true);
				} else {
					ui.changeButtonImage(R.id.bt_gps, R.drawable.ic_gps_small_details, true);
				}
				break;
		}
	}

	/*********************************************
	 * REG PICTURE
	 **********************************************/

	public void takePicture(View v) {
		if(permissionToPictureAccepted)
			picture();
		else
			requestPermission(REQUEST_PICTURE_PERMISSION);
	}

	private void picture(){
		File file = test.getPictureName();
		picture.showDialog(file);
	}

	public void pictureStatus( final int status )
	{
		runOnUiThread( () ->
		{
			switch ( status )
			{
				case Picture.WAIT:
					changeButtonImage( R.id.bt_picture, R.drawable.ic_wait, false );
					break;
				case Picture.FINISHED:
					changeButtonImage( R.id.bt_picture, R.drawable.ic_picture_small, true );
					break;
			}
		} );
	}

	public void changeButtonImage(int buttonId, int image, boolean enable){
		((ImageButton) findViewById(buttonId)).setImageResource(image);
		enableButtonImage(buttonId, enable);
	}

	private void enableButtonImage(int buttonId, boolean enable){
		ImageButton btn = findViewById(buttonId);
		int alpha = enable ? 255 : 51;
		btn.setEnabled(enable);
		btn.getDrawable().setAlpha(alpha);
	}

	/*********************************************
	 * REG END
	 **********************************************/

	public void showChart(View v) {
		showChart();
	}

	public void showChart() {
		if (this.findViewById(R.id.info_a) == null) {
			return;
		}

		showChart = !showChart;
		if (showChart) {
			this.findViewById(R.id.info_a).setLayoutParams(new LinearLayout.LayoutParams(LayoutParams.MATCH_PARENT, 0));
			this.findViewById(R.id.info_b).setLayoutParams(new LinearLayout.LayoutParams(LayoutParams.MATCH_PARENT, 0));

			if (table != null) {
				table.showTable(true);
			}

			this.findViewById(R.id.sliding_tabs).setLayoutParams(new LinearLayout.LayoutParams(LayoutParams.MATCH_PARENT, LayoutParams.WRAP_CONTENT));

			LinearLayout.LayoutParams lp = new LinearLayout.LayoutParams(LayoutParams.MATCH_PARENT, LayoutParams.MATCH_PARENT);
			lp.weight = 1;
			this.findViewById(R.id.pager).setLayoutParams(lp);

			ui.changeButtonImage(R.id.bt_chart, R.drawable.ic_info_screen, true);
		} else {
			LinearLayout.LayoutParams lp = new LinearLayout.LayoutParams(LayoutParams.MATCH_PARENT, LayoutParams.MATCH_PARENT);
			lp.weight = 1;
			this.findViewById(R.id.info_a).setLayoutParams(lp);
			this.findViewById(R.id.info_b).setLayoutParams(new LinearLayout.LayoutParams(LayoutParams.MATCH_PARENT, LayoutParams.WRAP_CONTENT));

			if (table != null) {
				table.showTable(false);
			}

			this.findViewById(R.id.pager).setLayoutParams(new LinearLayout.LayoutParams(LayoutParams.MATCH_PARENT, 0));
			this.findViewById(R.id.sliding_tabs).setLayoutParams(new LinearLayout.LayoutParams(LayoutParams.MATCH_PARENT, 0));

			ui.changeButtonImage(R.id.bt_chart, R.drawable.ic_chart_small, true);
		}
	}

	public void saveResults(View v) {
		String[] model_serial = ui.display().model().split(" ");
		test.setModel(model_serial[0]);
		if (model_serial.length > 1) {
			test.setSerial(model_serial[1]);
		}
		DateFormat dateFormat = new SimpleDateFormat("yyyy_MM_dd_HHmm", Locale.getDefault());
		Date date = new Date();
		String time = dateFormat.format(date);
		test.setTestDate(time);
		test.setTestNum(ui.display().testNumber());
		test.setDAI(ui.display().dai());
		test.setPI(ui.display().pi());
		test.setSVT(ui.display().svt());
		test.setCapacitance(ui.display().capacitance());

		test.Save();
		//ui.enableGpsButton(true);
		if (test.getLatitude().equals("")) {
			ui.changeButtonImage(R.id.bt_gps, R.drawable.ic_gps_small, true);
		} else {
			ui.changeButtonImage(R.id.bt_gps, R.drawable.ic_gps_small_details, true);
		}
		ui.enableRecordButton(true);
		ui.enablePictureButton(true);
		ui.enableSaveButton(false, test);
		//ui.enableExportButton(true);
	}


	public void exportChart(View v) {
		if(permissionToMemoryAccepted)
			export();
		else
			requestPermission(REQUEST_MEMORY_REPORT_PERMISSION);
	}

	public void export() {
		//report.createFolder();
		createFolder();
		if (test.getTestNum().equals("")) {
			return;
		}

		//Grafico nao esta sendo exibido
		if (!showChart) {
			showChart();
		}

		ui.changeButtonImage(R.id.bt_export, R.drawable.ic_wait, false);

		final MD10KVx parent = this;

		new Thread( () ->
		{
			try
			{
				Thread.sleep( 100 );
			}
			catch ( InterruptedException e )
			{
				e.printStackTrace();
			}
			File
					resistanceImage = saveChartImage( chartResistance ),
					voltageImage = saveChartImage( chartVoltage ),
					currentImage = saveChartImage( chartCurrent );

			//report.saveDocReport( test, resistanceImage, voltageImage, currentImage );
			reportPDF.generate( parent, test, test.getPictureName(), null, new File[]{ resistanceImage, voltageImage, currentImage } );
		} ).start();
	}

	private File saveChartImage(MD10KVxChartInterface chart) {
		String type = "";

		switch (chart.getChartType()) {
			case MD10KVxFragmentChart.RESISTANCE_CHART:
				type = "resistance";
				break;
			case MD10KVxFragmentChart.VOLTAGE_CHART:
				type = "voltage";
				break;
			case MD10KVxFragmentChart.CURRENT_CHART:
				type = "current";
				break;
		}

		Bitmap bitmap = chart.getChartBitmap();

		File dir, f = null;
		if ( Environment.getExternalStorageState().equals( Environment.MEDIA_MOUNTED ) )
		{
//			dir = report.getTempDir();
			dir = this.getTempDir();
			String location = dir.getAbsolutePath();
			String fileName = type + "-" + test.getPictureName().getName().replace( ".jpg", ".png" );
			f = new File( location + File.separator + fileName );
		}

		try {
			if(f!=null){
				FileOutputStream ostream = new FileOutputStream(f);
				bitmap.compress(CompressFormat.PNG, 10, ostream);
				ostream.close();

			}
			return f;
		}
		catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}

	public void viewResults(View v) {
		ui.openManager(this, test, test.getSavedFilesList());
	}

	public void loadResults(String fileName) {
		test.Clear();
		ui.display().clear();
		test.getResultsFromFile(fileName);
		ui.display().testNumber(test.getTestNum());
		ui.display().dai(test.getDAI());
		ui.display().pi(test.getPI());
		ui.display().svt(test.getSVT());
		ui.display().capacitance(test.getCapacitance());

		if (!test.getLongitude().equals("")) {
			ui.changeButtonImage(R.id.bt_gps, R.drawable.ic_gps_small_details, true);
		} else {
			ui.changeButtonImage(R.id.bt_gps, R.drawable.ic_gps_small, true);
		}

		resetCharts();

		showChart = false;
		this.showChart();

		ArrayList<Result> resultsArray;
		resultsArray = test.getResults();
		int lenght = resultsArray.size();

		for (int i = 0; i < lenght; i++) {
			int baseSeconds = Integer.parseInt(resultsArray.get(i).getSeconds());
			float resistance = Float.valueOf(resultsArray.get(i).getResistance());
			float voltage = Float.valueOf(resultsArray.get(i).getVoltage());
			float current = Float.valueOf(resultsArray.get(i).getCurrent());


			chartResistance.update(this, baseSeconds, resistance, voltage, current, false);
			chartVoltage.update(this, baseSeconds, resistance, voltage, current, false);
			chartCurrent.update(this, baseSeconds, resistance, voltage, current, false);
			table.update(this, baseSeconds, resistance, voltage, current, false);

			if (i == lenght - 1) { //Last value
				String formattedMilliseconds =
						String.format(Locale.US, "%02d", (baseSeconds / 1000) / 60)
								+ ":" + String.format(Locale.US, "%02d", (baseSeconds / 1000) % 60);
				String formattedResistance = resolutionR(resistance);
				String formattedVoltage = resolutionV(voltage / Units.KILO) + " kV";
				float tempCurrent = getSimpleVoltage(voltage) / resistance;
				String calculatedCurrent = resolutionA(tempCurrent);

				ui.display().elapsedTime(formattedMilliseconds);
				ui.display().resistance(formattedResistance);
				ui.display().current(calculatedCurrent);
				ui.display().voltage(formattedVoltage);
			}
		}

		chartResistance.repaint();
		chartVoltage.repaint();
		chartCurrent.repaint();
		table.repaint();

		ui.enableSaveButton(false, test);
		ui.enableRecordButton(true);
		ui.enablePictureButton(true);
	}

	private void createSwipe() {
		//Create page swipe				
		PagerAdapter mPagerAdapter = new PagerAdapter(this.getSupportFragmentManager(), this.getBaseContext());

		ViewPager mViewPager = findViewById(R.id.pager);

		if (mViewPager == null) {
			return;
		}

		mViewPager.setAdapter(mPagerAdapter);
		mViewPager.setOffscreenPageLimit(3);

		chartResistance = mPagerAdapter.chartResistance();
		chartVoltage = mPagerAdapter.chartVoltage();
		chartCurrent = mPagerAdapter.chartCurrent();
		table = mPagerAdapter.table();

		//Create customs Tabs
		SlidingTabLayout mSlidingTabLayout = findViewById(R.id.sliding_tabs);
		mSlidingTabLayout.setDistributeEvenly(true);
		mSlidingTabLayout.setCustomTabView(R.layout.custom_tab, 0);
		mSlidingTabLayout.setBackground(
				//this.getResources().getDrawable(R.drawable.chamfer_black_top)
				ResourcesCompat.getDrawable(getResources(), R.drawable.chamfer_black_top, null)
		);
		mSlidingTabLayout.setCustomTabColorizer(new SlidingTabLayout.TabColorizer() {
			public int getIndicatorColor(int position) {
				return Color.GRAY;
			}

			public int getDividerColor(int position) {
				return Color.WHITE;
			}
		});
		mSlidingTabLayout.setViewPager(mViewPager);
	}

	private void changeAlpha( int led, boolean show )
	{
		ImageView img = findViewById( R.id.status_led );
		if ( img != null )
		{
			LayerDrawable layer = (LayerDrawable) img.getDrawable();
			if ( layer != null )
			{
				if ( show )
				{
					layer.getDrawable( led ).setAlpha( 255 );
				}
				else
				{
					layer.getDrawable( led ).setAlpha( 0 );
				}
			}
		}
	}


	// ############################################
	// UTILITIES
	// ############################################

	private DecimalFormatSymbols separator;
	private DecimalFormat oneDigit;
	private DecimalFormat twoDigits;

	private void loadDecimalFormats() {
		separator = new DecimalFormatSymbols(Locale.getDefault());
		separator.setDecimalSeparator('.');
		oneDigit = new DecimalFormat("0.0", separator);
		oneDigit.setRoundingMode(RoundingMode.DOWN);
		twoDigits = new DecimalFormat("0.00", separator);
		twoDigits.setRoundingMode(RoundingMode.DOWN);
	}

	public String resolutionR(Float f) {
		try {
			String resistance;
			if (f >= Units.TERA) {
				resistance = twoDigits.format(f / Units.TERA) + " T\u03A9";
				if ((f / Units.TERA) >= 10) {
					resistance = ">10 T\u03A9";
				}
			} else if (f >= Units.GIGA) {
				float nrx = f / Units.GIGA;
				if (nrx >= 950) {
					resistance = twoDigits.format(nrx) + " G\u03A9";
				} else if (nrx >= 95) {
					int i = (int) nrx;
					resistance = i + "G\u03A9";
				} else if (nrx >= 10.5) {
					resistance = oneDigit.format(nrx) + " G\u03A9";
				} else if (nrx >= 1.05) {
					resistance = twoDigits.format(nrx) + " G\u03A9";
				} else {
					int i = (int) nrx * 1000;
					resistance = i + " M\u03A9";
				}
			} else if (f >= Units.MEGA) {
				float nrx = f / Units.MEGA;
				if (nrx >= 105.0) {
					int i = (int) nrx;
					resistance = i + " M\u03A9";
				}
				else if (nrx >= 10.5) {
					resistance = oneDigit.format(nrx) + " M\u03A9";
				}
				else {
					resistance = twoDigits.format(nrx) + " M\u03A9";
				}
			} else if (f > Units.KILO) {
				float nrx = f / Units.KILO;
				if (nrx >= 950) {
					resistance = twoDigits.format(nrx) + " k\u03A9";
				} else {
					int i = (int) nrx;
					resistance = i + " k\u03A9";
				}
			} else {
				resistance = f + " \u03A9";
			}
			return resistance;
		} catch (Exception e) {
			error("resolution", e);
			return "";
		}
	}

	public String resolutionV(Float f) {
		try {
			return twoDigits.format(f);
		} catch (Exception e) {
			error("resolution", e);
			return "";
		}
	}

	public String resolutionA(float f) {
		try {
			String a;
			if (f >= 1) {
				a = twoDigits.format(f) + " A";
			} else if (f * Units.MILI >= 1) {
				a = twoDigits.format(f * Units.MILI) + " mA";
			} else if (f * Units.MICRO >= 1) {
				if (f * Units.MICRO > 950) {
					a = twoDigits.format(f * Units.MICRO) + " \u00B5A";
				} else if (f * Units.MICRO > 105) {
					a = ((int) (f * Units.MICRO)) + " \u00B5A";
				} else if (f * Units.MICRO > 10.5) {
					a = oneDigit.format(f * Units.MICRO) + " \u00B5A";
				} else {
					a = twoDigits.format(f * Units.MICRO) + " \u00B5A";
				}
			} else if (f * Units.NANO >= 1) {
				if (f * Units.NANO > 105) {
					a = ((int) (f * Units.NANO)) + " nA";
				} else {
					a = oneDigit.format(f * Units.NANO) + " nA";
				}
			} else if (f * Units.PICO >= 1) {
				a = twoDigits.format(f * Units.PICO) + " pA";
			} else {
				a = "0 nA";
			}
			return a;
		} catch (Exception e) {
			error("resolution", e);
			return "";
		}
	}

	private String resolutionF(float f) {
		try {
			String a;
			DecimalFormat twoDigits = new DecimalFormat("0.00", separator);
			twoDigits.setRoundingMode(RoundingMode.DOWN);

			DecimalFormat oneDigit = new DecimalFormat("0", separator);
			oneDigit.setRoundingMode(RoundingMode.DOWN);

			if (f >= 1000) {
				a = twoDigits.format(f / 1000) + " \u00B5F";
			} else {
				a = oneDigit.format(f) + " nF";
			}
			return a;
		} catch (Exception e) {
			error("resolution", e);
			return "";
		}
	}

	private String checkPiDaiSvt(String value) {
		float f = Float.valueOf(MUtils.hexToFloat(value));
		if (f == 0) {
			return "???";
		} else {
			return f + "";
		}
	}

	public void ignoreChanges() {
		ignoreChange = true;
	}

	public void selectAnotherDevice() {
		Intent intent = new Intent(this, EnableBluetooth.class);
		startActivity(intent);
		finish();
	}

	@SuppressWarnings("StringConcatenationInLoop")
	public void error( String method, Exception error) {
		try {
			String msg = "";
			SimpleDateFormat dateFormatter = new SimpleDateFormat("dd/MM/yy HH:mm:ss.SSS", Locale.getDefault());
			String company = getResources().getString(R.string.company);
			String version = getPackageManager().getPackageInfo(getPackageName(), 0).versionName;
			String date = dateFormatter.format(new Date());
			StackTraceElement[] e = error.getStackTrace();

			msg += "DATE\t: \t" + date + "\r\n";
			msg += "BRAND\t: \t" + company + "\r\n";
			msg += "VERSION\t: \t" + version + "\r\n";
			msg += "CLASS\t: \t" + this.getClass().getName() + "\r\n";
			msg += "METHOD\t: \t" + method + "\r\n";
			msg += "CAUSE\t: \t" + error.toString() + "\r\n";
			msg += "STACK: \r\n";

			for ( StackTraceElement anE : e )
			{
				msg += anE.toString() + "\r\n";
			}
			msg += "\r\n";

			//writeErrorLog(msg);
			log("error log " + msg);
			//debbug("ERROR: " + msg);
		} catch (Exception e) {
			log("error log " + e.toString());
		}
	}

	private boolean isExternalStorageAvaiable()
	{
		String state = Environment.getExternalStorageState();
		return Environment.MEDIA_MOUNTED.equals( state );
	}

	@SuppressWarnings("ResultOfMethodCallIgnored")
	public boolean createFolder()
	{
		if ( !this.isExternalStorageAvaiable() )
		{
			return false;
		}

		String root = Environment.getExternalStorageDirectory().toString();
		rootDir = new File( root + "/" + getResources().getString( R.string.company ) );
		if ( !rootDir.exists() )
		{
			rootDir.mkdir();
		}

		dir = new File( root + "/" + getResources().getString( R.string.company ) + "/" + model );
		tempDir = new File( dir + "/temp" );

		if ( !dir.exists() )
		{
			dir.mkdir();
		}

		if ( !tempDir.exists() )
		{
			tempDir.mkdir();
		}

		reportPDF = new ReportPDF( this, dir, tempDir, handler );

		return dir.exists();
	}

	@SuppressWarnings("ResultOfMethodCallIgnored")
	public File getTempDir()
	{
		if ( !rootDir.exists() )
		{
			rootDir.mkdir();
		}
		if ( !dir.exists() )
		{
			dir.mkdir();
		}
		if ( !tempDir.exists() )
		{
			tempDir.mkdir();
		}
		return tempDir;
	}

	public File getDir()
	{
		return dir;
	}

	private void log( String msg )
	{
		if ( showlog )
		{
			long timeNow = System.currentTimeMillis();
			long timedelay = timeNow - logmillisec;
			if ( logmillisec == 0 )
			{
				timedelay = 0;
			}
			logmillisec = timeNow;
			Log.e( "LOG", String.format( Locale.US, "%04d", timedelay ) + "ms : " + msg );
		}
	}
}
