package com.megabras.bluelogg.md10kvx;

import android.app.Activity;
import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;

import com.megabras.bluelogg.extended.FileDetails;
import com.megabras.bluelogg.md10kvx.Database.DatabaseDescriptions;
import com.megabras.bluelogg.md10kvx.Database.DatabaseEntry;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

public class MD10KVxTest
{
	private final String
			TAG = "MD10KVxTest";

	private Context
			context;

	private Activity
			activity;

	private String
			MODEL = "",
			SERIAL = "",
			TEST_DATE = "",
			TEST_NUM = "",
			DAI = "",
			PI = "",
			SVT = "",
			CAP = "",
			GPS_LATITUDE = "",
			GPS_LONGITUDE = "",
			GPS_ALTITUDE = "",
			GPS_ACCURACY = "",
			TABLE_NAME = "";

	private File
			resultsFolder;

	private ArrayList<Result>
			resultsArray = new ArrayList<>();

	private Database.DbHelper
			mDbHelper;

	MD10KVxTest( Activity activity )
	{
		this.context = activity.getApplicationContext();
		this.activity = activity;
		createFolders();
		mDbHelper = new Database().new DbHelper( context );
	}

	@SuppressWarnings("ResultOfMethodCallIgnored")
	private void createFolders()
	{
		resultsFolder = new File( context.getFilesDir() + "/results" );
		if ( !resultsFolder.exists() )
		{
			resultsFolder.mkdir();
		}
	}

	public void setModel( String model )
	{
		MODEL = model;
	}

	public String getModel()
	{
		return MODEL;
	}

	void addResults( String seconds, String resistance, String voltage, String current )
	{
		resultsArray.add( new Result( seconds, resistance, voltage, current ) );
	}

	@SuppressWarnings("UseBulkOperation")
	public ArrayList<Result> getResults()
	{
		ArrayList<Result> s = new ArrayList<>();
		for ( Result r : resultsArray )
		{
			s.add( r );
		}
		return s;
	}

	public void setSerial( String serial )
	{
		SERIAL = serial;
	}

	public String getSerial()
	{
		return SERIAL;
	}

	void setTestDate( String date )
	{
		TEST_DATE = date;
	}

	String getTestDate()
	{
		return TEST_DATE;
	}

	String getTestDateAsString()
	{
		String[] datePieces = TEST_DATE.split( "_" );
		if ( datePieces.length >= 4 )
		{
			String date = datePieces[2] + "/" + datePieces[1] + "/" + datePieces[0];
			String time;
			if ( datePieces[3].length() == 4 )
			{
				time = datePieces[3].substring( 0, 2 ) + ":" + datePieces[3].substring( 2, 4 );
			}
			else
			{
				time = datePieces[3].substring( 0, 1 ) + ":" + datePieces[3].substring( 1, 3 );
			}
			return date + " " + time;
		}
		return TEST_DATE;
	}

	File getPictureName()
	{
		return new File( resultsFolder + "/" + TEST_NUM + TEST_DATE.replace( "_", "" ) + ".jpg" );
	}

	File getAudioName()
	{
		return new File( resultsFolder + "/" + TEST_NUM + TEST_DATE.replace( "_", "" ) + ".3gp" );
	}

	void setTestNum( String testNum )
	{
		TEST_NUM = testNum;
	}

	String getTestNum()
	{
		return TEST_NUM;
	}

	public void setDAI( String dai )
	{
		DAI = dai;
	}

	public String getDAI()
	{
		return DAI;
	}

	public void setPI( String pi )
	{
		PI = pi;
	}

	public String getPI()
	{
		return PI;
	}

	public void setSVT( String svt )
	{
		SVT = svt;
	}

	public String getSVT()
	{
		return SVT;
	}

	void setCapacitance( String capacitance )
	{
		CAP = capacitance;
	}

	public String getCapacitance()
	{
		return CAP;
	}

	public void setLatitude( String latitude )
	{
		GPS_LATITUDE = latitude;
	}

	public String getLatitude()
	{
		return GPS_LATITUDE;
	}

	public void setLongitude( String longitude )
	{
		GPS_LONGITUDE = longitude;
	}

	public String getLongitude()
	{
		return GPS_LONGITUDE;
	}

	void setAltidude( String altitude )
	{
		GPS_ALTITUDE = altitude;
	}

	public String getAltitude()
	{
		return GPS_ALTITUDE;
	}

	String getAltitudeAsString()
	{
		if ( !GPS_ALTITUDE.equals( "" ) )
		{
			return GPS_ALTITUDE + " m";
		}
		return GPS_ALTITUDE;
	}

	public void setAccuracy( String accuracy )
	{
		GPS_ACCURACY = accuracy;
	}

	public String getAccuracy()
	{
		return GPS_ACCURACY;
	}

	String getAccuracyAsString()
	{
		if ( !GPS_ACCURACY.equals( "" ) )
		{
			return "\u00B1" + GPS_ACCURACY + " m";
		}
		return GPS_ACCURACY;
	}

	List<FileDetails> getSavedFilesList()
	{
		try
		{
			List<FileDetails> filesList = new ArrayList<>();

			SQLiteDatabase db = mDbHelper.getReadableDatabase();
			Cursor c = db.rawQuery( "SELECT name FROM sqlite_master WHERE type='table'", null );

			if ( c.moveToFirst() )
			{
				while ( !c.isAfterLast() )
				{
					String tableName = c.getString( 0 );
					if ( !tableName.contains( "metadata" ) )
					{
						//android.util.Log.e(TAG, "TABLE: " + tableName);
						//MD10KVx_SERIALN_113_2016_04_12_1828
						FileDetails file = new FileDetails();
						String[] temp = tableName.split( "_" );
						file.FILENAME = tableName;
						file.MODEL = temp[0];
						file.SERIAL = temp[1];
						file.INDEX = temp[2];
						file.DATE = temp[5] + "/" + temp[4] + "/" + temp[3];
						String time;
						if ( temp[6].length() == 4 )
						{
							time = temp[6].substring( 0, 2 ) + ":" + temp[6].substring( 2, 4 );
						}
						else
						{
							time = temp[6].substring( 0, 1 ) + ":" + temp[6].substring( 1, 3 );
						}
						file.TIME = time;
						filesList.add( file );
					}

					c.moveToNext();
				}
			}
			c.close();
			return filesList;
		}
		catch ( Exception e )
		{
			return null;
		}
	}
	
	@SuppressWarnings({"ResultOfMethodCallIgnored", "StringConcatenationInLoop"})
	void deleteFile( String fileName ){
		
		File audioFile;		
		if(getAudioName().getName().equals(".3gp")){
			String[] pieces = fileName.split("_");
			String audioName=""; 
			for(int index = 2;index<pieces.length;index++){
				audioName += pieces[index];
			}
			audioFile = new File(resultsFolder + "/" + audioName + ".3gp");		
		}
		else{
			audioFile = getAudioName();
		}
		if(audioFile.exists()){
			audioFile.delete();
		}
		
		File picFile;		
		if(getPictureName().getName().equals(".jpg")){
			String[] pieces = fileName.split("_");
			String picName=""; 
			for(int index = 2;index<pieces.length;index++){
				picName += pieces[index];
			}
			picFile = new File(resultsFolder + "/" + picName + ".jpg");		
		}
		else{
			picFile = getPictureName();
		}
		if(picFile.exists()){
			picFile.delete();
		}
		
		SQLiteDatabase db = mDbHelper.getWritableDatabase();
		mDbHelper.deleteTable(db, fileName);
		
		if(fileName.equals(TABLE_NAME)){
			((MD10KVx) activity).clearAll();
		}
	}
	
	void Clear(){
		TABLE_NAME = "";
		TEST_DATE = "";	
		TEST_NUM = "";		
		DAI = "";
		PI = "";
		SVT = "";
		CAP = "";
		GPS_LATITUDE = "";
		GPS_LONGITUDE = "";
		GPS_ALTITUDE = "";
		GPS_ACCURACY = "";
		resultsArray.clear();
	}
	
	void updateGpsData(){
		try {
			
			if(TABLE_NAME.equals("")){
				return;
			}

			SQLiteDatabase db = mDbHelper.getWritableDatabase();
			
			db.beginTransaction();
			
			ContentValues values;

			values = new ContentValues();
			values.put(DatabaseEntry.DESCRIPTION, DatabaseDescriptions.GPS_LATITUDE );
			values.put(DatabaseEntry.COL_A, GPS_LATITUDE);				
			db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);
			
			values = new ContentValues();
			values.put(DatabaseEntry.DESCRIPTION, DatabaseDescriptions.GPS_LONGITUDE );
			values.put(DatabaseEntry.COL_A, GPS_LONGITUDE);				
			db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);
			
			values = new ContentValues();
			values.put(DatabaseEntry.DESCRIPTION, DatabaseDescriptions.GPS_ALTITUDE );
			values.put(DatabaseEntry.COL_A, GPS_ALTITUDE);				
			db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);
			
			values = new ContentValues();
			values.put(DatabaseEntry.DESCRIPTION, DatabaseDescriptions.GPS_ACCURACY );
			values.put(DatabaseEntry.COL_A, GPS_ACCURACY);				
			db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);
						
			android.util.Log.e(TAG, "END SAVING");
			
			db.setTransactionSuccessful();
			db.endTransaction();
			
		} catch (Exception e) {			
			android.util.Log.e(TAG, e.toString());
		}				
	}
	
	void Save(){
		try {			
			TABLE_NAME = MODEL + "_" + SERIAL + "_" + TEST_NUM + "_" + TEST_DATE;
			android.util.Log.e(TAG, "START SAVING");
			android.util.Log.e(TAG, TABLE_NAME);
			SQLiteDatabase db = mDbHelper.getWritableDatabase();
			mDbHelper.createTable(db, TABLE_NAME);				
			ContentValues values;
			
			db.beginTransaction();
			
			values = new ContentValues();
			values.put(DatabaseEntry.DESCRIPTION, DatabaseDescriptions.MODEL );
			values.put(DatabaseEntry.COL_A, MODEL);				
			db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);
			
			values = new ContentValues();
			values.put(DatabaseEntry.DESCRIPTION, DatabaseDescriptions.SERIAL );
			values.put(DatabaseEntry.COL_A, SERIAL);				
			db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);
			
			values = new ContentValues();
			values.put(DatabaseEntry.DESCRIPTION, DatabaseDescriptions.DATE );
			values.put(DatabaseEntry.COL_A, TEST_DATE);				
			db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);
			
			values = new ContentValues();
			values.put(DatabaseEntry.DESCRIPTION, DatabaseDescriptions.TEST_NUMBER );
			values.put(DatabaseEntry.COL_A, TEST_NUM);				
			db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);
			
			values = new ContentValues();
			values.put(DatabaseEntry.DESCRIPTION, DatabaseDescriptions.DAI );
			values.put(DatabaseEntry.COL_A, DAI);				
			db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);
			
			values = new ContentValues();
			values.put(DatabaseEntry.DESCRIPTION, DatabaseDescriptions.PI );
			values.put(DatabaseEntry.COL_A, PI);				
			db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);
			
			values = new ContentValues();
			values.put(DatabaseEntry.DESCRIPTION, DatabaseDescriptions.SVT );
			values.put(DatabaseEntry.COL_A, SVT);				
			db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);
			
			values = new ContentValues();
			values.put(DatabaseEntry.DESCRIPTION, DatabaseDescriptions.CAPACITANCE );
			values.put(DatabaseEntry.COL_A, CAP);				
			db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);
			
			values = new ContentValues();
			values.put(DatabaseEntry.DESCRIPTION, DatabaseDescriptions.GPS_LATITUDE );
			values.put(DatabaseEntry.COL_A, GPS_LATITUDE);				
			db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);
			
			values = new ContentValues();
			values.put(DatabaseEntry.DESCRIPTION, DatabaseDescriptions.GPS_LONGITUDE );
			values.put(DatabaseEntry.COL_A, GPS_LONGITUDE);				
			db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);
			
			values = new ContentValues();
			values.put(DatabaseEntry.DESCRIPTION, DatabaseDescriptions.GPS_ALTITUDE );
			values.put(DatabaseEntry.COL_A, GPS_ALTITUDE);				
			db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);
			
			values = new ContentValues();
			values.put(DatabaseEntry.DESCRIPTION, DatabaseDescriptions.GPS_ACCURACY );
			values.put(DatabaseEntry.COL_A, GPS_ACCURACY);				
			db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);
			
			int lenght = resultsArray.size();
							
			for(int i=0;i<lenght;i++){					
				values = new ContentValues();
				values.put(DatabaseEntry.DESCRIPTION, DatabaseDescriptions.RESULT );
				values.put(DatabaseEntry.COL_A, resultsArray.get(i).seconds);
				values.put(DatabaseEntry.COL_B, resultsArray.get(i).resistance);
				values.put(DatabaseEntry.COL_C, resultsArray.get(i).voltage);
				values.put(DatabaseEntry.COL_D, resultsArray.get(i).current);
				db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);
			}
			
			//resultsArray.clear();
			android.util.Log.e(TAG, "END SAVING");
			
			db.setTransactionSuccessful();
			db.endTransaction();
			
		} catch (Exception e) {			
			android.util.Log.e(TAG, e.toString());
		}				
	}
	
	@SuppressWarnings("IfCanBeSwitch")
	void getResultsFromFile( String tableName ){
		try {
			TABLE_NAME = tableName;
			SQLiteDatabase db = mDbHelper.getReadableDatabase();
			Cursor c = db.rawQuery("SELECT * FROM " + tableName, null);

			resultsArray.clear();
			
			if (c.moveToFirst()) {
			    while ( !c.isAfterLast() ) {			        
			        //android.util.Log.e(TAG, "RESULTS: " + c.getString(0) + " - " + c.getString(1) + " - " + c.getString(2) + " - " + c.getString(3) + " - " + c.getString(4));
			        
			        String colA = c.getString(0);
			        String colB = c.getString(1);
			        String colC = c.getString(2);
			        String colD = c.getString(3);
			        String colE = c.getString(4);
			        
			        if( colA.equals(DatabaseDescriptions.MODEL) ){
			        	MODEL = colB;
			        }
			        else if( colA.equals(DatabaseDescriptions.SERIAL) ){
			        	SERIAL = colB;
			        }
			        else if( colA.equals(DatabaseDescriptions.TEST_NUMBER) ){
			        	TEST_NUM = colB;
			        }
			        else if( colA.equals(DatabaseDescriptions.DATE) ){
			        	TEST_DATE = colB;
			        }
			        else if( colA.equals(DatabaseDescriptions.DAI) ){
			        	DAI = colB;
			        }
			        else if( colA.equals(DatabaseDescriptions.PI) ){
			        	PI = colB;
			        }
			        else if( colA.equals(DatabaseDescriptions.SVT) ){
			        	SVT = colB;
			        }
			        else if( colA.equals(DatabaseDescriptions.CAPACITANCE) ){
			        	CAP = colB;
			        }
			        else if( colA.equals(DatabaseDescriptions.GPS_LATITUDE) ){
			        	GPS_LATITUDE = colB;
			        }
			        else if( colA.equals(DatabaseDescriptions.GPS_LONGITUDE) ){
			        	GPS_LONGITUDE = colB;
			        }
			        else if( colA.equals(DatabaseDescriptions.GPS_ALTITUDE) ){
			        	GPS_ALTITUDE = colB;
			        }
			        else if( colA.equals(DatabaseDescriptions.GPS_ACCURACY) ){
			        	GPS_ACCURACY = colB;
			        }
			        else if( colA.equals(DatabaseDescriptions.RESULT) ){
						resultsArray.add(new Result(
							colB,
							colC,
							colD,
							colE ));	
			        }
			        

			        
			        c.moveToNext();
			    }
			}

			c.close();
		} catch (Exception e) {
			android.util.Log.e("TEST", e.toString());
		}
	}

	public class Result
	{
		private String
				seconds,
				resistance,
				voltage,
				current;

		Result( String seconds, String resistance, String voltage, String current )
		{
			this.seconds = seconds;
			this.resistance = resistance;
			this.voltage = voltage;
			this.current = current;
		}

		public String getSeconds()
		{
			return seconds;
		}

		public String getResistance()
		{
			return resistance;
		}

		public String getVoltage()
		{
			return voltage;
		}

		public String getCurrent()
		{
			return current;
		}
	}
}
