package com.megabras.bluelogg.md15kvr;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.Color;
import android.graphics.Paint;
import android.os.Bundle;
import android.util.DisplayMetrics;
import android.util.TypedValue;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.LinearLayout;
import android.widget.ListView;

import com.megabras.bluelogg.R;
import com.megabras.bluelogg.extended.Units;
import com.megabras.modbus.MD15KVR.Device;
import com.megabras.modbus.MD15KVR.DielectricDischarge;
import com.megabras.modbus.MD15KVR.Measurement;
import com.megabras.modbus.MD15KVR.Protocol;

import org.achartengine.ChartFactory;
import org.achartengine.GraphicalView;
import org.achartengine.model.XYMultipleSeriesDataset;
import org.achartengine.model.XYSeries;
import org.achartengine.renderer.XYMultipleSeriesRenderer;
import org.achartengine.renderer.XYSeriesRenderer;

import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;

import androidx.annotation.NonNull;
import androidx.fragment.app.Fragment;

@SuppressWarnings("WeakerAccess")
public class Charts extends Fragment
{
    @SuppressWarnings("unused")
    private final String
            TAG              = "Charts";

    public static final int
            RESISTANCE_CHART = 0,
            VOLTAGE_CHART    = 1,
            CURRENT_CHART    = 2,
            IDD_CHART        = 3,
            TABLE            = 4;

    protected int
            CHART_TYPE          = -1;

    @SuppressWarnings("FieldCanBeLocal")
    private final int
            MEASURE_GAP         = 5,
            MIN_TIME            = 15000;

    protected int
            BASE_OVERRANGE;

    private XYSeries
            mXYSeries;

    private GraphicalView
            mChart;

    protected double
            minY,
            maxY,
            minYdefault,
            maxYdefault;

    private long
            POWER            = 0,
            CHART_CORRECTION = 0;

    private XYMultipleSeriesRenderer
            mRenderer;

    protected DecimalFormat
            dosDigitos;

    private LinearLayout
            layout;

    private ListView
            list;

    private TableBaseAdapter
            adapter;

    protected String
            chartTitle = "";

    private Context
            context;

    protected com.megabras.modbus.MD15KVR.FormatData
            formatData;

    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState)
    {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(new Date());
        calendar.set(Calendar.MILLISECOND, 0);
        calendar.set(Calendar.SECOND     , 0);
        calendar.set(Calendar.MINUTE     , 0);
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        CHART_CORRECTION = calendar.getTimeInMillis();
        context          = inflater.getContext();
        return inflater.inflate(R.layout.md10kvx_fgr_online, container, false);
    }

    @Override
    public void onStart()
    {
        super.onStart();
        createChart();
    }

    public void setChartType(int type, String chartTitle)
    {
        CHART_TYPE      = type;
        this.chartTitle = chartTitle;
        formatData      = new com.megabras.modbus.MD15KVR.FormatData();
    }

    public void setOverrange(int OVERRANGE)
    {
        BASE_OVERRANGE = OVERRANGE;
    }

    public int getChartType()
    {
        return CHART_TYPE;
    }

    private void createChart()
    {
        POWER = 0;
        minY  = Double.MAX_VALUE;
        maxY  = Double.MIN_VALUE;
        dosDigitos = new DecimalFormat("#.##");

        switch (CHART_TYPE)
        {
            case Charts.RESISTANCE_CHART:
                Chart(chartTitle,"GOhm");
                break;
            case Charts.VOLTAGE_CHART:
                Chart(chartTitle,"V");
                break;
            case Charts.CURRENT_CHART:
                Chart(chartTitle,"A");
                break;
            case Charts.IDD_CHART:
                Chart(chartTitle,"A");
                break;
            case Charts.TABLE:
                Table();
                break;
        }
    }

    public Bitmap getChartBitmap()
    {
        if (mRenderer==null || mChart==null)
            return null;

        int gridColor   = mRenderer.getGridColor(0);
        int labelColor  = mRenderer.getLabelsColor();
        int yLabelColor = mRenderer.getYLabelsColor(0);
        int xLabelColor = mRenderer.getXLabelsColor();
        int axesColor   = mRenderer.getAxesColor();

        mRenderer.setGridColor   (Color.argb(150, 0, 0, 0));
        //mRenderer.setGridColor   (Color.BLACK);
        mRenderer.setLabelsColor (Color.BLACK);
        mRenderer.setYLabelsColor(0, Color.BLACK);
        mRenderer.setXLabelsColor(Color.BLACK);
        mRenderer.setAxesColor   (Color.BLACK);

        mChart.setDrawingCacheEnabled(true);
        Bitmap bitmap = mChart.getDrawingCache();

        mRenderer.setGridColor(gridColor);
        mRenderer.setLabelsColor(labelColor);
        mRenderer.setYLabelsColor(0, yLabelColor);
        mRenderer.setXLabelsColor(xLabelColor);
        mRenderer.setAxesColor(axesColor);
        return bitmap;
    }

    public void Table()
    {
        if( this.getView()==null )
            return;

        layout = this.getView().findViewById(R.id.chart);

        if( layout.getChildCount()>1 )
            return;

        list = new ListView( layout.getContext() );
        list.setDividerHeight(0);
        layout.addView(list);

        adapter = new TableBaseAdapter( layout.getContext() );
        list.setAdapter( adapter );
        list.setVisibility(View.VISIBLE);
    }

    public void setMode(int mode)
    {
        if (adapter!=null)
            adapter.setMode(mode);
    }

    public void setTestNumber(int testNumber)
    {
        showTable(true);
        if (adapter!=null)
            adapter.setTestNumber(testNumber);
    }

    public void setDate(String date)
    {
        if (adapter!=null)
            adapter.setDate(date);
    }

    public void setTitleDAI(String title)
    {
        if (adapter!=null)
            adapter.setTitleDAI(title);
    }

    public void setTitlePI(String title)
    {
        if (adapter!=null)
            adapter.setTitlePI(title);
    }

    public void setDAI(String dai)
    {
        if (adapter!=null)
            adapter.setDAI(dai);
    }

    public void setPI(String pi)
    {
        if (adapter!=null)
            adapter.setPI(pi);
    }

    public void setSVT(String svt)
    {
        if (adapter!=null)
            adapter.setSVT(svt);
    }

    public void setCAP(String cap)
    {
        if (adapter!=null)
            adapter.setCAP(cap);
    }

    public void setDD(String dd)
    {
        if (adapter!=null)
            adapter.setDD(dd);
    }

    public void showTable(boolean show)
    {
        if ( list!=null )
        {
            if(show)
                list.setVisibility(View.VISIBLE);
            else
                list.setVisibility(View.GONE);
        }
    }

    public void Chart(String chartTitle, String yTitle)
    {
        if(this.getView()==null)
            return;

        layout = this.getView().findViewById(R.id.chart);

        if(layout.getChildCount()>0)
            return;

        XYMultipleSeriesDataset mDataset = new XYMultipleSeriesDataset();
        mRenderer = new XYMultipleSeriesRenderer();
        XYSeriesRenderer mXYSeriesRenderer;
        mXYSeries = new XYSeries("Chart");
        mDataset.addSeries(mXYSeries);
        mXYSeriesRenderer = new XYSeriesRenderer();
        mXYSeriesRenderer.setColor(Color.RED);
        mXYSeriesRenderer.setFillPoints(true);
        mXYSeriesRenderer.setLineWidth(2);

        minYdefault = mRenderer.getYAxisMin();
        maxYdefault = mRenderer.getYAxisMax();

        DisplayMetrics metrics = context.getResources().getDisplayMetrics();

        mRenderer.setLabelsTextSize(
                TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, context.getResources().getDimension(R.dimen.chart_label), metrics)
        );

        mRenderer.setChartTitleTextSize(
                TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, context.getResources().getDimension(R.dimen.chart_title), metrics)
        );

        mRenderer.setAxisTitleTextSize(
                TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, context.getResources().getDimension(R.dimen.chart_title), metrics)
        );

        mRenderer.addSeriesRenderer(mXYSeriesRenderer);
        mRenderer.setAntialiasing(false);
        mRenderer.setShowLegend(false);
        mRenderer.setYTitle(yTitle);
        mRenderer.setYLabelsAlign(Paint.Align.RIGHT);
        mRenderer.setXTitle("h:mm:ss");
        mRenderer.setChartTitle(chartTitle);
        mRenderer.setApplyBackgroundColor(true);
        mRenderer.setBackgroundColor(Color.TRANSPARENT);
        mRenderer.setMarginsColor(Color.argb(0x00, 0x01, 0x01, 0x01));
        mRenderer.setAxesColor(Color.TRANSPARENT);
        mRenderer.setShowGrid(true);
        mRenderer.setGridColor(Color.argb(60, 255, 255, 255));
        mRenderer.setYLabels(6);
        mRenderer.setXLabels(8);
        mRenderer.setXAxisMin(CHART_CORRECTION);
        mRenderer.setMargins(new int[] { 25, 65, 0, 10 }); //top - left - bottom - right
        mRenderer.setPanEnabled(false, false);
        mRenderer.setZoomEnabled(false, false);



        if (this.getActivity()!=null)
        {
            mChart = ChartFactory.getTimeChartView(
                    this.getActivity().getApplicationContext(),
                    mDataset,
                    mRenderer,
                    "H:mm:ss");

            layout.addView(mChart);
        }
    }

    private double resolutionIDD(double value)
    {
        double r;
        if(POWER==0 && mRenderer!=null)
        {
            if(value * Units.MILI>=1)
            {
                r = (value * Units.MILI);
                mRenderer.setYTitle("mA");
                POWER = Units.MILI;
            }
            else if(value * Units.MICRO>=1)
            {
                r = (value * Units.MICRO);
                mRenderer.setYTitle("\u00B5A");
                POWER = Units.MICRO;
            }
            else if (value * Units.NANO>=1)
            {
                r = (value * Units.NANO);
                mRenderer.setYTitle("nA");
                POWER = Units.NANO;
            }
            else if (value * Units.PICO>=1)
            {
                r = (value * Units.PICO);
                mRenderer.setYTitle("pA");
                POWER = Units.PICO;
            }
            else
            {
                r = 0;
            }
        }
        else
        {
            r = (value * POWER);
        }
        updateScaleYIDD();
        return r;
    }

    private double resolution(double value, String unit)
    {
        double r;
        if(POWER==0 && mRenderer!=null){
            if (value>= Units.TERA){
                r = value / Units.TERA;
                mRenderer.setYTitle("T"+unit);
                POWER= Units.TERA;
            }
            else if(value>=Units.GIGA){
                r = value / Units.GIGA;
                mRenderer.setYTitle("G"+unit);
                POWER=Units.GIGA;
            }
            else if(value>=Units.MEGA){
                r = value / Units.MEGA;
                mRenderer.setYTitle("M"+unit);
                POWER=Units.MEGA;
            }
            else if(value>Units.KILO){
                r = value / Units.KILO;
                mRenderer.setYTitle("k"+unit);
                POWER=Units.KILO;
            }
            else if (value >= 1) {
                r = value;
                mRenderer.setYTitle(unit);
                POWER=1;
            }
            else if(value * Units.MILI>=1){
                r = (value * Units.MILI);
                mRenderer.setYTitle("m"+unit);
                POWER = Units.MILI;
            }
            else if(value * Units.MICRO>=1){
                r = (value * Units.MICRO);
                mRenderer.setYTitle("\u00B5"+unit);
                POWER = Units.MICRO;
            }
            else if (value * Units.NANO>=1) {
                r = (value * Units.NANO);
                mRenderer.setYTitle("n"+unit);
                POWER = Units.NANO;
            }
            else if (value * Units.PICO>=1) {
                r = (value * Units.PICO);
                mRenderer.setYTitle("p"+unit);
                POWER = Units.PICO;
            }
            else {
                r = 0;
            }
        }
        else {
            if(value >= 1){
                r = (value / POWER);
            } else {
                r = (value * POWER);
            }
        }
        updateScaleY();
        return r;
    }

    private void updateScaleYIDD()
    {
        double diff = mXYSeries.getMaxY() - mXYSeries.getMinY();
        double p10  = diff * 0.1; // 10%

        mRenderer.setYAxisMin(mXYSeries.getMinY() - p10);
        mRenderer.setYAxisMax(mXYSeries.getMaxY() + p10);
    }

//    private void updateScaleY(double newY)
//    {
//        if (newY < minY)
//            minY = newY;
//
//        if (newY > maxY)
//            maxY = newY;
//
//        double diff = maxY - minY;
//        double perc = (diff*100)/maxY;
//        perc = 100 - perc;
//        perc = perc/100;
//        double toAddMax = (maxY * perc)/2;
//        double toAddMin = (maxY * perc)/2;
//        double lowY     = minY - toAddMin;
//        double upY      = maxY + toAddMax;
//
//        if(lowY<0)
//            lowY=0;
//
//        mRenderer.setYAxisMin(lowY);
//        mRenderer.setYAxisMax(upY);
//    }

    private void updateScaleY()
    {
        double diff     = mXYSeries.getMaxY() - mXYSeries.getMinY();
        double perc     = (diff * 100) / mXYSeries.getMaxY();
        perc            = 100 - perc;
        perc            = perc / 100;
        double toAddMax = (mXYSeries.getMaxY() * perc) / 2;
        double toAddMin = (mXYSeries.getMaxY() * perc) / 2;
        double lowY     = mXYSeries.getMinY() - toAddMin;
        double upY      = mXYSeries.getMaxY() + toAddMax;

        if(lowY < 0)
            lowY = 0;

        mRenderer.setYAxisMin(lowY);
        mRenderer.setYAxisMax(upY);
    }

    public void update(Measurement measurement, Device device, boolean repaint, boolean updateLast)
    {
        if ( !updateLast )
        {
            if( measurement.getElapsed()%MEASURE_GAP!=0 )
                return;
        }

        if( (measurement.getElapsed()*1000)<MIN_TIME )
            return;

        long
                milliseconds = measurement.getElapsed()*1000;
        int
                voltage      = measurement.getVoltage();
        double
                current      = measurement.getCurrent(),
                resistance   = measurement.getResistance();
        boolean
                overrangeR   = resistance>=(BASE_OVERRANGE*Units.TERA),
                overrangeA   = current<(1/Units.PICO);

        if(overrangeR || overrangeA)
        {
            resistance = 0;
            current    = 0;
        }

        if( mXYSeries !=null && mXYSeries.getItemCount()>0 )
        {
            int index = mXYSeries.getItemCount()-1;
            if (mXYSeries.getX(index)==milliseconds+CHART_CORRECTION)
            {
                mXYSeries.remove(index);
            }
        }

        if(milliseconds==MIN_TIME && mRenderer!=null)
        {
            mChart.setVisibility(View.VISIBLE);
            mRenderer.setAxesColor(Color.WHITE);
        }

        switch (CHART_TYPE)
        {
            case Charts.RESISTANCE_CHART:
                if (mXYSeries!=null)
                    mXYSeries.add( milliseconds+CHART_CORRECTION, resolution(resistance, "\u03A9") );
                break;
            case Charts.VOLTAGE_CHART:
                if (mXYSeries!=null)
                    mXYSeries.add( milliseconds+CHART_CORRECTION, resolution(voltage, "V") );
                break;
            case Charts.CURRENT_CHART:
                if (mXYSeries!=null)
                    mXYSeries.add( milliseconds+CHART_CORRECTION, resolution(current, "A") );
                break;
            case Charts.TABLE:
//                if ( measurement.getElapsed()%MEASURE_GAP!=0 )
//                    return;
                String formattedResistance   = formatData.formatResistance( resistance );
                String formattedMilliseconds = formatData.formatElapsedTime((short)(milliseconds/1000));
                String formattedVoltage      = formatData.formatVoltage(voltage);
                String calculatedCurrent     = formatData.formatCurrent(current);

                if (adapter.getCount()>0)
                {
                    int index = adapter.getCount()-1;
                    if (adapter.getItem(index).getTime().equals(formattedMilliseconds))
                        adapter.remove(index);
                }

                if( device.getStatus(Device.ST_OVERRANGE) )
                {
                    calculatedCurrent = "0nA";
                    formattedResistance = ">" + BASE_OVERRANGE + "T\u03A9";
                }
//
//                if( device.getStatus(Device.ST_WAIT) ){
//                    calculatedCurrent = " - - - ";
//                    formattedResistance = " - - - ";
//                }

                adapter.add(
                        formattedMilliseconds,
                        formattedResistance,
                        formattedVoltage,
                        calculatedCurrent );

                if(repaint)
                    adapter.repaint();
                break;
        }

        if(repaint && mChart !=null)
            mChart.repaint();
    }

    public void update(Measurement measurement)
    {
        long
                milliseconds = measurement.getElapsed()*1000;
        int
                voltage      = measurement.getVoltage();
        double
                current      = measurement.getCurrent(),
                resistance   = measurement.getResistance();
        boolean
                overrangeR   = resistance>=(BASE_OVERRANGE*Units.TERA),
                overrangeA   = current<(1/Units.PICO);

        if(overrangeR || overrangeA)
        {
            resistance = 0;
            current    = 0;
        }

        if( mXYSeries !=null && mXYSeries.getItemCount()>0 )
        {
            int index = mXYSeries.getItemCount()-1;
            if (mXYSeries.getX(index)==milliseconds+CHART_CORRECTION)
            {
                mXYSeries.remove(index);
            }
        }

        if(milliseconds==MIN_TIME && mRenderer!=null)
        {
            mChart.setVisibility(View.VISIBLE);
            mRenderer.setAxesColor(Color.WHITE);
        }

        switch (CHART_TYPE)
        {
            case Charts.RESISTANCE_CHART:
                if (mXYSeries!=null)
                    mXYSeries.add( milliseconds+CHART_CORRECTION, resolution(resistance, "\u03A9") );
                break;
            case Charts.VOLTAGE_CHART:
                if (mXYSeries!=null)
                    mXYSeries.add( milliseconds+CHART_CORRECTION, resolution(voltage, "V") );
                break;
            case Charts.CURRENT_CHART:
                if (mXYSeries!=null)
                    mXYSeries.add( milliseconds+CHART_CORRECTION, resolution(current, "A") );
                break;
            case Charts.TABLE:
                String formattedResistance   = formatData.formatResistance  ( resistance );
                String formattedMilliseconds = formatData.formatElapsedTime ((short)(milliseconds/1000));
                String formattedVoltage      = formatData.formatVoltage     (voltage);
                String calculatedCurrent     = formatData.formatCurrent     (current);

                if (adapter.getCount()>0)
                {
                    int index = adapter.getCount()-1;
                    if (adapter.getItem(index).getTime().equals(formattedMilliseconds))
                        adapter.remove(index);
                }

                adapter.add(
                        formattedMilliseconds,
                        formattedResistance,
                        formattedVoltage,
                        calculatedCurrent );

                adapter.repaint();
                break;
        }

        if (mChart!=null)
            mChart.repaint();
    }

    public void update(DielectricDischarge dd)
    {
        if (dd.getElapsed()<1)
            return;

        long milliseconds = dd.getElapsed()*1000;
        double current    = dd.getAmps();

        if( mXYSeries!=null && mXYSeries.getItemCount()>0 )
        {
            int index = mXYSeries.getItemCount()-1;
            if (mXYSeries.getX(index)==milliseconds+CHART_CORRECTION)
            {
                mXYSeries.remove(index);
            }
        }

        if(mChart.getVisibility()!=View.VISIBLE && mRenderer!=null)
        {
            mChart.setVisibility(View.VISIBLE);
            mRenderer.setAxesColor(Color.WHITE);
        }

        switch (CHART_TYPE)
        {
            case Charts.IDD_CHART:
                if (mXYSeries!=null)
                    mXYSeries.add( milliseconds+CHART_CORRECTION, resolutionIDD(current) );
                break;
        }

        if (mChart!=null)
            mChart.repaint();
    }

    public void repaint()
    {
        switch (CHART_TYPE)
        {
            case Charts.RESISTANCE_CHART:
                mChart.repaint();
                break;
            case Charts.VOLTAGE_CHART:
                mChart.repaint();
                break;
            case Charts.CURRENT_CHART:
                mChart.repaint();
                break;
            case Charts.IDD_CHART:
                mChart.repaint();
                break;
            case Charts.TABLE:
                adapter.repaint();
                break;
        }
    }

    public void setAllValues(ArrayList<Measurement> measurements)
    {
        switch (CHART_TYPE)
        {
            case Charts.RESISTANCE_CHART:
                mXYSeries.clear();
                for ( Measurement m : measurements )
                {
                    double resistance = m.getResistance();
                    if( resistance>(BASE_OVERRANGE*Units.TERA) || m.getCurrent()<Protocol.MIN_CURRENT)
                        resistance = 0;
                    mXYSeries.add( (m.getElapsed()*1000)+CHART_CORRECTION, resolution(resistance, "\u03A9") );
                }
                mChart.setVisibility(View.VISIBLE);
                mRenderer.setAxesColor(Color.WHITE);
                mChart.repaint();
                break;
            case Charts.VOLTAGE_CHART:
                mXYSeries.clear();
                for ( Measurement m : measurements )
                {
                    mXYSeries.add( (m.getElapsed()*1000)+CHART_CORRECTION, resolution(m.getVoltage(), "V") );
                }
                mChart.setVisibility(View.VISIBLE);
                mRenderer.setAxesColor(Color.WHITE);
                mChart.repaint();
                break;
            case Charts.CURRENT_CHART:
                mXYSeries.clear();
                for ( Measurement m : measurements )
                {
                    double current = m.getCurrent();
                    if( m.getResistance()>(BASE_OVERRANGE*Units.TERA) || m.getCurrent()<Protocol.MIN_CURRENT)
                        current = 0;
                    mXYSeries.add( (m.getElapsed()*1000)+CHART_CORRECTION, resolution(current, "A") );
                }
                mChart.setVisibility(View.VISIBLE);
                mRenderer.setAxesColor(Color.WHITE);
                mChart.repaint();
                break;
            case Charts.TABLE:
                adapter.clear();
                for ( Measurement m : measurements )
                {
                    String formattedResistance   = formatData.formatResistance(m.getResistance());
                    String formattedMilliseconds = formatData.formatElapsedTime((short) m.getElapsed());
                    String formattedVoltage      = formatData.formatVoltage(m.getVoltage());
                    String calculatedCurrent     = formatData.formatCurrent(m.getCurrent());

                    adapter.add(
                            formattedMilliseconds,
                            formattedResistance,
                            formattedVoltage,
                            calculatedCurrent);
                }
                adapter.repaint();
                break;
        }
    }

    public void setAllIDD(ArrayList<DielectricDischarge> ddArray)
    {
        if (ddArray==null)
            return;
        switch (CHART_TYPE)
        {
            case Charts.IDD_CHART:
                mXYSeries.clear();
                for ( DielectricDischarge dd : ddArray )
                {
                    mXYSeries.add( (dd.getElapsed()*1000)+CHART_CORRECTION, resolutionIDD(dd.getAmps()));
                }
                if (ddArray.size()>0)
                {
                    mChart.setVisibility(View.VISIBLE);
                    mRenderer.setAxesColor(Color.WHITE);
                    mChart.repaint();
                }
                break;
        }
    }

    public void reset()
    {
        try
        {
            if (layout==null || adapter==null)
                createChart();

            if(CHART_TYPE==Charts.TABLE)
            {
                adapter.clear();
                adapter.repaint();
                layout.removeAllViews();
                layout.addView(list);
                showTable(false);
            }
            else
            {
                mXYSeries.clear();
                mChart.repaint();
                mChart.setVisibility(View.GONE);
                POWER = 0;
                minY  = Double.MAX_VALUE;
                maxY  = Double.MIN_VALUE;
            }
        }
        catch (Exception e)
        {
            android.util.Log.e("ERROR", e.getMessage()+"");
        }
    }
}
