package com.megabras.bluelogg.md15kvr;

import android.annotation.SuppressLint;
import android.app.Activity;
import android.app.Dialog;
import android.graphics.Bitmap;
import android.os.Environment;
import android.text.InputFilter;
import android.util.Log;
import android.util.TypedValue;
import android.view.View;
import android.view.Window;
import android.view.WindowManager;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.FrameLayout;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;
import android.widget.Spinner;
import android.widget.TableRow;
import android.widget.TextView;

import com.megabras.bluelogg.MUtils;
import com.megabras.bluelogg.R;
import com.megabras.bluelogg.extended.InputFilterMinMax;
import com.megabras.modbus.MD15KVR.Device;
import com.megabras.modbus.MD15KVR.Protocol;
import com.megabras.modbus.Units;

import java.io.File;
import java.io.FileOutputStream;
import java.text.DecimalFormat;

@SuppressWarnings("WeakerAccess")
@SuppressLint("SetTextI18n")
public class Helper
{
    @SuppressWarnings("FieldCanBeLocal")
    private String
            TAG = "HELPER";

    protected Activity
            activity;

    protected int
            MIN_VOLTAGE,
            MAX_VOLTAGE,
            STEP_VOLTAGE,
            MIN_STEP,
            MAX_STEP;

    private DecimalFormat
            format02 = new DecimalFormat("00");

    protected com.megabras.modbus.MD15KVR.FormatData
            formatData;

    public Helper(Activity activity)
    {
        this.activity = activity;
        MIN_VOLTAGE   = com.megabras.modbus.MD15KVR.Protocol.MIN_VOLTAGE;
        MAX_VOLTAGE   = com.megabras.modbus.MD15KVR.Protocol.MAX_VOLTAGE;
        STEP_VOLTAGE  = com.megabras.modbus.MD15KVR.Protocol.STEP_VOLTAGE;
        MIN_STEP      = com.megabras.modbus.MD15KVR.Protocol.MIN_STEP;
        MAX_STEP      = com.megabras.modbus.MD15KVR.Protocol.MAX_STEP;
        formatData    = new com.megabras.modbus.MD15KVR.FormatData();
    }

    public File saveChartImage(Charts chart, File dir)
    {
        String type = "";
        switch ( chart.getChartType() )
        {
            case Charts.RESISTANCE_CHART:
                type = "resistance";
                break;
            case Charts.VOLTAGE_CHART:
                type = "voltage";
                break;
            case Charts.CURRENT_CHART:
                type = "current";
                break;
            case Charts.IDD_CHART:
                type = "dielectric_discharge";
                break;
        }

        Bitmap bitmap = chart.getChartBitmap();
        if (bitmap==null)
            return null;

        File f = null;
        if ( Environment.getExternalStorageState().equals(Environment.MEDIA_MOUNTED) )
        {
            String location = dir.getAbsolutePath();
            String fileName = type + ".png";
            f = new File(location + File.separator + fileName);
        }

        try
        {
            if ( f!=null )
            {
                FileOutputStream ostream = new FileOutputStream(f);
                bitmap.compress(Bitmap.CompressFormat.PNG, 10, ostream);
                ostream.close();
            }
            return f;
        }
        catch (Exception e)
        {
            e.printStackTrace();
            return null;
        }
    }

    public void calculateFontSize(TableRow container, TextView label)
    {
        try
        {
            String text = "000.00 MO";
            label.setText(text);
            label.measure(View.MeasureSpec.UNSPECIFIED, label.getHeight());

            for (int i = 1; label.getMeasuredWidth() < container.getWidth() - 10 && label.getMeasuredHeight() < container.getHeight() - 5; i++)
            {
                label.setTextSize(TypedValue.COMPLEX_UNIT_PX, label.getTextSize() + 1);
                label.measure(View.MeasureSpec.UNSPECIFIED, label.getHeight());

                if ( i>100 )
                    break;
            }
        }
        catch (Exception e)
        {
            Log.e(TAG, e.getMessage());
        }
    }

    public LinearLayout getLL(int layoutID, View view)
    {
        return (LinearLayout) view.findViewById(layoutID);
    }

    public Spinner getSpinner(int spinnerID, int arrayID, View inflatedView)
    {
        ArrayAdapter<CharSequence> filter =
                ArrayAdapter.createFromResource(activity, arrayID, android.R.layout.simple_spinner_item);
        filter.setDropDownViewResource(android.R.layout.simple_list_item_activated_1);

        final Spinner spinner = inflatedView.findViewById(spinnerID);
        spinner.setAdapter(filter);
        return spinner;
    }

    public Spinner getSpinner(int spinnerID, String[] values, View inflatedView)
    {
        final Spinner spinner = inflatedView.findViewById(spinnerID);
        ArrayAdapter<CharSequence> filter = new ArrayAdapter<>(activity,
                android.R.layout.simple_spinner_item,
                values);
        filter.setDropDownViewResource(android.R.layout.simple_list_item_activated_1);
        spinner.setAdapter(filter);
        return spinner;
    }

    public void closeOverlay(FrameLayout v)
    {
        v.removeAllViews();
        v.setVisibility(View.GONE);
    }

    public void pagePickNumberDaiA(final TextView tv)
    {
        final Dialog dialog = new Dialog(activity);
        dialog.requestWindowFeature(Window.FEATURE_NO_TITLE);
        dialog.setCanceledOnTouchOutside(false);
        dialog.setContentView(R.layout.main_dialog_number);
//        if (dialog.getWindow()!=null)
//            dialog.getWindow().getAttributes().width = WindowManager.LayoutParams.MATCH_PARENT;

        final EditText etNumber = dialog.findViewById(R.id.et_number);
        final TextView message  = dialog.findViewById(R.id.tv_message);
        final TextView warning  = dialog.findViewById(R.id.tv_warning);
        final TextView tvUnit   = dialog.findViewById(R.id.tv_unit);

        tvUnit.setText("s");
        String dai = "DAI - Ra";
        message.setText(dai);

        final int min  = 30;
        final int max  = 600;
        final int step = 15;
        String warningMessage =
                activity.getString(R.string.range) + ": "
                        + min + " s - " + max + " s"
                        + " (" + activity.getString(R.string.step, (step+" s")) + ")";
        warning.setText(warningMessage);

        etNumber.setText( tv.getText().toString().replace("s","").trim() );

        Button buttonSave = dialog.findViewById(R.id.bt_save);
        buttonSave.setOnClickListener(v ->
        {
            if( !etNumber.getText().toString().equals("") )
            {
                int newValue = Integer.parseInt(etNumber.getText() + "");
                newValue = (newValue/step)*step;

                if (newValue >= min && newValue <= max)
                {
                    tv.setText(newValue + " s");
                    dialog.dismiss();
                }
                else
                {
                    warning.setTextColor(activity.getResources().getColor(R.color.common_signin_btn_default_background));
                }
            }
        });

        Button buttonCancel = dialog.findViewById(R.id.bt_cancel);
        buttonCancel.setOnClickListener(v -> dialog.dismiss());

        dialog.show();
    }

    public void pagePickNumberDaiB(final TextView tv)
    {
        final Dialog dialog = new Dialog(activity);
        dialog.requestWindowFeature(Window.FEATURE_NO_TITLE);
        dialog.setCanceledOnTouchOutside(false);
        dialog.setContentView(R.layout.main_dialog_number);
//        if (dialog.getWindow()!=null)
//            dialog.getWindow().getAttributes().width = WindowManager.LayoutParams.MATCH_PARENT;

        final EditText etNumber = dialog.findViewById(R.id.et_number);
        final TextView message  = dialog.findViewById(R.id.tv_message);
        final TextView warning  = dialog.findViewById(R.id.tv_warning);
        final TextView tvUnit   = dialog.findViewById(R.id.tv_unit);

        tvUnit.setText("s");

        message.setText("DAI - Rb");

        final int min  = 15;
        final int max  = 30;
        final int step = 5;
        String warningMessage =
                activity.getString(R.string.range) + ": "
                        + min + " s - " + max + " s"
                        + " (" + activity.getString(R.string.step, (step+" s")) + ")";
        warning.setText(warningMessage);

        etNumber.setText( tv.getText().toString().replace("s","").trim() );

        Button buttonSave = dialog.findViewById(R.id.bt_save);
        buttonSave.setOnClickListener(v ->
        {
            if( !etNumber.getText().toString().equals("") )
            {
                int newValue = Integer.parseInt(etNumber.getText() + "");
                newValue = (newValue/step)*step;

                if (newValue >= min && newValue <= max) {
                    tv.setText(newValue + " s");
                    dialog.dismiss();
                } else {
                    warning.setTextColor(activity.getResources().getColor(R.color.common_signin_btn_default_background));
                }
            }
        });

        Button buttonCancel = dialog.findViewById(R.id.bt_cancel);
        buttonCancel.setOnClickListener(v -> dialog.dismiss());

        dialog.show();
    }

    public void pagePickNumberPiA(final TextView tv)
    {
        final Dialog dialog = new Dialog(activity);
        dialog.requestWindowFeature(Window.FEATURE_NO_TITLE);
        dialog.setCanceledOnTouchOutside(false);
        dialog.setContentView(R.layout.main_dialog_number);
//        if (dialog.getWindow()!=null)
//            dialog.getWindow().getAttributes().width = WindowManager.LayoutParams.MATCH_PARENT;

        final EditText etNumber = dialog.findViewById(R.id.et_number);
        final TextView message  = dialog.findViewById(R.id.tv_message);
        final TextView warning  = dialog.findViewById(R.id.tv_warning);
        final TextView tvUnit   = dialog.findViewById(R.id.tv_unit);

        tvUnit.setText("m");

        message.setText("PI - Ra");

        final int min  = 10;
        final int max  = 90;
        final int step = 5;
        String warningMessage =
                activity.getString(R.string.range) + ": "
                        + min + " m - " + max + " m"
                        + " (" + activity.getString(R.string.step, (step+" m")) + ")";
        warning.setText(warningMessage);

        etNumber.setText( tv.getText().toString().replace("m","").trim() );

        Button buttonSave = dialog.findViewById(R.id.bt_save);
        buttonSave.setOnClickListener(v -> {
            if( !etNumber.getText().toString().equals("") )
            {
                int newValue = Integer.parseInt(etNumber.getText() + "");
                newValue = (newValue/step)*step;

                if (newValue >= min && newValue <= max) {
                    tv.setText(newValue + " m");
                    dialog.dismiss();
                } else {
                    warning.setTextColor(activity.getResources().getColor(R.color.common_signin_btn_default_background));
                }
            }
        });

        Button buttonCancel = dialog.findViewById(R.id.bt_cancel);
        buttonCancel.setOnClickListener(v -> dialog.dismiss());

        dialog.show();
    }

    public void pagePickNumberPiB(final TextView tv)
    {
        final Dialog dialog = new Dialog(activity);
        dialog.requestWindowFeature(Window.FEATURE_NO_TITLE);
        dialog.setCanceledOnTouchOutside(false);
        dialog.setContentView(R.layout.main_dialog_number);
//        if (dialog.getWindow()!=null)
//            dialog.getWindow().getAttributes().width = WindowManager.LayoutParams.MATCH_PARENT;

        final EditText etNumber = dialog.findViewById(R.id.et_number);
        final TextView message  = dialog.findViewById(R.id.tv_message);
        final TextView warning  = dialog.findViewById(R.id.tv_warning);
        final TextView tvUnit   = dialog.findViewById(R.id.tv_unit);

        tvUnit.setText("m");

        message.setText("PI - Rb");

        final int min  = 1;
        final int max  = 5;
        final int step = 1;
        String warningMessage =
                activity.getString(R.string.range) + ": "
                        + min + " m - " + max + " m"
                        + " (" + activity.getString(R.string.step, (step+" m")) + ")";
        warning.setText(warningMessage);

        etNumber.setText( tv.getText().toString().replace("m","").trim() );

        Button buttonSave = dialog.findViewById(R.id.bt_save);
        buttonSave.setOnClickListener(v -> {
            if( !etNumber.getText().toString().equals("") )
            {
                int newValue = Integer.parseInt(etNumber.getText() + "");
                newValue = (newValue/step)*step;

                if (newValue >= min && newValue <= max) {
                    tv.setText(newValue + " m");
                    dialog.dismiss();
                } else {
                    warning.setTextColor(activity.getResources().getColor(R.color.common_signin_btn_default_background));
                }
            }
        });

        Button buttonCancel = dialog.findViewById(R.id.bt_cancel);
        buttonCancel.setOnClickListener(v -> dialog.dismiss());

        dialog.show();
    }

    public void pagePickNumberTimer(final TextView tv)
    {
        final Dialog dialog = new Dialog(activity);
        dialog.requestWindowFeature(Window.FEATURE_NO_TITLE);
        dialog.setCanceledOnTouchOutside(false);
        dialog.setContentView(R.layout.main_dialog_timer);
        if (dialog.getWindow()!=null)
            dialog.getWindow().getAttributes().width = WindowManager.LayoutParams.MATCH_PARENT;

        final EditText etNumberMM = dialog.findViewById(R.id.et_number_mm);
        final EditText etNumberSS = dialog.findViewById(R.id.et_number_ss);
        final TextView message    = dialog.findViewById(R.id.tv_message);
        final TextView warning    = dialog.findViewById(R.id.tv_warning);

        message.setText( activity.getString(R.string.test_duration) );

        etNumberMM.setFilters(new InputFilter[]{ new InputFilterMinMax(1, 90)});
        etNumberSS.setFilters(new InputFilter[]{ new InputFilterMinMax(0, 59)});

        String[] parts = tv.getText().toString().split(":");
        etNumberMM.setText( Integer.parseInt(parts[0])+"" );
        etNumberSS.setText( parts[1] );

        final int min = 1;
        final int max = 90;
        String warningMessage =
                activity.getString(R.string.range) + ": " + min + " m - " + max + " m";

        warning.setText( warningMessage );

        Button buttonSave = dialog.findViewById(R.id.bt_save);
        buttonSave.setOnClickListener(v -> {
            if( etNumberMM.getText().toString().equals("") || etNumberSS.getText().toString().equals("") )
                return;

            int minutes = Integer.parseInt(etNumberMM.getText().toString());
            int seconds = Integer.parseInt(etNumberSS.getText().toString());
            int time    = (minutes*60) + seconds;

            if( time >= (min*60) && time <= (max*60) )
            {
                tv.setText( format02.format(minutes) + ":" + format02.format(seconds) );
                dialog.dismiss();
            }
            else
            {
                warning.setTextColor(activity.getResources().getColor(R.color.common_signin_btn_default_background));
            }
        });

        Button buttonCancel = dialog.findViewById(R.id.bt_cancel);
        buttonCancel.setOnClickListener(v -> dialog.dismiss());

        dialog.show();
    }

    public void pagePickNumberStepVoltage(final TextView tv)
    {
        final Dialog dialog = new Dialog(activity);
        dialog.requestWindowFeature(Window.FEATURE_NO_TITLE);
        dialog.setCanceledOnTouchOutside(false);
        dialog.setContentView(R.layout.main_dialog_number);
//        if (dialog.getWindow()!=null)
//            dialog.getWindow().getAttributes().width = WindowManager.LayoutParams.MATCH_PARENT;

        final int min  = MIN_STEP;
        final int max  = MAX_STEP;
        final int step = STEP_VOLTAGE;

        final EditText etNumber = dialog.findViewById(R.id.et_number);
        final TextView message  = dialog.findViewById(R.id.tv_message);
        final TextView warning  = dialog.findViewById(R.id.tv_warning);

        message.setText( activity.getString(R.string.step_voltage) );
        etNumber.setText( tv.getText().toString().replace("V", "").trim() );


        String warningMessage = activity.getString(R.string.step_var_v, min, max, step);

        warning.setText( warningMessage );

        Button buttonSave = dialog.findViewById(R.id.bt_save);
        buttonSave.setOnClickListener(v -> {
            if( etNumber.getText().toString().equals("") )
                return;

            String userStep = etNumber.getText().toString();
            int voltage     = Integer.parseInt(userStep);
            voltage         = (voltage/step)*step;

            if( voltage >= min && voltage <= max )
            {
                tv.setText( voltage + " V" );
                dialog.dismiss();
            }
            else
            {
                warning.setTextColor(activity.getResources().getColor(R.color.common_signin_btn_default_background));
            }
        });

        Button buttonCancel = dialog.findViewById(R.id.bt_cancel);
        buttonCancel.setOnClickListener(v -> dialog.dismiss());

        dialog.show();
    }

    public void pagePickNumberMaxVoltage(final int mode, final TextView tv_manual_v, final TextView tv_max_v)
    {
        final Dialog dialog = new Dialog(activity);
        dialog.requestWindowFeature(Window.FEATURE_NO_TITLE);
        dialog.setCanceledOnTouchOutside(false);
        dialog.setContentView(R.layout.main_dialog_number);
//        if (dialog.getWindow()!=null)
//            dialog.getWindow().getAttributes().width = WindowManager.LayoutParams.MATCH_PARENT;

        final EditText etNumber = dialog.findViewById(R.id.et_number);
        final TextView message  = dialog.findViewById(R.id.tv_message);
        final TextView warning  = dialog.findViewById(R.id.tv_warning);

        message.setText ( activity.getString(R.string.max_voltage) );
        etNumber.setText( tv_max_v.getText().toString().replace("V", "").trim() );

        String warningMessage = activity.getString(R.string.step_var_v, MIN_VOLTAGE, MAX_VOLTAGE, STEP_VOLTAGE);

        warning.setText( warningMessage );

        Button buttonSave = dialog.findViewById(R.id.bt_save);
        buttonSave.setOnClickListener(v ->
        {
            if( etNumber.getText().toString().equals("") )
                return;

            String userStep = etNumber.getText().toString();
            int voltage     = Integer.parseInt(userStep);
            voltage         = (voltage/STEP_VOLTAGE)*STEP_VOLTAGE;

            if( voltage >= MIN_VOLTAGE && voltage <= MAX_VOLTAGE )
            {
                // Check if manual voltage is higher than MAX. VOLTAGE
                if ( mode == Protocol.MODE_MANUAL || mode == Protocol.MODE_TIMER )
                {
                    int manual_voltage = Integer.parseInt( tv_manual_v.getText().toString().replace("V","").trim() );
                    if ( manual_voltage > voltage )
                        tv_manual_v.setText( voltage + " V" );
                }

                tv_max_v.setText( voltage + " V" );
                dialog.dismiss();
            }
            else
            {
                warning.setTextColor(activity.getResources().getColor(R.color.common_signin_btn_default_background));
            }
        });

        Button buttonCancel = dialog.findViewById(R.id.bt_cancel);
        buttonCancel.setOnClickListener(v -> dialog.dismiss());

        dialog.show();
    }

    public void pagePickNumberStepTime(final TextView tv)
    {
        final Dialog dialog = new Dialog(activity);
        dialog.requestWindowFeature(Window.FEATURE_NO_TITLE);
        dialog.setCanceledOnTouchOutside(false);
        dialog.setContentView(R.layout.main_dialog_number);
//        if (dialog.getWindow()!=null)
//            dialog.getWindow().getAttributes().width = WindowManager.LayoutParams.MATCH_PARENT;

        final int min  = 15;
        final int max  = 300;
        final int step = 5;

        final EditText etNumber = dialog.findViewById(R.id.et_number);
        final TextView message  = dialog.findViewById(R.id.tv_message);
        final TextView warning  = dialog.findViewById(R.id.tv_warning);

        message.setText( activity.getString(R.string.step_time) );
        etNumber.setText( tv.getText().toString().replace("s", "").trim() );

        String warningMessage = activity.getString(R.string.step_var_s, min, max, step);

        warning.setText( warningMessage );

        Button buttonSave = dialog.findViewById(R.id.bt_save);
        buttonSave.setOnClickListener(v -> {
            if( etNumber.getText().toString().equals("") )
                return;

            String userStep = etNumber.getText().toString();
            int voltage     = Integer.parseInt(userStep);
            voltage         = (voltage/step)*step;

            if( voltage >= min && voltage <= max )
            {
                tv.setText( voltage + " s" );
                dialog.dismiss();
            }
            else
            {
                warning.setTextColor(activity.getResources().getColor(R.color.common_signin_btn_default_background));
            }
        });

        Button buttonCancel = dialog.findViewById(R.id.bt_cancel);
        buttonCancel.setOnClickListener(v -> dialog.dismiss());

        dialog.show();
    }

    public void pagePickNumberManualVoltage(final Device device, final Controller controller, final com.megabras.modbus.MD15KVR.Protocol modbus)
    {
        final Dialog dialog = new Dialog(activity);
        dialog.requestWindowFeature(Window.FEATURE_NO_TITLE);
        dialog.setCanceledOnTouchOutside(false);
        dialog.setContentView(R.layout.main_dialog_number);
//        if (dialog.getWindow()!=null)
//            dialog.getWindow().getAttributes().width = WindowManager.LayoutParams.MATCH_PARENT;

        final EditText etNumber = dialog.findViewById(R.id.et_number);
        final TextView message  = dialog.findViewById(R.id.tv_message);
        final TextView warning  = dialog.findViewById(R.id.tv_warning);

        final int step = 25;

        message.setText( activity.getString(R.string.Test_voltage) );
        etNumber.setText( device.getTestVoltage()+"" );

        int maxVoltage = MAX_VOLTAGE;
        switch (device.getOperationMode())
        {
            case Protocol.MODE_MANUAL:
                maxVoltage = device.getParamManual().getMAX_VOLTAGE();
                break;
            case Protocol.MODE_TIMER:
                maxVoltage = device.getParamTimer().getMAX_VOLTAGE();
                break;
        }

        final int finalVoltage = maxVoltage;

        String warningMessage =
                activity.getString(R.string.range)
                        + ": " + MIN_VOLTAGE + " V - "
                        + finalVoltage + " V"
                        + " ("+activity.getString(R.string.step_25V)+")";

        warning.setText( warningMessage );

        Button buttonSave = dialog.findViewById(R.id.bt_save);
        buttonSave.setOnClickListener(v -> {
            if( etNumber.getText().toString().equals("") )
                return;

            int newVoltage = Integer.parseInt(etNumber.getText()+"");
            newVoltage = (newVoltage/step)*step;

            if( newVoltage >= MIN_VOLTAGE && newVoltage <= finalVoltage )
            {
                device.setTestVoltage(newVoltage);
                controller.sendMessage( modbus.write.voltage( device.getTestVoltage() ) );
                dialog.dismiss();
            }
            else
            {
                warning.setTextColor(activity.getResources().getColor(R.color.common_signin_btn_default_background));
            }
        });

        Button buttonCancel = dialog.findViewById(R.id.bt_cancel);
        buttonCancel.setOnClickListener(v -> dialog.dismiss());

        dialog.show();
    }

    public void pagePickNumber(final TextView tvMax, final TextView tv)
    {
        final Dialog dialog = new Dialog(activity);
        dialog.requestWindowFeature(Window.FEATURE_NO_TITLE);
        dialog.setCanceledOnTouchOutside(false);
        dialog.setContentView(R.layout.main_dialog_number);
//        if (dialog.getWindow()!=null)
//            dialog.getWindow().getAttributes().width = WindowManager.LayoutParams.MATCH_PARENT;

        final EditText etNumber = dialog.findViewById(R.id.et_number);
        final TextView message  = dialog.findViewById(R.id.tv_message);
        final TextView warning  = dialog.findViewById(R.id.tv_warning);

        message.setText( activity.getString(R.string.Test_voltage) );
        etNumber.setText( tv.getText().toString().replace("V", "").trim() );

        final int maxVoltage = Integer.parseInt(tvMax.getText().toString().replace("V", "").trim());

        String warningMessage =
                activity.getString(R.string.range)
                        + ": " + MIN_VOLTAGE + " V - "
                        + maxVoltage + " V"
                        + " ("+activity.getString(R.string.step_25V)+")";

        warning.setText( warningMessage );

        Button buttonSave = dialog.findViewById(R.id.bt_save);
        buttonSave.setOnClickListener(v -> {
            if( etNumber.getText().toString().equals("") )
                return;

            String userVoltage = etNumber.getText().toString();
            String lastDigits  = userVoltage.substring(userVoltage.length()-2);
            int step           = Integer.parseInt(lastDigits);

            if (step<=12)
                userVoltage = userVoltage.substring(0, userVoltage.length()-2) + "00";
            else if (step<=37)
                userVoltage = userVoltage.substring(0, userVoltage.length()-2) + "25";
            else if (step<=62)
                userVoltage = userVoltage.substring(0, userVoltage.length()-2) + "50";
            else
            {
                int mostSignificant = Integer.parseInt(userVoltage.substring(0, userVoltage.length() - 2));
                mostSignificant++;
                userVoltage = mostSignificant + "00";
            }

            int newVoltage = Integer.parseInt(etNumber.getText()+"");
            if( newVoltage >= MIN_VOLTAGE && newVoltage <= maxVoltage )
            {
                tv.setText( userVoltage + " V" );
                dialog.dismiss();
            }
            else
            {
                warning.setTextColor(activity.getResources().getColor(R.color.common_signin_btn_default_background));
            }
        });

        Button buttonCancel = dialog.findViewById(R.id.bt_cancel);
        buttonCancel.setOnClickListener(v -> dialog.dismiss());

        dialog.show();
    }

    public void pagePickNumberUnity(final TextView tv)
    {
        final Dialog dialog = new Dialog(activity);
        dialog.requestWindowFeature(Window.FEATURE_NO_TITLE);
        dialog.setCanceledOnTouchOutside(false);
        dialog.setContentView(R.layout.main_dialog_number_unity_selector);
//        if (dialog.getWindow()!=null)
//            dialog.getWindow().getAttributes().width = WindowManager.LayoutParams.MATCH_PARENT;

        final EditText etNumber = dialog.findViewById(R.id.et_number);
        final TextView message  = dialog.findViewById(R.id.tv_message);
        final TextView warning  = dialog.findViewById(R.id.tv_warning);

        message.setText( activity.getString(R.string.minimum_resistance) );

        warning.setVisibility(View.VISIBLE);
        String warningMessage = activity.getString(R.string.range) + ": " + "100 k" + Units.SYMBOL_OHM + " - " + "1 T" + Units.SYMBOL_OHM;
        warning.setText( warningMessage );

        final Spinner unity = dialog.findViewById(R.id.spManualLimUnity);

        String resistance = tv.getText().toString();

        if ( !resistance.equals( activity.getResources().getStringArray(R.array.enable_disable)[1] ))
        {
            String[] parts = resistance.split(" ");
            etNumber.setText(parts[0]);

            if (parts[1].startsWith("k"))
                unity.setSelection(0);
            else if (parts[1].startsWith("M"))
                unity.setSelection(1);
            else if (parts[1].startsWith("G"))
                unity.setSelection(2);
            else if (parts[1].startsWith("T"))
                unity.setSelection(3);
        }
        else
        {
            etNumber.setText("100");
            unity.setSelection(0);
        }

        Button buttonSave = dialog.findViewById(R.id.bt_save);
        buttonSave.setOnClickListener(v -> {
            if( etNumber.getText().toString().equals("") )
                return;

            long newValue = Integer.parseInt(etNumber.getText()+"");
            switch (unity.getSelectedItemPosition())
            {
                case 0:
                    newValue = newValue * Units.KILO;
                    break;
                case 1:
                    newValue = newValue * Units.MEGA;
                    break;
                case 2:
                    newValue = newValue * Units.GIGA;
                    break;
                case 3:
                    newValue = newValue * Units.TERA;
                    break;
            }

            if( newValue >= (100*Units.KILO) && newValue <= (Units.TERA) )
            {
                tv.setText( formatData.formatMaxResistance(newValue) );
                dialog.dismiss();
            }
            else
            {
                warning.setTextColor(activity.getResources().getColor(R.color.common_signin_btn_default_background));
            }
        });

        Button buttonCancel = dialog.findViewById(R.id.bt_cancel);
        buttonCancel.setOnClickListener(v -> dialog.dismiss());

        dialog.show();
    }

    public void changeMaxVoltageSelector(int mode, TextView tvVoltage, TextView tv, Device device)
    {
        int currentVoltage = Integer.parseInt( tvVoltage.getText().toString().replace("V","").trim() );
        switch (mode)
        {
            case Protocol.MODE_MANUAL:
                tv.setText( device.getParamManual().getMAX_VOLTAGE() + " V" );
                if ( currentVoltage>device.getParamManual().getMAX_VOLTAGE() )
                    tvVoltage.setText( device.getParamManual().getMAX_VOLTAGE() + " V" );
                break;
            case Protocol.MODE_TIMER:
                tv.setText( device.getParamTimer().getMAX_VOLTAGE() + " V" );
                if ( currentVoltage>device.getParamTimer().getMAX_VOLTAGE() )
                    tvVoltage.setText( device.getParamTimer().getMAX_VOLTAGE() + " V" );
                break;
            case Protocol.MODE_SVT:
                tv.setText( device.getParamSVT().getFINAL_VOLTAGE() + " V" );
                break;
            case Protocol.MODE_RAMP:
                tv.setText( device.getParamRamp().getFINAL_VOLTAGE() + " V" );
                break;
        }
    }

    public void changeTimerSelector(int mode, TextView tv, LinearLayout lineTime, Device device)
    {
        int time = 0;
        lineTime.setVisibility(View.GONE);
        switch (mode)
        {
            case Protocol.MODE_TIMER:
                lineTime.setVisibility(View.VISIBLE);
                time = device.getParamTimer().getTEST_TIME();
                break;
            case Protocol.MODE_RAMP:
                lineTime.setVisibility(View.VISIBLE);
                time = device.getParamRamp().getTEST_TIME();
                break;
        }
        tv.setText( formatData.formatElapsedTime((short)time) );
    }

    public void changePISelector(int mode, TextView tvA, TextView tvB, LinearLayout lineIndexes, Device device)
    {
        switch (mode)
        {
            case Protocol.MODE_MANUAL:
                lineIndexes.setVisibility(View.VISIBLE);
                tvA.setText( device.getParamManual().getPI_TIME_A() + " m" );
                tvB.setText( device.getParamManual().getPI_TIME_B() + " m" );
                break;
            case Protocol.MODE_TIMER:
                lineIndexes.setVisibility(View.VISIBLE);
                tvA.setText( device.getParamTimer().getPI_TIME_A() + " m" );
                tvB.setText( device.getParamTimer().getPI_TIME_B() + " m" );
                break;
            case Protocol.MODE_SVT:
                lineIndexes.setVisibility(View.GONE);
                break;
            case Protocol.MODE_RAMP:
                lineIndexes.setVisibility(View.GONE);
                break;
        }
    }

    public void changeDAISelector(int mode, TextView tvA, TextView tvB, LinearLayout lineIndexes, Device device)
    {
        switch (mode)
        {
            case Protocol.MODE_MANUAL:
                lineIndexes.setVisibility(View.VISIBLE);
                tvA.setText( device.getParamManual().getDAI_TIME_A() + " s" );
                tvB.setText( device.getParamManual().getDAI_TIME_B() + " s" );
                break;
            case Protocol.MODE_TIMER:
                lineIndexes.setVisibility(View.VISIBLE);
                tvA.setText( device.getParamTimer().getDAI_TIME_A() + " s" );
                tvB.setText( device.getParamTimer().getDAI_TIME_B() + " s" );
                break;
            case Protocol.MODE_SVT:
                lineIndexes.setVisibility(View.GONE);
                break;
            case Protocol.MODE_RAMP:
                lineIndexes.setVisibility(View.GONE);
                break;
        }
    }

    public void changeMaxResistanceSelector(int mode, TextView tv, Spinner sp, LinearLayout line, Device device)
    {
        double resistance = 0.0f;
        switch (mode)
        {
            case Protocol.MODE_MANUAL:
                resistance = device.getParamManual().getLIMIT_RESISTANCE();
                break;
            case Protocol.MODE_TIMER:
                resistance = device.getParamTimer().getLIMIT_RESISTANCE();
                break;
            case Protocol.MODE_SVT:
                resistance = device.getParamSVT().getLIMIT_RESISTANCE();
                break;
            case Protocol.MODE_RAMP:
                resistance = device.getParamRamp().getLIMIT_RESISTANCE();
                break;
        }

        if (resistance<=0)
        {
            tv.setText("100 k"+Units.SYMBOL_OHM);
            sp.setSelection(1);
            line.setVisibility(View.GONE);
        }
        else
        {
            tv.setText(formatData.formatMaxResistance((float)resistance));
            sp.setSelection(0);
            line.setVisibility(View.VISIBLE);
        }
    }

    @SuppressLint("ClickableViewAccessibility")
    public void warnAboutLock(int overlayID)
    {
        final View
                inflatedView = View.inflate(activity, R.layout.md10kvx_fgr_msg, null);

        final FrameLayout
                overlay = MUtils.getOverlay(activity, overlayID, inflatedView);

        //This is to prevent the ontouch propagate to others views
        RelativeLayout overlayBase = inflatedView.findViewById(R.id.overlay_base);
        overlayBase.requestDisallowInterceptTouchEvent(true);
        overlayBase.setOnTouchListener((v, event) -> true);

        TextView title = inflatedView.findViewById(R.id.tx_title);
        title.setText(activity.getResources().getString(R.string.md_remote_control_locked));

        TextView message = inflatedView.findViewById(R.id.tx_message);
        message.setText(activity.getResources().getString(R.string.md_remote_control_locked_explained));

        Button btClose = inflatedView.findViewById(R.id.bt_confirm);
        btClose.setVisibility(View.GONE);

        Button btCancel = inflatedView.findViewById(R.id.bt_cancel);
        btCancel.setText(activity.getResources().getString(R.string.md_ok));
        btCancel.setOnClickListener(v -> closeOverlay(overlay));
    }
}
