package com.megabras.bluelogg.md15kvr;

import android.content.Context;
import android.text.SpannableString;
import android.text.style.RelativeSizeSpan;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.TextView;

import com.megabras.bluelogg.R;
import com.megabras.bluelogg.extended.TableValues;
import com.megabras.modbus.MD15KVR.Protocol;

import java.util.ArrayList;
import java.util.List;

@SuppressWarnings("WeakerAccess")
public class TableBaseAdapter extends BaseAdapter
{
    private LayoutInflater
            inflater;

    private List<TableValues>
            tableValues;

    @SuppressWarnings("FieldCanBeLocal")
    private final int
            headerLayout = R.layout.md15kvr_table_header,
            lineLayout   = R.layout.table_item;

    private int
            modeOption   = 0;

    private String
            testNumber   = " - - -",
            dai          = " - - -",
            pi           = " - - -",
            cap          = " - - -",
            dd           = " - - -",
            mode         = " - - -",
            svt          = " - - -",
            daiTitle     = "DAI",
            piTitle      = "PI",
            date         = " - - -";

    public TableBaseAdapter(Context context)
    {
        tableValues  = new ArrayList<>();
        inflater     = LayoutInflater.from(context);
    }

    static class ViewHolder
    {
        TextView
                tvTestNumber,
                tvTestDate,
                tvMode,
                tvTitleDAI,
                tvTitlePI,
                tvTitleSVT,
                tvTitleDD,
                tvDD,
                tvDAI,
                tvPI,
                tvSVT,
                tvCAP,
                tv_elapsed,
                tv_resistance,
                tv_voltage,
                tv_current;
    }

    public long getItemId(int position)
    {
        return 0;
    }

    public int getCount()
    {
        return tableValues.size();
    }

    public TableValues getItem(int position)
    {
        return tableValues.get(position);
    }

    public void add(String time, String resistance, String voltage, String current)
    {
        tableValues.add( new TableValues(time, resistance, voltage, current) );
    }

    public void remove(int position)
    {
        if ( tableValues.size() > position )
            tableValues.remove(position);
    }

    public void clear()
    {
        if ( testNumber!=null )
            testNumber = " - - -";
        if ( dai!=null )
            dai = " - - -";
        if ( pi!=null )
            pi = " - - -";
        if ( cap!=null )
            cap = " - - -";
        if ( mode!=null )
            mode = " - - -";
        if ( svt!=null )
            svt = " - - -";
        if ( daiTitle!=null )
            daiTitle = "DAI";
        if ( piTitle!=null )
            piTitle = "PI";
        if ( date!=null )
            date = " - - -";
        if ( dd!=null )
            dd = " - - -";
        tableValues.clear();
    }

    public void repaint()
    {
        notifyDataSetChanged();
    }

    private TextView getTV(int id, View v)
    {
        return ( (TextView)v.findViewById(id) );
    }

    public void setTestNumber(int testNumber)
    {
        this.testNumber = testNumber+"";
    }

    public void setMode(int mode)
    {
        modeOption = mode;
        this.mode  = inflater.getContext().getResources().getStringArray(R.array.op_modes)[mode];
    }

    public void setDate(String date)
    {
        this.date = date;
    }

    public void setTitleDAI(String title)
    {
        this.daiTitle = title;
    }

    public void setTitlePI(String title)
    {
        this.piTitle = title;
    }

    public void setDAI(String dai)
    {
        this.dai = dai;
        notifyDataSetChanged();
    }

    public void setPI(String pi)
    {
        this.pi = pi;
        notifyDataSetChanged();
    }

    public void setSVT(String svt)
    {
        this.svt = svt;
        notifyDataSetChanged();
    }

    public void setCAP(String cap)
    {
        this.cap = cap;
        notifyDataSetChanged();
    }

    public void setDD(String dd)
    {
        this.dd = dd;
        notifyDataSetChanged();
    }

    @SuppressWarnings("RedundantCast")
    public View getView(int position, View vi, ViewGroup parent)
    {
        if ( position==0 ) // First result
        {
            if ( vi == null || (int)vi.getId()!= lineLayout)
            {
                ViewHolder h    = new ViewHolder();
                vi              = inflater.inflate(headerLayout, parent, false);
                h.tvTestNumber  = getTV(R.id.tv_number , vi);
                h.tvTestDate    = getTV(R.id.tv_date   , vi);
                h.tvMode        = getTV(R.id.tv_mode   , vi);
                h.tvTitleDAI    = getTV(R.id.tvTitleDAI, vi);
                h.tvTitleSVT    = getTV(R.id.tvTitleSVT, vi);
                h.tvTitlePI     = getTV(R.id.tvTitlePI , vi);
                h.tvTitleDD     = getTV(R.id.tvTitleDD , vi);
                h.tvDAI         = getTV(R.id.tvDAI     , vi);
                h.tvPI          = getTV(R.id.tvPI      , vi);
                h.tvSVT         = getTV(R.id.tvSVT     , vi);
                h.tvCAP         = getTV(R.id.tvCAP     , vi);
                h.tvDD          = getTV(R.id.tvDD      , vi);
                h.tv_elapsed    = getTV(R.id.col_a, vi);
                h.tv_resistance = getTV(R.id.col_c, vi);
                h.tv_voltage    = getTV(R.id.col_b, vi);
                h.tv_current    = getTV(R.id.col_d, vi);
                vi.setTag(h);
            }

            ViewHolder h = (ViewHolder) vi.getTag();
            if (Controller.firmware<Controller.FIRMWARE_DD)
            {
                h.tvDD.setVisibility(View.GONE);
                h.tvTitleDD.setVisibility(View.GONE);
            }

            h.tvTestNumber.setText(testNumber);
            h.tvTestDate.setText(date);
            h.tvMode.setText(mode);
            switch (modeOption){
                case Protocol.MODE_MANUAL:
                    h.tvTitleDAI.setVisibility(View.VISIBLE);
                    h.tvDAI.setVisibility(View.VISIBLE);
                    h.tvTitlePI.setVisibility(View.VISIBLE);
                    h.tvPI.setVisibility(View.VISIBLE);
                    h.tvTitleSVT.setVisibility(View.GONE);
                    h.tvSVT.setVisibility(View.GONE);
                    break;
                case Protocol.MODE_TIMER:
                    h.tvTitleDAI.setVisibility(View.VISIBLE);
                    h.tvDAI.setVisibility(View.VISIBLE);
                    h.tvTitlePI.setVisibility(View.VISIBLE);
                    h.tvPI.setVisibility(View.VISIBLE);
                    h.tvTitleSVT.setVisibility(View.GONE);
                    h.tvSVT.setVisibility(View.GONE);
                    break;
                case Protocol.MODE_SVT:
                    h.tvTitleDAI.setVisibility(View.GONE);
                    h.tvDAI.setVisibility(View.GONE);
                    h.tvTitlePI.setVisibility(View.GONE);
                    h.tvPI.setVisibility(View.GONE);
                    h.tvTitleSVT.setVisibility(View.VISIBLE);
                    h.tvSVT.setVisibility(View.VISIBLE);
                    break;
                case Protocol.MODE_RAMP:
                    h.tvTitleDAI.setVisibility(View.GONE);
                    h.tvDAI.setVisibility(View.GONE);
                    h.tvTitlePI.setVisibility(View.GONE);
                    h.tvPI.setVisibility(View.GONE);
                    h.tvTitleSVT.setVisibility(View.GONE);
                    h.tvSVT.setVisibility(View.GONE);
                    break;
            }
            SpannableString ssDAI = new SpannableString(daiTitle);
            ssDAI.setSpan(new RelativeSizeSpan(0.7f), 3, daiTitle.length(), 0); // set size
            h.tvTitleDAI.setText(ssDAI);
            SpannableString ssPI  = new SpannableString(piTitle);
            ssPI.setSpan(new RelativeSizeSpan(0.7f), 2, piTitle.length(), 0); // set size
            h.tvTitlePI.setText(ssPI);
            h.tvDAI.setText(dai);
            h.tvPI.setText(pi);
            h.tvSVT.setText(svt);
            h.tvCAP.setText(cap);
            h.tvDD.setText(dd);
            h.tv_elapsed.setText   ( tableValues.get(position).getTime()       );
            h.tv_voltage.setText   ( tableValues.get(position).getVoltage()    );
            h.tv_resistance.setText( tableValues.get(position).getResistance() );
            h.tv_current.setText   ( tableValues.get(position).getCurrent()    );
        }
        else // Second or higher result
        {
            if ( vi == null || (int)vi.getId()!=headerLayout)
            {
                ViewHolder h    = new ViewHolder();
                vi              = inflater.inflate(lineLayout, parent, false);
                h.tv_elapsed    = getTV(R.id.col_a, vi);
                h.tv_resistance = getTV(R.id.col_c, vi);
                h.tv_voltage    = getTV(R.id.col_b, vi);
                h.tv_current    = getTV(R.id.col_d, vi);
                vi.setTag(h);
            }
            ViewHolder h = (ViewHolder) vi.getTag();
            h.tv_elapsed.setText   ( tableValues.get(position).getTime()       );
            h.tv_voltage.setText   ( tableValues.get(position).getVoltage()    );
            h.tv_resistance.setText( tableValues.get(position).getResistance() );
            h.tv_current.setText   ( tableValues.get(position).getCurrent()    );
        }
        return vi;
    }

}

