package com.megabras.bluelogg.mpk257;

import android.app.Activity;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.os.Handler;

import com.itextpdf.text.Document;
import com.itextpdf.text.PageSize;
import com.itextpdf.text.pdf.PdfWriter;
import com.itextpdf.tool.xml.XMLWorkerHelper;
import com.megabras.bluelogg.Main;
import com.megabras.bluelogg.R;
import com.megabras.bluelogg.extended.ReportProvider;
import com.megabras.modbus.MPK257.Device;
import com.megabras.modbus.MPK257.FormatData;
import com.megabras.modbus.MPK257.Measurement;
import com.megabras.modbus.MPK257.Protocol;
import com.megabras.modbus.MPK257.TestData;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.Date;
import java.util.Locale;

public class Report extends ReportProvider
{
    @SuppressWarnings("unused")
    private final String
            TAG = "MPK257_Report";

    private int
            tableWidth = 450;

    public Report(Activity activity, File dir, File tempDir, Handler handler)
    {
        super(activity, dir, tempDir, handler);
    }

    void saveDocReport(Database.ObjectData obj, ArrayList<TestData> testData, Device device, File picture, File map, File reportFile){
        try
        {
            handler.obtainMessage(Controller.mHandler.REPORT_UPDATE, 1, -1, "Creating file").sendToTarget();
            Document  document  = new Document(PageSize.A4, 25, 25, 25, 15);
            PdfWriter pdfWriter = PdfWriter.getInstance(document, new FileOutputStream(reportFile));

            document.open();
            document.addTitle(activity.getString(R.string.test_report));

            InputStream fin     = activity.getAssets().open("MPK257-template.html");
            BufferedReader r    = new BufferedReader(new InputStreamReader(fin));
            StringBuilder total = new StringBuilder();
            String line;

            while ((line = r.readLine()) != null)
            {
                total.append(line);
            }

            XMLWorkerHelper worker = XMLWorkerHelper.getInstance();
            String html            = total.toString();

            String date = FormatData.getDateFormatted(
                    new Date(),
                    device.getDateFormat(),
                    device.getTimeFormat()
            );

            String
                    altitude = obj.getAltitude().equals("")    ? "" : obj.getAltitude() + " m",
                    accuracy = obj.getGpsAccuracy().equals("") ? "" : "\u00B1" + obj.getGpsAccuracy() + " m" ;

            html = html.replace( "{TITLE}"          , activity.getString(R.string.test_report)     );
            html = html.replace( "{LB_DATE}"        , activity.getString(R.string.date_time)       );
            html = html.replace( "{DATE}"           , date                                         );
            html = html.replace( "{LB_MODEL}"       , activity.getString(R.string.md_model)        );
            html = html.replace( "{MODEL}"          , testData.get(0).getModel()                   );
            html = html.replace( "{LB_SERIAL}"      , activity.getString(R.string.md_serial)       );
            html = html.replace( "{SERIAL}"         , testData.get(0).getSerial()                  );
            html = html.replace( "{LB_LATITUDE}"    , activity.getString(R.string.md_latitude)     );
            html = html.replace( "{LATITUDE}"       , obj.getLatitude()                            );
            html = html.replace( "{LB_LONGITUDE}"   , activity.getString(R.string.md_longitude)    );
            html = html.replace( "{LONGITUDE}"      , obj.getLongitude()                           );
            html = html.replace( "{LB_ALTITUDE}"    , activity.getString(R.string.md_altitude)     );
            html = html.replace( "{ALTITUDE}"       , altitude                                     );
            html = html.replace( "{LB_GPS_ACCURACY}", activity.getString(R.string.md_gps_accuracy) );
            html = html.replace( "{GPS_ACCURACY}"   , accuracy                                     );
            html = html.replace( "{PICTURE}"        , picture.getAbsolutePath()                    );
            html = html.replace( "{MAP}"            , map.getAbsolutePath()                        );
            html = html.replace( "{OBJ-NAME}"       , obj.getName()                                );

            boolean hasPics = (picture.exists() || map.exists());
            int lineCount = 0;
            int maxLines;
            String style;
            String table  = "";
            for( TestData data : testData )
            {
                handler.obtainMessage(
                        Controller.mHandler.REPORT_UPDATE,
                        1,
                        -1,
                        "Writting test data: " + data.getTestNumber()
                ).sendToTarget();
                if(hasPics)
                {
                    maxLines  = 59;
                }
                else
                {
                    maxLines  = 67;
                }

                if( (lineCount+data.getMeasurement().size()+2)>=maxLines)
                {
                    style = "page-break-before: always;margin-top:20px;";
                    lineCount = 0;
                }
                else
                {
                    style = "";
                }

                lineCount += data.getMeasurement().size()+2;

                Device d = data.getSetup();
                table += "<table class=\"table-results\" style=\"" + style + "\" width=\"" + tableWidth + "\" align=\"center\">";

                boolean isTempEnabled = false;
                for ( Measurement m : data.getMeasurement() )
                {
                    //CHECK IF TEMPERATURE COMP IS ENABLED
                    if ( m.getResistanceCompensated() > 0 && m.getTemperature() > 0 )
                    {
                        isTempEnabled = true;
                    }
                }

                String temp = isTempEnabled ? FormatData.getStringTempRange(device.getTempUnity(), d.getTempRange()) : "";
                String metal = isTempEnabled ? FormatData.getStringMetal    (d.getMetalType()) : "";

                table += getTableTestHeaderLine(
                        data.getTestNumber()+"",
                        FormatData.getDateFormatted(data.getDate(), device.getDateFormat(), device.getTimeFormat()),
                        temp,
                        metal,
                        "row-header"
                );

                table += getTableLine("MM:SS", "R", "Rc", "I", "T", "row-top");
                for ( Measurement mea : data.getMeasurement() )
                {
                    String
                            resistanceC = FormatData.formatResistance ( mea.getCurrentRange(), mea.getResistanceCompensated() ),
                            temperature = FormatData.formatTemperature( device.getTempUnity(), mea.getTemperature()           ),
                            resistance  = FormatData.formatResistance ( mea.getCurrentRange(), mea.getResistance()            );



                    if ( mea.getStatus()>0 )
                    {
                        resistance  = FormatData.getStringStatus(mea.getStatus(), Locale.getDefault().getLanguage());
                        resistanceC = FormatData.getStringStatus(mea.getStatus(), Locale.getDefault().getLanguage() );
                    }
                    if(mea.getResistanceCompensated()<=0)
                        resistanceC = "";

                    if ( mea.getTemperature()<0 || mea.getTemperature()> Protocol.TEMP_MAX )
                    {
                        resistanceC = "";
                        temperature = "";
                    }

                    table += getTableLine(
                            FormatData.formatElapsedTime( mea.getElapsedTime()                    ),
                            resistance,
                            resistanceC,
                            FormatData.formatCurrent    ( mea.getCurrentRange(), mea.getCurrent() ),
                            temperature,
                            "row-bottom"
                    );
                }
                table += "</table>";
            }

            html = html.replace("{TABLE}", table);

            if(!tempDir.exists())
                tempDir.mkdir();

            Bitmap bm                  = BitmapFactory.decodeResource( activity.getResources(), R.drawable.brand_report);
            File file                  = new File(tempDir, "ic_brand.png");
            FileOutputStream outStream = new FileOutputStream(file);

            int   newHeight = 25;
            float scale     = (float)(bm.getHeight() / newHeight);
            int   newWidth  = (int)  (bm.getWidth() / scale);

            bm.compress(Bitmap.CompressFormat.PNG, 100, outStream);
            outStream.flush();
            outStream.close();
            html = html.replace("{REPORT BRAND}", file.getAbsolutePath());
            html = html.replace("{LB_WIDTH}", newWidth+"");
            html = html.replace("{LB_HEIGHT}", newHeight+"");

            String generatedBy = activity.getString(R.string.md_report_generated) + "<br />" +
                    activity.getString(R.string.app_name) + "<br />" +
                    activity.getString(R.string.version) + ": " +
                    activity.getPackageManager().getPackageInfo(activity.getPackageName(), 0).versionName;
            html = html.replace("{LB_GENERATED}", generatedBy);

            html = html.replaceAll("\u03A9", "<span class=\"style-symbol\">W</span>");
            html = html.replaceAll("\u00B5", "<span class=\"style-symbol\">m</span>");
            html = html.replaceAll("\u2109", "°F");
            html = html.replaceAll("\u2103", "°C");

            handler.obtainMessage(Controller.mHandler.REPORT_UPDATE, 1, -1, activity.getString(R.string.please_wait)).sendToTarget();

            worker.parseXHtml(pdfWriter, document, new StringReader(html));

            document.close();
            fin.close();
            r.close();

            handler.obtainMessage(Controller.mHandler.REPORT_UPDATE, 1, -1, "Cleaning").sendToTarget();
            if(tempDir.exists()){
                File[] children = tempDir.listFiles();
                for (int i = 0; i < children.length; i++)
                {
                    children[i].delete();
                }
                tempDir.delete();
            }

            handler.obtainMessage(Controller.mHandler.REPORT_FINISHED, 1, -1, reportFile).sendToTarget();
        }
        catch(Exception e){
            handler.obtainMessage(Controller.mHandler.REPORT_ERROR, 1, -1, "ERROR").sendToTarget();
        }
    }

    private String getTableTestHeaderLine(String number, String date, String t, String metal, String style){
        String result = "";
        result += "<tr>";
        result += "<td class=\"" + style + "-left"  + "\" width=\"" + tableWidth/5 + "\">" + activity.getString(R.string.test_number) + ": " + number + "</td>" ;
        //if ( Controller.model.equals( Controller.fullModel ) )
        if(Main.equipmentId==257)
        {
            result += "<td class=\"" + style + "-left"  + "\" width=\"" + (tableWidth/5)*2 + "\" colspan=\"2\">" + activity.getString(R.string.date_time)   + ": " + date   + "</td>" ;
            result += "<td class=\"" + style + "-left"  + "\" width=\"" + tableWidth/5 + "\" >" + "Metal" + ": " + metal  + "</td>" ;
            result += "<td class=\"" + style + "-right" + "\" width=\"" + tableWidth/5 + "\" >" + "Tref" + ": " + t + "</td>" ;
        }
        else
        {
            result += "<td class=\"" + style + "-right"  + "\" width=\"" + (tableWidth/5)*2 + "\" colspan=\"2\">" + activity.getString(R.string.date_time)   + ": " + date   + "</td>" ;
        }
        result += "</tr>";

        return result;
    }

    private String getTableLine(String elapsed, String r, String rc, String i, String t, String style){
        String result = "";
        result+= "<tr>";
        result+= "<td class=\"" + style + "-left"  + "\" width=\"" + tableWidth/5 + "\">" + elapsed + "</td>";
        result+= "<td class=\"" + style + "-left"  + "\" width=\"" + tableWidth/5 + "\">" + r       + "</td>";
        //if ( Controller.model.equals( Controller.fullModel ) )
        if(Main.equipmentId==257)
        {
            result+= "<td class=\"" + style + "-left"  + "\" width=\"" + tableWidth/5 + "\">" + i       + "</td>";
            result+= "<td class=\"" + style + "-left"  + "\" width=\"" + tableWidth/5 + "\">" + rc      + "</td>";
            result+= "<td class=\"" + style + "-right" + "\" width=\"" + tableWidth/5 + "\">" + t       + "</td>";
        }
        else
        {
            result+= "<td class=\"" + style + "-right"  + "\" width=\"" + tableWidth/5 + "\">" + i       + "</td>";
        }
        result+= "</tr>";

        return result;
    }
}
