package com.megabras.bluelogg.mpk257;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.TextView;

import com.megabras.bluelogg.Main;
import com.megabras.bluelogg.R;

import java.util.ArrayList;
import java.util.List;

public class TableBaseAdapter extends BaseAdapter
{
    private LayoutInflater
            inflater;

    private List<TableValues>
            tableValues;

    @SuppressWarnings("FieldCanBeLocal")
    private int
            layoutID   = R.layout.mpk257_table_item,
            headerID   = R.layout.mpk257_table_header;

    private List<TableHeaderValues>
            headerValues;

    public TableBaseAdapter(Context context)
    {
        tableValues  = new ArrayList<>();
        headerValues = new ArrayList<>();
        inflater     = LayoutInflater.from(context);
    }

    static class ViewHolder{
        TextView
                tv_elapsed,
                tv_resistance,
                tv_resistance_comp,
                tv_current,
                tv_temperature;
    }

//    public void updateLastHeader(int testNumber, String date, String temperature, String metal)
//    {
//        if( headerValues.size() > 0 )
//        {
//            headerValues.get( headerValues.size() - 1 ).testNumber = testNumber;
//            headerValues.get( headerValues.size() - 1 ).date = date;
//            headerValues.get( headerValues.size() - 1 ).temperatureSet = temperature;
//            headerValues.get( headerValues.size() - 1 ).temperatureSet = metal;
//            //notifyDataSetChanged();
//        }
//    }

    public void setTestNumber(int testNumber)
    {
        if(headerValues.size()==0 || headerValues.get(headerValues.size()-1).testNumber!=testNumber){
            TableHeaderValues values = new TableHeaderValues( tableValues.size(), testNumber );
            headerValues.add(values);
        }
    }

    public int getTestNumber(){
        return headerValues.get( headerValues.size()-1 ).testNumber;
    }

    public void setDate(String date) {
        headerValues.get( headerValues.size()-1 ).date = date;
    }

    void setTemperatureSet( String temperatureSet ) {
        if(headerValues.get( headerValues.size()-1 ).temperatureSet.equals(""))
            headerValues.get( headerValues.size()-1 ).temperatureSet = temperatureSet;
    }

    public void setMetal(String metal) {
        if(headerValues.get( headerValues.size()-1 ).metal.equals(""))
            headerValues.get( headerValues.size()-1 ).metal = metal;
    }

    public long getItemId(int position)
    {
        return 0;
    }

    public int getCount()
    {
        return tableValues.size();
    }

    public TableValues getItem(int position)
    {
        return tableValues.get(position);
    }

    void addAll( List<TableValues> tableValues )
    {
        this.tableValues = tableValues;
        notifyDataSetChanged();
    }

    public void add(String time, String resistance, String resistanceC, String current, String temperature)
    {
        tableValues.add( new TableValues(time, resistance, resistanceC, current, temperature) );
    }

    public void repaint()
    {
        notifyDataSetChanged();
    }

    private TextView getTV(int id, View v)
    {
        return ( (TextView)v.findViewById(id) );
    }

    private TableHeaderValues getHeaderValues(int position)
    {
        for(TableHeaderValues val : headerValues)
        {
            if (val.index == position)
                return val;
        }
        return null;
    }

    @SuppressWarnings("RedundantCast")
    public View getView( int position, View vi, ViewGroup parent)
    {
        TableHeaderValues val = getHeaderValues(position);
        if( val != null )
        {
            vi = inflater.inflate(headerID, parent, false);
            String sNum = val.testNumber+"";
            getTV(R.id.tv_number, vi).setText(sNum);
            getTV(R.id.tv_date, vi).setText(val.date);
            getTV(R.id.tv_elapsed, vi).setText( tableValues.get(position).time );
            getTV(R.id.tv_resistance, vi).setText( tableValues.get(position).resistance );
            getTV(R.id.tv_current, vi).setText( tableValues.get(position).current );
            //if ( Controller.model.equals(Controller.fullModel) ){
            if( Main.equipmentId == 257 )
            {
                getTV(R.id.tv_temperature_set, vi).setText(val.temperatureSet);
                getTV(R.id.tv_metal, vi).setText(val.metal);
                getTV(R.id.tv_resistance_comp, vi).setText( tableValues.get(position).resistanceC );
                getTV(R.id.tv_temperature, vi).setText( tableValues.get(position).temperature );
            }
            else
            {
                getTV(R.id.tv_temperature_set, vi).setVisibility(View.GONE);
                getTV(R.id.tv_metal, vi).setVisibility(View.GONE);
                getTV(R.id.tv_resistance_comp, vi).setVisibility(View.GONE);
                getTV(R.id.tv_temperature, vi).setVisibility(View.GONE);
                getTV(R.id.lb_metal, vi).setVisibility(View.GONE);
                getTV(R.id.lb_temp, vi).setVisibility(View.GONE);
                getTV(R.id.lb_rc, vi).setVisibility(View.GONE);
                getTV(R.id.lb_temp_meas, vi).setVisibility(View.GONE);
            }
        }
        else
        {
            if (vi == null || (int)vi.getId() != layoutID)
            {
                ViewHolder h         = new ViewHolder();
                vi                   = inflater.inflate(layoutID, parent, false);
                h.tv_elapsed         = getTV(R.id.tv_elapsed, vi);
                h.tv_resistance      = getTV(R.id.tv_resistance, vi);
                h.tv_current         = getTV(R.id.tv_current, vi);
                h.tv_resistance_comp = getTV(R.id.tv_resistance_comp, vi);
                h.tv_temperature     = getTV(R.id.tv_temperature, vi);
                vi.setTag(h);
            }
            ViewHolder h = (ViewHolder) vi.getTag();
            h.tv_elapsed.setText( tableValues.get(position).time );
            h.tv_resistance.setText( tableValues.get(position).resistance );
            h.tv_current.setText( tableValues.get(position).current );
            //if ( Controller.model.equals(Controller.fullModel) ){
            if( Main.equipmentId == 257 )
            {
                h.tv_resistance_comp.setText( tableValues.get(position).resistanceC );
                h.tv_temperature.setText( tableValues.get(position).temperature );
            }
            else
            {
                h.tv_resistance_comp.setVisibility(View.GONE);
                h.tv_temperature.setVisibility(View.GONE);
            }
        }

        return vi;
    }

}

