package com.megabras.bluelogg.tm25r;

import java.io.BufferedOutputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Locale;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;
import java.util.zip.ZipOutputStream;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import android.app.Activity;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.os.Handler;
import android.util.Log;

import com.megabras.bluelogg.em4058.EM4058test;
import com.megabras.bluelogg.MUtils;
import com.megabras.bluelogg.R;

public class ReportODT {
	private final String TAG = "ReportODT";
	private Activity activity;
	private EM4058test testEM;
	private String testDate = "";
	private String selsite;
	private Locale local = Locale.getDefault();
	private String fileName;
	private Handler handler;
	public static final int MESSAGE_STATE_CHANGE = 1;
    public static final int MESSAGE_READ = 2;
    public static final int MESSAGE_WRITE = 3;
    public static final int MESSAGE_DEVICE_NAME = 4;
    public static final int MESSAGE_TOAST = 5;
    private double snapScale = 0;
    private double snapWidth = 7.25;
    private double snapHeight = 8.999;
    private double picScale = 0;
    private double picWidth = 6.75;
    private double picHeight = 9.00;

	public ReportODT(Activity activity, EM4058test test){
		this.activity = activity;
		this.testEM = test;
	}
	
	
	public void setHandler(Handler handler){
		this.handler = handler;
	}
	
	
	private String getFileName(String site){		
		selsite = site;
		String testSite = "";
		if (!site.equals("")){
			testSite = site;
		}

		File dir;
		if(testEM!=null){
			dir = testEM.getRootFolder();
		}
		else{
			return null;
		}

				
		int lenght = testSite.length();
		
		String siteName = testSite.substring(0, lenght - 15);
		String date = testSite.substring(lenght - 15);
		
		date = date.replace("_", "");
		date = date.replace("-", "");
		String YY = date.substring(0, 4);
		String MM = date.substring(4, 6);
		String DD = date.substring(6, 8);
		String hh = date.substring(8, 10);
		String mm = date.substring(10, 12);
		
		date     = YY + "-" + MM + "-" + DD + "_" + hh + mm;
		testDate = DD + "/" + MM + "/" + YY + " " + hh + ":" + mm;
		
		testSite = MUtils.removeSpecialChars(testSite);
		testSite = siteName + date + ".odt";
		
		File file = new File(dir, testSite);
		
		return file.toString();
	}
	
	
	public String getFilePath(){
		return fileName;
	}
	
	
	private void updateStatus(int id){		
		handler.obtainMessage(MESSAGE_READ, 1, -1, text(id)).sendToTarget();
	}
	
	
	public void saveReport(String site) {
		try {
			Log.e(TAG,"A--------------------------------------------");
			fileName = getFileName(site);
			File temp = new File(fileName);			
			if(temp.exists()){
				temp.delete();
				temp = null;
				//activity.sendBroadcast(new Intent(Intent.ACTION_MEDIA_MOUNTED, Uri.parse("file://" + Environment.getExternalStorageDirectory() + "/" + text(R.string.company) )));
			}

			updateStatus(R.string.loading_template);
			
			//Decompress
			InputStream fin = activity.getAssets().open("template.odt");
			ZipInputStream zin = new ZipInputStream(fin); 
			ZipEntry ze = null;
			
			//Compress
			FileOutputStream dest =  new FileOutputStream( fileName );
			ZipOutputStream  out  =  new ZipOutputStream( new BufferedOutputStream(dest) );
			
			while( (ze = zin.getNextEntry()) != null ) {
				if(ze.getName().equals("content.xml")){
					out.putNextEntry(ze);
					InputStream con = this.getContentXml();
					
					for (int c = con.read(); c != -1; c = con.read()) { 
						out.write(c); 
					}	
					zin.closeEntry();
				} 
				else if(ze.getName().equals("styles.xml")){
					out.putNextEntry(ze);
					InputStream con = getStylesXml();
					for (int c = con.read(); c != -1; c = con.read()) {
						out.write(c);
					}
					zin.closeEntry();
				}
				else if(ze.getName().equals("META-INF/manifest.xml")){
					out.putNextEntry(ze);
					InputStream con = getManifestXml();
					for (int c = con.read(); c != -1; c = con.read()) {
						out.write(c);
					}
					zin.closeEntry();
				}
				else {					
					out.putNextEntry(ze); 				
					for (int c = zin.read(); c != -1; c = zin.read()) { 
						out.write(c); 
					}				
					zin.closeEntry();
				}
			}

			/** IMAGES */
			String[] objects;
			objects = testEM.getObjectsFromFile(selsite);

			for (String obj : objects){
				File img;
				img = testEM.getPictureFileFromFile(selsite, obj);
				
				if(img.exists()){
					updateStatus(R.string.inserting_pictures);
					ZipEntry zeImg = new ZipEntry("Pictures/"+img.getName());
					out.putNextEntry(zeImg);
					InputStream inImg = new FileInputStream(img);
					byte[] bit = new byte[(int) img.length()];
					inImg.read(bit);					
					out.write(bit);
					inImg.close();
				}
			}
			
			File snap;
			snap = testEM.getSnapshotFile(selsite);
			
			if(snap.exists()){					
				updateStatus(R.string.inserting_pictures);
				ZipEntry zeImg = new ZipEntry("Pictures/"+snap.getName());
				out.putNextEntry(zeImg);
				InputStream inImg = new FileInputStream(snap);
				byte[] bit = new byte[(int) snap.length()];
				inImg.read(bit);					
				out.write(bit);
				inImg.close();					
			}			
			out.close();
			zin.close();
			//activity.sendBroadcast(new Intent(Intent.ACTION_MEDIA_MOUNTED, Uri.parse("file://" + Environment.getExternalStorageDirectory() + "/" + text(R.string.company) )));
			updateStatus(R.string.file_send);
		} catch (Exception e) {
			e.printStackTrace();
			Log.e("ReportODT",e.getMessage() + "");
		}
	}

	
	private String text(int id){
		return activity.getResources().getString(id).toUpperCase(local);
	}

	
	private void createNewTables(Document doc){		
		Node body = doc.getElementsByTagName("office:text").item(0);		

		String[] objects;
		objects = testEM.getObjectsFromFile(selsite);
		
		for (String obj : objects){			
			Element table = doc.createElement("table:table");				
			table.setAttributeNode( setAttribute( doc, "table:name", "table-results" ) );
			table.setAttributeNode( setAttribute( doc, "table:style-name", "table-results" ) );

			Element column;
			column = doc.createElement("table:table-column");
			column.setAttributeNode( setAttribute( doc, "table:style-name", "table-results.A" ) );
			column.setAttributeNode( setAttribute( doc, "table:number-columns-repeated", "3" ) );			
			table.appendChild(column);

			column = doc.createElement("table:table-column");
			column.setAttributeNode( setAttribute( doc, "table:style-name", "table-results.D" ) );
			table.appendChild(column);
		
			Element row;
			Element cell;
			Element line;

			//ROW 1		
			row = doc.createElement("table:table-row");		
			row.setAttributeNode( setAttribute( doc, "table:style-name", "table-results.1" ) );
			
			cell = doc.createElement("table:table-cell");
			cell.setAttributeNode( setAttribute( doc, "table:style-name", "table-results.A1" ) );
			cell.setAttributeNode( setAttribute( doc, "table:number-columns-spanned", "4" ) );
			cell.setAttributeNode( setAttribute( doc, "office:value-type", "string" ) );
						
			line = doc.createElement("text:p");
			line.setAttribute("text:style-name", "P5");
			line.setTextContent( text(R.string.object_name) );			
			cell.appendChild(line);
			row.appendChild(cell);
			
			cell = doc.createElement("table:covered-table-cell");
			row.appendChild(cell);
			cell = doc.createElement("table:covered-table-cell");
			row.appendChild(cell);
			cell = doc.createElement("table:covered-table-cell");
			row.appendChild(cell);
			
			table.appendChild(row);
			
			//ROW 2
			row = doc.createElement("table:table-row");			
			
			cell = doc.createElement("table:table-cell");
			cell.setAttributeNode( setAttribute( doc, "table:style-name", "table-results.A2" ) );
			cell.setAttributeNode( setAttribute( doc, "table:number-columns-spanned", "4" ) );
			cell.setAttributeNode( setAttribute( doc, "office:value-type", "string" ) );
						
			line = doc.createElement("text:p");
			line.setAttribute("text:style-name", "P11");
			line.setTextContent( obj.toUpperCase(Locale.getDefault()) );			
			cell.appendChild(line);
			row.appendChild(cell);
			
			cell = doc.createElement("table:covered-table-cell");
			row.appendChild(cell);
			cell = doc.createElement("table:covered-table-cell");
			row.appendChild(cell);
			cell = doc.createElement("table:covered-table-cell");
			row.appendChild(cell);
			
			table.appendChild(row);
			
			//ROW 3
			row = doc.createElement("table:table-row");	
			row.setAttributeNode( setAttribute( doc, "table:style-name", "table-results.1" ) );			
			cell = doc.createElement("table:table-cell");
			cell.setAttributeNode( setAttribute( doc, "table:style-name", "table-results.A3" ) );
			cell.setAttributeNode( setAttribute( doc, "office:value-type", "string" ) );						
			line = doc.createElement("text:p");
			line.setAttribute("text:style-name", "P5");
			line.setTextContent( text(R.string.latitude) );			
			cell.appendChild(line);			
			row.appendChild(cell);
			
			cell = doc.createElement("table:table-cell");
			cell.setAttributeNode( setAttribute( doc, "table:style-name", "table-results.A3" ) );
			cell.setAttributeNode( setAttribute( doc, "office:value-type", "string" ) );						
			line = doc.createElement("text:p");
			line.setAttribute("text:style-name", "P5");
			line.setTextContent( text(R.string.longitude) );			
			cell.appendChild(line);			
			row.appendChild(cell);
			
			cell = doc.createElement("table:table-cell");
			cell.setAttributeNode( setAttribute( doc, "table:style-name", "table-results.A3" ) );
			cell.setAttributeNode( setAttribute( doc, "office:value-type", "string" ) );						
			line = doc.createElement("text:p");
			line.setAttribute("text:style-name", "P5");
			line.setTextContent( text(R.string.altitude) );			
			cell.appendChild(line);			
			row.appendChild(cell);
			
			cell = doc.createElement("table:table-cell");
			cell.setAttributeNode( setAttribute( doc, "table:style-name", "table-results.A3" ) );
			cell.setAttributeNode( setAttribute( doc, "office:value-type", "string" ) );						
			line = doc.createElement("text:p");
			line.setAttribute("text:style-name", "P5");
			line.setTextContent( text(R.string.accuracy) );			
			cell.appendChild(line);			
			row.appendChild(cell);
			
			table.appendChild(row);
			
			//ROW 4
			String[] gps;
			gps = testEM.getGpsDataFromFile(obj,selsite);

			
			row = doc.createElement("table:table-row");	
			row.setAttributeNode( setAttribute( doc, "table:style-name", "table-results.4" ) );			
			cell = doc.createElement("table:table-cell");
			cell.setAttributeNode( setAttribute( doc, "table:style-name", "table-results.A4" ) );
			cell.setAttributeNode( setAttribute( doc, "office:value-type", "string" ) );						
			line = doc.createElement("text:p");
			line.setAttribute("text:style-name", "P12");
			line.setTextContent( gps[1] );			
			cell.appendChild(line);			
			row.appendChild(cell);
			
			cell = doc.createElement("table:table-cell");
			cell.setAttributeNode( setAttribute( doc, "table:style-name", "table-results.A4" ) );
			cell.setAttributeNode( setAttribute( doc, "office:value-type", "string" ) );						
			line = doc.createElement("text:p");
			line.setAttribute("text:style-name", "P12");
			line.setTextContent( gps[0] );			
			cell.appendChild(line);			
			row.appendChild(cell);
			
			cell = doc.createElement("table:table-cell");
			cell.setAttributeNode( setAttribute( doc, "table:style-name", "table-results.A4" ) );
			cell.setAttributeNode( setAttribute( doc, "office:value-type", "string" ) );						
			line = doc.createElement("text:p");
			line.setAttribute("text:style-name", "P12");
			line.setTextContent( gps[2] );			
			cell.appendChild(line);			
			row.appendChild(cell);
			
			cell = doc.createElement("table:table-cell");
			cell.setAttributeNode( setAttribute( doc, "table:style-name", "table-results.A4" ) );
			cell.setAttributeNode( setAttribute( doc, "office:value-type", "string" ) );						
			line = doc.createElement("text:p");
			line.setAttribute("text:style-name", "P12");
			line.setTextContent( gps[3] );			
			cell.appendChild(line);			
			row.appendChild(cell);
			
			table.appendChild(row);
			
			//ROW 5			
			row = doc.createElement("table:table-row");			
			row.setAttributeNode( setAttribute( doc, "table:style-name", "table-results.1" ) );
			
			cell = doc.createElement("table:table-cell");
			cell.setAttributeNode( setAttribute( doc, "table:style-name", "table-results.A3" ) );
			cell.setAttributeNode( setAttribute( doc, "table:number-columns-spanned", "4" ) );
			cell.setAttributeNode( setAttribute( doc, "office:value-type", "string" ) );
						
			line = doc.createElement("text:p");
			line.setAttribute("text:style-name", "P5");
			line.setTextContent( text(R.string.results) );			
			cell.appendChild(line);
			row.appendChild(cell);
			
			cell = doc.createElement("table:covered-table-cell");
			row.appendChild(cell);
			cell = doc.createElement("table:covered-table-cell");
			row.appendChild(cell);
			cell = doc.createElement("table:covered-table-cell");
			row.appendChild(cell);
			
			table.appendChild(row);
			
			//ROW 6
			row = doc.createElement("table:table-row");
			row.setAttributeNode( setAttribute( doc, "table:style-name", "table-results.6" ) );

			cell = doc.createElement("table:table-cell");
			cell.setAttributeNode( setAttribute( doc, "table:style-name", "table-results.A6" ) );
			cell.setAttributeNode( setAttribute( doc, "table:number-columns-spanned", "4" ) );
			cell.setAttributeNode( setAttribute( doc, "office:value-type", "string" ) );

			//HERE
			ReportTable tabla;
			ArrayList<EM4058test.EM4058values> list = testEM.getEM4058TestResultsFromFile(selsite, obj);
			tabla = new ReportTable(activity, doc);
			tabla.createTable("table-result");
			tabla.setColumns(11);
			tabla.setRowsEM4058(list);

			cell.appendChild(tabla.getTable());
			row.appendChild(cell);

			cell = doc.createElement("table:covered-table-cell");
			row.appendChild(cell);
			cell = doc.createElement("table:covered-table-cell");
			row.appendChild(cell);
			cell = doc.createElement("table:covered-table-cell");
			row.appendChild(cell);

			table.appendChild(row);

			File img;
			img = testEM.getPictureFileFromFile(selsite, obj);

			Log.e(TAG,"FILE: " + img.getAbsolutePath());

			if (img.exists()){
				//ROW 7			
				row = doc.createElement("table:table-row");			
				row.setAttributeNode( setAttribute( doc, "table:style-name", "table-results.1" ) );			
				cell = doc.createElement("table:table-cell");
				cell.setAttributeNode( setAttribute( doc, "table:style-name", "table-results.A3" ) );
				cell.setAttributeNode( setAttribute( doc, "table:number-columns-spanned", "4" ) );
				cell.setAttributeNode( setAttribute( doc, "office:value-type", "string" ) );						
				line = doc.createElement("text:p");
				line.setAttribute("text:style-name", "P5");
				line.setTextContent( text(R.string.picture) );			
				cell.appendChild(line);
				row.appendChild(cell);			
				cell = doc.createElement("table:covered-table-cell");
				row.appendChild(cell);
				cell = doc.createElement("table:covered-table-cell");
				row.appendChild(cell);
				cell = doc.createElement("table:covered-table-cell");
				row.appendChild(cell);			
				table.appendChild(row);
				
				if (picScale <= 0){
					BitmapFactory.Options options = new BitmapFactory.Options();
					options.inPreferredConfig = Bitmap.Config.ARGB_8888;
					Bitmap bitmap = BitmapFactory.decodeFile(img.getAbsolutePath(), options);
					picScale = (double) bitmap.getWidth() / (double) bitmap.getHeight();					
					if (picScale < 1){
						picScale = 1 / picScale;
					}					
					picWidth = picHeight / picScale;
				}
				
				//ROW 8
				row = doc.createElement("table:table-row");
				row.setAttributeNode( setAttribute( doc, "table:style-name", "table-results.5" ) );	
				
				cell = doc.createElement("table:table-cell");
				cell.setAttributeNode( setAttribute( doc, "table:style-name", "table-results.A5" ) );
				cell.setAttributeNode( setAttribute( doc, "table:number-columns-spanned", "4" ) );
				cell.setAttributeNode( setAttribute( doc, "office:value-type", "string" ) );				
				line = doc.createElement("text:p");
				line.setAttribute("text:style-name", "P10");
				Element drawFrame = doc.createElement("draw:frame");				
				drawFrame.setAttribute("draw:name", "figura1");
				drawFrame.setAttribute("text:anchor-type", "as-char");
				drawFrame.setAttribute("svg:width", picWidth+"cm");
				drawFrame.setAttribute("svg:height", picHeight+"cm");
				
				Element drawImage = doc.createElement("draw:image");
				drawImage.setAttribute("xlink:href", "Pictures/"+img.getName());
				drawImage.setAttribute("xlink:type", "simple");
				drawImage.setAttribute("xlink:show", "embed");
				drawImage.setAttribute("xlink:actuate", "onLoad");				
				drawFrame.appendChild(drawImage);
				line.appendChild(drawFrame);	
				cell.appendChild(line);
				row.appendChild(cell);
				
				cell = doc.createElement("table:covered-table-cell");
				row.appendChild(cell);
				cell = doc.createElement("table:covered-table-cell");
				row.appendChild(cell);
				cell = doc.createElement("table:covered-table-cell");
				row.appendChild(cell);
				table.appendChild(row);
			}
			
			body.appendChild(table);
		}
	}
	
		
	private Attr setAttribute(Document doc, String attributeName, String attributeValue){
		Attr attr = doc.createAttribute(attributeName);
		attr.setValue(attributeValue);
		return attr;
	}
	
	
	public InputStream getContentXml(){		
		try{
			
			DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();			
			DocumentBuilder	documentBuilder = documentBuilderFactory.newDocumentBuilder();			 
			Document doc = documentBuilder.parse( activity.getAssets().open("content_em4058.xml") );
			
			String[] operator;
			operator = testEM.getOperatorFromFile(selsite);

			updateStatus(R.string.creating_tables);
			
			this.createNewTables(doc);
						
			doc.getElementById("L-GENERAL-INFO").setTextContent( text(R.string.general_information) );			
			doc.getElementById("L-TEST-DATE").setTextContent( text(R.string.test_date) );			
			doc.getElementById("F-TEST-DATE").setTextContent( testDate );			
			doc.getElementById("L-EQUIPMENT").setTextContent( text(R.string.equipment) );



			doc.getElementById("F-EQUIPMENT").setTextContent( testEM.getEquipmentModel() );

			doc.getElementById("L-OPERATOR").setTextContent( text(R.string.operator) );
			doc.getElementById("F-OPERATOR").setTextContent( operator[0] + " / " + operator[1] + " / " + operator[2] );			
			doc.getElementById("L-COMMENTS").setTextContent( text(R.string.comments) );			
			doc.getElementById("L-COMP").setTextContent( text(R.string.firm) );
			doc.getElementById("L-MAP").setTextContent( text(R.string.map) );
			
			String[] company;
			company = testEM.getCompany();

			if (company!=null){
				doc.getElementById("F-COMP-NAME").setTextContent( company[0] );
				doc.getElementById("F-COMP-ADDRESS").setTextContent( company[1] );
				doc.getElementById("F-COMP-ADDRESS-2").setTextContent( company[2] + " - " + company[3] + " - " + company[4] );
				doc.getElementById("F-COMP-PHONE").setTextContent( text(R.string.phone) + " " + company[5] + " / " + text(R.string.fax) + " " + company[6] );				
			}			
			
			doc.getElementById("L-SITE").setTextContent( text(R.string.test_site) );
			
			String[] testsite;
			testsite = testEM.getTestsiteFromFile(selsite);

			if (testsite!=null){
				doc.getElementById("F-SITE-NAME").setTextContent( testsite[0] );
				doc.getElementById("F-SITE-CONTACT").setTextContent( testsite[1] );
				doc.getElementById("F-SITE-ADDRESS").setTextContent( testsite[2] );
				doc.getElementById("F-SITE-ADDRESS-2").setTextContent( testsite[3] + " - " + testsite[4] + " - " + testsite[5] );
				doc.getElementById("F-SITE-PHONE").setTextContent( text(R.string.phone) + " " + testsite[6] + " / " + text(R.string.fax) + " " + testsite[7] );				
			}
			
			Node com = doc.getElementById("F-COMMENTS").getParentNode();
			
			com.removeChild( doc.getElementById("F-COMMENTS") );

			String[] comments;
			comments = testEM.getComments(selsite).split("\\n");
			
			if (comments!=null){
				for (String li : comments){
					Element line = doc.createElement("text:p");
					line.setAttribute("text:style-name", "P8");
					line.setTextContent(li);
					com.appendChild(line);					
				}												
			}
			
			File snap;
			snap = testEM.getSnapshotFile(selsite);
			
			if (snap.exists()){
				if (snapScale <= 0){
					BitmapFactory.Options options = new BitmapFactory.Options();
					options.inPreferredConfig = Bitmap.Config.ARGB_8888;
					Bitmap bitmap = BitmapFactory.decodeFile(snap.getAbsolutePath(), options);
					snapScale = (double) bitmap.getWidth() / (double) bitmap.getHeight();
					if (snapScale < 1){
						snapScale = 1 / snapScale;
					}					
					snapWidth = snapHeight / snapScale;
				}
				Element drawFrame = doc.createElement("draw:frame");
				drawFrame.setAttribute("draw:name", "figura1");
				drawFrame.setAttribute("text:anchor-type", "as-char");
				drawFrame.setAttribute("svg:width", snapWidth+"cm");
				drawFrame.setAttribute("svg:height", snapHeight+"cm");
				
				Element drawImage = doc.createElement("draw:image");
				drawImage.setAttribute("xlink:href", "Pictures/"+snap.getName());
				drawImage.setAttribute("xlink:type", "simple");
				drawImage.setAttribute("xlink:show", "embed");
				drawImage.setAttribute("xlink:actuate", "onLoad");				
				drawFrame.appendChild(drawImage);
				doc.getElementById("snapshot").appendChild(drawFrame);
			} else {				
				Element mainTable = doc.getElementById("tablegeneral");
				Node map1 = doc.getElementById("map-1");
				Node map2 = doc.getElementById("map-2");
				mainTable.removeChild(map1);
				mainTable.removeChild(map2);
			}			
			
			ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
			Source xmlSource = new DOMSource(doc);
			Result outputTarget = new StreamResult(outputStream);
			TransformerFactory.newInstance().newTransformer().transform(xmlSource, outputTarget);
			InputStream is = new ByteArrayInputStream(outputStream.toByteArray());
			
			return is;
			
		} catch (Exception e) {
			return null;
		}		
	}
	
	
	public InputStream getStylesXml(){		
		try{			
			DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();			
			DocumentBuilder	documentBuilder = documentBuilderFactory.newDocumentBuilder();			 
			Document doc = documentBuilder.parse( activity.getAssets().open("styles.xml") );

			doc.getElementById("L-TITLE").setTextContent( text(R.string.ground_tester_report) );
			
			ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
			Source xmlSource = new DOMSource(doc);
			Result outputTarget = new StreamResult(outputStream);
			TransformerFactory.newInstance().newTransformer().transform(xmlSource, outputTarget);
			InputStream is = new ByteArrayInputStream(outputStream.toByteArray());
			
			return is;
			
		} catch (Exception e) {
			return null;
		}		
	}
	
	
	public InputStream getManifestXml(){		
		try{			
			DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();			
			DocumentBuilder	documentBuilder = documentBuilderFactory.newDocumentBuilder();			 
			Document doc = documentBuilder.parse( activity.getAssets().open("manifest.xml") );
			
			Element manifest = (Element) doc.getElementsByTagName("manifest:manifest").item(0);	
			
			String[] objects;
			objects = testEM.getObjectsFromFile(selsite);

			for (String obj : objects){
				File img;
				img = testEM.getPictureFileFromFile(selsite, obj);
				if(img.exists()){
					Element el = doc.createElement("manifest:file-entry");
					el.setAttribute("manifest:media-type", "image/jpeg");
					el.setAttribute("manifest:full-path", "Pictures/"+img.getName());					
					manifest.appendChild(el);					
				}
			}
			
			File snap;
			snap = testEM.getSnapshotFile(selsite);

			if (snap.exists()){
				Element el = doc.createElement("manifest:file-entry");
				el.setAttribute("manifest:media-type", "image/jpeg");
				el.setAttribute("manifest:full-path", "Pictures/"+snap.getName());					
				manifest.appendChild(el);
			}
			
			ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
			Source xmlSource = new DOMSource(doc);
			Result outputTarget = new StreamResult(outputStream);
			TransformerFactory.newInstance().newTransformer().transform(xmlSource, outputTarget);
			InputStream is = new ByteArrayInputStream(outputStream.toByteArray());
			
			return is;
			
		} catch (Exception e) {
			return null;
		}		
	}
	
	
}
